	PROGRAM DELSOL
c      SUBROUTINE DELSOL
c
c This version restores the original DELSOL3 receiver and tower cost algorithms described
C in SAND86-8018.  These were restored by Greg Kolb in November 2010.
C
C Copyright 6/29/2009 Sandia Corporation. 
C Under the terms of Contract DE-AC04-94AL85000 
C with Sandia Corporation, the U.S. Government retains certain rights in this software.
C
c
      IMPLICIT REAL*8 (A-H,O-Z)
c
c #####################################################################
c Mike Wagner modified the code January 2008 so that it would run with 
c the Compaq Visual Fortran 6.5 compiler and the GCC (gfortran) 
c compiler.
c - IMPLICIT REAL statements were uncommented
c - COMMON blocks were rearranged to place integer variables after real 
c   numbers
c - Hollerith constants that wrapped over two lines were broken into 
c   character strings.
c - To accomodate compilation on the GCC compiler and to speed up run 
c   time, PRINT output lines were commented.
c - "NML=" specifiers were added to "READ (5," statements
c - Changed PROGRAM declaration to SUBROUTINE
c - Commented "STOP" command in subroutine delsol to allow it to run as 
c   a called subroutine and changed it to "RETURN". See comments at 
c   approximately line 295
c - There was a call to write an "end of program" statement to a unit 
c   that had never been opened, so the unit number was corrected so 
c   that the statement would write to the file output that was 
c   currently being written. 
c #####################################################################
c
C Lee Walton from CRAy consulting modified the code on 8/1/88
C so that it would run with the new complier and operating system
c
c G. Kolb changed idens=4 heliostat spacing option for external receivers.
c idens=4 now
c matches heliostat spacing algorithm suggested by University of Houston
c for stretched membrane heliostats in the 1988 utility study.
c See APS report pg. c-21.
c
c This version also contains SERI mod to emulate aim points used in
c PG&E study
c
C     ******************************************************************
C
C                       ********** NOTICE  **********
C
C         THIS PROGRAM WAS PREPARED AS AN ACCOUNT OF WORK SPONSORED
C     BY THE UNITED STATES GOVERNMENT. NEITHER THE UNITED STATES NOR
C     THE UNITED STATES DEPARTMENT OF ENERGY NOR ANY OF THEIR EMPLOYEES,
C     NOR ANY OF THEIR CONTRACTORS, SUBCONTRACTORS, OR THEIR EMPLOYEES
C     MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL
C     LIABILITY OF RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS OR
C     USEFULNESS OF ANY INFORMATION, APPARATUS, PRODUCT OR PROCESS
C     DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT INFRINGE OWNED
C     RIGHTS.
C
C     ******************************************************************
C
C
C                    VERSION III - JUNE 1986
C
C
C          THE BASIC REFERENCE FOR THE CODE IS
C
C          SANDIA NATIONAL LABORATORIES REPORT SAND86-XXXX
C
C            A USER'S MANUAL FOR DELSOL3:  A COMPUTER CODE FOR
C            CALCULATING THE OPTICAL PERFORMANCE AND OPTIMAL SYSTEM
C            DESIGN FOR SOLAR THERMAL CENTRAL RECEIVER PLANTS
C
C            THEODORE A. DELLIN
C            SANDIA NATIONAL LABORATORIES, ALBUQUERQUE, NM  87185
C
C            MIRIAM J. FISH
C            SANDIA NATIONAL LABORATORIES, LIVERMORE, CA  94550
C
C            CHRISTINE L. YANG
C            SANDIA NATIONAL LABORATORIES, LIVERMORE, CA  94550
C
C             BRUCE L. KISTLER
C             SANDIA NATIONAL LABORATORIES, LIVERMORE, CA 94550
C
C     ******************************************************************
C
C
C         THE DELSOL PROGRAM AND ITS SUBROUTINES CALCULATES FIELD
C     PEFORMANCE, FIELD LAYOUT, AND OPTIMAL SYSTEM DESIGN FOR SOLAR
C     CENTRAL RECEIVER PLANTS. THEY PERFORM AN INITIAL,
C     DETAILED FIELD PERFORMANCE CALCULATION FOR A USER DEFINED OR
C     DEFAULT FIELD, OR FOR A CIRCULAR GRID OF ZONES. THE LATTER OPTION
C     IS USED FOR AN OPTIMAL DESIGN RUN, WHILE EITHER OF THE FIRST TWO
C     APPLIES WHEN THE FIELD PERFORMANCE OF A SPECIFIC SYSTEM IS
C     DESIRED. PERFORMANCE AND OPTIMIZATION SUBROUTINES ARE CALLED BY
C     DELSOL.
C
C
      COMMON WORK(5000)
C     ABOVE BLANK COMMON FOR DISSPLA FOR FLUX MAP PLOTTING
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /DEL3/ IRERUN
C
      COMMON /DEL4/ H2O,INSOL,IPRINT(9),ITAPE,NUAZ,NUEL,NYEAR,
     1  PRES,SOLCON,UAZ(20),UDAY,UEL(9),UTIME
C
C PC VERSION -- DEFINE TITL AS CHARACTER
      dimension TITL(10)
      character TITL*8
C
       NAMELIST /BASIC/  IPROB,NYEAR,HRDEL,UDAY,UTIME,NUAZ,NUEL,UAZ,UEL,
     1                  DHOPT,IPRINT,ITAPE,TDESP,PLAT,ALT,INSOL,SOLCON,
     2                  IWEATH,WEATH,DWEATH,H2O,DH2O,PRES,DPRES,NSUN,
     3                  NSUNPT,SUNI,SUNR,REFDAY,REFTIM,REFSOL,ASTART,
     4                  IATM,ATM1,ATM2,ATM3,ATM4
      IFUDGE = 0
C>>>>I
      IRERUN = 0
c commented next line on 1-18-91 to fix for unicos
c     open(unit=1,file='tty',status='new')
      open(unit=5,file='input',status='old')
      open(unit=6,file='output',status='unknown')
C change 3/27/87 change status of tapes from new to unknown
C this will allow tapes to be overwritten---cray consulting change
      open(unit=10,file='tape10',form='unformatted',status='unknown')
      open(unit=11,file='tape11',status='unknown')
      open(unit=20,file='tape20',status='unknown')
      open(unit=30,file='tape30',form='unformatted',status='unknown')
      open(unit=32,file='tape32',status='unknown')
      open(unit=31,file='tape31',status='unknown')
c     call link("
c    $t1=tty,unit1=(t1,text),
c    $unit5=(input,open,text),read5,
c    $unit6=(output,hc,col1=remove,line=137,text,create),print6,
c    $unit10=(tape10,create),
c    $t11=tape11,unit11=(t11,text,create,line=137),
c    $unit20=(tape20),
c    $unit30=(tape30),
c    $unit32=(tape32,seq,create),
c    $unit31=(tape31,create,text)
c    $//"h)
      WRITE (6,1215)
 1215 FORMAT(1H1,' THE FOLLOWING IS YOUR INPUT FOR THIS JOB'///)
 1213 CONTINUE
      READ (5,26101,END=1214) TITL
      WRITE (6,1211) TITL
      GO TO 1213
26101 FORMAT(10A8)
 1211 FORMAT(1X,10A8)
 1214 CONTINUE
      REWIND 5
      WRITE (6,1212)
 1212 FORMAT(1H1,///////////30X,7HDELSOL3//
     *       30X,43HA COMPUTER CODE FOR CALCULATING THE OPTICAL,/
     *       30X,41HPERFORMANCE AND OPTIMAL SYSTEM DESIGN FOR,/
     *       30X,37HSOLAR THERMAL CENTRAL RECEIVER PLANTS,///
     *       30X,39HSANDIA NATIONAL LABORATORIES, LIVERMORE,//
     *       30X,38HLAST MODIFIED 10-7-86 BY B. L. KISTLER )
C
C             DEFAULT VALUES FOR NAMELIST BASIC
C
c     call ddiopon('ibm')
  100 CONTINUE
      ITAPE = 0
      DHOPT = .2
      NYEAR=5
      HRDEL=1.0
      DO 110 I=1,9
      IPRINT(I)=0
110   CONTINUE
         IF (IFUDGE.NE.0) GO TO 200
C
C             IFUDGE=1 SIGNALS A RERUN OF AN OPTIMIZED SYSTEM
C
      TDESP=100.
      PLAT=35.0
      ALT=0.65
      SOLCON=.95
      NSUN=1
      REFDAY=81.0
      REFTIM=0.0
      REFSOL=0.95
      INSOL=0
      WEATH=0.83
      IPROB=0
      DO 190 I=1,9
      DWEATH(I)=0.83
      DH2O(I)=20.
190   DPRES(I)=1.0
      H2O=20.
      PRES=1.0
      NUEL=6
      NUAZ=7
      UEL(1)=0.5
      UEL(2)=25.
      UEL(3)=45.
      UEL(4)=65.
      UEL(5)=75.
      UEL(6)=85.
      IWEATH=0
      ATM1=.006789
      ATM2=.1046
      ATM4=.002845
      ATM3=-.017
      UAZ(1)=0.
      UAZ(2)=30.
      UAZ(3)=60.
      UAZ(4)=75.
      UAZ(5)=90.
      UAZ(6)=110.
      UAZ(7)=130.
      ASTART=75.0
      UDAY=81.
      UTIME=0.
      IATM=0
C
C             READ IN INPUT
C
  200 CONTINUE
      READ (5,2610) HEAD
c     call ddi(BASIC,5,1,1)
      READ (5,NML=BASIC)
      IF(ASTART.EQ.90.0) WRITE (6,*)
     1'0****ERROR, ASTART CANNOT=90. RESET TO 89.5********'
      IF(ASTART.EQ.90.0) WRITE (1,*)
     1'0****ERROR, ASTART CANNOT=90. RESET TO 89.5********'
      IF(ASTART.EQ.90.0) ASTART=89.5
      WRITE (6,9876) '$BASIC'
 9876 FORMAT(1H1,A8)
C
C     OPTIONAL WEATHER FACTOR BASED ON LATITUDE 9-5-85
      IF(IWEATH.EQ.2) WEATH=2.16255*EXP(-0.02804*PLAT)
      IF(IWEATH.EQ.2) IWEATH=0
C
C      PRINT BASIC
      CALL GENER
      IF ( W .LT. 0.0 ) GO TO 400
         IF (ITAPE.EQ.2) GO TO 300
C>>>>I
      IF ( IRERUN .NE. 0 ) ITAPE = 3
C
C               CALL IN PERCAL OVERLAY TO PERFORM PERFORMANCE CALCULATIO
C               ROUTINES IN OVERLAY ARE
C                     PERCAL
C                     HELPOS
C                     MOMENT
C                     POWER
C                     SHADOW
C
C                    TO RUN CODE WITHOUT OVERLAYS REPLACE
C                    CALL OVERLAY(PERCAL,1,0,RECALL) WITH
C                    CALL PERCAL
C
      CALL PERCAL
        IF ( W .LT. 0.0 ) GO TO 400
  300 CONTINUE
C
C               CALL IN OPTCAL OVERLAY TO COMPUTE OPTIMIZATION
C               ROUTINES IN OVERLAY ARE
C                     OPTCAL
C                     HCOORD
C                     HVALUE
C                     MAX
C
C                    TO RUN CODE WITHOUT OVERLAYS REPLACE
C                    CALL OVERLAY(OPTCAL,2,0,RECALL) WITH
C                    CALL OPTCAL
C
      CALL OPTCAL
         IF ( W .LT. 0.0 ) GO TO 400
         IF ( IRERUN .NE. 0 ) GO TO 100
  400 CONTINUE
c mjw    WRITE (1,3010)  commented 1/7/08.. Unit 6 is output, not unit 1
      WRITE (6,3010)
      close(5)
      close(6)
      close(10)
      close(20)
      close(30)
      close(31)
      close(32)
c      EXIT
c mjw commented next line to get it to run as a subroutine
      STOP
c mjw NOTE: if DELSOL3 is being run as a subroutine, uncomment the next line
c           However, if the DELSOL3 is being used as the main program, then
c	      the "STOP" command should be uncommented.
c	RETURN
C
 2610 FORMAT (5A8)
 3010 FORMAT (///20X,27H ----- END OF PROGRAM -----)
      END
      FUNCTION FLUX(XX,YY)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         FLUX = FLUX DENSITY AT THE POINT (X,Y) IN THE IMAGE
C                PLANE DUE TO A HELIOSTAT IN THE (K,L) ZONE
C
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
C
      DIMENSION HX(9),HY(9)
C
C
      DATA (HX(I),I=1,2)/1.,0./,(HY(I),I=1,2)/1.,0./
C
C
      FX=-2.
      DO 110 I=1,NORDER
      FX=FX+1.
      HX(I+2)=XX*HX(I+1)-FX*HX(I)
110   HY(I+2)=YY*HY(I+1)-FX*HY(I)
      IPAK=0
      FLUX=0.
      DO 120 I=1,NORDER
      JMAX=JMX(I)
      JMIN=JMN(I)
      DO 120 J=JMIN,JMAX,ISTEP
      IPAK=IPAK+1
      FLUX=FLUX+HC(IPAK)*HX(I+2)*HY(J+2)
120   CONTINUE
      RETURN
      END
      SUBROUTINE FLXMAP (K,L)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE CALCULATES THE FLUX AT NFLXPTS
C     PRODUCED BY HELIOSTATS IN THE (K,L) ZONE
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C          PERCAL
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /FLUX1/ CFA(169),CFP(169),DIAMFN,FAZMAS,FAZMIS,FLX(169),
     1  FZMAXN,FZMINN,SFA(169),SFP(169),XFCN,XFLX(169),
     2  XNFLX(169),YFCN,YFLX(169),YNFLX(169),ZFCN,ZFLX(169),NFLXPT
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /REC1/ CAZMMN(4),CAZMMX(4),JREC(12,2),NREC(12),RCRA(4),
     1  RSRA(4)
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C
      DO 110 N=1,NFLXPT
110   FLX(N)=0.
C
C             FLUX IS CALCULATED AT SEVERAL POINTS IN A ZONE
C             USE IMAGE SIZE TO CALCULATE NUMBER OF POINTS
C
C>>>>RII=INT(WN*SIN(DAZM*RR)/AX2)+1
      II = INT(WN*SIN(AZMDEL*RR)/AX2) + 1
      RSTART=RADIUS(K)
      DRAD=0.
      JJ=1
         IF (IREC.LT.1.OR.IREC.GT.2) GO TO 120
      RMAX=RSTART+RADDEL/2.*(1.-FLOAT(K/NRAD))
      RMIN=RSTART-RADDEL/2.*(1.-FLOAT((NRAD-K)/(NRAD-1)))
      JJ=INT((WN/RMIN-WN/RMAX)/AY2)+1
      IF (JJ.LT.2) JJ=2
      DRAD=(RMAX-RMIN)/FLOAT(JJ-1)
      RSTART=RMIN
120   CONTINUE
         IF (IUSERF .NE. 3) GO TO 130
      II=1
      JJ=1
      RSTART=RADIUS(K)
130   CONTINUE
C>>>>RTT=DAZM/FLOAT(II)
      TT = AZMDEL/FLOAT(II)
      WTP=1./FLOAT(II*JJ)
      DO 220 KKL=1,JJ
      RADIUK=RSTART+FLOAT(KKL-1)*DRAD
C>>>>RSSTART=-DAZM/2.-TT/2.
      SSTART = - AZMDEL/2. - TT/2.
      DO 220 J=1,II
      SSTART=SSTART+TT
      CP=COS(RR*SSTART)
      SP=SIN(RR*SSTART)
      CPA=CTA*CP-STA*SP
      SPA=STA*CP+CTA*SP
      T11=CPA
      T13=0.
      T12=-SPA
      T21=CTP*SPA
      T22=CTP*CPA
      T23=STP
      R12=0.
      R11=-1.
      R21=0.
      R22=STP
      SAIMX=1.
      SAIMY=1.
      CPP=CTP
      SPP=STP
      NN=1
         IF (IREC.EQ.0) GO TO 140
      NN=NREC(L)
         IF (NN.EQ.0) GO TO 240
140   CONTINUE
      WREC=1.
      DO 210 IJK=1,NN
         IF (IREC.EQ.0) GO TO 150
      I=JREC(L,IJK)
         IF (ICAVF(I).EQ.0) GO TO 210
      SAIMY=RYN(I)/RYN(1)
      SAIMX=RXN(I)/RXN(1)
      XRECVR=-W2*RSRA(I)
      YRECVR=-W2*RCRA(I)
      ZRECVR=0.
      XHEL=SPA*RADIUK
      YHEL=CPA*RADIUK
      RSAVE=SQRT((XHEL-XRECVR)**2+(YHEL-YRECVR)**2+(1.+ZRECVR)**2)
      CPP=(1.+ZRECVR)/RSAVE
      SPP=SQRT(1.-CPP**2)
      SPA=-(XRECVR-XHEL)/(RSAVE*SPP)
      CPA=-(YRECVR-YHEL)/(RSAVE*SPP)
      T11=CPA
      T12=-SPA
      T13=0.
      T21=CPP*SPA
      T22=CPP*CPA
      T23=SPP
      WREC=WTREC(L,IJK)
      CBRT=CPA*RCRA(I)+SPA*RSRA(I)
      SBRT=CPA*RSRA(I)-SPA*RCRA(I)
      R11=CBRT
      R12=SBRT*RCRP(I)
      R21=-CPP*SBRT
      R22=SPP*RSRP(I)+CPP*RCRP(I)*CBRT
      SAVE=R11*R22-R21*R12
         GO TO 160
150   CONTINUE
      XRECVR=W2*SPA
      YRECVR=W2*CPA
      ZRECVR=0.
160   CONTINUE
C
C             EXTERNAL CYLINDRICAL RECEIVER
C
      DO 200 N=1,NFLXPT
C     ONLY CALCULATE FLUX AT SPECIFIED FLUX POINTS, MAX OF 4
C     FOR OPTIMIZATION, ALL FOR PERFORMANCE RUN
      IF (IPROB.NE.4.OR.IFUDGE.NE.0) GO TO 202
      DO 201 IC=1,NFLXMX
      IF(N.EQ.NMXFLX(IC)) GO TO 202
 201   CONTINUE
      GO TO 200
 202  CONTINUE
      FDOTT=CFP(N)*CPP+SFP(N)*SPP*(CPA*CFA(N)+SPA*SFA(N))
C
C             TEST TO SEE THAT IMAGE HITS FRONT OF FLUX SURFACE
C
         IF (FDOTT.GT.1.E-05) GO TO 200
      FDOTT=- FDOTT
C
C             PROJECT FLUX POINT TO IMAGE PLANE
C
      XTS=T11*(XFLX(N)-XRECVR)+T12*(YFLX(N)-YRECVR)+T13*ZFLX(N)
      YTS=T21*(XFLX(N)-XRECVR)+T22*(YFLX(N)-YRECVR)+T23*ZFLX(N)
      DO 190 M=1,NAIMPT
C
C             DISPLACE FLUX POINT BY PROJECTION OF AIMPOINT - NORMALIZE
C
      YT=(YTS-R22*SAIMY*DYAIM(M)-R21*SAIMX*DXAIM(M))/AY2
      XT=(XTS-R12*SAIMY*DYAIM(M)-R11*SAIMX*DXAIM(M))/AX2
         IF (ABS(XT)+ABS(YT).GT.4.) GO TO 190
         IF (IREC.EQ.0) GO TO 180
         IF (IREC.GT.2) GO TO 180
C
C             PROJECT FLUX POINT TO IMAGE PLANE
C
C     XRF=(R22*XTS-R12*YTS)/SAVE+DXAIM(M)*SAIMX
C     YRF=(-R21*XTS+R11*YTS)/SAVE+DYAIM(M)*SAIMY
C     7-31-85 B.L.KISTLER--LOGIC OF ABOVE WRONG SINCE
C     CHECKING USES NONAIMPOINT SHIFTED XTS,YTS RATHER
C     THAN XT, YT
C     PROJECTS FLUX POINT ON IMAGE PLANE TO APERATURE PLANE
      XRF=(R22*XTS-R12*YTS)/SAVE
      YRF=(-R21*XTS+R11*YTS)/SAVE
         IF (IREC.EQ.2) GO TO 170
C
C             TEST TO MAKE SURE LIGHT GOES THRU APERTURE, NOT WALLS
C
         IF (XRF**2/RA2(I)+YRF**2/RB2(I).GT.1.) GO TO 190
         GO TO 180
C>>>>R   IF (ABS(XRF/RXN(I)).GT.1.0.OR.ABS(YRF/RYN(I)).GT.1.0) GO TO 190
C  170    IF ( ABS(XRF) .GT. RXN(I) .OR. ABS(YRF) .GT. RYN(I) )
C    C      GO TO 190
 170  CONTINUE
      IF(YFLX(N).GE.W2-1.0E-08) GO TO 180
C     ABOVE LINE MAY BOMB WITH MULTIPLE APERATURES
          IF ( ABS(XRF) .GT. RXN(I) .OR. ABS(YRF) .GT. RYN(I) )
     C      GO TO 190
180   CONTINUE
      FLX(N)=FLX(N)+FDOTT*FLUX(XT,YT)*EXP(-.5*XT**2-.5*YT**2)*WTAIMP(M)*
     1WREC*WTP
190   CONTINUE
200   CONTINUE
210   CONTINUE
220   CONTINUE
240   CONTINUE
      RETURN
      END
      SUBROUTINE GENER
      IMPLICIT REAL*8 (A-H,O-Z)
C
C     THIS ROUTINE GENERATES VARIABLES NECESSARY FOR EITHER A
C     PERFORMANCE OR OPTIMIZATION CALCULATION
C
C     THIS ROUTINE IS CALLED BY
C          DELSOL
C
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /DEL4/ H2O,INSOL,IPRINT(9),ITAPE,NUAZ,NUEL,NYEAR,
     1  PRES,SOLCON,UAZ(20),UDAY,UEL(9),UTIME
C
      COMMON /GEN1/ ATM1S,ATM2S,ATM3S,ATM4S,CTAANG(13),CTPANG(13),
     1  DCANT,DIAMF,FAZMAX,FAZMIN,FZMAX,FZMIN,HCANT,
     2  HCS(13,7,50),HEAD1,HEAD11,HEAD12,HEAD2,HEAD3,HEAD4,HPANL,
     3  HXCANT(25),HYCANT(25),
     4  RADMAX,RADMIN,RANGE(13),
     5  RCANT(13),RMIRL,ROUND,SIGAZ,SIGEL,SIGSX,SIGSY,SIGTX,
     6  SIGTY,SIPAK(16),STAANG(13),TOWD,TOWL,WM,WPANL,XFC,XFOCUS,
     7  YFC,YFOCUS,ZFC,DENSMR,
     8  ICANT,ICPANL,IFLX,IFOCUS,IFXOUT(9,20),IDENS,
     9  IROTFL,NCANTX,NCANTY,NUMK(16)
C>>>>R LINE 7 ABOVE
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /PERF/ HREFIN(16,13,7),REFTOT(13,12),YATMOS(13,12),
     1  YBLOCK(13,7),YCOSI(13,7),YSHAD(13,7),YSPILL(13,7),YTOTAL(13,7)
     2  ,HROP,RREFL,RWCAV(4),NHREF
C>>>>R LINE 2 ABOVE
C
      COMMON /AIM/ IAUTOP,INDC,NAX(13,12),NAY(13,12),NUMPT(13,12),
     1  XAIM(13,12,2),YAIM(13,12,2)
C
C
      COMMON /FLUXXX/ AZMF,POLF,IFLAUT,NXFLX,NYFLX
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /EFF/ ACWOSY,AREF,CCMULC,CCMULY,CSTREF,CSTRMD,EFFSTR,
     1  EMPTY,ESTREF,ETAREF,FCR,FEFF,FLXLIM(4),FPLH,FSP,
     2  REFLP,REFPIP,REFPRL,REFRC,REFRC1,REFRC2,REFRC3,REFTHP,RHOML,
     X  RNHOML,
     3  SMULT,VMAX,VSTREF,XST,ITHEL,IRADFL
C>>>>R LINE 1 ABOVE
C
      COMMON /EFF1/ HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5
     *             ,PARL6,PARL7,PARL8,PARL9,PARL10,TPRE,TSTRT
C
C
C>>>>RDIMENSION FLXLIM(4),HEADTP(5)
      DIMENSION HEADTP(5)
C
C
      DATA IHSAB/0/,NOMAX/7/,NORDER/7/
C
C
      NAMELIST /FIELD/  NAZM,NRAD,RADMIN,RADMAX,INORTH,AMAXN,
     1                  ILAY,IDENS,IUSERF,IHPR,NLAND,ALP,CLE,CLN,SLEW,
     2                  SLNS,XTOWER,YTOWER,FLAND,IROTFL,DENSIT,AZMSEP,
     3                  NRADMN,NRADMX,FSLIP
      NAMELIST /HSTAT/  WM,HM,ICPANL,WPANL,HPANL,HXCANT,HYCANT,DENSMR,
     1                  IROUND,RMIRL,SIGEL,SIGAZ,SIGSX,SIGSY,SIGTX,
     2                  SIGTY,ICANT,NCANTX,NCANTY,HCANT,DCANT,RCANT,
     3                  XFOCUS,YFOCUS,IFOCUS,XFOCAL,YFOCAL,INDC,ISB
      NAMELIST /NLFLUX/ IFLX,IFXOUT,IFLAUT,XFC,YFC,ZFC,POLF,AZMF,DIAMF,
     1                  NXFLX,FAZMIN,FAZMAX,NYFLX,FZMIN,FZMAX,ICAVF,
     2                  NFLXMX,NMXFLX,FLXLIM
      NAMELIST /NLEFF/  REFTHP,AREF,REFRC,REFLP,REFPIP,FPLH,FPLC,ITHEL,
     1                  ETAREF,FEFF,REFPRL,FSP,FEP,EFFSTR,PF,SMULT,IPH,
     2                  REFRC1,REFRC2,REFRC3,IRADFL
     *,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7,PARL8,PARL9
     *,PARL10,TPRE,TSTRT
      NAMELIST /REC/    THT,TOWL,TOWD,IREC,W,H,RRECL,IAUTOP,NAY,NAX,
     1                  XAIM,YAIM,NUMPT,NUMCAV,RELV,RAZM,RX,RY,RWCAV
      NAMELIST /FMTP/   THT,WM,HM,IROUND,NCANTX,NCANTY,ROUND,NRAD,NAZM,
     1                  RMIRL,DENSMR,DENSIT,XFOCUS,YFOCUS,SIGAZ,SIGEL,
     2                  SIGSX,SIGSY,SIGTX,SIGTY,HEAD,IFOCUS,XFOCAL,
     3                  YFOCAL,HRDEL,NYEAR,HCANT,DCANT,RADMIN,RADMAX,
     4                  RCANT,WEATH,DWEATH,IWEATH,ILAY,IATM,ATM1,ATM2,
     5                  ATM3,ATM4,INDC,IROTFL,INSOL,
     6                  NSUN,SUNI,SUNR,INORTH,AMAXN,RAD,
     7                  AZM,AZMSEP
C
C
      ROUND = 1.
      NUMFF = 5
      NUMTT = 5
      PI = 4.*ATAN(1.)
      RR = PI/180.
      HEAD1 =7HCOSINE 
      HEAD2 =7HSPILL  
      HEAD3 =7HATTEN  
      HEAD4 =7HTOTAL  
      HEAD5 =7HDENSITY
      HEAD11 =7HSHADOW   
      HEAD12 =7HBLOCK     
      HEAD19 =7HRAD SEP   
      HEAD20 =7HAZM SEP   
      HEAD67 =7HMISS. H   
C
C     TEST TO SEE IF THIS IS A RERUN OF AN OPTIMIZED SYSTEM
C
      IF ( IFUDGE .EQ. 0 ) GO TO 110
C
C               SET FIELD TO SYSTEM THAT WAS JUST OPTIMIZED
C
      IUSERF=2
      IDENS=3
      DO 100 I=1,13
      NRADMN(I)=NTMIN(I)
      NRADMX(I)=NTMAX(I)
      DO 100 J=1,12
      DENSIT(I,J)=DENTR(I,J)
      AZMSEP(I,J)=AZMTR(I,J)
  100 CONTINUE
      GO TO 350
C
C             INITIAL VALUES
C
C             SET UP FOR HERMITE EXPANSION
C
  110 CONTINUE
      FACT(1)=1.
      DO 120 I=2,NOMAX
C
C             FACT=FACTORIAL
C
120   FACT(I)=FACT(I-1)*FLOAT(I-1)
C
C             BINOMIAL COEFFICIENTS
C
      DO 151 I=1,NOMAX
      DO 150 J=1,NOMAX
      HXN(I,J)=0.
150   CONTINUE
151   CONTINUE
      HXN(1,1)=1.
      HXN(2,2)=1.
      DO 161 I=3,NOMAX
      FI=FLOAT(I-2)
      HXN(I,1)=-FI*HXN(I-2,1)
      DO 160 J=2,NOMAX
      HXN(I,J)=HXN(I-1,J-1)-FI*HXN(I-2,J)
160   CONTINUE
161   CONTINUE
C
C             MISCELLANEOUS DEFAULT VALUES
C
      HEAD41=7HNO XAIM
      HEAD42=7HNO YAIM
      HEAD43=7H  X MIN
C
C             DEFAULT VALUES FOR NAMELIST FIELD
C
      NAZM=12
      INORTH=0
      IHPR=0
      AMAXN=82.5
      NRAD=12
      RADMIN=0.75
      RADMAX=7.50
      ILAY=0
      IDENS=1
      IUSERF=0
      IROTFL=0
      NLAND=0
      DO 220 JJ=1,5
      ALP(JJ)=0.0
      CLN(JJ)=0.0
      CLE(JJ)=0.0
      SLEW(JJ)=0.0
220   SLNS(JJ)=0.0
      XTOWER=0.0
      YTOWER=0.0
      DO 231 I=1,13
      NRADMN(I)=1
      NRADMX(I)=1
      DO 230 J=1,12
      REALD(I,J)=1.0
      FLAND(I,J)=1.
      DENSIT(I,J)=1.0
      AZMSEP(I,J)=1.05
230   CONTINUE
231   CONTINUE
      FSLIP=4./3.
C
C             DEFAULT VALUES FOR NAMELIST HSTAT
C
      WM=9.91
      HM=9.93
      ISB=0
      DENSMR=0.970
      IROUND=0
      RMIRL=0.91
      SIGEL=0.00075
      SIGAZ=0.00000
      SIGSX=0.0010
      SIGSY=0.0000
      SIGTX=0.
      SIGTY=0.
      ICANT=1
      NCANTX=2
      NCANTY=8
      ICPANL=0
      WPANL=4.88
      HPANL=1.22
      II=NCANTX*NCANTY
      HYCANT(1)=4.27
      HYCANT(2)=3.05
      HYCANT(3)=1.83
      HYCANT(4)=0.61
      HYCANT(5)=-0.61
      HYCANT(6)=-1.83
      HYCANT(7)=-3.05
      HYCANT(8)=-4.27
      DO 270 I=1,NCANTY
      HXCANT(I)=2.475
      HXCANT(II-I+1)=-2.475
      HYCANT(II-I+1)=HYCANT(NCANTY-I+1)
270   CONTINUE
      HCANT=0.
      DCANT=81.0
      DO 280 I=1,13
      XFOCAL(I) = 6.00
      YFOCAL(I) = 6.00
      RCANT(I)=6.00
280   CONTINUE
      XFOCUS=1.
      YFOCUS=1.
      IFOCUS=1
      INDC=0.
C
C               DEFAULT VALUES FOR NAMELIST REC
C
      DO 301 K=1,13
      DO 300 L=1,12
      NUMPT(K,L)=1
      NAX(K,L)=1
      NAY(K,L)=1
      DO 300 N=1,2
      XAIM(K,L,N)=0.
      YAIM(K,L,N)=0.
  300 CONTINUE
301   CONTINUE
      THT=175.
      TOWL=175.
      TOWD=10.
      IREC=0
      W=16.
      H=16.
      RRECL=.965
      IAUTOP=0
      NUMCAV=1
      DO 310 I=1,4
      RELV(I)=90.
      RWCAV(I)=1.
      RX(I)=6.
      RY(I)=6.
  310 CONTINUE
      RAZM(1)=180.
      RAZM(2)=270.
      RAZM(3)=0.
      RAZM(4)=90.
C
C             DEFAULT VALUES FOR NAMELIST NLFLUX
C
      DO 316 I = 1,9
      DO 315 J = 1,20
         IFXOUT(I,J) = 0
  315 CONTINUE
316   CONTINUE
      IFLX=0
      IFLAUT=4
      NXFLX=1
      FAZMIN=180.0
      FAZMAX=180.0
      NYFLX=1
      FZMIN=0.
      FZMAX=0.
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 320
C
C             DEFAULT VALUE OF FZMIN, FZMAX AT CENTER OF
C             MAXIMUM POSSIBLE HEAT ABSORBING HEIGHT
C             FOR CAVITY RECEIVERS
C
      W2=0.5*W/THT
      RY2=0.5*RY(1)/THT
      ANG=180.-RELV(1)
      HTP=(1.+RY2*SIN(ANG*RR))*(RADMIN-W2*(1.-RWCAV(1)))/(RADMIN-W2-RY2*
     1COS(ANG*RR))
      HBT=(1.-RY2*SIN(ANG*RR))*(RADMAX-W2*(1.-RWCAV(1)))/(RADMAX-W2+RY2*
     1COS(ANG*RR))
      HTP=HBT+H/THT
      HCTR=0.5*(HTP+HBT)*THT-THT
      FZMIN=HCTR
      FZMAX=HCTR
320   CONTINUE
      DO 330 I=1,4
      NMXFLX(I)=I
      FLXLIM(I)=1.0E+06
330   CONTINUE
      FLXLIM(1)=0.6E+06
      NFLXMX=1
      DIAMF=16.
      ICAVF(1)=1
      ICAVF(2)=0
      ICAVF(3)=0
      ICAVF(4)=0
      POLF=90.
      AZMF=180.
      XFC=0.
      YFC=0.
      ZFC=0.
C
C             DEFAULT VALUES FOR NAMELIST NLEFF
C
      PF=0.9
      REFRC=0.83
      AREF=2780.0
      IF(IREC.EQ.1.OR.IREC.EQ.2) AREF=761.0
      REFTHP=4.10E+08
      IF(IREC.EQ.1.OR.IREC.EQ.2) REFTHP=2.10E+08
      REFPIP=0.990
      REFLP=170.0
      FPLH=2.6
      FPLC=2.6
      ITHEL=0
      ETAREF=0.42
      FEFF=0.90
      IRADFL=0
      IF(IREC.GT.2)IRADFL=1
      REFRC2=460.0
      IF(IREC.EQ.1.OR.IREC.EQ.2) REFRC2=7631.0
      REFRC1=16403.0
      REFRC3=0.0
      PARL1=0.5
      PARL2=0.10300
      PARL3=0.5
      PARL4=0.10300
      PARL5=0.50000
      PARL6=0.00800
      PARL7=0.50000
      PARL8=0.00800
      PARL9=0.18
      PARL10=0.009
      TPRE=1.0
      TSTRT=1.0
      REFPRL=0.103
      FSP=0.66
      SMULT=1.5
      EFFSTR=1.0
      FEP=0.0
      IPH=0
350   CONTINUE
      DFFF=DHOPT/2.
      DTTT=DFFF
         IF (ITAPE.NE.2) GO TO 380
C
C             READ TAPE20 - PERFORMANCE DATA FOR OPTIMIZATION RUN
C
      READ (20) THT,WM,HM,IROUND,ICANT,NCANTX,NCANTY,ROUND,ASTAT,AMIRRO,
     1AZM,RADIUS,RADDEL,AZMDEL,NRAD,NAZM,RR,RMIRL,DAREA,DENSMR,DENSIT,
     2STAANG,CTPANG,STPANG,CTAANG,XFOCUS,YFOCUS,SIGAZ,SIGEL,SIGSX,SIGSY,
     3SIGTX,SIGTY,WTAREA,HEADTP,NHREF,IFOCUS,REFFLU,HREFIN,AZMSEP,
     4XFOCAL,YFOCAL,REFTOT,YINSOL,REFSOL,HRDEL,YCOSI,YSHAD,YBLOCK,HCS,
     5HCANT,DCANT,RADMIN,RADMAX,RCANT,RANGE,HROP,WEATH,DPRES,DH2O,
     6IWEATH,NYEAR,DWEATH,INSOL,SOLCON,NSUN,NSUNPT,SUNI,SCS,HCSDP,
     7NUMFF,NUMTT,IFILL,ILAY,REALD,LZONE,MZONE,HCANT,DCANT,RCANT,INDC,
     8IROTFL,ATM1,ATM2,ATM3,ATM4,IATM,INORTH,AMAXN,PLAT,ALT,REFDAY,
     9REFTIM,NSUNPT,SUNR,SUNI,NSUM,IPROB,ICPANL,WPANL,HPANL,HXCANT,
     AHYCANT,RDMIN,RDMAX
     B,H,IAUTOP,IREC,NAX,NAY,NUMCAV,NUMPT,RAZM,RELV,RRECL,RWCAV,RX,RY
     C,TOWD,TOWL,W,XAIM,YAIM,AREF,EFFSTR,ETAREF,FEFF,FEP,FPLC,FPLH,FSP
     D,IPH,ITHEL,PF,REFLP,REFPIP,REFPRL,REFRC,REFTHP,SMULT,AZMF,DIAMF
     E,FAZMAX,FAZMIN,FLXLIM,FZMAX,FZMIN,ICAVF,IFLAUT,IFLX,IFXOUT
     F,NFLXMX,NMXFLX,NXFLX,NYFLX,POLF,XFC,YFC,ZFC,ISB,ALP,ASTART,CLE
     G,CLN,FLAND,FSLIP,IDENS,IHPR,IUSERF,NLAND,NRADMN,NRADMX,SLEW,SLNS
     H,XTOWER,YTOWER,REFRC1,REFRC2,REFRC3,IRADFL
     *,HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7
     *,PARL8,PARL9,PARL10,TPRE,TSTRT
C>>>7I ABOVE LINES
C>>>>R LINE 6 ABOVE
      WRITE (6,9876) '$FMTP'
C mjw     PRINT FMTP
      WRITE (6,2620) (HEADTP(J),J=1,5)
C>>>>D
C>>>>D          CONVERT FROM UNITS OF THT TO REGULAR UNITS
C>>>>D
C>>>>DDO 375 K = 1,NRAD
C>>>>DDO 375 L = 1,NAZM
C>>>>DDAREA(K,L) = DAREA(K,L)/REALD(K,L)/FLAND(K,L)
C>>>>DCONTINUE
         GO TO 440
380      IF (ITAPE.NE.3) GO TO 430
C
C             READ RESULT OF PREVIOUS OPTIMIZATION
C
      WRITE (6,2630)
      rewind 30
      READ (30) WM,HM,IROUND,ICANT,NCANTX,NCANTY,ROUND,ASTAT,AMIRRO,NRAD
     1,NAZM,RMIRL,DENSMR,XFOCUS,YFOCUS,SIGAZ,SIGEL,SIGSX,SIGSY,SIGTX,
     2SIGTY,HEADTP,IFOCUS,XFOCAL,YFOCAL,HCANT,DCANT,RADMIN,RADMAX,RCANT,
     3RANGE,ILAY,INDC,IROTFL,REFSOL,ATM1,ATM2,ATM3,ATM4,IATM,INORTH,
     4AMAXN,PLAT,ALT,REFDAY,REFTIM,NSUNPT,SUNR,SUNI,NSUN,ICPANL,WPANL,
     5HPANL,HXCANT,HYCANT,NUMCAV,IREC,RELV,RAZM,RRECL,IAUTOP,II,HROP,
     6WEATH,DPRES,DH2O,DWEATH,IWEATH,ALP,ASTART,REFTHP,AREF,REFRC,
     7REFLP,REFPIP,FPLH,FPLC,ITHEL,ETAREF,FEFF,REFPRL,FSP,PF,SMULT,
     8CCMUL,CCMULY,CCMULC,FCR,RHOML,RNHOML,CSTREF,VSTREF,ESTREF,XST,
     9CSTRMD,VMAX,EMPTY,ISTR,NSTR
     A,NAX,NAY,NUMPT,XAIM,YAIM,EFFSTR,FEP,IPH,FLXLIM,ICAVF,IFLAUT,IFLX
     B,IFXOUT,NFLXMX,NMXFLX,ISB,AZMSEP,DENSIT,FSLIP,IDENS,IHPR,IUSERF
     C,NRADMN,NRADMX,REFRC1,REFRC2,REFRC3,IRADFL
     *,HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7
     *,PARL8,PARL9,PARL10,TPRE,TSTRT
C>>>3I ABOVE LINES
      WRITE (6,2640) (HEADTP(J),J=1,5)
         IF (II.LT.1) GO TO 420
      DO 400 N=1,II
      READ (30) SAVE3,NTMIN,NTMAX,AZMSEP,DENSIT,FLAND,XTOWER,YTOWER,
     1SLEW,SLNS,CLE,CLN,NLAND,THT,RX,RY,W,H,RWCAV,TOWD,TOWL,ACWOSY,
     2ACWOSC,ACHY,ACHC,ESTOR,XFC,YFC,ZFC,DIAMF,POLF,AZMF,NXFLX,FAZMIN,
     3FAZMAX,NYFLX,FZMIN,FZMAX
      WRITE (6,2650) N,SAVE3,TDESP
         IF (ABS(SAVE3-TDESP).GT.0.01*TDESP) GO TO 400
      IDENS=3
      DO 390 I=1,13
      NRADMN(I)=NTMIN(I)
      NRADMX(I)=NTMAX(I)
390   CONTINUE
         GO TO 410
400   CONTINUE
      WRITE (6,2660)
      W = - 100
         GO TO 2605
410   CONTINUE
      WRITE (6,9876) '$FMTP'
C mjw     PRINT FMTP
      IFUDGE=1
420   CONTINUE
      IUSERF=2
430   CONTINUE
c     call ddi(FIELD,5,1,1)
      READ (5,NML=FIELD)
      WRITE (6,9876) '$FIELD'
 9876 FORMAT(1H1,A8)
C mjw     PRINT FIELD
      RADDEL=(RADMAX-RADMIN)/FLOAT(NRAD-1)
         IF (IUSERF.NE.2) GO TO 431
      DO 432 L=1,NAZM
      N=NRADMX(L)+1
         IF (N.GT.NRAD) GO TO 432
      DO 433 K=N,NRAD
      FLAND(K,L)=0.
433   CONTINUE
432   CONTINUE
431   CONTINUE
c     call ddi(HSTAT,5,1,1)
      READ (5,NML=HSTAT)
      WRITE (6,9876) '$HSTAT'
C mjw     PRINT HSTAT
c     call ddi(REC,5,1,1)
      READ (5,NML=REC)
      IF (ITAPE.NE.1) GO TO 435
      IF (IREC.GT.2) IRADFL=1
      IF (IREC.NE.1.AND.IREC.NE.2) GO TO 435
      AREF=761.0
      REFTHP=2.10E+08
      REFRC2=7631.0
      REFRC3=5077.0
C
C             DEFAULT VALUE OF FZMIN, FZMAX AT CENTER OF
C             MAXIMUM POSSIBLE HEAT ABSORBING HEIGHT
C             FOR CAVITY RECEIVERS
C
      W2=0.5*W/THT
      RY2=0.5*RY(1)/THT
      ANG=180.-RELV(1)
      HTP=(1.+RY2*SIN(ANG*RR))*(RADMIN-W2*(1.-RWCAV(1)))/(RADMIN-W2-RY2*
     1COS(ANG*RR))
      HBT=(1.-RY2*SIN(ANG*RR))*(RADMAX-W2*(1.-RWCAV(1)))/(RADMAX-W2+RY2*
     1COS(ANG*RR))
C         2-4-86 NORMALIZE H TO MATCH REST OF FZMIN CALC
      HTP=HBT+H/THT
      HCTR=0.5*(HTP+HBT)*THT-THT
      FZMIN=HCTR
      FZMAX=HCTR
435   CONTINUE
      WRITE (6,9876) '$REC'
C mjw     PRINT REC
440   CONTINUE
C
C             CHANGE ATMOSPHERIC ATTENUATION COEFFICIENTS TO
C             UNITS OF TOWER HEIGHT
C
         IF (IATM.EQ.2) GO TO 460
         IF ( IATM .EQ. 1 ) GO TO 450
      ATM1 = .6789
      ATM2 = 10.46
      ATM3 = - 1.7
      ATM4 = .2845
         GO TO 460
  450 CONTINUE
      ATM1 = 1.293
      ATM2 = 27.48
      ATM3 = - 3.394
      ATM4 = 0.
  460 CONTINUE
      ATM1S=ATM1/100.
      ATM2S=ATM2/1.0E+05
      ATM3S=ATM3/1.0E+08
      ATM4S=ATM4/1.0E+11
      NAZM2P = NAZM/2 + 1
      IF ( IROUND .GT. 0 ) ROUND = PI/4.
C
C     SCALE RATIO OF MIRROR AREA ACCORDING TO SHAPE
C          ROUND = 1., RECTANGULAR
C          ROUND = PI/4., ROUND
C
      DENSMS = DENSMR*ROUND
         IF (ITAPE.EQ.2) GO TO 620
c     call ddi(NLFLUX,5,1,1)
      READ (5,NML=NLFLUX)
      WRITE (6,9876) '$NLFLUX'
C mjw     PRINT NLFLUX
c     call ddi(NLEFF,5,1,1)
      READ (5,NML=NLEFF)
         IF (IPH.EQ.0) GO TO 461
      ITHEL=0
      ETAREF=1.0
      FEFF=1.0
      REFPRL=0.0
          PARL1=0.0
       PARL2=0.0
       PARL3=0.0
       PARL4=0.0
       PARL5=0.0
       PARL6=0.0
       PARL7=0.0
       PARL8=0.0
       PARL9=0.0
       PARL10=0.0
461   CONTINUE
      WRITE (6,9876) '$NLEFF'
C mjw     PRINT NLEFF
         IF (IUSERF.NE.1) GO TO 470
      NAZM=12
      NRAD=11
C>>>>I
      NAZM2P = NAZM/2 + 1
      RADMIN=0.80
      RADMAX=7.15
470   RADDEL=(RADMAX-RADMIN)/FLOAT(NRAD-1)
      AZMDEL=360./FLOAT(NAZM)
      IF (INORTH.GT.0) AZMDEL=AMAXN*2./FLOAT(NAZM-1)
C
C             CALCULATE AREA OF ZONES, DAREA
C
      DO 481 I=1,NRAD
      RAD = RADMIN + FLOAT(I-1)*RADDEL
      DO 480 L=1,NAZM
      DAREA(I,L)=RAD*RADDEL
      IF (I.EQ.1) DAREA(I,L)=0.5*DAREA(I,L)+0.125*RADDEL**2
      IF (I.EQ.NRAD) DAREA(I,L)=0.5*DAREA(I,L)-0.125*RADDEL**2
      FLAND(I,L)=FLAND(I,L)+1.E-10
      DAREA(I,L)=DAREA(I,L)*AZMDEL*RR*FLAND(I,L)
480   CONTINUE
481   CONTINUE
      NAZM2=NAZM/2
      NAZM2A=NAZM2+2
C
C             LZONE TELLS THE SYMMETRIC ZONE USED TO STORE ANNUAL
C             AVERAGE RESULTS
C
      DO 510 L=1,NAZM2P
C>>>>I
      WTAREA(L) = 2.
      LL=2+NAZM-L
      IF (LL.GT.NAZM) LL=1
      MZONE(L)=LL
      LZONE(L)=L
510   CONTINUE
      LL1=1
      DO 520 L=NAZM2A,NAZM
      LL1=LL1+1
      LZONE(L) = L - 2*LL1 + 2
520   CONTINUE
      WTAREA(1)=1.0
      WTAREA(NAZM2P)=1.0
C>>>>DDO 530 L=2,NAZM2
C>>>>DWTAREA(L)=2.0
C>>>>DCONTINUE
         IF (IUSERF.EQ.0.OR.IUSERF.EQ.2) GO TO 550
C
C             DEFAULT FIELD BOUNDARIES
C
      NRADMN(1)=1
      NRADMX(1)=11
      NRADMN(NAZM2P)=1
      NRADMX(NAZM2P)=6
      DO 540 J=2,NAZM2
C>>>>RJ1=NAZM-J+2
      J1 = MZONE(J)
      NRADMN(J)=1
      NRADMX(J)=13-J
      NRADMN(J1)=1
      NRADMX(J1)=13-J
540   CONTINUE
C
C             DENSITY OF FIELD POINTS CHOSEN TO ELIMINATE BLOCKING
C
  550 CONTINUE
         IF (IDENS.EQ.3) GO TO 580
C
C             DEFAULT DENSITIES - FUNCTION OF THT
C
         IF (IROUND .EQ. 1 ) SAVE = WM
      IF ( IROUND .EQ. 0 ) SAVE=SQRT(WM**2+HM**2)
      AMIN=SAVE/WM
      DO 570 I=1,NRAD
      RAD=RADMIN+FLOAT(I-1)*RADDEL
      ANGL=ATAN((THT+HM/2.0)/(THT*RAD))
      ETATHE=FEFF*ETAREF
      ECON=TDESP/ETATHE
      DO 570 J=1,NAZM
      AZMX=FLOAT(J-1)*AZMDEL+180.0
      ASEP=(1.7490871+0.63964099*ANGL+0.028726279/(ANGL-0.049023315))
      RSEP=1.1442399/TAN(ANGL)-1.093519+3.0683558*ANGL-1.1255617*ANGL**2
      IF (IDENS.EQ.2) ASEP=1.609666+0.29654848*ANGL+0.019137019/(ANGL-0.
     1012341664)
      IF(IDENS.EQ.4.AND.IREC.GT.0.AND.IREC.LT.3)
     1ASEP=2.16597-0.01*ECON/100.-0.10273*COS(RR*AZMX)
     2+(0.4965+3.3247*COS(RR*AZMX))*ANGL/RR/100.0
     3-(3.3815+11.1843*COS(RR*AZMX))*(ANGL/RR/100.0)**2
      IF(IDENS.EQ.4.AND.IREC.GT.0.AND.IREC.LT.3)
     *RSEP=
     1(62.324+0.63*ECON/100.+(3.151+0.65*ECON/100.)*COS(RR*AZMX))
     2/ANGL/RR
     3-(0.5375+0.085*ECON/100.+(0.1984+0.065*ECON/100.)*COS(RR*AZMX))
     4+(2.253+0.21*ECON/100.+(0.842+0.16*ECON/100.)*COS(RR*AZMX))
     5*ANGL/RR/100.0
      IF(IDENS.EQ.4.AND.IREC.NE.1.AND.IREC.NE.2)
C change 11/22/89 replaced next three lines to input utility study
c spacing algorithm
c    1ASEP=2.46812-0.0401054*ANGL/RR+0.000923594*(ANGL/RR)**2
c    2+COS(RR*AZMX)*(0.1734593-0.00911259*ANGL/RR+0.00012761
c    3*(ANGL/RR)**2)
     1ASEP=(1.9516+0.1079*COS(RR*AZMX))-(0.02239+
     2 0.00797*COS(RR*AZMX))*(ANGL/RR)+
     3 (4.575+1.260*COS(RR*AZMX))*1.E-4*(ANGL/RR)*(ANGL/RR)
C change 1/29/87 switched resp to rsep and divided by angl/rr according
C to kistlers 1/23/87 memo for case IDENS EQ 4 AND IREC NE 1 AND IREC NE 2
      IF(IDENS.EQ.4.AND.IREC.NE.1.AND.IREC.NE.2)
c change 11/22/89 replaced next 4 lines to use spacing algorithm employed
c in utility study.
c    1RSEP=63.0093-0.587313*ANGL/RR+0.018423909*(ANGL/RR)**2
c    2+COS(RR*AZMX)*(2.808733-0.1480498*ANGL/RR+0.001489201
c    3*(ANGL/RR)**2)
c     IF(IDENS.EQ.4.AND.IREC.NE.1.AND.IREC.NE.2)RSEP=RSEP/(ANGL/RR)
     1RSEP=(57.080+1.935*COS(RR*AZMX))/(ANGL/RR)-
     2(0.2019+0.02239*COS(RR*AZMX))+
     3(0.01012-0.000776*COS(RR*AZMX))*(ANGL/RR)
C
C             TEST FOR MECHANICAL LIMITATIONS ON
C             HELIOSTAT SPACINGS
C
      IF (ILAY.LT.2) ASEP=ASEP/(1.0-RSEP*HM/(THT*2.*RAD))
      IF (ASEP.LT.AMIN) ASEP=AMIN
      IF (RSEP.LT.SAVE/HM) RSEP=SAVE/HM
      SAVE2=(SAVE**2-(RSEP*HM/2.)**2)*4./WM**2
      IF (ILAY.NE.2*(ILAY/2)) SAVE2=0.
      IF (ASEP**2.LT.SAVE2) ASEP=SQRT(SAVE2)
      DENS=DENSMS/(ASEP*RSEP/2.0)
      ASEP2=0.5*ASEP
      DENSIT(I,J)=DENS
      AZMSEP(I,J)=ASEP2
570   CONTINUE
580   CONTINUE
C
C             CORRECTION FOR MISSING HELIOSTATS IN GAPS IN SLIP
C             PLANE 1 - REALD = FRACTION MISSING
C
         IF (ILAY.GT.1) GO TO 600
      DO 590 K=1,NRAD
      RAD=RADMIN+FLOAT(K-1)*RADDEL
      DO 590 L=1,NAZM
      RSEP=DENSMS/DENSIT(K,L)/AZMSEP(K,L)
      SAVE=RAD*THT*(FSLIP-1.)/(RSEP*HM*(1.+FSLIP))
      REALD(K,L)=(2.*SAVE+1./FSLIP)/(2.*SAVE+1.)
      DAREA(K,L)=DAREA(K,L)*REALD(K,L)
590   CONTINUE
600   CONTINUE
      RREFL=RMIRL*RRECL
      WRITE (6,2670) RMIRL,RRECL,RREFL
      IF (ICANT.EQ.0) NCANTX=1
      IF (ICANT.EQ.0) NCANTY=1
C
C             PRINT OUT HELIOSTAT DATA
C
620   CONTINUE
      WRITE (6,2680)
         IF (IROUND.EQ.0) WRITE (6,2690) WM,HM
      NCANT=NCANTX*NCANTY
         IF (NCANT.LE.25) GO TO 621
      WRITE (6,2691)
      W = - 99
        GO TO 2605
621   CONTINUE
         IF (ICPANL.LT.1) GO TO 640
      WRITE (6,2700) WPANL,HPANL
      IHPR=1
      DO 630 I=1,NCANT
630   WRITE (6,2710) I,HXCANT(I),HYCANT(I)
640   CONTINUE
         IF (IROUND.NE.0) WRITE (6,2720) WM
C>>>2RASTAT = WM*HM*ROUND
      ASTAT = WM*HM
      SAVE = ASTAT*ROUND
C>>>>RSAVE2=DENSMR*ASTAT
      SAVE2 = DENSMR*SAVE
C>>>>RWRITE (6,2730) ASTAT,SAVE2,DENSMR
      WRITE (6,2730) SAVE,SAVE2,DENSMR
         IF (ISB.EQ.0) WRITE (6,2740)
         IF (ISB.EQ.1) WRITE (6,2750)
      WRITE (6,2760)
      WRITE (6,2770)
      WRITE (6,2780) SIGAZ
      WRITE (6,2790) SIGEL
      WRITE (6,2800) SIGSX
      WRITE (6,2810) SIGSY
      WRITE (6,2820) SIGTX
      WRITE (6,2830) SIGTY
         IF (NSUN.LT.3) GO TO 650
      WRITE (6,2840) NSUNPT
C
C             FTIME IS WEIGHT OF TIME STEP USED IN CALCULATING
C             ANNUAL AVERAGE
C
650   CONTINUE
         IF (ICANT.EQ.0) GO TO 680
         IF (ICANT.EQ.-1) WRITE (6,2850) NCANT
         IF (ICANT.EQ.1) WRITE (6,2860) NCANT
         IF (ICANT.NE.1) GO TO 670
      DO 660 K=1,NRAD
      SAVE=THT*RCANT(K)
      WRITE (6,2870) K,SAVE
660   CONTINUE
670   CONTINUE
         IF (ICANT.EQ.3) WRITE (6,2880) NCANT,HCANT,DCANT
      WRITE (6,2890) NCANTX,NCANTY
         GO TO 690
680   CONTINUE
      WRITE (6,2900)
690   CONTINUE
         IF (INDC.EQ.1) WRITE (6,2910)
         IF (XFOCUS+YFOCUS-1.00) 700,710,730
700   WRITE (6,2920)
         GO TO 750
710      IF (YFOCUS.GT.0.) GO TO 720
      WRITE (6,2930)
         GO TO 740
720   WRITE (6,2940)
730   WRITE (6,2950)
740   CONTINUE
         IF (IFOCUS.EQ.0) WRITE (6,2960)
         IF (IFOCUS.EQ.1) WRITE (6,2970)
750   CONTINUE
         IF (NSUN.EQ.1) WRITE (6,2980)
         IF (NSUN.NE.3) GO TO 760
      WRITE (6,2990)
      WRITE (6,3000) (N,SUNR(N),SUNI(N),N=1,NSUNPT)
760   CONTINUE
C
C             CALCULATE TIME INDEPENDENT QUANTITIES NEEDED TO
C             DETERMINE HERMITE EXPANSION OF HELIOSTAT IMAGES
C
      NPAK=(NORDER*(NORDER+1))/2-((NORDER-1)*(1+(NORDER-1)/2))/2
      DO 770 IPAK=1,NPAK
770   HC(IPAK)=0.
      ISTEP=2-IHSAB
      IHSABM=1-IHSAB
      NORDEB=NORDER+2
      DO 780 I=1,NORDER
      JMX(I)=NORDER+1-I
      IMN(I)=2-I+2*(I/2)
780   JMN(I)=ISTEP+IHSABM*(2*(I/2)-I)
      IPAK=0
      DO 790 M=1,NORDER
      NMIN=JMN(M)
      JMAX=JMX(M)
      DO 790 N=NMIN,JMAX,ISTEP
      IPAK=IPAK+1
      SIPAK(IPAK)=(-1.)**(M+1)
      NUMK(IPAK)=(M+N-2)/2+1
790   CONTINUE
 2605 CONTINUE
      RETURN
2620  FORMAT  (///39H----- PERFORMANCE DATA FROM RUN TITLED ,5A8//)
2630  FORMAT (1H1,20X,85HSYSTEM DESCRIPTION READ OFF OF TAPE THAT WAS GE
     1NERATED IN A PREVIOUS OPTIMIZATION RUN//)
2640  FORMAT (18H LABEL OF TAPE IS ,5A8)
2650  FORMAT (4H THE,I3,22H DESIGN POINT ON TAPE=,F8.2,28H MW, DESIRED D
     1ESIGN POINT IS,F8.2,3H MW)
2660  FORMAT (//59H ----- PROGRAM STOP, DESIGN POWER REQUESTED NOT ON TH
     1E TAPE)
2670  FORMAT (//26H CONSTANT LOSSES - MIRROR=,F5.3,10H REC REFL=,F5.3,7H
     1 TOTAL=,F5.3//)
2680  FORMAT (1H1,20X,9HHELIOSTAT////)
2690  FORMAT (32H RECTANGULAR HELIOSTAT - WIDTH =,F6.2,12H M, HEIGHT =,F
     16.2,1HM)
c 2691 FORMAT (//59H ----- PROGRAM STOP, HELIOSTAT HAS MORE THAN 25 CANT
c     1PANELS)
 2691 FORMAT (//' ----- PROGRAM STOP, HELIOSTAT HAS MORE THAN 25 CANT ',
     1'PANELS')
2700  FORMAT (/51H USER DEFINED CANT PANEL DIMENSIONS - PANEL WIDTH =,F6
     1.3,17HM, PANEL HEIGHT =,F6.3,1HM)
2710  FORMAT (10X,11HPANEL NO. =,I3,63H LOCATION OF PANEL CENTER RELATIV
     1E TO HSTAT CENTER, HORIZONTAL=,F6.3,12HM, VERTICAL=,F6.3,1HM)
2720  FORMAT (32H CIRCULAR HELIOSTAT - DIAMETER =,F6.2,2H M)
2730  FORMAT (10X,19HOVERALL AREA/HSTAT=,F7.3,27H M2, REFLECTIVE AREA/HS
     1TAT=,F7.3,26H RATIO (REFLECTIVE/TOTAL)=,F5.3)
2740  FORMAT (//54H THE SHADOWING AND BLOCKING ARE ASSUMED NOT TO OVERLA
     1P//)
2750  FORMAT (//68H THE SHADOWING AND BLOCKING ARE ASSUMED TO BE COMPLET
     1ELY OVERLAPPING//)
2760  FORMAT (//87H THE HELIOSTAT PERFORMANCE ERRORS HAVE NORMAL DISTRIB
     1UTIONS WITH STANDARD DEVIATIONS OF)
2770  FORMAT (10X,47H  NAME   VARIABLE EFFECTED       STD. DEV (RAD))
2780  FORMAT (10X,37H SIGAZ - AZIMUTH ANGLE               ,F7.5)
2790  FORMAT (10X,37H SIGEL - ELEVATION ANGLE             ,F7.5)
2800  FORMAT (10X,37H SIGSX - HSTAT SURFACE, HORIZ.       ,F7.5)
2810  FORMAT (10X,37H SIGSY - HSTAT SURFACE, VERT.        ,F7.5)
2820  FORMAT (10X,37H SIGTX - REFLECTED RAY, HORIZ.       ,F7.5)
2830  FORMAT (10X,37H SIGTY - REFLECTED,RAY, VERT.        ,F7.5)
2840  FORMAT (//27H USER DEFINED SUNSHAPE HAS ,I3,34H RADIAL INTENSITY V
     1S RADIUS POINTS/)
2850  FORMAT (//4H THE,I3,67H MIRROR PANELS ARE CANTED ON AXIS WITH A FO
     1CAL LENGTH = SLANT RANGE)
2860  FORMAT (//5H THE ,I3,97H MIRROR PANELS ARE CANTED ON-AXIS WITH THE
     1 FOLLOWING FOCAL LENGTHS - RADIAL ZONE  FOCAL LENGTH(M))
2870  FORMAT (81X,I2,11X,F7.1)
2880  FORMAT (//4H THE,I3,32H MIRROR PANELS ARE CANTED ON THE,F4.2,23H H
     1OURS PAST NOON ON THE,F6.2,16H DAY OF THE YEAR)
2890  FORMAT (10X,35H NO. OF PANELS IN HORIZ. DIRECTION=,I3,18H VERT. DI
     1RECTION =,I3)
2900  FORMAT (//12H NO CANTING )
2910  FORMAT (/112H INDIVIDUAL IMAGES FROM EACH CANT PANEL WILL BE GENER
     1ATED - THIS MUST BE A PERFORMANCE ONLY RUN, NO OPTIMIZATION)
2920  FORMAT (//26H FLAT PANELS - NO FOCUSING)
2930  FORMAT (//51H MIRROR PANELS FOCUSED ONLY IN HORIZONTAL DIRECTION)
2940  FORMAT (//49H MIRROR PANELS FOCUSED ONLY IN VERTICAL DIRECTION)
2950  FORMAT (//40H MIRROR PANELS FOCUSED IN TWO DIMENSIONS)
2960  FORMAT (10X,27H FOCAL LENGTH = SLANT RANGE)
2970  FORMAT (10X,30H FOCAL LENGTHS DEFINED BY USER)
2980  FORMAT (//38H DELSOL DEFAULT LIMB DARKENED SUNSHAPE/)
2990  FORMAT (//22H USER DEFINED SUNSHAPE)
3000  FORMAT (20X,4H THE,I3,19H POINT HAS AN ANGLE,F7.5,36H RADIANS AND
     1A RELATIVE INTENSITY OF,E13.4)
      END
      SUBROUTINE HERMIT
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE EVALUATES THE HERMITE COEFFICIENTS.
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C          MOMENT(PERCAL)
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
C
      DIMENSION AXI(7),AYI(7)
C
C
      AX2=SQRT(UF(3,1))
      AY2=SQRT(UF(1,3))
      DO 110 I=1,NPAK
110   HSPL(I)=0.
         IF (ITYPE.EQ.1) GO TO 120
      CALL SPILL
      SPL=0.
         GO TO 130
120   SPL=1.
130   CONTINUE
      AXI(1)=1.
      AYI(1)=1.
      DO 140 I=2,NORDER
      AXI(I)=AXI(I-1)/AX2
140   AYI(I)=AYI(I-1)/AY2
C
C             EVALUATE THE HERMITE COEFFICIENTS
C
      IPAK=0
      DO 160 I=1,NORDER
      JMIN=JMN(I)
      JMAX=JMX(I)
      KMIN=IMN(I)
      DO 160 J=JMIN,JMAX,ISTEP
      LMIN=IMN(J)
      IPAK=IPAK+1
      SAVE=0.
      DO 150 K=KMIN,I,2
      DO 150 L=LMIN,J,2
C
C             PRINT 2566,I,J,K,L,SAVE,HXN(I,K),HXN(K,L),UF(K,L),
C             AXI(K),AYI(L)
C
150   SAVE=SAVE+HXN(I,K)*UF(K,L)*AXI(K)*AYI(L)*HXN(J,L)
      SAVE=SAVE/(FACT(I)*FACT(J)*6.2832)*AXI(2)*AYI(2)
      SPL=SPL+HSPL(IPAK)*SAVE
      HC(IPAK)=SAVE
160   CONTINUE
      RETURN
      END
      SUBROUTINE HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,
     1WT)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE CALCULATES THE INTEGRAL OF
C     HI(X/AX)*HJ(Y/AY)*EXP(-.5*(X**2/AX2+Y**2/AY2)) OVER
C     OVER THE PROJECTION OF THE RECEIVER ON THE IMAGE PLANE
C
C     8 POINT GAUSSIAN INTEGRATION FORMULAE ARE USED
C
C     THIS ROUTINE IS CALLED BY
C          SPILL(HERMIT(OPTCAL,MOMENT(PERCAL)))
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
C
      DIMENSION H1(31),H2(31),H3(31),XG(16),AG(16)
      DIMENSION CI(4)
      DATA (CI(I),I=1,4)/.196584,.115194,.000344,.019527/
C
C     DATA XG /.96028986,.79666648,.52553241,.18343464,-.18343464,
C    1         -.52553241,-.79666648,-.96028986/
C     DATA AG /.10122854,.22381034,.31370665,.36268378,.3628378,
C    1         .31370665,.22381034,.10122854/
C
      DATA (AG(I),I=1,16) /  .02715246, .06225352, .09515851, .12462897,
     1           .14959599, .16915652, .18260341, .18945061,
     2           .14959599, .16915652, .18260341, .18945061,
     3           .02715246, .06225352, .09515851, .12462897/
      DATA (XG(I),I=1,16)  / .98940093, .94457502, .86563120, .75540441,
     1           .61787624, .45801678, .28160355, .09501251,
     2          -.61787624,-.45801678,-.28160355,-.09501251,
     3          -.98940093,-.94457502,-.86563120,-.75540441/
C
C             EVALUATE THE HERMITE POLYNOMIALS AND EXPONENTIALS
C
      SAVE=(XMAX-XMIN)/2.
      SAVE2=2.50663
      H1(2)=0.
      H3(2)=0.
      DO 140 N=1,16
      XX=XMIN+SAVE*(1.+XG(N))
      X12=XX**2
      X2=(G1+G2*XX+G3*SQRT(G4+G5*X12))*AYI-YTA*AYI
      X3=(F1+F2*XX+F3*SQRT(F4+F5*X12))*AYI-YTA*AYI
      X1=(XX-XTA)*AXI
      H1(1)=0.
      H2(1)=0.
      H3(1)=0.
      X12=X1**2
      X22=X2**2
      X32=X3**2
      IF (X12.LT.100.) H1(1)=EXP(-X12/2.)*WT/AYI*SAVE
      IF (X22.LT.100.) H2(1)=EXP(-X22/2.)
      IF (X32.LT.100.) H3(1)=EXP(-X32/2.)
      H2(2)=0.
      FK=-2.
      DO 110 K=3,NORDEB
      FK=FK+1.
      H1(K)=X1*H1(K-1)-FK*H1(K-2)
      H2(K)=X2*H2(K-1)-FK*H2(K-2)
110   H3(K)=X3*H3(K-1)-FK*H3(K-2)
      S2=1.
      S3=1.
      SIGN3=(X3+1.23456789E-10)/ABS(X3+1.23456789E-10)
      SIGN2=(X2+1.23456789E-10)/ABS(X2+1.23456789E-10)
      X2=ABS(X2)
      X3=ABS(X3)
      DO 120 J=1,4
      S2=S2+CI(J)*X2**J
120   S3=S3+CI(J)*X3**J
      S2=SAVE2*((-1.+SIGN2)/(-2.)+SIGN2*(1.-.5*S2**(-4)))
      S3=SAVE2*((-1.+SIGN3)/(-2.)+SIGN3*(1.-.5*S3**(-4)))
      H2(2)=S3-S2
      IPAK=0.
      DO 130 I=1,NORDER
      JMIN=JMN(I)
      JMAX=JMX(I)
      DO 130 J=JMIN,JMAX,ISTEP
      IPAK=IPAK+1
130   HSPL(IPAK)=HSPL(IPAK)+AG(N)*(H1(I+2)*(H2(J+1)-H3(J+1)))
140   CONTINUE
      RETURN
      END
      SUBROUTINE LAND
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE LAND DETERMINES THE FRACTION, F(K,L) OF
C     THE AREA OF THE (K,L) ZONE THAT SATISFIES THE LAND
C     CONSTRAINT WHEN THE TOWER IS XTOWER*THT METERS TO
C     THE EAST AND YTOWER*THT METERS TO THE NORTH WITH
C     RESPECT TO THE ORIGIN USED TO DEFINE THE LAND
C     CONSTRAINT.
C
C     THIS SUBROUTINE CALCULATES THE FRACTION OF EACH
C     HELIOSTAT ZONE THAT SATISFIES A USER DEFINED
C     CONSTRAINT ON THE LAND AVAILABLE FOR HELIOSTATS
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C          PERCAL
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C
      DIMENSION X(6),Y(6),XMAX(6),YMAX(6),XMIN(6),YMIN(6),WTINT(6),XI(6)
     1  ,CLEN(5),CLNN(5),CLP(5),SLP(5)
C
C
      DATA (WTINT(I),I=1,6) / 0.0856622,.180380785,.233956965,
     1             .233956965,.18038078,0.0856622/
C>>>>RDATA (XI(I),I=1,6) / 0.033765245,.169395305,0.38069045,0.619309595
      DATA (XI(I),I=1,6)/ .033765245,.169395305,.380690405,.619309595
     1                      ,0.830604695    , 0.966234755/
C
C
      XTOWEN = XTOWES/THTI
      YTOWEN = YTOWES/THTI
      SAVE=0.
      SAVE1=0.
C>>>>DDO 110 K=1,NRAD
C>>>>DDO 110 L=1,NAZM
C>>>>DFLAND(K,L)=0.
C
C          NORMALIZE VALUES TO UNITS OF TOWER HEIGHT
C
      DO 120 N=1,NLAND
      CLP(N) = COS(RR*ALP(N))
      SLP(N) = SIN(RR*ALP(N))
      SLEWN = SLEW(N)/THTI
      XMAX(N)=SLEWN/2.
      XMIN(N)=XMAX(N)-SLEWN
      SLNSN = SLNS(N)/THTI
      YMAX(N)=SLNSN/2.
      YMIN(N)=YMAX(N)-SLNSN
      CLEN(N) = CLE(N)/THTI
      CLNN(N) = CLN(N)/THTI
120   CONTINUE
      DAZ=(AZM(2)-AZM(1))/5.
      DO 170 L=1,NAZM
      AZ=AZM(L)
C
C             EACH ZONE IS DIVIDED INTO A 6X6 GRID OF SUBZONES
C             IF CENTER POINT OF SUBZONE SATISFIES LAND CONSTRAINT
C             ENTIRE SUBZONE IS ASSUMED TO SATISFY CONSTRAINT
C             AND VICE VERSA
C
      DO 130 N=1,6
      A=AZ+XI(N)*DAZ-DAZ/2.
      X(N)=SIN(A*RR)
      Y(N)=COS(A*RR)
130   CONTINUE
      DO 170 K=1,NRAD
      ATOT=0.
      AIN=0.
      RMIN=RDMIN(K)
      DRR=RDMAX(K)-RMIN
      DO 160 KK=1,6
      R=RMIN+XI(KK)*DRR
      ATOT=ATOT+R*WTINT(KK)
      DO 150 M=1,6
      XXX=R*X(M)+XTOWEN
      YYY=R*Y(M)+YTOWEN
      DO 140 N=1,NLAND
      XX=CLP(N)*(XXX-CLEN(N))-SLP(N)*(YYY-CLNN(N))
      YY=SLP(N)*(XXX-CLEN(N))+CLP(N)*(YYY-CLNN(N))
C
C             TEST TO SEE IF POINT IS WITHIN ANY ALLOWED RECTANGLE
C
         IF ((XX-XMIN(N))*(XX-XMAX(N)).GT.0.) GO TO 140
         IF ((YY-YMIN(N))*(YY-YMAX(N)).GT.0.) GO TO 140
      AIN=AIN+R*WTINT(M)*WTINT(KK)
         GO TO 150
140   CONTINUE
150   CONTINUE
160   CONTINUE
      SAVE=SAVE+ATOT
      SAVE1=SAVE1+AIN
      FLAND(K,L)=AIN/ATOT
170   CONTINUE
C>>>>DDO 180 K=1,NRAD
C>>>>DDO 180 L=1,NAZM2P
C>>>>DLL=NAZM+2-L
C>>>>DIF (LL.GT.NAZM) LL=1
C>>>>DFLAND(K,L)=(FLAND(K,L)+FLAND(K,LL))/2.
C>>>>DCONTINUE
      FLTOT=SAVE1/SAVE
      WRITE (6,200) XTOWES,YTOWES,FLTOT
C>>>>RIF (IPROPT.EQ.0) RETURN
      IF ( IPROPT .NE. 1 ) RETURN
      WRITE (6,210)
      WRITE (6,220) (AZM(L),L=1,NAZM)
      DO 190 K=1,NRAD
      WRITE (6,230) RADIUS(K),(FLAND(K,L),L=1,NAZM)
190   CONTINUE
      RETURN
200   FORMAT (20X,9H TOWER IS,F7.1,12H M EAST AND ,F7.1,62H M NORTH, FRA
     1CTION OF TOTAL POSSIBLE FIELD WITHIN CONSTRAINT =,F5.2)
210   FORMAT (/20X,52H FRACTION OF ZONE THAT SATISFIES THE LAND CONSTRAI
     1NT)
220   FORMAT (19X,12(F6.0,2X))
230   FORMAT (1X,F5.2,13X,12F8.3)
      END
      SUBROUTINE SETCAV
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE SETCAV IS USED WHEN A CAVITY RECEIVER IS SPECIFIED.
C     IT ASSIGNS EACH ZONE TO THE CAVITY AT WHICH IT SHOULD POINT. THIS
C     IS NECESSARY FOR THE SPILLAGE CALCULATION. FOR ZONES SPLIT BETWEEN
C     TWO APERTURES, SETCAV DETERMINES THE FRACTION POINTED TOWARD EACH
C     OPENING.
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C          PERCAL
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /REC1/ CAZMMN(4),CAZMMX(4),JREC(12,2),NREC(12),RCRA(4),
     1  RSRA(4)
C
C
      DIMENSION AXSID(2),CAVANG(4)
C
C
      DATA (AXSID(I),I=1,2)/5HAXIS=,5HSIDE=/
C
C
C             CAVITY RECEIVERS
C
C             FIND CAVITY APERTURE(S) THAT EACH ZONE POINTS AT
C
      DO 110 I=1,NUMCAV
      RAZM(I)=RAZM(I)-180.
      IF (RAZM(I).LT.-1.E-10) RAZM(I)=RAZM(I)+360.
110   CONTINUE
         IF (INORTH.GT.0) GO TO 260
C>>>>DDAZM=AZMDEL
      DO 120 L=1,NAZM
      NREC(L)=0
      DO 120 K=1,2
      JREC(L,K)=0
      WTREC(L,K)=0.
120   CONTINUE
      SAVE = RAZM(NUMCAV)
      DO 250 N=1,NUMCAV
C>>>>RNSTART=INT(RAZM(N)/DAZM)+1
      NSTART = INT(RAZM(N)/AZMDEL) + 1
         IF (NUMCAV.GT.1) GO TO 130
      RMAX=RAZM(N)+90.
      RMIN=RAZM(N)-90.
         GO TO 180
130   CONTINUE
         IF (N.GT.1) SAVE = RAZM(N-1)
      IF (SAVE.GT.RAZM(N)) SAVE=SAVE-360.
      RMIN=(RAZM(N)+SAVE)/2.
      IF (RMIN.LT.RAZM(N)-90.) RMIN=RAZM(N)-90.
         IF (N.EQ.NUMCAV) GO TO 160
      SAVE=RAZM(N+1)
         GO TO 170
160   SAVE=RAZM(1)
170   CONTINUE
      IF (SAVE.LT.RAZM(N)) SAVE=SAVE+360.
      RMAX=(RAZM(N)+SAVE)/2.
      IF (RMAX.GT.RAZM(N)+90.) RMAX=RAZM(N)+90.
180   CONTINUE
      NZONE=NSTART
C>>>>RRSAVE=AZM(NZONE)+DAZM/2.
      RSAVE = AZM(NZONE) + AZMDEL/2.
      SSAVE=RAZM(N)
190   CONTINUE
      NREC(NZONE) = NREC(NZONE) + 1
      M = NREC(NZONE)
      JREC(NZONE,M) = N
C>>>>R   IF (RMAX-RSAVE) 210,210,200
         IF ( RMAX .LE. (RSAVE+1.0E-10) ) GO TO 210
C>>>>DCONTINUE
C>>>>RWTREC(NZONE,M)=WTREC(NZONE,M)+(RSAVE-SSAVE)/DAZM
      WTREC(NZONE,M) = WTREC(NZONE,M) + (RSAVE - SSAVE)/AZMDEL
      NZONE=NZONE+1
      IF (NZONE.GT.NAZM) NZONE=1
      SSAVE=RSAVE
C>>>>RRSAVE=RSAVE+DAZM
      RSAVE = RSAVE + AZMDEL
         GO TO 190
210   CONTINUE
C>>>>RWTREC(NZONE,M)=WTREC(NZONE,M)+(RMAX-SSAVE)/DAZM
      WTREC(NZONE,M) = WTREC(NZONE,M) + (RMAX - SSAVE)/AZMDEL
C>>>>RCAZMMX(N)=SSAVE+WTREC(NZONE,M)*DAZM
      CAZMMX(N) = SSAVE + WTREC(NZONE,M)*AZMDEL
      NZONE=NSTART
      NREC(NZONE)=NREC(NZONE)-1
      RSAVE=RAZM(N)
C>>>>RSSAVE=AZM(NZONE)-DAZM/2.
      SSAVE = AZM(NZONE) - AZMDEL/2.
220   CONTINUE
      NREC(NZONE) = NREC(NZONE) + 1
      M = NREC(NZONE)
      JREC(NZONE,M) = N
C>>>>R   IF (RMIN-SSAVE) 230,240,240
         IF ( RMIN .GE. (SSAVE-1.0E-10) ) GO TO 240
C>>>>DCONTINUE
C>>>>RWTREC(NZONE,M)=WTREC(NZONE,M)+(RSAVE-SSAVE)/DAZM
      WTREC(NZONE,M) =  WTREC(NZONE,M) + (RSAVE - SSAVE)/AZMDEL
      NZONE=NZONE-1
      IF (NZONE.LT.1) NZONE=NAZM
      RSAVE=SSAVE
C>>>>RSSAVE=SSAVE-DAZM
      SSAVE = SSAVE - AZMDEL
         GO TO 220
240   CONTINUE
C>>>>RWTREC(NZONE,M)=WTREC(NZONE,M)+(RSAVE-RMIN)/DAZM
      WTREC(NZONE,M) = WTREC(NZONE,M) + (RSAVE - RMIN)/AZMDEL
C>>>>RCAZMMN(N)=RSAVE-WTREC(NZONE,M)*DAZM
      CAZMMN(N) = RSAVE - WTREC(NZONE,M)*AZMDEL
250   CONTINUE
         GO TO 280
260   CONTINUE
      DO 270 L=1,NAZM
      NREC(L)=1
      JREC(L,1)=1
270   WTREC(L,1)=1.0
      NREC(NAZM2P)=0
      CAZMMN(1)=AMAXN
      CAZMMX(1)=360.-AMAXN
280   CONTINUE
         IF (IREC.LT.3) WRITE (6,360)
         IF (IREC.GT.2) WRITE (6,350)
C
C             PRINT OUT
C
      DO 290 I=1,NUMCAV
290   RAZM(I)=RAZM(I)+180.
         IF (IREC.EQ.2.OR.IREC.EQ.4) GO TO 300
      WRITE (6,370) NUMCAV
      J = 1
         GO TO 310
300   WRITE (6,390) NUMCAV
      J = 2
  310 CONTINUE
      WRITE (6,400) (I,RAZM(I),RELV(I),AXSID(J),RX(I),AXSID(J),RY(I),
     1I=1,NUMCAV)
      DO 320 I=1,NUMCAV
      CAVANG(I)=0.
      RCRA(I)=COS(RR*RAZM(I))
      RSRA(I)=SIN(RR*RAZM(I))
      RCRP(I)=COS(RR*RELV(I))
      RSRP(I)=SIN(RR*RELV(I))
  320 CONTINUE
      DO 330 L=1,NAZM
      N=NREC(L)
330   WRITE (6,410) L,AZM(L),N,(JREC(L,M),WTREC(L,M),M=1,N)
C
C             CAVANG(M) IS THE ANGULAR WIDTH OF THE PORTION OF THE
C             HELIOSTAT FIELD THAT POINTS AT CAVITY M
C
      DO 340 L=1,NAZM
      N=NREC(L)
      DO 340 I=1,N
      M=JREC(L,I)
C>>>>RCAVANG(M)=CAVANG(M)+DAZM*WTREC(L,I)
      CAVANG(M) = CAVANG(M) + AZMDEL*WTREC(L,I)
340   CONTINUE
      RETURN
350   FORMAT (///20X,19HFLAT PLATE RECEIVER/)
360   FORMAT (///20X,15HCAVITY RECEIVER/)
370   FORMAT (1X,10H THERE ARE,I2,43H APERTURES OR PLATES WITH ELLIPTICA
     1L SHAPES)
390   FORMAT (1X,10H THERE ARE,I2,49H APERTURES OR FLAT PLATES WITH RECT
     1ANGULAR SHAPES)
400   FORMAT (5X,10H    NUMBER,I2,9H AZIMUTH=,F5.1,11H ELEVATION=,F5.1,1
     12H HORIZONTAL ,A5,F6.2,7H OTHER ,A5,F6.2)
410   FORMAT (10X,4H THE,I2,23H AZIMUTHAL ZONES,ANGLE=,F6.1,13H ARE AIME
     1D AT,I2,17H APERTURES - NO.=,I2,4H WT=,F5.2,6H, NO.=,I2,4H WT=,F5.
     22)
      END
      SUBROUTINE SETFLX
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE SETFLX GENERATES THE COORDINATES OF THE POINTS ON
C     AN EXTERNAL RECEIVER AT WHICH THE FLUX IS TO BE CALCULATED
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C          PERCAL
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /FLUXXX/ AZMF,POLF,IFLAUT,NXFLX,NYFLX
C
      COMMON /FLUX1/ CFA(169),CFP(169),DIAMFN,FAZMAS,FAZMIS,FLX(169),
     1  FZMAXN,FZMINN,SFA(169),SFP(169),XFCN,XFLX(169),
     2  XNFLX(169),YFCN,YFLX(169),YNFLX(169),ZFCN,ZFLX(169),NFLXPT
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C
C             IFLAUTO = 1 EXTERNAL RECEIVER - VERTICAL CYLINDER
C                         NXFLX AZIMUTHAL POINTS FROM FAZMIN TO FAZMAX
C                         NYFLX VERTICAL POINTS FROM FZMIN TO FZMAX
C             IFLAUTO = 2 CAVITY RECEIVER - FLUX POINTS ON INSIDE
C                         OF CYLINDER
C             IFLAUTO = 3 FLAT PLATE RECEIVER - MAY BE BEHIND A
C                         CAVITY APERTURE
C
      N=0
      FSAVE=1.
      DAZMF=0.
      DHF=0.
      IF (IFLAUT.EQ.2) FSAVE=-1.
      IF (NXFLX.GT.1) DAZMF=(FAZMAS-FAZMIS)/FLOAT(NXFLX-1)
      IF (NYFLX.GT.1) DHF=(FZMAXN-FZMINN)/FLOAT(NYFLX-1)
         IF (IFLAUT.GT.2) GO TO 120
      DAZMF=DAZMF*RR
      DO 110 J=1,NYFLX
      SAVE2=FZMAXN-FLOAT(J-1)*DHF
      YNFLX(J)=SAVE2*THTI
      SAVE=RR*FAZMIS-DAZMF
      DO 110 I=1,NXFLX
      N=N+1
      SAVE=SAVE+DAZMF
      XNFLX(I)=SAVE/RR
      CFA(N)=COS(SAVE)
      SFA(N)=SIN(SAVE)
      CFP(N)=0.
      SFP(N)=1.
      ZFLX(N)=ZFCN+SAVE2
      XFLX(N)=-DIAMFN*SFA(N)*FSAVE/2.+XFCN
      YFLX(N)=-DIAMFN*CFA(N)*FSAVE/2.+YFCN
110   CONTINUE
         GO TO 140
120   CONTINUE
      FAZMNN = FAZMIS/THTI
      DAZMF=DAZMF/THTI
      CRA = COS(RR*AZMF)
      SRA = SIN(RR*AZMF)
      CRP = COS(RR*POLF)
      SRP = SIN(RR*POLF)
      DO 130 J=1,NYFLX
      SAVE2=FZMAXN-FLOAT(J-1)*DHF
      YNFLX(J)=SAVE2*THTI
      SAVE=FAZMNN-DAZMF
      DO 130 I=1,NXFLX
      SAVE=SAVE+DAZMF
      XNFLX(I)=SAVE*THTI
      N=N+1
      CFA(N) = CRA
      SFA(N) = SRA
      CFP(N) = CRP
      SFP(N) = SRP
      XFLX(N)=XFCN+SAVE*CFA(N)+SAVE2*CFP(N)*SFA(N)
      YFLX(N)=YFCN-SAVE*SFA(N)+SAVE2*CFP(N)*CFA(N)
      ZFLX(N)=ZFCN+SAVE2*SFP(N)
130   CONTINUE
140   CONTINUE
      RETURN
      END
      SUBROUTINE SPILL
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE CALCULATES THE AMOUNT OF FLUX
C     INTERCEPTED BY THE RECEIVER FOR EACH TERM IN THE
C
C     THIS ROUTINE IS CALLED BY
C          HERMIT(OPTCAL,MOMENT(PERCAL))
C
c change
c seri modification: (NW, 3-7-88)
C This routine has been modified to use alternate weighting factors
c the external receiver aim points.  If the number of aim points
c calculated by DELSOL is 3 or 5, alternate weights are used.  The values
c are based loosely on results from the utility study as reported by PG&E
c and Bechtel
c seri change add next line
      real*8 wfctrs(5,2)
C
C
      COMMON /DEL1/ AX2,AY2,DHOPT,DXAIM(100),DYAIM(100),HSPL(16),
     1  NAIMPT,NORDEB,RCRP(4),RSRP(4),TDESP,WTAIMP(100),WTREC(12,2),
     2  XTA,YTA
C
C>>>>RDOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /AIM/ IAUTOP,INDC,NAX(13,12),NAY(13,12),NUMPT(13,12),
     1  XAIM(13,12,2),YAIM(13,12,2)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /REC1/ CAZMMN(4),CAZMMX(4),JREC(12,2),NREC(12),RCRA(4),
     1  RSRA(4)
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
      COMMON /HELIO/ XCENT(25),YCENT(25)
C
C
      DIMENSION XTAP(100),YTAP(100), WTA(100)
C
C
c seri change, nw
c changed default aimpoint spillage cutoff from 2.5 to 1.8 sigma
c     DATA FAIMPT/2.5/
      data faimpt/1.8/
c
      data wfctrs/.50,.00,.50,0.0,0.0,.34,.11,.10,.11,.34/
c
C
C
      K=KK
      L=LL
      AXI=1./AX2
      AYI=1./AY2
C
C             DELSOL PICKS THE NUMBER AND LOCATION OF THE AIM POINTS
C             FOR A FLUX LIMITED RECEIVER.
C                       THIS DOES NOT AFFECT SPILLAGE MORE THAN
C                       A PERCENT
C                       PRODUCES A MORE UNIFORM FLUX PROFILE
C
      NN=1
      IF (IREC.GT.0) NN=NREC(LL)
      IF (NN.LT.1) RETURN
      H2 = HN/2.
      DO 430 NNN=1,NN
      I=JREC(LL,NNN)
         IF (INDC.EQ.0) GO TO 150
C
C             INDIVIDUAL IMAGES FROM EACH CANT PANEL
C               DONE FOR PERFORMANCE CALCULATIONS ONLY
C
      NAIMPT=NCANT
      NAIMPA=NCANT
         IF (IREC.EQ.0) GO TO 110
C>>>>DI=JREC(LL,NNN)
      WT=WTREC(LL,NNN)
      CBRT=CTA*RCRA(I)+STA*RSRA(I)
      SBRT=CTA*RSRA(I)-STA*RCRA(I)
      R11=CBRT
      R12=SBRT*RCRP(I)
      R21=-CTP*SBRT
      R22=STP*RSRP(I)+CTP*RCRP(I)*CBRT
      SAVE=R11*R22-R21*R12
         GO TO 120
110   CONTINUE
      R11=1./STP
      R12=0.
      R21=0.
      R22=-1.
      SAVE=1.
120   CONTINUE
C
C             CONSIDER EACH CANT PANEL AS A SEPARATE HELIOSTAT WITH AN
C             AIM POINT SET WHERE PANEL CENTER HITS RECEIVER PLANE
C
      WTAC = 1./FLOAT(NCANT)
      DO 140 N=1,NCANT
      WTA(N) = WTAC
      WTAIMP(N) = WTAC
      DXAIM(N)=(R22*XCENT(N)-R12*YCENT(N))/SAVE
      DYAIM(N)=(-R21*XCENT(N)+R11*YCENT(N))/SAVE
         IF (IAUTOP.NE.6) GO TO 130
      DXAIM(N)=DXAIM(N)+XAIM(K,L,1)/THTI
      DYAIM(N)=DYAIM(N)+YAIM(K,L,1)/THTI
130   CONTINUE
      XTAP(N)=DXAIM(N)
140   YTAP(N)=DYAIM(N)
C>>>>R   GO TO 310
         GO TO 300
150   CONTINUE
         IF (IAUTOP.NE.0) GO TO 160
      NAIMPA=1
      NAIMPT=1
      NUMPT(K,L)=1
      NAX(K,L)=1
      NAY(K,L)=1
      DXAIM(1)=0.
      DYAIM(1)=0.
      WTAIMP(1)=1.
      WTA(1)=1.
      XTAP(1)=0.
      YTAP(1)=0.
C>>>>R   GO TO 310
         GO TO 300
160   CONTINUE
         IF (IAUTOP.NE.6) GO TO 180
      NAIMPA=NUMPT(K,L)
      NAIMPT = NAIMPA
      WTAC = 1./FLOAT(NAIMPT)
      DO 170 N=1,NAIMPA
      WTA(N) = WTAC
      XTAP(N)=XAIM(K,L,N)/THTI
      YTAP(N)=YAIM(K,L,N)/THTI
      WTAIMP(N) = WTAC
      DXAIM(N)=XTAP(N)
      DYAIM(N)=YTAP(N)
170   CONTINUE
C>>>>R   GO TO 310
         GO TO 300
180      IF (IAUTOP.EQ.5) GO TO 230
      AYS=SQRT(UF(1,3))*FAIMPT/STP
      AYS2=AYS
      AXS=1.E+20
      IF (IAUTOP.EQ.2*(IAUTOP/2)) AXS=SQRT(UF(3,1))*FAIMPT
      IF(IAUTOP.LT.0) AXS=SQRT(UF(3,1))*FAIMPT
      AXS2=AXS
      SAIMY=HN
      SAIMX=WN*.866
         IF (IREC.EQ.0) GO TO 190
      SAIMY=2.*RYN(I)
      SAIMX=2.*RXN(I)
      CBRT=CTA*RCRA(I)+STA*RSRA(I)
      SBRT=CTA*RSRA(I)-STA*RCRA(I)
      R11=CBRT
      R12=SBRT*RCRP(I)
      R21=-CTP*SBRT
      R22=STP*RSRP(I)+CTP*RCRP(I)*CBRT
      AX3=AXI**2
      AY3=AYI**2
      BB=R12**2*AX3+R22**2*AY3
      AA=R11**2*AX3+R21**2*AY3
      CC=2.*(R11*R12*AX3+R21*R22*AY3)
      DD=-1.
      SAVE=CC**2-4.*AA*BB
      IF ((AA*SAVE)**2.LT.1.E-15) RETURN
      XMAX=DD*CC**2/AA/SAVE
      XMAX=ABS(XMAX)
      XMIN=SQRT(XMAX)
      IF (CC.GT.0.) XMIN=-XMIN
      XMAX=ABS(-CC*XMIN/2./BB+SQRT(XMAX*SAVE-4.*DD*BB)/2./BB)
      AYS2=XMAX*FAIMPT
      SAVE3=1.0+.4*FLOAT(IREC-(IREC/2)*2)
      AYS=1./SQRT(BB)*FAIMPT
      AYS2=(AYS+AYS2)*SAVE3/2.
      AXS=1.E+20
      AXS2=AXS
         IF (IAUTOP.NE.2*(IAUTOP/2).AND.IAUTOP.GE.0) GO TO 190
      SAVE=CC**2-4.*AA*BB
      XMAX=DD*CC**2/BB/SAVE
      XMIN=SQRT(XMAX)
      IF (CC.GT.0) XMIN=-XMIN
      XMAX=ABS(-CC*XMIN/2./AA+SQRT(XMAX*SAVE-4.*AA*DD)/2./AA)
      AXS=1./SQRT(AA)*FAIMPT
      AXS2=XMAX*FAIMPT
      AXS2=(AXS+AXS2)*SAVE3/2.
190   CONTINUE
      DYAIM(1)=SAIMY/2.-AYS2
      DXAIM(1)=SAIMX/2.-AXS2
         IF (DYAIM(1).GT.0.) GO TO 200
      DYAIM(1)=0.
      SAVE3=0.
      NAIMY=1
         GO TO 210
200   NAIMY=INT((SAIMY-AYS2*2.)/AYS)+2
      IF (NAIMY.GT.25) NAIMY=25
      NAIMY=2*(NAIMY/2)+1
      SAVE3=(SAIMY-2.*AYS2)/FLOAT(NAIMY-1)
210      IF (DXAIM(1).GT.0.) GO TO 220
      DXAIM(1)=0.
      SAVE4=0.
      NAIMX=1
         GO TO 240
220   NAIMX=INT((SAIMX-AXS2*2.)/AXS/.8)+2
      NAIMX=2*(NAIMX/2)+1
      IF (NAIMX*NAIMY.GT.100) NAIMX=100/NAIMY
      SAVE4=(SAIMX-2.*AXS2)/FLOAT(NAIMX-1)
         GO TO 240
230   NAIMX=NAX(K,L)
      NAIMY=NAY(K,L)
      DXAIM(1)=XAIM(K,L,2)/THTI
      DYAIM(1)=YAIM(K,L,2)/THTI
      SAVE3=0.
      SAVE4=0.
      IF (NAIMX.GT.1) SAVE4=(XAIM(K,L,2)-XAIM(K,L,1))/FLOAT(NAIMX-1)/
     1  THTI
      IF (NAIMY.GT.1) SAVE3=(YAIM(K,L,2)-YAIM(K,L,1))/FLOAT(NAIMY-1)/
     1  THTI
240   CONTINUE
      IF ((IAUTOP.GE.0.AND.IAUTOP.LT.3).OR.IAUTOP.GT.4) GO TO 250
      IF(IAUTOP.LT.0.AND.K.GE.(-IAUTOP)) GO TO 250
      NAIMY=1
      DYAIM(1)=-DYAIM(1)
      SAVE3=0.
      IF (IAUTOP.LT.0) GO TO 250
         IF (IAUTOP.EQ.4) GO TO 250
      NAIMX=1
      SAVE4=0.
250   CONTINUE
      NAIMPT=NAIMX*NAIMY
c seri change for alternate external aim point weights
c
      iaim=0
      if (irec.eq.0 .and. naimpt.eq.3) iaim=1
      if (irec.eq.0 .and. naimpt.eq.5) iaim=2
c
      WTAC = 1./FLOAT(NAIMPT)
      NUMPT(K,L)=NAIMPT
      NAX(K,L)=NAIMX
      NAY(K,L)=NAIMY
      YAIM(K,L,2)=DYAIM(1)*THTI
      XAIM(K,L,2)=DXAIM(1)*THTI
      MN=0
      DO 260 N=1,NAIMY
      DO 260 M=1,NAIMX
      MN=MN+1
      DYAIM(MN)=DYAIM(1)-FLOAT(N-1)*SAVE3
      DXAIM(MN)=DXAIM(1)-FLOAT(M-1)*SAVE4
c seri change for alternate external aim points weights
c
      if(irec.eq.0 .and. iaim.ne.0) wtac=wfctrs(n,iaim)
c
      WTAIMP(MN) = WTAC
260   CONTINUE
      YAIM(K,L,1)=DYAIM(NAIMPT)*THTI
      XAIM(K,L,1)=DXAIM(NAIMPT)*THTI
      NAIM2X=NAIMX
      IF (NAIM2X.GT.3) NAIM2X=3
      NAIM2Y=NAIMY
      IF (NAIM2Y.GT.3) NAIM2Y=3
      SAVE=1./FLOAT(NAIMPT)
      NAIMPA=NAIM2X*NAIM2Y
      MN=0
      DO 270 N=1,NAIM2Y
      NM=N+(N/2-N/3)*(NAIMPT/2-1)+N/3*(NAIMPT-N)
      DO 270 M=1,NAIM2X
      MM=M+(M/2-M/3)*(NAIMX/2-1)+M/3*(NAIMX-M)
      MN=MN+1
      XTAP(MN)=DXAIM(MM)
      YTAP(MN)=DYAIM(NM)
270   WTA(MN)=SAVE
         IF (NAIM2X.LT.3) GO TO 280
      WTA(2)=FLOAT(NAIMX-2)*SAVE
      WTA(NAIMPA-1)=WTA(2)
         IF (NAIM2Y.LT.3) GO TO 300
      WTA(5)=FLOAT((NAIMX-2)*(NAIMY-2))*SAVE
         GO TO 290
280      IF (NAIM2Y.LT.3) GO TO 300
290   WTA(NAIM2X+1)=FLOAT(NAIMY-2)*SAVE
      WTA(2*NAIM2X)=WTA(NAIM2X+1)
300   CONTINUE
C>>>>DCONTINUE
         IF (IREC.GT.0) GO TO 330
C
C             EXTERNAL CYLINDRICAL RECEIVER
C
      G5=-1.
      F5=-1.
      G2=0.
      F2=0.
      G3=CTP
      F3=G3
      DO 320 I=1,NAIMPA
      XMAX=W2
      XMIN=-W2
      IF (XMAX/AX2.GT.4) XMAX=4.*AX2
      IF (XMIN/AX2.LT.-4.) XMIN=-4.*AX2
      G1=XMIN*CTP-H2*STP
      F1=XMIN*CTP+H2*STP
      XTA=XTAP(I)
      YTA=YTAP(I)*STP
      G4=XMAX**2
      F4=G4
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WTA(I))
320   CONTINUE
         GO TO 440
330   CONTINUE
C
C             CAVITY RECEIVER
C
C>>>>DI=JREC(LL,NNN)
      WT=WTREC(LL,NNN)
      CBRT=CTA*RCRA(I)+STA*RSRA(I)
      SBRT=CTA*RSRA(I)-STA*RCRA(I)
      R11=CBRT
      R12=SBRT*RCRP(I)
      R21=-CTP*SBRT
      R22=STP*RSRP(I)+CTP*RCRP(I)*CBRT
      DO 430 M=1,NAIMPA
      SAVE=XTAP(M)
      SAVE2=YTAP(M)
C
C             CONVERT AIMPOINT FROM RECEIVER TO IMAGE PLANE
C
      XTA=R11*SAVE+R12*SAVE2
      YTA=R21*SAVE+R22*SAVE2
      WT=WTA(M)*WTREC(LL,NNN)
         IF (IREC.EQ.2.OR.IREC.EQ.4) GO TO 340
C
C             CAVITY WITH ELLIPTICAL OPENING
C
      AR2=RA2(I)
      BR2=RB2(I)
      AA=(R12**2/AR2+R11**2/BR2)
      BB=-2.*(R12*R22/AR2+R11*R21/BR2)
      CC=R22**2/AR2+R21**2/BR2
      DD=-(R11*R22-R12*R21)**2
      SAVE=BB*BB-4.*AA*CC
      XMAX=DD*BB**2/AA/SAVE
      XMIN=SQRT(XMAX)
      IF (BB.GT.0.) XMIN=-XMIN
      XMAX=ABS(-BB*XMIN/2./CC+SQRT(XMAX*SAVE-4.*CC*DD)/2./CC)
      XMIN=-XMAX
      G1=0.
      F1=0.
      G2=-BB/AA/2.
      F2=G2
      G3=-.5/AA
      F3=-G3
      G5=BB**2-4.*AA*CC
      F5=G5
      G4=-4.*AA*DD
      F4=G4
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WT)
         GO TO 430
340   CONTINUE
C
C             CAVITY WITH RECTANGULAR OPENING
C
      SAVE=RXN(I)
      SAVE2=RYN(I)
      IF (R11.LT.0.) SAVE=- SAVE
      IF (R12.LT.0.) SAVE2=- SAVE2
      X1=R11*SAVE+R12*SAVE2
      Y1=R21*SAVE+R22*SAVE2
      X2=-R11*SAVE+R12*SAVE2
      Y2=-R21*SAVE+R22*SAVE2
      X3=-X1
      Y3=-Y1
      X4=-X2
      Y4=-Y2
      G3=0.
      G4=0.
      G5=0.
      F3=0.
      F4=0.
      F5=0.
         IF (X2+1.E-08.LT.X4) GO TO 350
      SAVE=X2
      X2=X4
      X4=SAVE
      SAVE=Y2
      Y2=Y4
      Y4=SAVE
350   CONTINUE
         IF ((X1-X4)**2.GT.1.E-08) GO TO 370
      XMIN=X3
      XMAX=X1
      G1=(X2*Y1-X1*Y2)/(X2-X1)
      G2=(Y2-Y1)/(X2-X1)
      F2=(Y3-Y4)/(X3-X4)
      F1=(X3*Y4-X4*Y3)/(X3-X4)
         IF (F1+F2*X2.GE.G1+G2*X2) GO TO 360
      SAVE=G1
      G1=F1
      F1=SAVE
      SAVE2=G2
      G2=F2
      F2=SAVE2
360   CONTINUE
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WT)
         GO TO 430
370   CONTINUE
         IF ((X2-X3)**2.LT.1.E-08) GO TO 390
      XMIN=X3
      XMAX=X2
      F1=(X3*Y4-X4*Y3)/(X3-X4)
      F2=(Y3-Y4)/(X3-X4)
      G1=(X2*Y3-X3*Y2)/(X2-X3)
      G2=(Y2-Y3)/(X2-X3)
         IF (F1+F2*X2.GE.G1+G2*X2) GO TO 380
      SAVE=G1
      G1=F1
      F1=SAVE
      SAVE2=G2
      G2=F2
      F2=SAVE2
380   CONTINUE
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WT)
390   CONTINUE
         IF ((X2-X4)**2.LT.1.E-08) GO TO 410
      XMIN=X2
      XMAX=X4
      G1=(X2*Y1-X1*Y2)/(X2-X1)
      G2=(Y2-Y1)/(X2-X1)
      F2=(Y3-Y4)/(X3-X4)
      F1=(X3*Y4-X4*Y3)/(X3-X4)
         IF (F1+F2*X2.GE.G1+G2*X2) GO TO 400
      SAVE=G1
      G1=F1
      F1=SAVE
      SAVE2=G2
      G2=F2
      F2=SAVE2
400   CONTINUE
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WT)
410   CONTINUE
      XMIN=X4
      XMAX=X1
      F1=(X4*Y1-Y4*X1)/(X4-X1)
      F2=(Y4-Y1)/(X4-X1)
      G1=(X2*Y1-X1*Y2)/(X2-X1)
      G2=(Y2-Y1)/(X2-X1)
         IF (F1+F2*X4.GE.G1+G2*X4) GO TO 420
      SAVE=G1
      G1=F1
      F1=SAVE
      SAVE2=G2
      G2=F2
      F2=SAVE2
420   CONTINUE
      CALL HINT (G1,G2,G3,G4,G5,F1,F2,F3,F4,F5,XMAX,XMIN,AXI,AYI,WT)
430   CONTINUE
440   CONTINUE
      RETURN
      END
      SUBROUTINE OPTCAL
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE OPTCALL IS ACCESSED FOR OPTIMIZATION RUNS. THE
C     USER SPECIFIED ARRAYS FOR TOWER HEIGHT AND RECEIVER DIMENSIONS ARE
C     SELECTIVELY SEARCHED AT EACH REQUESTED POWER LEVEL TO FIND THE
C     SYSTEM DESIGN GIVING THE LOWEST COST ENERGY. FOR EACH TOWER AND
C     RECEIVER COMBINATION CONSIDERED, THE AVERAGE YEARLY PERFORMANCE
C     IS RECALCULATED. SUBROUTINE MAX IS THEN CALLED TO PERFORM THE
C     FIELD LAYOUT AND ENERGY COST CALCULATION.
C
C     THIS ROUTINE IS CALLED BY
C          DELSOL
C
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /DEL3/ IRERUN
C
      COMMON /GEN1/ ATM1S,ATM2S,ATM3S,ATM4S,CTAANG(13),CTPANG(13),
     1  DCANT,DIAMF,FAZMAX,FAZMIN,FZMAX,FZMIN,HCANT,
     2  HCS(13,7,50),HEAD1,HEAD11,HEAD12,HEAD2,HEAD3,HEAD4,HPANL,
     3  HXCANT(25),HYCANT(25),
     4  RADMAX,RADMIN,RANGE(13),
     5  RCANT(13),RMIRL,ROUND,SIGAZ,SIGEL,SIGSX,SIGSY,SIGTX,
     6  SIGTY,SIPAK(16),STAANG(13),TOWD,TOWL,WM,WPANL,XFC,XFOCUS,
     7  YFC,YFOCUS,ZFC,DENSMR,
     8  ICANT,ICPANL,IFLX,IFOCUS,IFXOUT(9,20),IDENS,
     9  IROTFL,NCANTX,NCANTY,NUMK(16)
C>>>>R LINE 7 ABOVE
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>4I
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /PERF/ HREFIN(16,13,7),REFTOT(13,12),YATMOS(13,12),
     1  YBLOCK(13,7),YCOSI(13,7),YSHAD(13,7),YSPILL(13,7),YTOTAL(13,7)
     2  ,HROP,RREFL,RWCAV(4),NHREF
C>>>>R LINE 2 ABOVE
C
      COMMON /AIM/ IAUTOP,INDC,NAX(13,12),NAY(13,12),NUMPT(13,12),
     1  XAIM(13,12,2),YAIM(13,12,2)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
      COMMON /FLUXXX/ AZMF,POLF,IFLAUT,NXFLX,NYFLX
C
      COMMON /FLUX1/ CFA(169),CFP(169),DIAMFN,FAZMAS,FAZMIS,FLX(169),
     1  FZMAXN,FZMINN,SFA(169),SFP(169),XFCN,XFLX(169),
     2  XNFLX(169),YFCN,YFLX(169),YNFLX(169),ZFCN,ZFLX(169),NFLXPT
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /EFF/ ACWOSY,AREF,CCMULC,CCMULY,CSTREF,CSTRMD,EFFSTR,
     1  EMPTY,ESTREF,ETAREF,FCR,FEFF,FLXLIM(4),FPLH,FSP,
     2  REFLP,REFPIP,REFPRL,REFRC,REFRC1,REFRC2,REFRC3,REFTHP,RHOML,
     X  RNHOML,
     3  SMULT,VMAX,VSTREF,XST,ITHEL,IRADFL
C>>>>R LINE 1 ABOVE
C
      COMMON /EFF1/ HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5
     *             ,PARL6,PARL7,PARL8,PARL9,PARL10,TPRE,TSTRT
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C
C>>>>RCOMMON /OPT1/ CH,DATM(93),DSAVE(13,12),FLUXMB(13,7,4),
c      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
c     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
c     2  HVAL(93),HWDA,HWDR,HWR,IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
c     3  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH,THT2
c     4  ,ICOUNT,IHOPT,AZMSER(13,12),DENSIR(13,12),DAREAF(13,7)
      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
     2  HVAL(93),HWDA,HWDR,HWR,THT2,
     3  AZMSER(13,12),DENSIR(13,12),DAREAF(13,7),ICOUNT,IHOPT,
     4  IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
     5  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH
C>>>>R LINE 3 ABOVE
C>>>>R LINE 4 ABOVE
C
c      COMMON /OPT2/ AEVMAX,AEVREF,ALLATM(20),ALLBUA(20),ALLBUS(20),
c     1  ALLCC(20),ALLCI(20),ALLCIC(20),ALLCIY(20),ALLCOS(20),
c     2  ALLETA(20),ALLFLU(20,4),ALLHR(20),ALLKWH(20),ALLPCA(20),
c     3  ALLPCB(20),ALLPCE(20),ALLPCF(20),ALLPCH(20),ALLPCL(20),
c     4  ALLPCP(20),ALLPCR(20),ALLPCS(20),ALLPCT(20),ALLPCW(20),
c     5  ALLRCP(20),ALLRCR(20),ALLSAB(20),ALLSPL(20),ALLSTR(20),
c     6  ALLTC(20),ALLTOA(20),ALLTOB(20),ALLTP(20),ANGMAX(20,4),
c     7  ANGMIN(20,4),APHMAX,APHREF,AREC,ARECRC,ARECRF,ARESAV(20),
c     8  ARHMAX,ARHREF,ASHMAX,ASHREF,AVGPRL,BBHTW(20),BBW(20),CCPREF,
c     9  CEGREF,CEVREF,CFIXED,CHEREF,CHPREF,CL,CPHREF,CREC,CREC1,
c     A  CRHREF,CRPU,CSHREF,CSPU,CTOW,CWOSCH(20),CWOSCL(20),CWOSYH(20),
c     B  CWOSYL(20),FLAST(20),HBOT(20,4),HSTSAV(20),
c     C  HTOP(20,4),ICHE,IOPTHT(20),IOPTL,IOPTPO(20),IOPTRE(20),
c     D  IOPTTH(20),IOPT1,IOPT2,IOPT20,IOPT21,IOPT3,IOPT30,IOPT40,
c     E  IOPT6,IOPT7,IOPT8,IOPT9,IPLFL(20),IPOS,
c     F  ISET(20,20),KLAST(20),LLAST(20),MHTW,MREC,MTHT,NUMOPT,PEGREF,
c     G  PEVREF,PHEREF,PMXH,POPT(21),PPHREF,PPIREF,PRHREF,PSHREF,PXHTW,
c     H  SMPI,XEPGS,XHEA,XHEP,XPI,XREC,XRP,XSP
c     I ,CKNREF,PKNREF,XKN,ICKN
c     J ,ALLXFLX(20,4,3)
      COMMON /OPT2/ AEVMAX,AEVREF,ALLATM(20),ALLBUA(20),ALLBUS(20),
     1  ALLCC(20),ALLCI(20),ALLCIC(20),ALLCIY(20),ALLCOS(20),
     2  ALLETA(20),ALLFLU(20,4),ALLHR(20),ALLKWH(20),ALLPCA(20),
     3  ALLPCB(20),ALLPCE(20),ALLPCF(20),ALLPCH(20),ALLPCL(20),
     4  ALLPCP(20),ALLPCR(20),ALLPCS(20),ALLPCT(20),ALLPCW(20),
     5  ALLRCP(20),ALLRCR(20),ALLSAB(20),ALLSPL(20),ALLSTR(20),
     6  ALLTC(20),ALLTOA(20),ALLTOB(20),ALLTP(20),ANGMAX(20,4),
     7  ANGMIN(20,4),APHMAX,APHREF,AREC,ARECRC,ARECRF,ARESAV(20),
     8  ARHMAX,ARHREF,ASHMAX,ASHREF,AVGPRL,BBHTW(20),BBW(20),CCPREF,
     9  CEGREF,CEVREF,CFIXED,CHEREF,CHPREF,CL,CPHREF,CREC,CREC1,
     A  CRHREF,CRPU,CSHREF,CSPU,CTOW,CWOSCH(20),CWOSCL(20),CWOSYH(20),
     B  CWOSYL(20),FLAST(20),HBOT(20,4),HSTSAV(20),
     C  HTOP(20,4),
     D  PEGREF,
     E  PEVREF,PHEREF,PMXH,POPT(21),PPHREF,PPIREF,PRHREF,PSHREF,PXHTW,
     F  SMPI,XEPGS,XHEA,XHEP,XPI,XREC,XRP,XSP
     G ,CKNREF,PKNREF,XKN,ALLXFLX(20,4,3),
     H  ICHE,IOPTHT(20),IOPTL,IOPTPO(20),IOPTRE(20),
     I  IOPTTH(20),IOPT1,IOPT2,IOPT20,IOPT21,IOPT3,IOPT30,IOPT40,
     J  IOPT6,IOPT7,IOPT8,IOPT9,IPLFL(20),IPOS,
     K  ISET(20,20),KLAST(20),LLAST(20),MHTW,MREC,MTHT,NUMOPT,ICKN
C>>>>R LINE D AND E ABOVE
C>>>>R LINE B ABOVE
C
C
      DIMENSION BBHTWL(20),BBWL(20),HTWSAV(20),RECSAV(20),THTSAV(20),
     1  Y2(20),RXI(4),RYI(4)
     2  ,APEREC(2),ATEWID(2),MINMAX(2),PREC(2),UREWID(2)
C
      DATA APER/7HAPERTUR/,EWID/7HE WIDTH/,PLATE/7H   PLAT/
     1  ,RECE/7HRECEIVE/,DIAM/7HR DIAM /,RECEI/6H RECEI/,VER/6HVER HT/
     2  ,(APEREC(I),I=1,2)/6H APERT,6H RECVR/,(UREWID(I),I=1,2)
     3  /6HURE HT,6H WIDTH/,(PREC(I),I=1,2)/6H    PL,6H RECER/
     4  ,(ATEWID(I),I=1,2)/6HATE HT,6H WIDTH/,(MINMAX(I),I=1,2)/4H MIN
     5  ,4H MAX/
C>>>>I 8 LINES ABOVE
C
C
      NAMELIST /NLEFF/  REFTHP,AREF,REFRC,REFLP,REFPIP,FPLH,FPLC,ITHEL,
     1                  ETAREF,FEFF,REFPRL,FSP,FEP,EFFSTR,PF,SMULT,IPH,
     2                  REFRC1,REFRC2,REFRC3,IRADFL
     *,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7,PARL8,PARL9
     *,PARL10,TPRE,TSTRT
      NAMELIST /NLCOST/ CH,CL,CWR,CWDR,CWDA,ITHT,CTOW1,CTOW2,CTOW3,
     1                  XTOW,CREC1,ARECRF,XREC,CRPREF,TRPREF,SMRP,
     2                  PRPREF,XRP,CSPREF,PSPREF,XSP,CHPREF,CCPREF,SMPI,
     3                  PPIREF,XPI,CSTREF,CSTRMD,VSTREF,ESTREF,XST,
     4                  VMAX,EMPTY,ICHE,CHEREF,PHEREF,XHEP,CPHREF,
     5                  APHREF,PPHREF,APHMAX,CEVREF,AEVREF,PEVREF,
     6                  AEVMAX,CSHREF,ASHREF,PSHREF,ASHMAX,CRHREF,
     7                  ARHREF,PRHREF,ARHMAX,XHEA,CEGREF,PEGREF,
     8                  XEPGS,CFIXED
     9                 ,ICKN,XKN,CKNREF,PKNREF
      NAMELIST /NLFLUX/ IFLX,IFXOUT,IFLAUT,XFC,YFC,ZFC,POLF,AZMF,DIAMF,
     1                  NXFLX,FAZMIN,FAZMAX,NYFLX,FZMIN,FZMAX,ICAVF,
     2                  NFLXMX,NMXFLX,FLXLIM
      NAMELIST /REC/    THT,TOWL,TOWD,IREC,W,H,RRECL,IAUTOP,NAY,NAX,
     1                  XAIM,YAIM,NUMPT,NUMCAV,RELV,RAZM,RX,RY,RWCAV
      NAMELIST /OPT/    IHOPT,NUMTHT,THTST,THTEND,NUMREC,WST,WEND,
     1                  NUMHTW,HTWST,HTWEND,IOPTUM,RYTRX,RX2TRX,RX3TRX,
     2                  RX4TRX,NUMOPT,POPTMN,POPTMX,NUMPOS,XTPST,YTPST,
     3                  XTPEND,YTPEND,NLAND,ALP,CLE,CLN,SLEW,SLNS,SMULT,
     4                  IPLFL,IPROPT,IHOPTP,IOTAPE,IRERUN,IALL,ISTR,NSTR
      NAMELIST /NLECON/ CONT,SPTS,EXT,ESC,RINF,NYTCON,AFDC,IFCR,FCR,
     1                  DISRT,PTI,TC,TR,FDEBT,RDEBT,ROE,IDEP,NDEP,NYOP,
     2                  RHOM,RNHOM
C
C
C             DEFAULT VALUES FOR NAMELIST NLCOST
C
      HEAD17 = 7HHELCOST
C     HEL COST CH BASED ON 2-20-85 MEMO FROM H. NORRIS TO P. FALCONE
      CH=120.0
      CL=0.62
      CWR=4.0/130.0
      CWDR=15.0000
      CWDA=9.00
      ITHT=0
      CTOW1=0.78232E+06
      CTOW2=0.0113
      CTOW3=1.09025E+06
      XTOW=0.00879
      CREC1=23.0E+06
      XREC=0.8
      ARECRF=0.758E+03
      CRPREF=2.100E+06
      TRPREF=170.0
      SMRP=1.5
      PRPREF=2.6E+08
      XRP=0.85
      CSPREF=0.470E+06
      PSPREF=3.0E+08
      XSP=0.15
      CHPREF=2.84E+04
      CCPREF=0.0
      SMPI=1.5
      PPIREF=2.6E+08
      XPI=1.06
      CSTREF=9.700E+06
      CSTRMD=6.80E+06
      ESTREF=6.88E+08
      VSTREF=0.374E+04
      VMAX=1.23E+04
      EMPTY=0.0
      XST=0.6
      ICKN=0
      CKNREF=9.0E6
      PKNREF=320.0E6
      XKN=0.8
      ICHE=0
      XHEP=0.8
      CHEREF=15.20E+06
      PHEREF=1.73E+08
      ARHREF=310.0
      ARHMAX=1.0E+10
      CRHREF=1.38E+06
      ASHREF=400.0
      ASHMAX=1.0E+10
      CSHREF=1.24E+06
      AEVREF=1300.0
      AEVMAX=1.0E+10
      CEVREF=3.77E+06
      APHREF=1.0
      APHMAX=1.0E+10
      CPHREF=0.0
      PRHREF=2.6E+08
      PSHREF=2.6E+08
      PEVREF=2.6E+08
      PPHREF=1.0
      XHEA=0.6
      CEGREF=37.5E+06
      PEGREF=1.12E+08
      XEPGS=0.8
      CFIXED=0.00000
C
C             DEFAULT VALUES FOR NAMELIST NLECON
C
      CONT=0.12
      SPTS=0.01
      EXT=0.16
      ESC=0.00
      RINF=0.00
      NYTCON=0
      AFDC=0.09
      IFCR=0
      FCR=0.06150
      DISRT=0.0315
      PTI=0.010
      TC=0.1
      TR=0.48
      FDEBT=0.5431
      RDEBT=0.11
      ROE=0.15
      IDEP=2
      NDEP=24
      NYOP=30
      RNHOM=0.015
      RHOM=0.015
C
C             DEFAULT VALUES FOR NAMELIST NLEFF
C
      PF=0.9
      REFRC=0.83
      AREF=2780.0
      IF(IREC.EQ.1.OR.IREC.EQ.2) AREF=761.0
      REFTHP=4.10E+08
      IF(IREC.EQ.1.OR.IREC.EQ.2) REFTHP=2.10E+08
      REFPIP=0.990
      REFLP=170.0
      FPLH=2.6
      FPLC=2.6
      ITHEL=0
      ETAREF=0.42
      FEFF=0.90
      IRADFL=0
      IF(IREC.GT.2)IRADFL=1
      REFRC2=460.0
      IF(IREC.EQ.1.OR.IREC.EQ.2) REFRC2=7631.0
      REFRC1=16403.0
      REFRC3=0.0
      PARL1=0.5
      PARL2=0.10300
      PARL3=0.5
      PARL4=0.10300
      PARL5=0.50000
      PARL6=0.00800
      PARL7=0.50000
      PARL8=0.00800
      PARL9=0.18
      PARL10=0.009
      TPRE=1.0
      TSTRT=1.0
      REFPRL=0.103
      FSP=0.66
      EFFSTR=1.0
      FEP=0.0
      IPH=0
      K=0
      DO 110 J=1,NAZM2P
      DO 110 I=1,NRAD
      K=K+1
C>>>>DFLAND(I,J)=1.0
C>>>>DM=MZONE(J)
C>>>>DFLAND(I,M)=1.0
      REALD(I,J)=1.000
      LMAX(K)=J
      KMAX(K)=I
110   CONTINUE
      NMAX=NRAD*NAZM2P
C
C             READ IN THE NEXT PROBLEM
C
C             DEFAULT VALUES FOR NAMELIST OPT
C
      ICOUNT=0
      NUMTHT=1
      THTST=75.0
      THTEND=400.
      NUMREC=1
      WST=8.0
      WEND=26.0
      DO 120 I=1,20
      Y2(I)=0.
120   CONTINUE
      NUMHTW=1
      HTWST=1.0
      HTWEND=1.0
         IF (IREC.NE.0) GO TO 130
      HTW = H/W
      RYTRX=1.0
      RX2TRX=1.0
      RX3TRX=1.0
      RX4TRX=1.0
130   CONTINUE
      NUMOPT=20
      POPTMN=2.E+07
      POPTMX=4.E+08
      SMULT=1.5
      IOPTUM=1
      NLAND=0
      IHOPTP=0
      IALL=0
      DO 140 JJ=1,5
      CLN(JJ)=0.
      CLE(JJ)=0.
      ALP(JJ)=0.
      SLNS(JJ)=0.
      SLEW(JJ)=0.
140   CONTINUE
      NUMPOS=1
      YTPST=0.
      YTPEND=0.
      XTPST=0.
      XTPEND=0.
      DO 150 I=1,20
150   IPLFL(I)=0
      IPROPT=-1
      IOTAPE=0
      IRERUN=0
      ISTR=0
      NSTR=1
      ITYPE = 0
C
C          CONVERT FROM UNITS OF THT TO REGULAR UNITS
C
      DO 155 K = 1,NRAD
      RAD = RADIUS(K)*THT
      DO 155 L = 1,NAZM
      RSEP = DENSMS/DENSIT(K,L)/AZMSEP(K,L)
      SAVE = 1. - RSEP*HM/(2.*RAD)
      AZMSER(K,L) = AZMSEP(K,L)*SAVE
      DENSIR(K,L) = DENSIT(K,L)/SAVE
      AZMTR(K,L) = AZMSEP(K,L)
      DENTR(K,L) = DENSIT(K,L)
  155 CONTINUE
      THTS = THT
160   CONTINUE
c     call ddi(REC,5,1,1)
      READ (5,NML=REC)
      IF((IREC.EQ.1.OR.IREC.EQ.2).AND.H.EQ.16.0) H=1.1*RY(1)
      IF((IREC.EQ.1.OR.IREC.EQ.2).AND.H.GT.(2.0*RY(1)))
     1WRITE (6,*) ' WARNING****H MAY BE LIMITED BY FIELD GEOM'
C     1ST CORR MOVED THTS HERE.  2ND MOVED BACK. 3RD-IGNORE TOWL,TOWD
      WRITE (6,9876) '$REC'
 9876 FORMAT(1H1,A8)
C mjw     PRINT REC
         IF ( W .LT. 0.0 ) GO TO 1720
      DIAMF=W
      IF (IREC.GT.2) IRADFL=1
      IF (IREC.GT.0) GO TO 165
      HTW=H/W
      RYTRX=1.0
      RX2TRX=1.0
      RX3TRX=1.0
      RX4TRX=1.0
 165  CONTINUE
      IF (IREC.NE.1.AND.IREC.NE.2) GO TO 170
      AREF=761.0
      REFTHP=2.10E+08
      REFRC2=7631.0
      REFRC3=5077.0
 170  CONTINUE
         IF (IREC.EQ.0) GO TO 180
      CALL SETCAV
      RYTRX=RY(1)/RX(1)
      IF (NUMCAV.GT.1) RX2TRX=RX(2)/RX(1)
      IF (NUMCAV.GT.2) RX3TRX=RX(3)/RX(1)
      IF (NUMCAV.GT.3) RX4TRX=RX(4)/RX(1)
      HTW=W/RX(1)
180   CONTINUE
      DO 190 I=2,4
      FLXLIM(I)=1.0E+08
190   CONTINUE
      FLXLIM(1)=0.6E+06
      NFLXMX=1
      IHOPT = 0
c     call ddi(OPT,5,1,1)
      READ (5,NML=OPT)
C>>>>I
      IF ( IRERUN .EQ. 1 ) IOTAPE = 1
      WRITE (6,9876) '$OPT'
C mjw     PRINT OPT
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 200
C
C             DEFAULT VALUE OF FZMIN,FZMAX AT CENTER OF
C             MAXIMUM POSSIBLE HEAT ABSORBING HEIGHT
C             FOR CAVITY RECEIVERS
C
      W2=0.5*W/THT
      RYREC2=0.5*RY(1)/THT
      ANG=180.-RELV(1)
      HTP=(1.+RYREC2*SIN(ANG*RR))*(RADMIN-W2*(1.-RWCAV(1)))/(RADMIN-W2-R
     1YREC2*COS(ANG*RR))
      HBT=(1.-RYREC2*SIN(ANG*RR))*(RADMAX-W2*(1.-RWCAV(1)))/(RADMAX-W2+R
     1YREC2*COS(ANG*RR))
      HTP=HBT+H/THT
      HCTR=0.5*(HTP+HBT)*THT-THT
      FZMIN=HCTR
      FZMAX=HCTR
200   CONTINUE
c     call ddi(NLFLUX,5,1,1)
      READ (5,NML=NLFLUX)
      WRITE (6,9876) '$NLFLUX'
C mjw     PRINT NLFLUX
      NFLXPT=NXFLX*NYFLX
         IF (IFLAUT*IFLX.NE.4) GO TO 240
C
C             FLUX POINTS SET TO HEAT ABSORBING SURFACE
C
         IF (IREC.GT.0) GO TO 210
      DIAMF=W
      IFLAUT=1
      XFC=0.
      YFC=0.
      ZFC=0.
         GO TO 240
210      IF (IREC.GT.2) GO TO 220
      DIAMF=W*RWCAV(1)
      IFLAUT=2
         GO TO 230
220   IFLAUT=3
      POLF=RELV(1)
      AZMF=RAZM(1)
      IF (FAZMIN.GT.100) FAZMIN=0.
      IF (FAZMAX.GT.100.) FAZMAX=0.
230   ZFC=0.
      XFC=-W/2.*SIN(RR*RAZM(1))
      YFC=-W/2.*COS(RR*RAZM(1))
240   CONTINUE
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 250
C
C             CALCULATE RATIOS FOR SCALING FLUX POINTS ON
C             FLUX SURFACE  (RFZMIN,RFZMAX)
C
      FCTR=((W2*(-SIN(RR*RAZM(1)))-XFC/THT)**2
     1 +(W2*(-COS(RR*RAZM(1)))-YFC/THT)**2)**0.5
      HTP=(1.+RYREC2*SIN(RR*ANG))*(RADMIN-W2+FCTR+0.5*DIAMF/THT)/(RADMIN
     1 -W2-RYREC2*COS(RR*ANG))
      HBT=(1.-RYREC2*SIN(RR*ANG))*(RADMAX-W2+FCTR+0.5*DIAMF/THT)/(RADMAX
     1 -W2+RYREC2*COS(RR*ANG))
      HTP=HBT+H/THT
      THCAV=(HTP-HBT)*THT
      IF(IFLAUT.NE.2) GOTO 250
      FZMIN=FZMIN+(1.0-HBT)*THT
      FZMAX=FZMAX+(1.0-HBT)*THT
      RFZMIN=FZMIN/THCAV
      RFZMAX=FZMAX/THCAV
      RZFC=(ZFC+(1.-HBT)*THT)/THCAV
250   CONTINUE
      DO 260 K=1,NRAD
      DO 260 L=1,NAZM2P
      DO 260 II=1,4
      FLUXMB(K,L,II)=0.
260   CONTINUE
c     call ddi(NLEFF,5,1,1)
      READ (5,NML=NLEFF)
      WRITE (6,9876) '$NLEFF'
C mjw     PRINT NLEFF
c     call ddi(NLCOST,5,1,1)
      READ (5,NML=NLCOST)
      IF(ICKN.EQ.0) CKNREF=0.0
      WRITE (6,9876) '$NLCOST'
C mjw     PRINT NLCOST
c     call ddi(NLECON,5,1,1)
      READ (5,NML=NLECON)
      WRITE (6,9876) '$NLECON'
C mjw     PRINT NLECON
         IF (IPH.EQ.0) GO TO 270
      ITHEL=0
      ETAREF=1.0
      FEFF=1.0
      REFPRL=0.0
          PARL1=0.0
       PARL2=0.0
       PARL3=0.0
       PARL4=0.0
       PARL5=0.0
       PARL6=0.0
       PARL7=0.0
       PARL8=0.0
       PARL9=0.0
       PARL10=0.0
      CEGREF=0.0
C>>>>RIPROP2=0
  270 CONTINUE
C>>>>DIF (IPROPT.NE.0) IPROP2=1
C>>>>DIF (IPROPT.LT.1) IPROPT=0
      CRPU=CRPREF/(PRPREF*TRPREF*SMRP)**XRP
      CSPU=CSPREF/PSPREF**XSP
C
C             ECONOMIC PARAMETER CALCULATION - INPUT TO MAX FOR
C             LEVELIZED ENERGY COST CALCULATION
C
C             CONVERSION FACTORS FOR DIRECT COSTS TO TOTAL COSTS,
C             ESCALATED AND DEFLATED TO CURRENT DOLLARS
C
      CCMUL=1.0+CONT+SPTS+EXT
      CCMULY=CCMUL*(1.0+AFDC)*(1.0+ESC)**NYTCON
      CCMULC=CCMULY/(1.0+RINF)**NYTCON
C
C             DISCOUNTING FACTOR
C
         IF ( IFCR .NE. 0 ) DISRT = (1. - TR)*FDEBT*RDEBT + (1. -
     1          FDEBT)*ROE
      DISF=(1.0-(1.0+DISRT)**(-NYOP))/DISRT
         IF (IFCR.EQ.0) GO TO 320
C
C             DEPRECIATION CALCULATION
C
      DEP=0.0
C
C               STRAIGHT LINE SCHEDULE
C
      DEPI = 1./FLOAT(NDEP)
      DO 310 I=1,NDEP
         IF (IDEP.NE.2) GO TO 300
C
C             SUM-OF-YEARS DIGITS SCHEDULE
C
      DEPI=2.0*FLOAT(NDEP-I+1)/FLOAT(NDEP*(NDEP+1))
300   DEPI=DEPI/(1.0+DISRT)**I
      DEP=DEP+DEPI
310   CONTINUE
C
C             FIXED CHARGE RATE CALCULATION
C
      FCR=PTI+(1.0-TC)/(1.0-TR)/DISF-TR*DEP/(1.0-TR)/DISF
C
C             LEVELIZED O+M FACTORS
C
320   R=(1.0+RINF)/(1.0+DISRT)
      DFOM=(1.0-(1.0/R)**(-NYOP))/(1.0/R-1.0)
      RHOML=RHOM*DFOM/DISF
      RNHOML=RNHOM*DFOM/DISF
      POPT(1)=POPTMN
      ALLBUA(1)=1.E+06
      DELP=0.
      IF (NUMOPT.GT.1) DELP=(POPTMX-POPTMN)/FLOAT(NUMOPT-1)
      DO 330 I=2,NUMOPT
      POPT(I)=POPT(I-1)+DELP
      ALLBUA(I)=1.E+06
330   CONTINUE
      POPT(NUMOPT+1)=1.E+20
      PMIN = POPTMN/1.E+06
      PMAX = POPTMX/1.E+06
      DTHT=0.
         IF (NUMHTW.GT.1) GO TO 340
      IF (IREC.EQ.0) HTWST=HTW
      IF (IREC.GT.0.AND.IOPTUM.EQ.1) HTWST=RYTRX
      IF (IREC.GT.0.AND.IOPTUM.EQ.2) HTWST=W
      IF (IREC.GT.0.AND.IOPTUM.EQ.3) HTWST=RELV(1)
340   CONTINUE
      DHTW=1.
C
C             STEP SIZES IN ITERATIONS
C
C>>>>DMOPT=1
      IF (NUMHTW.GT.1) DHTW=(HTWEND-HTWST)/FLOAT(NUMHTW-1)
      IF (NUMREC.EQ.1.AND.IREC.EQ.0) WST=W
      IF (NUMREC.EQ.1.AND.IREC.NE.0) WST=RX(1)
      IF (NUMTHT.EQ.1) THTST=THT
      IF (NUMTHT.GT.1) DTHT=-(THTST-THTEND)/FLOAT(NUMTHT-1)
      DPOS=0.
      IF (NUMPOS.GT.1) DPOS=(YTPEND-YTPST)/FLOAT(NUMPOS-1)
      DPOSX=0.
      IF (NUMPOS.GT.1) DPOSX=(XTPEND-XTPST)/FLOAT(NUMPOS-1)
C
C             PRINT OUT THE OPTIMIZATION VARIABLES
C
      WRITE (6,1800)
         IF (IALL.EQ.0) WRITE (6,1810)
         IF (IALL.EQ.1) WRITE (6,1820)
         IF (IHOPT.EQ.0) WRITE (6,1830)
         IF (IHOPT.EQ.1) WRITE (6,1840)
      WRITE (6,1850) NUMTHT
      WRITE (6,1860) THTST
         IF (NUMTHT.GT.1) WRITE (6,1870) THTEND
         IF (IHOPT*NUMTHT.LT.2) GO TO 350
      WRITE (6,1880)
      W = - 96
        GO TO 1720
350   CONTINUE
         IF (IREC.GT.0) GO TO 360
      WRITE (6,1890)
      WRITE (6,1900) NUMREC
      WRITE (6,1860) WST
         IF (NUMREC.GT.1) WRITE (6,1870) WEND
      WRITE (6,1910) NUMHTW
         GO TO 430
360      IF (IREC.GT.1) GO TO 370
      WRITE (6,1920) NUMCAV
         GO TO 400
370      IF (IREC.GT.2) GO TO 380
      WRITE (6,1930) NUMCAV
         GO TO 400
380      IF (IREC.GT.3) GO TO 390
      WRITE (6,1940) NUMCAV
         GO TO 400
390   WRITE (6,1950) NUMCAV
400   CONTINUE
      WRITE (6,1960) NUMREC
      WRITE (6,1860) WST
         IF (NUMREC.GT.1) WRITE (6,1870) WEND
         IF (IOPTUM.EQ.2) GO TO 410
      WRITE (6,1970) NUMHTW
         GO TO 430
410   CONTINUE
      WRITE (6,1980) NUMHTW
      DO 420 I=1,NUMCAV
      WRITE (6,1990) I,RWCAV(I)
420   CONTINUE
  430 CONTINUE
      WRITE (6,1860) HTWST
         IF (NUMHTW.GT.1) WRITE (6,1870) HTWEND
      WRITE (6,2000) NUMOPT
      WRITE (6,1860) PMIN
         IF (NUMOPT.GT.1) WRITE (6,1870) PMAX
         IF (NLAND.EQ.0) GO TO 450
      WRITE (6,2010) NUMPOS
      WRITE (6,2020) YTPST,XTPST
         IF (NUMPOS.GT.1) WRITE (6,2030) YTPEND,XTPEND
450   CONTINUE
      WRITE (6,2040)
         IF (IFLX.EQ.0) GO TO 490
      DO 460 II=1,NFLXMX
      WRITE (6,2050) II,FLXLIM(II)
460   CONTINUE
      WRITE (6,2060)
      SAVE=XFC/W
      SAVE2=YFC/W
      SAVE3=ZFC/W
      WRITE (6,2070) SAVE,SAVE2,SAVE3
         IF (IFLAUT.EQ.3) GO TO 480
      SAVE = DIAMF/W
         IF (IFLAUT.EQ.2) GO TO 470
      WRITE (6,2080) SAVE
         GO TO 475
470   CONTINUE
      WRITE (6,2100) SAVE
  475 CONTINUE
      WRITE (6,2090)
         GO TO 490
480   CONTINUE
      WRITE (6,2110)
490   CONTINUE
         IF (NLAND.EQ.0) GO TO 510
      WRITE (6,2120) NLAND
      DO 500 N=1,NLAND
      WRITE (6,2130) N,CLE(N),CLN(N)
      WRITE (6,2140) N,ALP(N)
      WRITE (6,2150) SLEW(N),SLNS(N)
500   CONTINUE
510   CONTINUE
      WRITE (6,2160)
      DO 520 I=1,5
      DO 520 J=1,NAZM2P
      ILLNR(I,J)=0
520   ILLRMI(I,J)=0
      J=1
      DO 530 I=1,NUMOPT
         IF (IPLFL(I).EQ.0) GO TO 530
      IPLFL(I)=J
      J=J+1
530   CONTINUE
      DO 540 I=1,NUMTHT
      DO 540 J=1,NUMREC
540   ISET(I,J)=0
      DO 550 I=1,20
      IOPTRE(I)=NUMREC
550   IOPTTH(I)=NUMTHT
      THTI = THTST-DTHT
      DW=0.
      IF (NUMREC.GT.1) DW=(WEND-WST)/FLOAT(NUMREC-1)
      IOPT3=0
      HMINS=0.
      HTWM2=0.
      MINHTW=1
C
C             ITERATE OVER TOWER HEIGHT
C
      IOPT6=0
      IOPT20=1
      HI = H
      ATM1N = ATM1S
      RXI(1) = RX(1)
      RYI(1) = RY(1)
      RYN(1) = RY(1)
C**********************************************************************
C               START OF OPTIMIZATION LOOP
      DO 1290 III=1,NUMTHT
      MTHT=III
      THTI=THTI+DTHT
      THT2=THTI**2
      THTSAV(MTHT) = THTI
C
C               NORMALIZE DISTANCES TO UNITS OF NEW TOWER HEIGHT
C
      WMN = WM/THTI
      HMN = HM/THTI
      ASTATN = ASTAT/THT2
      ATM2N = ATM2S*THTI
      ATM3N = ATM3S*THT2
      ATM4N = ATM4S*THTI**3
      IF (IOPT3.EQ.0) MOPT=1
         IF (MOPT.GT.NUMREC) GO TO 1300
560   CONTINUE
      IOPT1=0
      IOPT7=0
      IOPT21=NUMOPT+1
      DO 570 IJJ=1,NUMOPT
      BBW(IJJ)=1.E+10
570   BBWL(IJJ)=1.E+10
      IOPTLW=0
      PXW=0.
      PMXWL=0.
      PMXW=0.
C
C             ITERATE FIRST RECEIVER VARIABLE
C
      IOPT2=0
      IF (MOPT.GT.1) MOPT=MOPT-1
      DO 1260 JJJ=MOPT,NUMREC
      DO 580 IJJ=1,NUMOPT
      BBHTW(IJJ)=1.E+10
580   BBHTWL(IJJ)=1.E+10
      PXHTW=0.
      PMXHL=0.
      PMXH=0.
      IOPTL=0
      MREC=JJJ
      WI = WST+FLOAT(MREC-1)*DW
      RECSAV(MREC)=WI
      WSAVE=WI
         IF (IREC.NE.0) RXI(1) = WI
      IF (IOPTUM.EQ.1) HMINS=HMINS/WI**2
      IF (IOPT2.EQ.1) MINHTW=INT((HMINS-HTWST-1.E-6)/DHTW)+1
C     ABOVE LINE 8-15-85 TO ALWAYS TRY 1 SIZE SMALLER
      HMINS=1.E+10
      IOPT2=0
      IF (IOPTUM.EQ.1) HTWM2=HTWM2/WI**2
      MIN2=INT((HTWM2-HTWST)/DHTW)+1
      IF (MINHTW.LT.MIN2) MINHTW=MIN2
      IOPT40=0
      HTWM2=0.
      IF (MINHTW.LT.1.OR.JJJ.EQ.MOPT) MINHTW=1
         IF (IALL.EQ.1) MINHTW=1
C
C             ITERATE OVER THE SECOND RECEIVER VARIALBLE
C
      DO 1200 KKK=MINHTW,NUMHTW
      MHTW=KKK
      HTW=HTWST+FLOAT(MHTW-1)*DHTW
      HTWSAV(MHTW)=HTW
C>>>>DIOPT5=0
         IF (IREC.NE.0) GO TO 600
      HI = WI*HTW
      HSAVE=HI
         GO TO 660
600   CONTINUE
         IF (IOPTUM.EQ.2) GO TO 610
      RYTRX=HTW
      HSAVE=RXI(1)*RYTRX
      WI = W
         GO TO 620
610   CONTINUE
      WI = HTW
      HSAVE=WI
620   CONTINUE
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 621
         IF (RXI(1).GT.WI*RWCAV(1)) GO TO 1200
621   CONTINUE
         IF (NUMCAV.LT.2) GO TO 630
      RXI(2)=RXI(1)*RX2TRX
         IF (NUMCAV.LT.3) GO TO 630
      RXI(3)=RXI(1)*RX3TRX
         IF (NUMCAV.LT.4) GO TO 630
      RXI(4)=RXI(1)*RX4TRX
630   CONTINUE
      DO 650 JJ=1,NUMCAV
      RYI(JJ) = RXI(JJ)*RYTRX
      RXN(JJ)=RXI(JJ)/2./THTI
      RYN(JJ)=RXN(JJ)*RYTRX
      RA2(JJ)=RXN(JJ)**2
      RB2(JJ)=RYN(JJ)**2
650   CONTINUE
660   CONTINUE
      RREFL=RMIRL*RRECL
C
C             SCALE DENSITY OF RADIAL LAYOUTS WITH TOWER HEIGHT
C
C             CORRECT FOR MISSING HELIOSTATS IN SLIP PLANES (RADIAL)
C
         IF (ILAY.GT.1) GO TO 690
      DO 680 K=1,NRAD
      RAD=RADIUS(K)*THTI
      DO 680 L=1,NAZM
      RSEP=DENSMS/DENSIR(K,L)/AZMSER(K,L)
      SAVE=1.0-RSEP*HM/(2.0*RAD)
      AZMTR(K,L) = AZMSER(K,L)/SAVE
      DENTR(K,L) = DENSIR(K,L)*SAVE
      RSEP=DENSMS/DENTR(K,L)/AZMTR(K,L)
      SAVE=RAD*(FSLIP-1.)/(RSEP*HM*(1.+FSLIP))
      REALD(K,L)=(2.*SAVE+1./FSLIP)/(2.*SAVE+1.)
      DAREA(K,L)=DAREA(K,L)*REALD(K,L)
680   CONTINUE
690   CONTINUE
C
C             NORMALIZE DISTANCES TO UNITS OF TOWER HEIGHT
C
      WN = WI/THTI
      HN = HI/THTI
      W2=WN/2.
         IF (IFLX.EQ.0) GO TO 730
C
C             SET UP FOR FLUX CALCULATION - SCALE LOCATION
C             OF FLUX POINTS ACCORDING TO RECEIVER TYPE
C
      SAVE3=1.
      SAVE=WI/W
      SAVE2=HI/H
C
C             SCALE FAZMIN, FAZMAX WITH RECEIVER WIDTH
C             IF EXTERNAL OR CAVITY RECEIVER (IREC.LT.3) HAS
C             FLAT PLATE FLUX SURFACE (IFLAUT.EQ.3)
C
      IF (IFLAUT.EQ.3) SAVE3=SAVE
C CHANGE 10/12/87 KISTLER ERROR FIX- REMOVED .5 FACTOR
C FROM FOLLOWING TWO EQUATIONS
      IF (IREC.GT.2) SAVE3=(RXI(1))/RX(1)
      IF (IREC.GT.2) SAVE2=(RYI(1))/RY(1)
      XFCN = XFC*SAVE/THTI
      YFCN = YFC*SAVE/THTI
      ZFCN = ZFC*SAVE/THTI
      FAZMIS = FAZMIN*SAVE3
      FAZMAS = FAZMAX*SAVE3
      DIAMFN =DIAMF*SAVE/THTI
      FZMINN = FZMIN*SAVE2/THTI
      FZMAXN = FZMAX*SAVE2/THTI
         IF (IFLAUT.NE.2.AND.IREC.NE.1.AND.IREC.NE.2) GO TO 721
      FCTR=SQRT((W2*(-SIN(RR*RAZM(1)))-XFCN)**2+
     1(W2*(-COS(RR*RAZM(1)))-YFCN)**2)
      THTP=(1.+RYN(1)*SIN(RR*ANG))*(RADMIN-W2+FCTR+0.5*DIAMFN)/(RADMIN
     1 -W2-RYN(1)*COS(RR*ANG))
      THBT=(1.-RYN(1)*SIN(RR*ANG))*(RADMAX-W2+FCTR+0.5*DIAMFN)/(RADMAX
     1 -W2+RYN(1)*COS(RR*ANG))
      THTP1=THBT+H/THTI*RYI(1)/RY(1)
      IF(THTP1.GT.THTP) WRITE (6,*) 'WARNING**DELSOL HCAV USED'
      IF(THTP1.LT.THTP) THTP=THTP1
      HN=H*2.0/RY(1)
C     ABOVE DEFINITION FOR CAV FOR USE IN MAX--SCALING
      THCAV=THTP-THBT
       IF(IFLAUT.NE.2) GO TO 721
      FZMINN=RFZMIN*THCAV
      FZMAXN=RFZMAX*THCAV
C     ZFC=RZFC*THCAV+(HBT-1.)*THT
      ZFCN=THBT-1.0
721   CONTINUE
      CALL SETFLX
730   CONTINUE
C
C             CALCULATE HELIOSTAT, LAND AND WIRE COSTS
C
C>>>>RHL=CL/DENSMR
      HL = CL/DENSMS
      SAVE=DENSMS*ASTAT
      HWDA=CWDA*WM/SAVE
      HWDR=CWDR*HM/SAVE
      HWR=CWR*THTI/SAVE
      DO 740 K=1,NRAD
      RAD = RADIUS(K)*THTI
      DO 740 L=1,NAZM2P
      RSPC=DENSMS*ASTATN/(AZMTR(K,L)*DENTR(K,L))*THTI/WMN
      CW=CWR*RAD+CWDA*AZMTR(K,L)*WM+CWDR*RSPC
      HCOST(K,L)=CH+CL/DENTR(K,L)/REALD(K,L)+CW/SAVE
740   CONTINUE
      HCOST1=HCOST(1,1)
      DO 750 K=1,NRAD
      DO 750 L=1,NAZM2P
      HCOST(K,L)=HCOST(K,L)/HCOST1
750   CONTINUE
C
C             CALCULATE TOWER AND RECEIVER COSTS
C
      AREC=PI*WI*HI
         IF (IREC.LT.1) GO TO 770
      AREC=0.
      DO 760 JJ=1,NUMCAV
760   AREC=AREC+RXI(1)*RYI(1)
         IF (IREC.NE.4) AREC=AREC*PI/4.
770   CONTINUE
      CREC=CREC1*(AREC/ARECRF)**XREC
      SAVE=HN
      IF (IREC.GT.0) SAVE=RYN(1)*SIN(RELV(1)*RR)*2.
C     TOWER HEIGHT ADJUSTED FOR TRANSITION REGION 2-27-85
      SAVE=THTI*(1.+HMN/2.-SAVE/2.-WN)
      CTOW=CTOW1*EXP(CTOW2*SAVE)
C     CTOW=CTOW1+CTOW2*SAVE+CTOW3*SAVE**XTOW
         IF (ITHT.NE.0) GO TO 780
      IF (SAVE.LT.120.0) CTOW=CTOW3*EXP(XTOW*SAVE)
C     IF (THTI.LT.120) CTOW=162.92*SAVE**2
780    CONTINUE
C
C             AREA OF RECEIVER FOR RADIATION/CONVECTION
C
         IF (IREC.NE.2) GO TO 800
      ARECRC=0.0
      DO 790 I=1,NUMCAV
      ARECRC=ARECRC+RXI(I)*RYI(I)
790   CONTINUE
         GO TO 810
800   ARECRC=AREC
810   CONTINUE
      IF(IRADFL.EQ.0.AND.ICAV.GT.1)
     1WRITE (6,*) ' ERROR***CAN.T USE NEW LOSS WITH MULT CAVS'
      IF(IRADFL.EQ.0.AND.ICAV.GT.1) IRADFL=1
      IF(IRADFL.EQ.0.AND.(IREC.EQ.1.OR.IREC.EQ.2))
     1AREC=THCAV*RR*180.0*W2*RWCAV(1)*THT2
C
C             SCALE PERFORMANCE TO NEW TOWER AND RECEIVER
C
      DO 900 K=1,NRAD
      SAVE=SQRT(1.+(RADIUS(K)-W2)**2)
      STP=(RADIUS(K)-W2)/SAVE
      CTP=1./SAVE
      DO 900 L=1,NAZM2P
      CTA=CTAANG(L)
      STA=STAANG(L)
      IPAK=0
      IPAK2=0
      DO 830 I=1,NORDER
      JMIN=JMN(I)
      JMAX=JMX(I)
      DO 830 J=JMIN,JMAX,ISTEP
      IPAK=IPAK+1
      UF(I,J)=0.
      NN=NUMK(IPAK)
      DO 830 M=1,NN
      IPAK2=IPAK2+1
      UF(I,J)=UF(I,J)+HCS(K,L,IPAK2)*(THTS/THTI)**(2*M-2)
830   CONTINUE
      KK=K
      LL=L
C
C             DO NOT DO SPILLAGE FOR DUMMY ZONE IN NORTH ONLY FIELD
C
      SPL=0.
         IF (INORTH*L.NE.NAZM2P) CALL HERMIT
      FL = SPL+1.E-08
      YSPILL(K,L)=FL
      YATMOS(K,L)=1.0-ATM1N-ATM2N*RANGE(K)-ATM3N*RANGE(K)**2-ATM4N*
     1  RANGE(K)**3
      DO 850 J=1,NHREF
      HREFIN(J,K,L)=HREFIN(J,K,L)*YSPILL(K,L)*YATMOS(K,L)
850   CONTINUE
      YTOTAL(K,L)=YCOSI(K,L)*YSHAD(K,L)*YBLOCK(K,L)*YATMOS(K,L)*FL*RREFL
C>>>>DLL=2*NAZM2P-L
C>>>>RIF (LL.GT.NAZM) LL=1
      LL = MZONE(L)
      REFTOT(K,L)=(REFFLU(K,L)+REFFLU(K,LL))/2.*FL*YATMOS(K,L)
         IF (IFLX.EQ.0.OR.INORTH*L.EQ.NAZM2P) GO TO 900
C
C             CALCULATE FLUX DENSITY
C
      CALL FLXMAP (K,L)
      CON1=REFSOL*REFFLU(K,L)*YATMOS(K,L)*1000./RRECL/THT2/WTAREA(L)
      DO 860 JJ=1,NFLXMX
      II=NMXFLX(JJ)
860   FLUXMB(K,L,JJ)=FLX(II)*CON1
C>>>>R   IF (WTAREA(L).LT.2.0) GO TO 900
         IF ( LL .LE. MZONE(NAZM2P) ) GO TO 900
C>>>>DLL=2*NAZM2P-L
      STA=-STA
      CON1=CON1*REFFLU(K,LL)/REFFLU(K,L)
      DO 870 II=1,16
870   HC(II)=HC(II)*SIPAK(II)
      CALL FLXMAP (K,LL)
      DO 880 JJ=1,NFLXMX
      II=NMXFLX(JJ)
880   FLUXMB(K,L,JJ)=FLX(II)*CON1+FLUXMB(K,L,JJ)
900   CONTINUE
      WRITE (6,2170) THTI,WSAVE,HSAVE
C>>>>R   IF (IPROPT.EQ.0) GO TO 920
         IF ( IPROPT .NE. 1 ) GO TO 920
      WRITE (6,1780)
C
C             PRINT EFFICIENCIES
C
      WRITE (6,1790)
      WRITE (6,1730) HEAD
      WRITE (6,1790)
      WRITE (6,1770)
      WRITE (6,1750) (AZM(L),L=1,NAZM2P)
      WRITE (6,1760)
      DO 910 K=1,NRAD
      WRITE (6,1740) RADIUS(K),HEAD1,(YCOSI(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD2,(YSPILL(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD3,(YATMOS(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD11,(YSHAD(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD12,(YBLOCK(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD4,(YTOTAL(K,L),L=1,NAZM2P)
      WRITE (6,1740) RADIUS(K),HEAD17,(HCOST(K,L),L=1,NAZM2P)
      WRITE (6,1760)
910   CONTINUE
920   CONTINUE
C
C             ITERATE THE TOWER POSITION (LAND CONSTRAINT ONLY)
C
C>>>4I
      DO 925 L = 1,NAZM
      DO 925 K = 1,NRAD
      FLAND(K,L) = 1.
  925 CONTINUE
      IOPT9=0
      NVALH=1000000
      ISKIP=0
      DO 1010 JKL=1,NUMPOS
      IPOS=JKL
      IOPT8=0
      YTOWES = YTPST+FLOAT(IPOS-1)*DPOS
      XTOWES = XTPST+FLOAT(IPOS-1)*DPOSX
C>>>>RDO 930 L=1,NAZM
      DO 930 L = 1,NAZM2P
      DO 930 K=1,NRAD
C>>>>I
      DAREAF(K,L) = DAREA(K,L)
C>>>>RFLAND(K,L)=1.0
  930 CONTINUE
      FLTOT=1.
C>>>2RIF (NLAND.GT.0) CALL LAND
         IF ( NLAND .EQ. 0 ) GO TO 945
      CALL LAND
C>>>>RDO 940 L=1,NAZM
      DO 940 L = 1,NAZM2P
      DO 940 K=1,NRAD
C>>>>DSAVE=DAREA(K,L)
C>>>2RDAREA(K,L)=DAREA(K,L)*FLAND(K,L)
      LL = MZONE(L)
      DAREAF(K,L) = DAREA(K,L)*((FLAND(K,L) + FLAND(K,LL))/2.)
C>>>>DFLAND(K,L)=SAVE
940   CONTINUE
C>>>>I
  945 CONTINUE
      IF (IHOPT.GT.0) CALL HVALUE
C
C              HELIOSTAT DENSITY OPTIMIZATION SET UP
C
         IF (FLTOT.LT.1.E-06) GO TO 1010
      DO 950 K=1,NRAD
      DO 950 L=1,NAZM2P
950   HCOST(K,L)=YTOTAL(K,L)/HCOST(K,L)
C
C             BUILD UP OPTIMAL FIELD
C
      IOPT30=1
C     DEFAULT 8-15-85 (MATCHES DEFAULT IN SUBROU MAX)
      CALL MAX
C>>>>D   IF (IHOPT.EQ.0) GO TO 990
C>>>>DDO 980 M=1,NMAX
C>>>>DK=KMAX(M)
C>>>>DL=LMAX(M)
C>>>>DYCOSI(K,L)=DCOS(M)
C>>>>DYSHAD(K,L)=DSHD(M)
C>>>>DYBLOCK(K,L)=DBLK(M)
C>>>>DYATMOS(K,L)=DATM(M)
C>>>>DYSPILL(K,L)=DSPL(M)
C>>>>DYTOTAL(K,L)=DTOT(M)
C>>>>DHCOST(K,L)=HCST(M)
C>>>>DDO 960 I=1,NFLXMX
C>>>>DFLUXMB(K,L,I)=FLXM(I,M)
C>>>>DDO 970 I=1,NHREF
C>>>>DHREFIN(I,K,L)=HREFS(I,M)
C>>>>DCONTINUE
C>>>>DCONTINUE
      DO 1000 L=1,NAZM2P
      DO 1000 K=1,NRAD
C>>>>DDAREA(K,L)=FLAND(K,L)
1000  HCOST(K,L)=YTOTAL(K,L)/HCOST(K,L)
      ISKIP=1
         IF (IOPT9.EQ.1.AND.IOPT8.EQ.0) GO TO 1020
1010  CONTINUE
1020  CONTINUE
         IF (ILAY.GT.1) GO TO 1050
      DO 1040 K=1,NRAD
      DO 1040 L=1,NAZM
      DAREA(K,L)=DAREA(K,L)/REALD(K,L)
1040  CONTINUE
1050  CONTINUE
      DO 1070 J=1,NHREF
      DO 1070 K=1,NRAD
      DO 1070 L=1,NAZM2P
      HREFIN(J,K,L)=HREFIN(J,K,L)/(YSPILL(K,L)*YATMOS(K,L))
1070  CONTINUE
      IF (IOPT40.GT.0.OR.IOPT30.EQ.0.OR.PMXH.LT.PMXHL) GO TO 1090
C     ABOVE LINE 8-15-85 IF POWER DECR. AS SIZE INCR, DONT LIMIT SIZE
C
C             THIS RECEIVER COULD NOT MEET MINIMUN POWER OF INTEREST
C             DO NOT USE RECEIVERS OF SMALLER AREA IN REST OF SEARCH
C
         IF (IREC.EQ.0) GO TO 1080
      IF (IOPTUM.EQ.1) HTWM2=RXI(1)*RYI(1)/4.
      IF (IOPTUM.EQ.2) HTWM2=WI
         GO TO 1090
1080  HTWM2=HI*WI
1090  CONTINUE
C>>>>R   IF (IOPT5.EQ.0) GO TO 1120
         IF ( IOPT9 .EQ. 0 ) GO TO 1120
         IF ( IREC .EQ. 0 ) GO TO 1110
C
C             THIS RECEIVER SET A NEW MINIMUM ENERGY COST
C             TEST TO SEE IF THIS IS SMALLEST REC AREA TO SET NEW MIN
C             IF IT IS, DO NOT USE SMALLER RECEIVERS IN REST OF SEARCH
C     EXCEPT(8-15-85) TRY ONE SIZE SMALLER, AND DONT USE FOR IOPTUM=2
C     SINCE WI IS DEPTH OF CAV AND OPTIMIZATION DOESNT DO RIGHT
C
         IF (IOPTUM.LT.2) GO TO 1100
      HMINS=HTWST
C     ABOVE LINE 8-15-85 DONT LIMIT CAVITY DEPTH SMALLER
         GO TO 1120
1100  CONTINUE
      IF (RYI(1)*RXI(1).LT.HMINS) HMINS=RYI(1)*RXI(1)
         GO TO 1120
1110  CONTINUE
      IF (HI*WI.LT.HMINS) HMINS=HI*WI
1120  CONTINUE
         IF (IALL.EQ.1) GO TO 1200
C
C             IF THE FIRST RECEIVER IS OPTIMUM TRY SMALLER SIZE
C
         IF (MOPT.EQ.1.OR.JJJ.GT.MOPT.OR.IOPT1.EQ.0) GO TO 1130
         GO TO 560
1130  CONTINUE
      IF (PXHTW.GT.PXW) PXW=PXHTW
      IF (KKK.EQ.1) PXW=PXHTW
      IF (KKK.EQ.1.OR.PMXH.GT.PMXW) PMXW=PMXH
      IF (IOPTL.GT.IOPTLW) IOPTLW=IOPTL
         IF (NUMHTW.EQ.1) GO TO 1170
C
C             IF CANT REACH MINIMUM POWER, STOP AFTER 2 ITERATIONS
C
         IF (PXHTW.LT.POPT(IOPT20)) GO TO 1210
      IF(IOPTL.EQ.0.AND.IREC.GT.0.AND.IOPTUM.EQ.2.AND.IOPT30.EQ.1)
     1GO TO 1210
C     ABOVE 2 LINES 8-15-85 IF CAV DEPTH OPT POWER BELOW MIN, 'STOP'
         IF (IOPTL.EQ.0) GO TO 1200
C
C             IF A FLUX LIMIT WAS REACHED CONTINUE ITERATION
C
         IF (IOPT30.EQ.0) GO TO 1150
C
C             IF THE ENERGY COST IS DECREASING CONTINUE ITERATION
C
      DO 1140 IJJ=IOPT20,IOPTL
         IF (BBHTW(IJJ).LT.BBHTWL(IJJ)) GO TO 1150
1140  CONTINUE
C
C             IF POWER DECREASED FROM LAST CASE STOP ITERATION
C
         IF (PMXH.LT.PMXHL) GO TO 1210
C
C             IT NEXT POWER LEVEL CANNOT BE REACHED, STOP ITERATION
C
         IF (PXHTW.LT.POPT(IOPTL+1)) GO TO 1210
1150  DO 1160 IJJ=IOPT20,IOPTL
      BBHTWL(IJJ)=BBHTW(IJJ)
      IF (BBHTW(IJJ).LT.BBW(IJJ)) BBW(IJJ)=BBHTW(IJJ)
1160  CONTINUE
      PMXHL=PMXH
         GO TO 1200
1170  CONTINUE
      DO 1180 IJJ=IOPT20,IOPTL
1180  BBW(IJJ)=BBHTW(IJJ)
1200  CONTINUE
1210  CONTINUE
         IF (IALL.EQ.1) GO TO 1260
C
C             IF MINIMUM POWER CANNOT BE MET STOP ITERATION
C
         IF (PXW.LT.POPT(IOPT20)) GO TO 1270
         IF (IOPT30.EQ.0) GO TO 1230
         IF (IOPTLW.EQ.0) GO TO 1260
      DO 1220 IJJ=IOPT20,IOPTLW
         IF (BBW(IJJ).LT.BBWL(IJJ)) GO TO 1230
1220  CONTINUE
         IF (PMXW.LT.PMXWL) GO TO 1270
         IF (PXW.LE.POPT(IOPTLW+1)) GO TO 1270
1230  CONTINUE
         IF (IOPTLW.EQ.0) GO TO 1260
      PMXWL=PMXW
      DO 1240 IJJ=IOPT20,IOPTLW
1240  BBWL(IJJ)=BBW(IJJ)
1260  CONTINUE
1270  CONTINUE
         IF (IALL.EQ.1) GO TO 1290
         IF (IOPT6.EQ.1.AND.IOPT7.EQ.0) GO TO 1300
      IF (IOPT21.LE.NUMOPT) IOPT20=IOPT21
      LL=MOPT
      DO 1280 I=LL,NUMREC
         IF (ISET(MTHT,I).NE.0.OR.IOPT1.EQ.0) GO TO 1290
1280  MOPT=MOPT+1
1290  CONTINUE
C               END OF OPTIMIZATION LOOP
C**********************************************************************
1300  CONTINUE
      WRITE (6,2290)
      WRITE (6,1730) HEAD
      WRITE (6,2180) PMIN,PMAX,SMULT,FLXLIM
      WRITE (6,2190) REFDAY,REFTIM,REFSOL
C
C             CHECK IF ANY OPTIMIZED SYSTEM DESIGNS FOUND
C             DURING OPTIMIZATION
C
      DO 1310 I=1,NUMOPT
      IF (ALLBUA(I).LT.1.E+06) GO TO 1320
1310  CONTINUE
      WRITE (6,2200)
      W = - 95
        GO TO 1720
1320  CONTINUE
         IF (IHOPT.GT.0) WRITE (6,1840)
C
C             PRINT OUT BUSBAR, TOWER, RECEIVER
C
         IF (IREC.GT.0) GO TO 1330
C>>>>RWRITE (6,2250)
      WRITE (6,2210) RECE,DIAM,RECEI,VER
         GO TO 1350
1330     IF (IREC.GT.2) GO TO 1340
C>>>>R   IF (IOPTUM.EQ.1) WRITE (6,2210)
      WRITE (6,2210) APER,EWID,APEREC(IOPTUM),UREWID(IOPTUM)
C>>>>D   IF (IOPTUM.EQ.2) WRITE (6,2220)
         GO TO 1350
C>>>>R   IF (IOPTUM.EQ.1) WRITE (6,2230)
1340  WRITE (6,2210) PLATE,EWID,PREC(IOPTUM),ATEWID(IOPTUM)
C>>>>D   IF (IOPTUM.EQ.2) WRITE (6,2240)
1350  CONTINUE
C>>>>D   IF (IREC.EQ.1.OR.IREC.EQ.2) GO TO 1360
      WRITE (6,2260)
C>>>>D   GO TO 1370
C>>>>DWRITE (6,2270)
C>>>>DCONTINUE
      DO 1400 I=1,NUMOPT
         IF (ALLBUA(I).GT.1.0E+05) GO TO 1400
      II=IOPTTH(I)
      JJ=IOPTRE(I)
      MM=IOPTHT(I)
      SAVE2=HTWSAV(MM)*RECSAV(JJ)
      IF (IOPTUM.EQ.2) SAVE2=HTWSAV(MM)
      Y2(I)=POPT(I)/1.E+06
C>>>>D   IF (IREC.EQ.1.OR.IREC.EQ.2) GO TO 1380
C>>>>DWRITE (6,2390) Y2(I),ALLBUS(I),THTSAV(II),RECSAV(JJ),SAVE2,HSTSAV(
C>>>>DI),ARESAV(I)
C>>>>D   GO TO 1390
1380  WRITE (6,2280) Y2(I),ALLBUS(I),THTSAV(II),RECSAV(JJ),SAVE2,
     1HSTSAV(I),ARESAV(I)
C>>>>DCONTINUE
      NP=I
1400  CONTINUE
C
C             PRINT OUT CAVITY CONFIGURATION
C
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 1430
      WRITE (6,2300)
      WRITE (6,2310)
      DO 1420 I=1,NUMOPT
      JJ=IOPTRE(I)
      KK=IOPTHT(I)
      XAPER=RECSAV(JJ)
      IF (IOPTUM.EQ.2) W2=0.5*HTWSAV(KK)/THTI
      DO 1420 MM=1,NUMCAV
      IF (MM.EQ.2) XAPER=XAPER*RX2TRX
      IF (MM.EQ.3) XAPER=XAPER*RX3TRX/RX2TRX
      IF (MM.EQ.4) XAPER=XAPER*RX4TRX/RX3TRX
      YAPER=HTWSAV(KK)*XAPER
      IF (IOPTUM.EQ.2) YAPER=RYTRX*XAPER
      W2D=W2*RWCAV(MM)*THTI
      HCAV=HTOP(I,MM)-HBOT(I,MM)
      ANGCAV=ANGMAX(I,MM)-ANGMIN(I,MM)
      IF (MM.EQ.1) WRITE (6,2320) Y2(I),MM,RAZM(MM),RELV(MM),XAPER,
     1YAPER
     2,W2D,HTOP(I,MM),HBOT(I,MM),HCAV,ANGMIN(I,MM),ANGMAX(I,MM),ANGCAV
      IF (MM.NE.1) WRITE (6,2330) MM,RAZM(MM),RELV(MM),XAPER,YAPER,W2D,H
     1TOP(I,MM),HBOT(I,MM),HCAV,ANGMIN(I,MM),ANGMAX(I,MM),ANGCAV
1420  CONTINUE
      WRITE (6,2340)
1430  CONTINUE
C
C             PRINT OUT THE LAND CONSTRAINED TOWER LOCATION
C
         IF (NLAND.EQ.0) GO TO 1450
      WRITE (6,2400)
      WRITE (6,2350)
      WRITE (6,2360)
      WRITE (6,2370)
      DO 1440 I=1,NP
      II=IOPTTH(I)
      IPOS=IOPTPO(I)
      YTOWES = YTPST+FLOAT(IPOS-1)*DPOS
      XTOWES = (XTPST+FLOAT(IPOS-1)*DPOSX)
      WRITE (6,2380) Y2(I),THTSAV(II),XTOWES,YTOWES
1440  CONTINUE
1450  CONTINUE
C
C             PRINT OUT PERFORMANCES
C
      WRITE (6,2400)
      WRITE (6,2450)
      WRITE (6,2410) RMIRL,RRECL
      WRITE (6,2420)
      WRITE (6,2430)
      DO 1460 I=1,NP
      WRITE (6,2440) Y2(I),ALLCOS(I),ALLSAB(I),ALLATM(I),ALLSPL(I),ALLRC
     1R(I),ALLRCP(I),ALLTOB(I),ALLETA(I),ALLTOA(I)
1460  CONTINUE
         IF (IFLX.EQ.0) GO TO 1480
C
C             PRINT OUT FLUX ON RECEIVER
C
      WRITE (6,2400)
      WRITE (6,2460)
      WRITE (6,1760)
      WRITE (6,1760)
      WRITE (6,2470)
      DO 1470 I=1,NP
      WRITE (6,2480) Y2(I),(J,ALLFLU(I,J),J=1,NFLXMX)
1470  CONTINUE
1480  CONTINUE
      DO 1485 I=1,NP
      WRITE (6,3200)
      WRITE (6,3210)
      WRITE (6,3220)
      DO 1486 N=1,NFLXMX
      WRITE (6,3230) N,(ALLXFLX(I,N,J),J=1,3)
 1486 CONTINUE
 1485 CONTINUE
3200  FORMAT (///20X,27H COORDINATES OF FLUX POINTS)
3210  FORMAT (  /34H NUMBER      COORDINATES IN METERS)
3220  FORMAT (35H            EAST     NORTH       UP)
3230  FORMAT (I5,4X,3(F7.2,3X))
C
C             PRINT OUT CAPITAL COSTS
C
      WRITE (6,2400)
      WRITE (6,2490)
      WRITE (6,2500)
      WRITE (6,2510)
      DO 1490 I=1,NP
      WRITE (6,2520) Y2(I),ALLCC(I),ALLPCL(I),ALLPCW(I),ALLPCH(I),ALLPCT
     1(I),ALLPCR(I),ALLPCP(I),ALLPCB(I),ALLPCS(I),ALLPCE(I),ALLPCA(I),AL
     2LPCF(I)
1490  CONTINUE
      ESC1=ESC*100.0
      CONT1=CONT*100.0
      SPTS1=SPTS*100.0
      EXT1=EXT*100.0
      RINF1=RINF*100.0
      CMDC1=AFDC*100.0
      FCR1=FCR*100.0
      RNHOMA=RNHOML*100.0
      RHOML1=RHOML*100.0
C
C             PRINT OUT ENERGY BREAKDOWN
C
      WRITE (6,2400)
      WRITE (6,2530)
      WRITE (6,2540)
      WRITE (6,2550) PF,WEATH,REFPRL,AVGPRL
      WRITE (6,2560)
      WRITE (6,2570)
      DO 1500 I=1,NP
      CAPFAC=ALLKWH(I)*1000./POPT(I)/8760.
      WRITE (6,2580) Y2(I),ALLTOA(I),ALLHR(I),ALLKWH(I),CAPFAC
1500  CONTINUE
C
C             PRINT OUT BBEC BREAKDOWN
C
      WRITE (6,2400)
      WRITE (6,2590)
      WRITE (6,2600)
      WRITE (6,2610)
      WRITE (6,2620) CONT1,SPTS1,EXT1,ESC1,RINF1,NYTCON,CMDC1,FCR1,
     1RHOML1,RNHOMA
      WRITE (6,2630)
      WRITE (6,2640)
      DO 1510 I=1,NP
      DPKWE=ALLCIC(I)*1.0E+06/(Y2(I)*1000.)
      WRITE (6,2650) Y2(I),ALLCI(I),ALLCIY(I),ALLCIC(I),DPKWE,
     1ALLBUA(I),ALLBUS(I)
1510  CONTINUE
         IF (IOTAPE.EQ.0) GO TO 1530
      II=0
      DO 1520 I=1,NP
      IF (IPLFL(I).GT.0) II=II+1
1520  CONTINUE
      WRITE (30) WM,HM,IROUND,ICANT,NCANTX,NCANTY,ROUND,ASTAT,AMIRRO,
     1NRAD,NAZM,RMIRL,DENSMR,XFOCUS,YFOCUS,SIGAZ,SIGEL,SIGSX,SIGSY,
     2SIGTX,SIGTY,HEAD,IFOCUS,XFOCAL,YFOCAL,HCANT,DCANT,RADMIN,RADMAX,
     3RCANT,RANGE,ILAY,INDC,IROTFL,REFSOL,ATM1,ATM2,ATM3,ATM4,IATM,
     4INORTH,AMAXN,PLAT,ALT,REFDAY,REFTIM,NSUNPT,SUNR,SUNI,NSUN,ICPANL,
     5WPANL,HPANL,HXCANT,HYCANT,NUMCAV,IREC,RELV,RAZM,RRECL,IAUTOP,II,
     6HROP,WEATH,DPRES,DH2O,DWEATH,IWEATH,ALP,ASTART,REFTHP,AREF,REFRC,
     7REFLP,REFPIP,FPLH,FPLC,ITHEL,ETAREF,FEFF,REFPRL,FSP,PF,SMULT,
     8CCMUL,CCMULY,CCMULC,FCR,RHOML,RNHOML,CSTREF,VSTREF,ESTREF,XST,
     9CSTRMD,VMAX,EMPTY,ISTR,NSTR
     A,NAX,NAY,NUMPT,XAIM,YAIM,EFFSTR,FEP,IPH,FLXLIM,ICAVF,IFLAUT,IFLX
     B,IFXOUT,NFLXMX,NMXFLX,ISB,AZMSEP,DENSIT,FSLIP,IDENS,IHPR,IUSERF
     C,NRADMN,NRADMX,REFRC1,REFRC2,REFRC3,IRADFL
     *,HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7
     *,PARL8,PARL9,PARL10,TPRE,TSTRT
C>>>3I LINES ABOVE
1530  CONTINUE
C
C             PRINT OUT FIELD LAYOUT
C
      IFUDGE=1
      WRITE (6,2400)
      WRITE (6,2660)
      TOWLS = TOWL
      TOWDS = TOWD
      HI = H
      DO 1540 I = 1,NUMCAV
      RXI(I) = RX(I)
 1540 CONTINUE
      DO 1680 N=1,NP
         IF (IPLFL(N).EQ.0) GO TO 1680
      J=IPLFL(N)
      K=IOPTTH(N)
      THTI = THTSAV(K)
      THT2=THTI**2
      K=IOPTRE(N)
      WI = RECSAV(K)
      MM=IOPTHT(N)
      IF (IREC.EQ.0) HI=WI*HTWSAV(MM)
      WRITE (6,2670) Y2(N)
      DO 1550 I=1,NAZM2P
         IF (ILLRMI(J,I).GT.0.AND.ILLRMI(J,I).LT.NRAD+1) GO TO 1550
      ILLRMI(J,I)=0
      ILLNR(J,I)=0
1550  CONTINUE
      WRITE (6,2680) (AZM(I),I=1,NAZM2P)
C>>>>RWRITE (6,2690) (ILLRMI(J,I),I=1,NAZM2P)
      WRITE (6,2690) MINMAX(1),(ILLRMI(J,I),I=1,NAZM2P)
      DO 1560 I=1,NAZM2P
1560  ILLNR(J,I)=ILLNR(J,I)+ILLRMI(J,I)-1
C>>>>RWRITE (6,2700) (ILLNR(J,I),I=1,NAZM2P)
      WRITE (6,2690) MINMAX(2),(ILLNR(J,I),I=1,NAZM2P)
      DO 1570 I=1,NAZM
      L=LZONE(I)
      NTMIN(I)=ILLRMI(J,L)
1570  NTMAX(I)=ILLNR(J,L)
         IF (IREC.EQ.0) GO TO 1595
      RXI(1)=WI
         IF (IOPTUM.EQ.2) GO TO 1580
      RYTRX=HTWSAV(MM)
      WI = W
         GO TO 1590
1580  WI = HTWSAV(MM)
1590  CONTINUE
      IF (NUMCAV.GT.1) RXI(2)=RXI(1)*RX2TRX
      IF (NUMCAV.GT.2) RXI(3)=RXI(1)*RX3TRX
      IF (NUMCAV.GT.3) RXI(4)=RXI(1)*RX4TRX
 1595 CONTINUE
      DO 1600 IJJ=1,NUMCAV
      RYI(IJJ)=RYTRX*RXI(IJJ)
1600  CONTINUE
      SAVE=WI/W
      SAVE2=HI/H
      IF (IREC.GT.0) SAVE2=RYI(1)/RY(1)
      SAVE3=1.
      IF (IREC.GT.2) SAVE3=RXI(1)/RX(1)
      XFCS = XFC*SAVE
      YFCS = YFC*SAVE
      ZFCS = ZFC*SAVE
      FAZMIS = FAZMIN*SAVE3
      FAZMAS = FAZMAX*SAVE3
      DIAMFS =DIAMF*SAVE
      FZMINS = FZMIN*SAVE2
      FZMAXS = FZMAX*SAVE2
         IF (IFLAUT.NE.2) GO TO 1620
C
C             SCALE FZMIN, FZMAX FOR OPTIMIZED SYSTEM DATA ON TAPE30
C
      W2=0.5*WI
      RY2=0.5*RYI(1)
      ANG=180.-RELV(1)
      FCTR=((W2*(-SIN(RR*RAZM(1)))-XFCS)**2
     1 +(W2*(-COS(RR*RAZM(1)))-YFCS)**2)**0.5
      THTP=(THTI+RY2*SIN(RR*ANG))*(RADMIN*THTI-W2+FCTR+0.5*DIAMFS)
     1 /(RADMIN*THTI-W2-RY2*COS(RR*ANG))
      THBT=(THTI-RY2*SIN(RR*ANG))*(RADMAX*THTI-W2+FCTR+0.5*DIAMFS)
     1 /(RADMAX*THTI-W2+RY2*COS(RR*ANG))
      THTP1=THBT+H*RYI(1)/RY(1)
      IF(THTP1.GT.THTP) WRITE (6,*) 'WARNING**DELSOL HCAV USED'
      IF(THTP1.LT.THTP) THTP=THTP1
      THCAV=THTP-THBT
      FZMINS = THCAV*RFZMIN+(THBT-THTI)
      FZMAXS = THCAV*RFZMAX+(THBT-THTI)
1620  CONTINUE
      KLST=KLAST(J)
      LLST=LLAST(J)
      FLST=FLAST(J)
      DO 1640 L=1,NAZM
      DO 1640 K=1,NRAD
      FLAND(K,L)=1.
1640  CONTINUE
C>>>>DDO 1640 L=1,NAZM2P
C>>>>DKKK=ILLRMI(J,L)
C>>>>DLLL=ILLNR(J,L)
C>>>>DDO 1640 K=1,NRAD
C>>>>DFLAND(K,L)=0.
C>>>>DM=MZONE(L)
C>>>>DFLAND(K,M)=0.
C>>>>D   IF (K.LT.KKK.OR.K.GT.LLL) GO TO 1640
C>>>>DFLAND(K,M)=1.
C>>>>DFLAND(K,L)=1.
C>>>>DCONTINUE
         IF (NLAND.EQ.0) GO TO 1670
      IPOS=IOPTPO(J)
      YTOWES = (YTPST+FLOAT(IPOS-1)*DPOS)
      XTOWES = (XTPST+FLOAT(IPOS-1)*DPOSX)
      CALL LAND
1670  CONTINUE
C>>>9I
      DO 1675 L = 1,NAZM2P
      KKK = ILLRMI(J,L)
      LLL = ILLNR(J,L)
      DO 1675 K = 1,NRAD
         IF ( K .GE. KKK .AND. K .LE. LLL) GO TO 1675
      M = MZONE(L)
      FLAND(K,L) = 0.
      FLAND(K,M) = 0.
 1675 CONTINUE
      FLAND(KLST,LLST)=FLAND(KLST,LLST)*FLST
      M=MZONE(LLST)
C>>>>RIF (WTAREA(LLST).GT.1.1) FLAND(KLST,M)=FLAND(KLST,M)*FLST
      IF ( M .GT. MZONE(NAZM2P) ) FLAND(KLST,M) = FLAND(KLST,M)*FLST
C
C             PRINT OUT FIELD
C
      CALL HCOORD (J)
         IF (IOTAPE.EQ.0) GO TO 1680
      ACWOSY=CWOSYL(N)
      ACWOSC=CWOSCL(N)
      ACHY=CWOSYH(N)
      ACHC=CWOSCH(N)
      ESTOR=ALLSTR(N)
      TOWLS = TOWLS*THTI/THTS
      TOWDS = TOWDS*THTI/THTS
C change 1/29/87 fixed htop and hbot problem according to kistlers
C 1/23/87 memo
      IF(IREC.EQ.1.OR.IREC.EQ.2) HI=HTOP(N,1)-HBOT(N,1)
      WRITE (30) Y2(N),NTMIN,NTMAX,AZMTR,DENTR,FLAND,XTOWES,YTOWES,SLEW,
     1SLNS,CLE,CLN,NLAND,THTI,RXI,RYI,WI,HI,RWCAV,TOWDS,TOWLS,ACWOSY,
     2ACWOSC,ACHY,ACHC,ESTOR,XFCS,YFCS,ZFCS,DIAMFS,POLF,AZMF,NXFLX,
     3FAZMIS,FAZMAS,NYFLX,FZMINS,FZMAXS
1680  CONTINUE
         IF (IOTAPE.EQ.0) GO TO 1700
      WRITE (30) NP
      DO 1690 N=1,NP
      WRITE (30) Y2(N),ALLBUS(N),HSTSAV(N),ARESAV(N),ALLCC(N),ALLPCL(N),
     1ALLPCW(N),ALLPCH(N),ALLPCT(N),ALLPCR(N),ALLPCP(N),ALLPCB(N),ALLPCS
     2(N),ALLPCE(N),ALLPCA(N),ALLPCF(N),ALLTOA(N),ALLHR(N),ALLKWH(N),ALL
     3COS(N),ALLSAB(N),ALLATM(N),ALLSPL(N),ALLRCR(N),ALLRCP(N),ALLTOB(N)
     4,ALLETA(N),ALLTOA(N),(ALLFLU(N,I),I=1,4)
      ENDFILE 30
1690  CONTINUE
      WRITE (6,2710)
1700  CONTINUE
C
C             CAPITAL COST OF THERMAL ENERGY AT THE BASE OF THE TOWER
C             INCLUDES FIELD, TOWER, RECEIVER, PUMPS, AND PIPES
C
      WRITE (6,2400)
      WRITE (6,2720)
      WRITE (6,2730)
      WRITE (6,2740)
      WRITE (6,2750)
      DO 1710 I=1,NP
      SAVE=ALLTP(I)/1.E+06
      Y3=ALLTC(I)/ALLTP(I)*1000.
      Y1=POPT(I)/1.0E+06
      WRITE (6,2760) Y1,SAVE,Y3
1710  CONTINUE
      IF (IRERUN.EQ.0) GO TO 160
C>>>7I
      THT = THTI
      H = HI
      W = WI
      DO 1715 I = 1,NUMCAV
      RX(I) = RXI(I)
      RY(I) = RYI(I)
 1715 CONTINUE
 1720 CONTINUE
      RETURN
C
1730  FORMAT (1X,5A8)
1740  FORMAT (1X,F5.2,3X,A7,3X,12F8.3)
1750  FORMAT (19X,12(F6.0,2X))
1760  FORMAT (/)
1770  FORMAT (/1X,82H TOTAL PERFORMANCE INCLUDES REFLECTIVITIES BUT NOT
     1RADIATION AND CONVECTION LOSSES/)
1780  FORMAT (////20X,27H YEARLY AVERAGE PERFORMANCE/)
1790  FORMAT (/)
1800  FORMAT (1H1,20X,25H ----- OPTIMIZATION -----///)
1810  FORMAT (/98H DELSOL WILL DO A SMART SEARCH OVER THE OPTIMIZATION V
     1ARIABLES - NOT EVERY CASE WILL BE CALCULATED/)
1820  FORMAT (/75H DELSOL WILL CALCULATE EVERY POSSIBLE COMBINATION OF O
     1PTIMIZATION VARIABLES/)
1830  FORMAT (39H NO OPTIMIZATION OF HELIOSTAT DENSITIES//)
1840  FORMAT (34H HELIOSTAT DENSITIES ARE OPTIMIZED//)
1850  FORMAT (1X,9HTHERE ARE,I3,27H VALUES OF THE TOWER HEIGHT)
1860  FORMAT (10X,15HMINIMUM VALUE =,F8.3)
1870  FORMAT (10X,15HMAXIMUM VALUE =,F8.3)
1880  FORMAT (//96H -----PROGRAM STOPPED - ONLY ONE TOWER HT. CAN BE USE
     1D WHEN OPTIMIZING HELIOSTAT DENSITIES -----)
1890  FORMAT (///30H EXTERNAL CYLINDRICAL RECEIVER)
1900  FORMAT (/10H THERE ARE,I3,32H VALUES OF THE CYLINDER DIAMETER)
1910  FORMAT (/11H THERE ARE ,I3,36H VALUES OF THE HEIGHT TO WIDTH RATIO
     1)
1920  FORMAT (///21H CAVITY RECEIVER WITH,I2,21H ELLIPTICAL APERTURES)
1930  FORMAT (///21H CAVITY RECEIVER WITH,I2,22H RECTANGULAR APERTURES)
1940  FORMAT (///10H THERE ARE,I2,32H ELLIPTICAL FLAT PLATE RECEIVERS)
1950  FORMAT (///10H THERE ARE,I2,33H RECTANGULAR FLAT PLATE RECEIVERS)
1960  FORMAT (/' THERE ARE',I3,' VALUES OF THE HORIZONTAL DIMENSION',
     1'RX(1)')
1970  FORMAT (/10H THERE ARE,I3,35H VALUES OF THE RATIO OF RX(1)/RY(1))
1980  FORMAT (/10H THERE ARE,I3,32H VALUES OF THE RECEIVER DIAMETER)
1990  FORMAT (10X,19H THE CAVITY OF THE ,I2,30H APERTURE HAS A DEPTH THA
     1T IS ,F6.3,26H TIMES THE RECEIVER RADIUS)
2000  FORMAT (///10H THERE ARE,I3,61H VALUES OF THE DESIGN POINT POWER L
     1EVEL IN MEGAWATTS ELECTRIC)
2010  FORMAT (//11H THERE ARE ,I2,29H VALUES OF THE TOWER POSITION)
2020  FORMAT (10X,14HMINIMUM NORTH=,F10.1,6H EAST=,F10.1)
2030  FORMAT (10X,14HMAXIMUM NORTH=,F10.1,6H EAST=,F10.1)
2040  FORMAT (///30H CONSTRAINTS ON DESIGN(IF ANY)//)
2050  FORMAT (//20X,17H THE FLUX AT THE ,I3,41H POINT ON THE RECEIVER MU
     1ST BE LESS THAN ,E12.3,5H W/M2)
2060  FORMAT (10X,' THE ORIGIN OF THE FLUX SURFACE IS PROPORTIONAL TO ',
     1'THE RECEIVER DIAMETER')
2070  FORMAT (10X,80H THE ORIGIN COORDINATES EXPRESSED AS FRACTIONS OF T
     1HE   REC. DIAMETER ARE, EAST=,F6.3,7H NORTH=,F6.3,4H UP=,F6.3)
2080  FORMAT (10X,60H FLUX SURFACE IS THE OUTSIDE OF A CYLINDER WHOSE DI
     1AMETER IS,F6.3,28H TIMES THE RECEIVER DIAMETER)
2090  FORMAT (10X,118H THE FLUX POINTS HAVE A CONSTANT AZIMUTH AND A HEI
     1GHT   THAT SCALES WITH REC. HT. (EXT. REC.) OR REC. DIAM.(CAV. REC
     2.))
2100  FORMAT (10X,63H THE FLUX SURFACE IS THE INSIDE OF A CYLINDER WHOSE
     1 DIAMETER IS,F6.3,28H TIMES THE RECEIVER DIAMETER)
2110  FORMAT (10X,120H FLUX SURFACE IS A PLANE, THE HORIZONTAL (VERTICAL
     1) COMPONENT OF THE FLUX POINTS SCALES WITH THE RECEIVER WIDTH (HEI
     2GHT))
2120  FORMAT (//20X,36H THE HELIOSTAT FIELD MUST BE WITHIN ,I2,11H RECTA
     1NGLES)
2130  FORMAT (//19H THE CENTER OF THE ,I2,14H RECTANGLE IS ,F7.1,12H M E
     1AST AND ,F7.1,45H M NORTH OF THE CENTER OF THE FIRST RECTANGLE)
2140  FORMAT (10X,3HTHE,I3,34H RECTANGLE IS ROTATED CLOCKWISE BY,F6.2,28
     1H DEGREES FROM THE NS/EW AXES)
2150  FORMAT (20X,48H BEFORE ROTATION THE SIDE PARALLEL TO E-W AXIS =,F8
     1.1,20H  M LONG, N-S SIDE =,F8.1,7H M LONG)
2160  FORMAT (1H1)
2170  FORMAT (/1X,16H ITERATING, THT=,F6.1,25H FIRST RECEIVER VARIABLE=,
     1F7.3,9H SECOND =,F7.3)
2180  FORMAT (21H DESIGN POWER RANGE =,F7.2,3H TO,F7.2,4H MWE,/
     115H SOLAR MULTIPLE,F5.2,17H   FLUX LIMIT(S)=,4E12.4,5H W/M2)
2190  FORMAT (/19H DESIGN POINT, DAY=,F7.3,24H  HOURS PAST SOLAR NOON=,F
     17.3,16H    INSOLATION =,F7.4,6H KW/M2/)
2200  FORMAT  (/////10X,110(1H*)///15X,33H..... NO OPTIMUM SYSTEM DESIGN
     1(S),60H FOUND FOR COMBINATION OF OPTIMIZATION VARIABLES GIVEN ....
     2.///10X,110(1H*))
C>>>>DFORMAT (88H DES.POWER  BUSBAR COST   TOWER HT  APERTURE WIDTH APER
C>>>2RTURE HT  NO.HELIOSTATS  LAND AREA)
 2210 FORMAT (36H DES.POWER  BUSBAR COST   TOWER HT  ,2A7,2A6,
     126H  NO.HELIOSTATS  LAND AREA)
C>>>>DFORMAT (88H DES.POWER  BUSBAR COST   TOWER HT  APERTURE WIDTH RECV
C>>>>DR WIDTH  NO.HELIOSTATS  LAND AREA)
C>>>>DFORMAT (88H DES.POWER  BUSBAR COST   TOWER HT     PLATE WIDTH    P
C>>>>DLATE HT  NO.HELIOSTATS  LAND AREA)
C>>>>DFORMAT (89H DES.POWER  BUSBAR COST   TOWER HT     PLATE WIDTH RECV
C>>>>DR WIDTH  NO. HELIOSTATS  LAND AREA)
C>>>>DFORMAT (88H DES.POWER  BUSBAR COST   TOWER HT  RECEIVER DIAM  RECE
C>>>>DIVER HT  NO.HELIOSTATS  LAND AREA)
2260  FORMAT (31H   (MWE)    (MILLS/KWHR)    (M),9X,3H(M),11X,3H(M),24X,
     15H(KM2))
C>>>>DFORMAT  (32H   (MWE)    (MILLS/KWHR)     (M),9X,4H (M),11X,4H (M),
C>>>>D23X,5H(KM2))
2280  FORMAT  (F9.2,2X,F9.2,4X,F9.2,3X,F9.2,5X,F9.2,7X,F9.0,2X,F9.3)
2290  FORMAT (1H1,20X,13HSYSTEM DESIGN///)
2300  FORMAT  (1H1,20X,20HCAVITY CONFIGURATION///)
2310  FORMAT  (' DESIGN',49X,'CAVITY    HEAT ABSORBING SURFACE         '
     1,'CAVITY ANGLES'/' POWER    CAVITY    RAZM     RELV      RX',
     2'       RY      DEPTH     TOP     BOTTOM   HEIGHT     MIN      ',
     3'MAX     TOTAL'/' (MWE)',14X,'(DEG)    (DEG)     (M)      (M)',
     4'      (M)      (M)      (M)      (M)      (DEG)    (DEG)    ',
     5'(DEG)')
2320  FORMAT  (F7.2,I6,3X,8F9.2,1X,3F9.2)
2330  FORMAT  (7X,I6,3X,8F9.2,1X,3F9.2)
2340  FORMAT  (//54H ----- TOP, BOTTOM OF HEAT ABSORBING SURFACE RELATIV
     1E ,37HTO TOP OF TOWER (CENTER OF APERTURE)./23H ----- MIN, MAX CAV
     2ITY ,55HANGLES MEASURED COUNTERCLOCKWISE FROM PLANE OF APERTURE,33
     3H  (ORIGIN AT CENTER OF APERTURE).)
2350  FORMAT (68H OPTIMUM LOCATION OF TOWER WITH RESPECT TO CENTER OF LA
     1ND CONSTRAINT///)
2360  FORMAT (44H DES. POWER   TOWER HT.       TOWER LOCATION)
2370  FORMAT (47H                 M          EAST(M)    NORTH(M))
2380  FORMAT (2X,F7.2,2X,F9.2,2(3X,F9.1))
C>>>>DFORMAT (2X,F7.2,2X,F9.2,4X,F9.2,3X,F9.2,5X,F9.2,7X,F9.0,2X,F9.3)
2400  FORMAT (1H1)
2410  FORMAT (32H CONSTANT - MIRROR REFLECTIVITY=,F5.3,25H   RECEIVER AB
     1SORPTIVITY=,F5.3/)
2420  FORMAT (103H DES.POWER  COSINE  SHAD+BLOCK  ATM.TRANSMIT.  INTERCE
     1PT  REC.RAD-CON  PIPING   TOTAL   THERMAL-  TOTAL)
2430  FORMAT (8H   (MWE),71X,17HTHERMAL  ELECTRIC)
2440  FORMAT (2X,F7.2,3X,F5.3,5X,F5.3,9X,F5.3,8X,F5.3,7X,F5.3,5X,F5.3,
     1        3(4X,F5.3))
2450  FORMAT (///20X,29H ANNUAL PERFORMANCE BREAKDOWN)
2460  FORMAT (///20X,50HFLUX ON RECEIVER AT POINTS TESTED FOR MAXIMUM FL
     1UX/)
2470  FORMAT (115H DES.POWER(MWE)  FLUX PT. MAX.FLUX(W/M2)  FLUX PT. MAX
     1.FLUX(W/M2)  FLUX PT. MAX.FLUX(W/M2)  FLUX PT. MAX.FLUX(W/M2))
2480  FORMAT (4X,F7.2,9X,4(I2,6X,E9.3,8X))
2490  FORMAT (///20X,22HCAPITAL COST BREAKDOWN/)
2500  FORMAT (28H DES.POWER   DIRECT CAP.COST,15X,8HPERCENT-)
2510  FORMAT (3X,117H(MWE)    (M$,CURRENT EST.)      LAND    WIRE    HEL
     1     TOW     REC     PIPE    PUMP    STOR    EPGS    HTXCHG  FIXED
     2/)
2520  FORMAT (1X,F7.2,7X,F8.2,9X,11(F6.2,2X))
2530  FORMAT (///20X,24H ANNUAL ENERGY BREAKDOWN)
2540  FORMAT (/60H PLANT FACTOR  WEATHER FACTOR  DES.PT.PAR.LOAD  AVG.PA
     1R.LOAD)
2550  FORMAT (4X,F5.3,11X,F5.3,11X,F5.3,10X,F5.3/)
2560  FORMAT (79H DES.POWER  OVERALL EFFICIENCY  HRS.STORAGE  TOT.KWHR P
     1ER YEAR  CAPACITY FACTOR)
2570  FORMAT (3X,5H(MWE)/)
2580  FORMAT (2X,F7.2,9X,F5.3,12X,F5.2,9X,E11.4,10X,F5.3)
2590  FORMAT (///20X,26HBBEC CALCULATION BREAKDOWN/)
2600  FORMAT (127H CONTINGENCY  SPARE PARTS  INDIRECT  CAP.ESCALATION  I
     1NFLATION  NO.YRS  INT.DURING  FIXED CHG  LEV.HEL.O-M(PCT  LEV.BAL.
     2O-M(PCT)
2610  FORMAT (12X,15H(PCT CAP.COSTS),13X,8H(PCT/YR),5X,72H(PCT/YR)  TO C
     1ONS. CONS(PCT) RATE(PCT/YR) HEL.CAP.COST)  OTHER CAP.COST))
2620  FORMAT (3X,F6.2,7X,F6.2,2(6X,F6.2),8X,F6.2,6X,I2,6X,F6.2,5X,F6.2,
     1   2(9X,F6.2))
2630  FORMAT (/120H DES.POWER  TOT.CAP.COST EST.(M$)  TOT.INVESTMENT(M$)
     1  TOT.INVESTMENT(M$)    $/KWE    BBEC(MILLS/KWHR)  BBEC(MILLS/KWHR
     2))
2640  FORMAT (3X,51H(MWE)    INDIR.,ETC.,INCLUDED   BY 1ST YR OPERATION,
     16X,9HCURRENT $,6X,41HCURRENT $  1ST YR OPERATION     CURRENT $/)
2650  FORMAT (2X,F7.2,2X,2(F13.2,9X),2(F13.2,2X),F12.2,5X,F13.2)
2660  FORMAT (//20X,12HFIELD LAYOUT/)
2670  FORMAT (/19H DESIGN POWER(MWE)=,E11.3)
2680  FORMAT (21H AZIMUTHAL ANGLE     ,10F10.1)
C>>>>RFORMAT (21H MIN. RADIAL ZONE NO.,10(7X,I2,1X))
 2690 FORMAT(A4,17H. RADIAL ZONE NO.,10(7X,I2,1X))
C>>>>DFORMAT (21H MAX. RADIAL ZONE NO.,10(7X,I2,1X))
2710  FORMAT (///10X,77H----- OUTPUT WRITTEN ON LOCAL FILE TAPE30 - USER
     1 SHOULD STORE THIS DATA -----//)
2720  FORMAT (///20X,58H CAPITAL COST OF THERMAL ENERGY NEAR THE BASE OF
     1 THE TOWER)
2730  FORMAT (30X,55HCOSTS INCLUDE FIELD, RECEIVER, TOWER , PIPING AND P
     1UMPS)
2740  FORMAT (40H DES.ELEC.POWER  THERMAL POWER  CAP.COST)
2750  FORMAT (5X,5H(MWE),10X,6H(MWTH),6X,8H($/KWTH))
2760  FORMAT (4X,F7.2,8X,F8.2,5X,F8.3)
      END
      SUBROUTINE HCOORD (II)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE CALCULATES AND PRINTS THE FIELD LAYOUT
C     PARAMETERS FOR AN OPTIMIZED SYSTEM CONFIGURATION.
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C>>>>RCOMMON /OPT1/ CH,DATM(93),DSAVE(13,12),FLUXMB(13,7,4),
      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
     2  HVAL(93),HWDA,HWDR,HWR,THT2,
     3  AZMSER(13,12),DENSIR(13,12),DAREAF(13,7),ICOUNT,IHOPT,
     4  IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
     5  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH
C>>>>R LINE 3 ABOVE
C>>>>R LINE 4 ABOVE
C
C
      DIMENSION RSEP(13,7),ASEP(13,7)
C
C
      DO 110 K=1,13
      DO 110 L=1,12
      DENTR(K,L)=1.0E-08
      REFFLU(K,L)=0.
110   AZMTR(K,L)=.5
      DO 120 K=1,NRAD
      DO 120 L=1,NAZM2P
      ASEP(K,L)=0.
120   CONTINUE
      I1=II+NUMFF4
      I2=II+NUMFF3
      FLAREA=0.
      AMIRRN=0.
      DO 160 L=1,NAZM2P
      K1=ILLRMI(II,L)
      K2=ILLNR(II,L)
C>>>>DLL=NAZM+2-L
C>>>>RIF (LL.GT.NAZM) LL=1
      LL = MZONE(L)
      NTMIN(L)=K1
      NTMIN(LL)=K1
      NTMAX(L)=K2
      NTMAX(LL)=K2
      DO 160 K=1,NRAD
      RAD=RADIUS(K)*THTI
         IF (IHOPT.EQ.0) GO TO 130
C>>>>RDENSIR(K,L)=HCSDP(K,L,I1)*DENSMS
      F = HCSDP(K,L,I1)
C>>>>DF=DENSIR(K,L)/DENSMR
      T=HCSDP(K,L,I2)
      RSEP(K,L)=SQRT(T+SQRT(T**2+1./F**2))
      ASEP(K,L)=1./(F*RSEP(K,L))
C>>>>RDENTR(K,L)=DENSIR(K,L)
      DENTR(K,L) = F*DENSMS
      AZMTR(K,L)=ASEP(K,L)
         IF (L*INORTH.LT.NAZM2P) GO TO 140
C
C             DUMMY ZONE IN NORTH ONLY FIELD
C
C>>>>RAZMTR(K,L)=AZMSER(K,1)
      AZMTR(K,L) = AZMTR(K,1)
C>>>>RDENTR(K,L)=DENSIR(K,1)
      DENTR(K,L) = DENTR(K,1)
         GO TO 140
130   CONTINUE
      RSEP(K,L)=DENSMS/DENSIR(K,L)/AZMSER(K,L)
      SAVE=1.0
      IF (ILAY.LT.2) SAVE=1.0-HM*RSEP(K,L)/(2.*RAD)
      DENTR(K,L)=DENSIR(K,L)*SAVE
      AZMTR(K,L)=AZMSER(K,L)/SAVE
      ASEP(K,L)=AZMTR(K,L)
140   CONTINUE
C>>>>DLL=NAZM+2-L
C>>>>DIF (LL.GT.NAZM) LL=1
      SAVE2=(FLAND(K,L)+FLAND(K,LL))/2.
      SAVE3=RAD*(FSLIP-1.)/(RSEP(K,L)*HM*(1.+FSLIP))
      SAVE3=(2.*SAVE3+1./FSLIP)/(2.*SAVE3+1.)
      IF (ILAY.GT.1) SAVE3=1.0
      REALD(K,L)=SAVE3
      SAVE=DAREA(K,L)*DENTR(K,L)*WTAREA(L)*SAVE2*SAVE3
C>>>>RREFFLU(K,L)=SAVE*THT2/ASTAT/DENSMR/WTAREA(L)
      REFFLU(K,L) = SAVE*THT2/ASTAT/DENSMS/WTAREA(L)
      AMIRRN=AMIRRN+SAVE
      FLAREA=FLAREA+SAVE/DENTR(K,L)/SAVE3
      DENTR(K,LL)=DENTR(K,L)
      AZMTR(K,LL)=AZMTR(K,L)
160   CONTINUE
      AMIRRO = AMIRRN*THT2
      FLAREA=FLAREA*THT2
C>>>>RHELN=AMIRRO/(ASTAT*DENSMR)
      HELN = AMIRRO/(ASTAT*DENSMS)
      AVGDEN=AMIRRO/FLAREA
      FLAREA=1.3*FLAREA+0.18E6
C     CHANGE IN AREA TO ADD CENTER AND ROADS, ETC, BASED ON H. NORRIS
      WRITE (6,220) FLAREA,AMIRRO,HELN,AVGDEN
      WRITE (6,180)
      WRITE (6,210)
      WRITE (6,190) (AZM(L),L=1,NAZM2P)
      WRITE (6,180)
      HEAD84 = 7HHSTATS  
      HEAD85 = 7HF LAND   
      DO 170 K=1,NRAD
      WRITE (6,200) RADIUS(K),HEAD5,(DENTR(K,L),L=1,NAZM2P)
      WRITE (6,200) RADIUS(K),HEAD67,(REALD(K,L),L=1,NAZM2P)
      WRITE (6,200) RADIUS(K),HEAD19,(RSEP(K,L),L=1,NAZM2P)
      WRITE (6,200) RADIUS(K),HEAD20,(ASEP(K,L),L=1,NAZM2P)
      WRITE (6,230) RADIUS(K),HEAD84,(REFFLU(K,L),L=1,NAZM2P)
      WRITE (6,200) RADIUS(K),HEAD85,(FLAND(K,L),L=1,NAZM2P)
      WRITE (6,180)
170   CONTINUE
      RETURN
180   FORMAT (/)
190   FORMAT (19X,12F8.1)
200   FORMAT (1X,F5.2,3X,A7,3X,12F8.3)
210   FORMAT (7H RADIUS,14X,52HAZIMUTHAL ANGLE OF FIELD POINT  (NORTH=0.
     1, EAST=90.))
220   FORMAT (//35H FIELD DESCRIPTION - LAND AREA(M2)=,E11.3,15H  MIRROR
     1 AREA =,E11.3,16H  NO. OF HSTATS=,F7.0,19H  AVERAGE DENSITY =,F5.3
     2)
230   FORMAT (1X,F5.2,3X,A7,5X,12(F6.1,2X))
      END
      SUBROUTINE HVALUE
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE DOES THE HELIOSTAT LAYOUT OPTIMIZATION
C     THE INPUT IS A GRID OF NUMFF DENSITIES X NUMTT ASPECT
C     RATIOS. AT EACH GRID POINT THE ANNUAL AVERAGE AND DESIGN
C     POINT SHADOWING AND BLOCKING LOSSES HAVE BEEN DETERMINED
C     BY THE INITIAL PERFORMANCE RUN. THE LAYOUTS ARE
C     CONSTRAINED TO LIE WITHIN THIS GRID.
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /PERF/ HREFIN(16,13,7),REFTOT(13,12),YATMOS(13,12),
     1  YBLOCK(13,7),YCOSI(13,7),YSHAD(13,7),YSPILL(13,7),YTOTAL(13,7)
     2  ,HROP,RREFL,RWCAV(4),NHREF
C>>>>R LINE 2 ABOVE
C
C>>>>RCOMMON /OPT1/ CH,DATM(93),DSAVE(13,12),FLUXMB(13,7,4),
      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
     2  HVAL(93),HWDA,HWDR,HWR,THT2,
     3  AZMSER(13,12),DENSIR(13,12),DAREAF(13,7),ICOUNT,IHOPT,
     4  IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
     5  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH
C>>>>R LINE 3 ABOVE
C>>>>R LINE R ABOVE
C
C
      DIMENSION CSTOUT(5,5),DA(93),DCOEF(5,5),DENHEL(5),DM(93),
     1  DREF(93),DSDF(5,5),DSDT(5,5),FF(5),SABD1(5,5),
     2  SABD2(5,5),SABHEL(5),SD1HEL(5),SD2HEL(5),TT(5),TTTHEL(5),
     3  VALHEL(5),VHEL(5,5)
C
C
      DATA ((DCOEF(I,J),I=1,5),J=1,5)/-2.,-1.,0.,0.,0.,
     1                                2.,0.,-1.,0.,0.,
     2                                0.,1.,0.,-1.,0.,
     3                                0.,0.,1.,0.,-2.,
     4                                0.,0.,0.,1.,2./
C
C
C>>>>DNMAX = NRAD*NAZM2P
      ISKIP=1
         IF (ICOUNT.GT.0) GO TO 350
C
C             FIND BEST ASPECT RATIO AND VALUE AT EACH DENSITY
C             DO THIS ONLY FOR FIRST RECEIVER
C
      ICOUNT=ICOUNT+1
      NUMFF2=2*NUMFF
      NUMFF3=3*NUMFF
      NUMFF4=4*NUMFF
      NFMID=NUMFF/2+1
      NTMID=NUMTT/2+1
      IF ( IROUND .EQ. 0 ) HAVE=SQRT(WMN**2+HMN**2)
      IF ( IROUND .EQ. 1 ) HAVE = WMN
      AMIN=HAVE/WMN
         IF (IHOPTP.LT.1) GO TO 110
      WRITE (6,590)
      WRITE (6,600)
      WRITE (6,610)
110   CONTINUE
C
C             DO EVERY ZONE
C
      DO 340 K=1,NRAD
      DO 340 L=1,NAZM2P
C>>>>DLL2=2*NAZM2P-L
C>>>>RIF (LL2.GT.NAZM) LL2=1
      LL2 = MZONE(L)
C>>>>DM=LZONE(L)
C>>>>RDENS=DENTR(K,L)*AZMTR(K,L)/DENSMR
      DENS = DENTR(K,L)*AZMTR(K,L)/DENSMS
      SAVE=1./DENS*HMN
      SAVE1=WMN*AZMTR(K,L)
      IPAK=0
      DO 130 I=1,NUMFF
      DO 130 J=1,NUMTT
      IPAK=IPAK+1
      SABD1(I,J)=HCSDP(K,L,IPAK)
      SABD2(I,J)=HCSDP(K,LL2,IPAK)
130   SAB(I,J)=SCS(K,L,IPAK)
      REFFLU(K,LL2)=REFFLU(K,LL2)/SABD2(NFMID,NTMID)
      REFFLU(K,L)=REFFLU(K,L)/SABD1(NFMID,NTMID)
      FFFST=WMN*HMN/SAVE/SAVE1
      TTTST=.5*((SAVE/HMN)**2-AZMTR(K,L)**2)
      DO 140 I=1,NUMTT
      TT(I)=TTTST*(1.+FLOAT(I-NUMTT/2-1)*DTTT)
140   CONTINUE
      DO 150 II=1,NUMFF
      FF(II)=FFFST*(1.+FLOAT(II-NUMFF/2-1)*DFFF)
150   CONTINUE
      VSAVE=YTOTAL(K,L)/YSHAD(K,L)/YBLOCK(K,L)
      DO 210 I=1,NUMFF
      DO 200 J=1,NUMTT
      DSDF(I,J)=0.
      DSDT(I,J) = 0.
      DO 170 N=1,NUMFF
      DSDF(I,J)=DSDF(I,J)+DCOEF(I,N)*SAB(N,J)
      DSDT(I,J) = DSDT(I,J) + DCOEF(J,N)*SAB(I,N)
  170 CONTINUE
      DSDF(I,J)=DSDF(I,J)/(FF(2)-FF(1))/2.
      DSDT(I,J) = DSDT(I,J)/(TT(2) - TT(1))/2.
      VHEL(I,J)=SAB(I,J)+DSDF(I,J)*FF(I)
      RRR = SQRT(TT(J) + SQRT(TT(J)**2 + 1./FF(I)**2))
      ZZZ = 1./(FF(I)*RRR)
      SAVE=CH+HWR*RADIUS(K)+HWDR*RRR+HWDA*ZZZ-1./(FF(I)*(RRR**2+ZZZ**2))
     1*(HWDR*ZZZ+HWDA*RRR)
      VHEL(I,J)=VHEL(I,J)/SAVE
C
C             VHEL = VALUE OF ADDING ONE MORE HELIOSTAT TO ZONE
C
      SAVE2=1.0
         IF (ILAY.GT.1) GO TO 180
      SAVE2=RADIUS(K)*(FSLIP-1.)/(RRR*HMN*(1.+FSLIP))
      SAVE2=(2.*SAVE2+1./FSLIP)/(2.*SAVE2+1.)
180   CONTINUE
C
C             CSTOUT = AVERAGE VALUE OF HELIOSTAT IN A ZONE
C
      CSTOUT(I,J)=SAB(I,J)/(CH+HL/FF(I)/SAVE2+HWR*RADIUS(K)+HWDR*RRR+HWD
     1A*ZZZ)
         IF (IHOPTP.LT.1) GO TO 200
      SAVE3=CSTOUT(I,J)*VSAVE
      SAVE=VHEL(I,J)*VSAVE
      VAL=CH+HL/FF(I)/SAVE2+HWR*RADIUS(K)+HWDR*RRR+HWDA*ZZZ
      SAVE2=DSDT(I,J)/VAL
C>>>>DIF (HWDR*RRR-HWDA*ZZZ.NE.0.) SAVE2=DSDT(I,J)*VSAVE*(RRR**2+ZZZ**2)
C>>>2R/(HWDR*RRR-HWDA*ZZZ)
      IF ( HWDR*RRR .NE. HWDA*ZZZ ) SAVE2 = DSDT(I,J)*VSAVE*(RRR**2
     1     + ZZZ**2)/(HWDR*RRR - HWDA*ZZZ)
      SAVE7=FF(I)*DENSMS
      WRITE (6,620) K,L,SAVE7,RRR,ZZZ,SAB(I,J),SABD1(I,J
     1),SABD2(I,J),VAL,SAVE3,SAVE,SAVE2
200   CONTINUE
210   CONTINUE
      NN=0
      VMX=0.
      III=(NUMFF-1)*4+1
      JJJ=(NUMTT-1)*4+1
C
C             WHEN ZONE IS FIRST ADDED TO FIELD ITS DENSITY AND
C             ASPECT RATIO ARE THOSE THAT MAXIMIZE THE AVERAGE VALUE
C             THIS MAXIMUM IS FOUND BY INTERPOLATION
C
      DO 250 I=1,III
      I1=(I-1)/4+1
      I2=I1+1
      IF (I2.GT.NUMFF) I2=I1
      NNN=0
      F=FLOAT(I-(I1-1)*4-1)*.25
      F1=1.0-F
      FFF=F1*FF(I1)+F*FF(I2)
      DO 240 J=1,JJJ
      J1=(J-1)/4+1
      J2=J1+1
      IF (J2.GT.NUMTT) J2=J1
      G=FLOAT(J-(J1-1)*4-1)*.25
      G1=1.0-G
      SB=F1*G1*SAB(I1,J1)+F*G1*SAB(I2,J1)+F1*G*SAB(I1,J2)+F*G*SAB(I2,J2)
      TTT=G1*TT(J1)+G*TT(J2)
      RRR=SQRT(TTT+SQRT(TTT**2+1./FFF**2))
      ZZZ=1./RRR/FFF
C
C             TEST FOR MECHANICAL LIMITS
C
         IF (ZZZ.LT.AMIN/2.) GO TO 240
         IF (RRR.LT.HAVE/HMN) GO TO 240
      SAVE2=(HAVE**2-(RRR*HMN/2.)**2)/WMN**2
         IF (ZZZ**2.LT.SAVE2) GO TO 240
      SAVE2=1.0
         IF (ILAY.GT.1) GO TO 220
      SAVE2=RADIUS(K)*(FSLIP-1.)/(RRR*HMN*(1.+FSLIP))
      SAVE2=(2.*SAVE2+1./FSLIP)/(2.*SAVE2+1.)
220   CONTINUE
      VAL=SB/(CH+HL/FFF/SAVE2+HWR*RADIUS(K)+HWDR*RRR+HWDA*ZZZ)
      SAVE3=F1*G1*DSDF(I1,J1)+F*G1*DSDF(I2,J1)+F1*G*DSDF(I1,J2)+F*G*DSDF
     1(I2,J2)
      SAVE3=(SB+FFF*SAVE3)/(CH+HWR*RADIUS(K)+HWDR*RRR+HWDA*ZZZ-(HWDA*RRR
     1+HWDR*ZZZ)/(RRR**2+ZZZ**2)/FFF)
C
C             TEST TO SEE IF VALUE GREATER THAN CURRENT MAXIMUM
C
         IF (VAL.LT.VMX) GO TO 240
      VMX=VAL
      NNN=1
      VALHEL(1)=VAL
      DENHEL(1)=FFF
      TTTHEL(1)=TTT
      SABHEL(1)=SB
      SD1HEL(1)=F1*G1*SABD1(I1,J1)+F*G1*SABD1(I2,J1)+F1*G*SABD1(I1,J2)+F
     1*G*SABD1(I2,J2)
      SD2HEL(1)=F1*G1*SABD2(I1,J1)+F*G1*SABD2(I2,J1)+F1*G*SABD2(I1,J2)+F
     1*G*SABD2(I2,J2)
C>>>>R LINE ABOVE
      NN=1
      IF (VAL.GT.SAVE3) VAL=SAVE3
      SCS(K,L,NUMFF4+1)=VAL
      IF (I.EQ.1) SCS(K,L,NUMFF4+1)=F1*VHEL(I1,J1)+F*VHEL(I1,J2)
240   CONTINUE
         IF (NNN.LT.1.AND.NN.GT.0) GO TO 260
250   CONTINUE
260   CONTINUE
      I1=(I-1)/4+2
      M=1
         IF (I1.GT.NUMFF) GO TO 300
C
C             TEST TO SEE IF MAXIMUM AVERAGE VALUE OCCURED
C             AT MAXIMUM ALLOWED DENSITY
C
      DO 290 I=I1,NUMFF
      M=M+1
      FFF=FF(I)
      MMM=0
C
C             FOR DENSITY VALUES BEYOND THE DENSITY THAT GIVES THE
C             MAXIMUM AVERAGE VALUE THE OPTIMAL ASPECT RATIO IS
C             FOUND BY REQUIRING
C                  D(VALUE)/D(DENSITY) = D(ASPECT)/D(DENSITY)
C
      DO 280 J=1,JJJ
      J1=(J-1)/4+1
      J2=J1+1
      IF (J2.GT.NUMTT) J2=J1
      G=FLOAT(J-(J1-1)*4-1)*.25
      G1=1.0-G
      TTT=G1*TT(J1)+G*TT(J2)
      RRR=SQRT(TTT+SQRT(TTT**2+1./FFF**2))
      ZZZ=1./RRR/FFF
C
C             TEST FOR MECHANICAL LIMITS
C
         IF (ZZZ.LT.AMIN/2.) GO TO 280
         IF (RRR.LT.HAVE/HMN) GO TO 280
      SAVE2=(HAVE**2-(RRR*HMN/2.)**2)/WMN**2
         IF (ZZZ**2.LT.SAVE2) GO TO 280
      SAVE=G1*DSDT(I,J1)+G*DSDT(I,J2)
      SB=G1*SAB(I,J1)+G*SAB(I,J2)
      SAVE3=(SB+FFF*(G1*DSDF(I,J1)+G*DSDF(I,J2)))/(CH+HWR*RADIUS(K)+HWDR
     1*RRR+HWDA*ZZZ-(HWDR*ZZZ+HWDA*RRR)/(RRR**2+ZZZ**2)/FFF)
      SAVE4=SAVE3*(HWDR*RRR-HWDA*ZZZ)/(RRR**2+ZZZ**2)
      SAVE=SAVE-SAVE4
      MMM=MMM+1
         IF (SAVE.GT.0) GO TO 270
         IF (MMM.EQ.1) GO TO 270
      F1=SAVE/(SAVE-SAVEL)
      F=1.-F1
      TTTHEL(I)=F1*TTTHEL(I)+F*TTT
      SABHEL(I)=F1*SABHEL(I)+F*(G1*SAB(I,J1)+G*SAB(I,J2))
      SD2HEL(I)=F1*SD2HEL(I)+F*(G1*SABD2(I,J1)+G*SABD2(I,J2))
      SD1HEL(I)=F1*SD1HEL(I)+F*(G1*SABD1(I,J1)+G*SABD1(I,J2))
      DENHEL(I)=FFF
      TTT=TTTHEL(I)
      RRR=SQRT(TTT+SQRT(TTT**2+1./FFF**2))
      ZZZ=1./RRR/FFF
      DSDF(1,1)=F1*DSDF(1,1)+F*(G1*DSDF(I,J1)+G*DSDF(I,J2))
      VALHEL(I)=(SABHEL(I)+FFF*DSDF(1,1))/(CH+HWR*RADIUS(K)+HWDR*RRR+HWD
     1A*ZZZ-(HWDR*ZZZ+HWDA*RRR)/(RRR**2+ZZZ**2)/FFF)
      SCS(K,L,NUMFF4+M)=VALHEL(I)
         GO TO 290
270   CONTINUE
      SAVEL=SAVE
      DSDF(1,1)=G1*DSDF(I,J1)+G*DSDF(I,J2)
      VALHEL(M)=SAVE3
      SCS(K,L,NUMFF4+M)=SAVE3
      SABHEL(M)=G1*SAB(I,J1)+G*SAB(I,J2)
      SD1HEL(M)=G1*SABD1(I,J1)+G*SABD1(I,J2)
      SD2HEL(M)=G1*SABD2(I,J1)+G*SABD2(I,J2)
C>>>>RDENHEL(M)=FF(I)
      DENHEL(M) = FFF
      TTTHEL(M)=TTT
         IF (MMM.EQ.1.AND.SAVE.LT.0) GO TO 290
280   CONTINUE
         IF (MMM.GT.0) GO TO 290
      M=M-1
         GO TO 300
290   CONTINUE
         IF (M.EQ.NUMFF) GO TO 320
300   MM=M+1
C
C             MAKE THE DENSITY THAT MAXIMIZES AVERAGE VALUE THE FIRST
C             DENSITY
C             MAKE THE FIRST DISCRETE DENSITY CHOICE AFTER THE FIRST
C             DENSITY THE SECOND DENSITY, ETC.
C
C             IF THE MAXIMUM AVERAGE VALUE OCCURS AT A DENSITY
C             BEYOND THE SECOND DISCRETE DENSITY CHOICE THIS ZONE
C             WILL NOT HAVE A FULL SET OF NUMFF DENSITY CHOICES
C
      DO 310 I=MM,NUMFF
      VALHEL(I)=-1.E-08-FLOAT(MM)*1.E-09
      SCS(K,L,NUMFF4+I)=VALHEL(I)
      SABHEL(I)=SABHEL(M)
      SD1HEL(I)=SD1HEL(M)
      SD2HEL(I)=SD2HEL(M)
      TTTHEL(I)=TTTHEL(M)
      DENHEL(I)=DENHEL(M)
310   CONTINUE
320   CONTINUE
      DO 330 I=1,NUMFF
      SCS(K,L,I)=VALHEL(I)
      SCS(K,L,I+NUMFF)=DENHEL(I)
      SCS(K,L,I+NUMFF2)=SABHEL(I)
      SCS(K,L,I+NUMFF3)=TTTHEL(I)
      HCSDP(K,L,I)=SD1HEL(I)
      HCSDP(K,LL2,I)=SD2HEL(I)
330   CONTINUE
340   CONTINUE
350   CONTINUE
C
C             MODIFY VALUES FOR SPILLAGE FOR THIS SIZE RECEIVER
C
      NMX=0
      DO 360 N=1,NMAX
      K=KMAX(N)
      L=LMAX(N)
      HVAL(N)=SCS(K,L,1)*YTOTAL(K,L)/YSHAD(K,L)/YBLOCK(K,L)
      NMX=NMX+1
C>>>2R   IF (DAREA(K,L).GT.1.E-06.AND.YTOTAL(K,L).GT.0.001) GO TO 360
         IF ( DAREAF(K,L) .GT. 1.E-06 .AND. YTOTAL(K,L) .GT. .001)
     C        GO TO 360
      NMX=NMX-1
      HVAL(N)=0.
360   CONTINUE
C
C             RANK ORDER ZONES ACCORDING TO BEST AVERAGE VALUE
C
      DO 390 N=2,NMAX
      K=KMAX(N)
      L=LMAX(N)
      NS=N-1
370   CONTINUE
         IF (HVAL(NS+1).LE.HVAL(NS)) GO TO 380
      SAVE=HVAL(NS+1)
      HVAL(NS+1)=HVAL(NS)
      HVAL(NS)=SAVE
      LMAX(NS+1)=LMAX(NS)
      KMAX(NS+1)=KMAX(NS)
      NS=NS-1
         IF (NS.GT.0) GO TO 370
380   KMAX(NS+1)=K
      LMAX(NS+1)=L
390   CONTINUE
      NV=NMAX+2
      DO 420 N=1,NV
      NVP = NV + 1 - N
      NVM = NV - 1 - N
      IF (N.LT.NV-1) HVAL(NVP)=HVAL(NVM)
      DREF(N)=0.
      HCST(N)=0.
      DM(N)=0.
C>>>>DDCOS(N)=0.
C>>>>DDSHD(N)=0.
      DBLK(N)=0.
      DATM(N)=0.
      DSPL(N)=0.
      DTOT(N)=0.
      DA(N)=0.
      DO 410 J=1,NFLXMX
410   FLXM(J,N)=0.
      DO 420 J=1,NHREF
      HREFS(J,N)=0.
420   CONTINUE
         IF (IHOPTP.LT.1) GO TO 430
      WRITE (6,630)
      WRITE (6,640)
      WRITE (6,650)
      WRITE (6,660)
430   CONTINUE
C
C             BUILD UP THE OPTIMAL FIELD
C
      DO 520 NNI=1,NMX
      K=KMAX(NNI)
      L=LMAX(NNI)
C
C             DO NOT OPTIMIZE ZONES WITH ZERO AREA OR PERFORMANCE
C
         IF (YTOTAL(K,L).LT.0.001) GO TO 530
C>>>>R   IF (DAREA(K,L)*YTOTAL(K,L).LT.1.E-07) GO TO 530
         IF ( DAREAF(K,L)*YTOTAL(K,L) .LT. 1.E-07 ) GO TO 530
      VSAVE=YTOTAL(K,L)/YSHAD(K,L)/YBLOCK(K,L)
C>>>>DLL2=NAZM+2-L
C>>>>RIF (LL2.GT.NAZM) LL2=1
      LL2 = MZONE(L)
      SAVE3=YATMOS(K,L)*YSPILL(K,L)
      DO 440 M=1,NUMFF
      SD1HEL(M)=HCSDP(K,L,M)
      SD2HEL(M)=HCSDP(K,LL2,M)
      TTTHEL(M)=SCS(K,L,M+NUMFF3)
      DENHEL(M)=SCS(K,L,M+NUMFF)
      SABHEL(M)=SCS(K,L,M+NUMFF2)
      HCSDP(K,L,M+NUMFF2)=VSAVE*SCS(K,L,M+NUMFF4)
C
C             PRINT OPTIMUM SEPARATIONS IN ORDER OF BUILDUP
C
         IF (IHOPTP.LT.1) GO TO 440
      SAVE=HCSDP(K,L,M+NUMFF2)
         IF (SAVE.LT.1.E-06) GO TO 440
      RRR=SQRT(TTTHEL(M)+SQRT(TTTHEL(M)**2+1./DENHEL(M)**2))
      ZZZ=1./(RRR*DENHEL(M))
      SAVE2=DENHEL(M)*DENSMS
      SAVE6=SAVE3*SABHEL(M)
      SAVE7=REFFLU(K,L)*SD1HEL(M)
      SAVE8=REFFLU(K,LL2)*SD2HEL(M)
      WRITE (6,670) K,L,SAVE2,RRR,ZZZ,SAVE6,SAVE7,SAVE8,SAVE
440   CONTINUE
      MM=2
      DO 520 I=NNI,NMX
      N=I+2
      VAL=HVAL(N)
      DO 450 M=MM,NUMFF
         IF (HCSDP(K,L,M+NUMFF2)+1.E-05.LT.VAL) GO TO 460
450   CONTINUE
      M=M-1
460   MM=M
      F=(VAL-HCSDP(K,L,M+NUMFF2))/(HCSDP(K,L,M-1+NUMFF2)-HCSDP(K,L,M+NUM
     1FF2))
      IF (F.GT.1.0) F=1.0
      DENS=F*DENHEL(M-1)+(1.0-F)*DENHEL(M)
      SB=F*SABHEL(M-1)+(1.-F)*SABHEL(M)
C>>>>RDA(N)=DA(N)+DAREA(K,L)*WTAREA(L)/REALD(K,L)
      DA(N) = DA(N) + DAREAF(K,L)*WTAREA(L)/REALD(K,L)
      TTT=F*TTTHEL(M-1)+(1.0-F)*TTTHEL(M)
      RRR=SQRT(TTT+SQRT(TTT**2+1./DENS**2))
C>>>>RDMIRA=DAREA(K,L)*WTAREA(L)*DENS*THT2*DENSMS
      DMIRA = DAREAF(K,L)*WTAREA(L)*DENS*THT2*DENSMS
      SAVE2=REALD(K,L)
         IF (ILAY.GT.1) GO TO 470
      SAVE2=RADIUS(K)*(FSLIP-1.)/(RRR*HMN*(1.+FSLIP))
      SAVE2=(2.*SAVE2+1./FSLIP)/(2.*SAVE2+1.)
470   CONTINUE
      DMIRA=SAVE2/REALD(K,L)*DMIRA
      DM(N)=DM(N)+DMIRA
      DO 480 J=1,NFLXMX
480   FLXM(J,N)=FLXM(J,N)+FLUXMB(K,L,J)*DMIRA
      ZZZ=1./(DENS*RRR)
      HCST(N)=HCST(N)+(CH+HL/DENS/SAVE2+HWDA*ZZZ+HWR*RADIUS(K)+HWDR*RRR)
     1*DMIRA
      SAVE=SAVE3*DMIRA/2.*(F*(REFFLU(K,L)*SD1HEL(M-1)+REFFLU(K,LL2)*SD2H
     1EL(M-1))+(1.-F)*(REFFLU(K,L)*SD1HEL(M)+REFFLU(K,LL2)*SD2HEL(M)))
      DREF(N)=DREF(N)+SAVE
         IF (I.GT.NNI) GO TO 490
      HCSDP(K,L,NUMFF+1)=SAVE
      HCSDP(K,L,NUMFF+2)=DMIRA
C>>>>RHCSDP(K,L,NUMFF+3)=DAREA(K,L)*WTAREA(L)*THT2/REALD(K,L)
      HCSDP(K,L,NUMFF+3) = DAREAF(K,L)*WTAREA(L)*THT2/REALD(K,L)
      HCSDP(K,L,NUMFF+4)=DMIRA*(HWR*RADIUS(K)+HWDR*RRR+HWDA*ZZZ)
490   CONTINUE
C>>>>DDCOS(N)=DCOS(N)+DMIRA
      SAVEIN=YCOSI(K,L)*DMIRA
      DBLK(N)=DBLK(N)+SAVEIN
      SAVEIN=SAVEIN*SB
      DATM(N)=DATM(N)+SAVEIN
      SAVEIN=SAVEIN*YATMOS(K,L)
      DSPL(N)=DSPL(N)+SAVEIN
      DTOT(N)=DTOT(N)+SAVEIN*YSPILL(K,L)
      DO 520 J=1,NHREF
      HREFS(J,N)=HREFS(J,N)+HREFIN(J,K,L)*DMIRA
520   CONTINUE
C>>>>I
      NNI = NMX + 1
530   CONTINUE
C>>>>DNMAX=NNI-1
C>>>>DF=0.
C>>>>DDSPL(2)=1.
C>>>>DDATM(2)=1.
C>>>>DDBLK(2)=1.
C>>>>DDCOS(2)=1.
C>>>>RNVALH=NMAX
      NVALH = NNI - 1
C>>>>RDO 580 NNI=1,NMAX
      DO 580 NNI = 1,NVALH
      K=KMAX(NNI)
      L=LMAX(NNI)
C>>>>DDA(NNI)=DAREA(K,L)
      DCOS(NNI)=YCOSI(K,L)
      DSHD(NNI)=YSHAD(K,L)
      DBLK(NNI)=YBLOCK(K,L)
      DATM(NNI)=YATMOS(K,L)
      DSPL(NNI)=YSPILL(K,L)
      DTOT(NNI)=YTOTAL(K,L)
      HCST(NNI)=HCOST(K,L)
C>>>>DDREF(NNI)=REFTOT(K,L)
      DO 540 I=1,NHREF
540   HREFS(I,NNI)=HREFIN(I,K,L)
      DO 550 I=1,NFLXMX
550   FLXM(I,NNI)=FLUXMB(K,L,I)
C>>>>RDAREA(K,L)=(DA(NNI+2)-F*DA(NNI+1))/WTAREA(L)*REALD(K,L)
      DAREAF(K,L) = (DA(NNI+2) - DA(NNI+1))/WTAREA(L)*REALD(K,L)
C>>>>RDSAVE(K,L)=(DM(NNI+2)-F*DM(NNI+1))
      DSAVE(NNI) = DM(NNI+2) - DM(NNI+1)
C>>>>R   IF (DSAVE(K,L).LT.1.E-07) GO TO 580
         IF ( DSAVE(NNI) .LT. 1.E-07 ) GO TO 580
      DO 560 I=1,NFLXMX
C>>>2RFLUXMB(K,L,I)=(FLXM(I,NNI+2)-F*FLXM(I,NNI+1))/DSAVE(K,L)
      FLUXMB(K,L,I) = (FLXM(I,NNI+2) - FLXM(I,NNI+1))/DSAVE(NNI)
  560 CONTINUE
C>>>>RYSPILL(K,L)=(DTOT(NNI+2)-F*DTOT(NNI+1))/(DSPL(NNI+2)-F*
      YSPILL(K,L) = (DTOT(NNI+2) - DTOT(NNI+1))/(DSPL(NNI+2) -
     1  DSPL(NNI+1))
C>>>>RYATMOS(K,L)=(DSPL(NNI+2)-F*DSPL(NNI+1))/(DATM(NNI+2)-F*
      YATMOS(K,L) = (DSPL(NNI+2) - DSPL(NNI+1))/(DATM(NNI+2) -
     1  DATM(NNI+1))
C>>>2RYSHAD(K,L)=(DATM(NNI+2)-F*DATM(NNI+1))/(DBLK(NNI+2)-F*DBLK(NNI+1))
      YSHAD(K,L) = (DATM(NNI+2) - DATM(NNI+1))/(DBLK(NNI+2)
     C           - DBLK(NNI+1))
C>>>>RYCOSI(K,L)=(DBLK(NNI+2)-F*DBLK(NNI+1))/(DCOS(NNI+2)-F*DCOS(NNI+1))
      YCOSI(K,L) = (DBLK(NNI+2) - DBLK(NNI+1))/DSAVE(NNI)
C>>>>RYTOTAL(K,L)=(DTOT(NNI+2)-F*DTOT(NNI+1))/DSAVE(K,L)*RREFL
      YTOTAL(K,L) = (DTOT(NNI+2) - DTOT(NNI+1))/DSAVE(NNI)*RREFL
      YBLOCK(K,L)=1.
C>>>>RHCOST(K,L)=(HCST(NNI+2)-F*HCST(NNI+1))/DSAVE(K,L)/HCOST1
      HCOST(K,L) = (HCST(NNI+2) - HCST(NNI+1))/DSAVE(NNI)/HCOST1
C>>>>RREFTOT(K,L)=(DREF(NNI+2)-DREF(NNI+1)*F)/DSAVE(K,L)
      REFTOT(K,L) = (DREF(NNI+2) - DREF(NNI+1))/DSAVE(NNI)
      DO 570 I=1,NHREF
C>>>2RHREFIN(I,K,L)=(HREFS(I,NNI+2)-F*HREFS(I,NNI+1))/DSAVE(K,L)
      HREFIN(I,K,L) = (HREFS(I,NNI+2) - HREFS(I,NNI+1))/DSAVE(NNI)
  570 CONTINUE
C>>>>DF=1.
580   CONTINUE
      RETURN
590   FORMAT (1H1,10X,30HHELIOSTAT DENSITY OPTIMIZATION//)
600   FORMAT (119H     ZONE          DENSITY  HSTAT SEPARATION       SHA
     1DOWING AND BLOCKING        HSTAT     EFFICIENCY/HSTAT COST PER M2)
610   FORMAT (129H RADIAL AZIMUTHAL           RADIAL AZIMUTHAL    ANNUAL
     1 DESIGN PT. SYM. DES. PT.  COST  T.   AVERAGE   DENSITY CHANGE  AS
     2PECT CHG.)
620   FORMAT (3X,I2, 6X,I2,6X,F5.3,4X,F5.3,4X,F5.3,5X,F7.5,3X,F5.3, 7X,
     1 F5.3,4X,F7.3,5X,F8.6,5X,F8.6,5X,F8.6)
630   FORMAT (////10X,43HZONES LISTED IN ORDER USED IN FIELD BUILDUP)
640   FORMAT (////10X,37HOPTIMUM HSTAT SEPARATIONS VS. DENSITY)
650   FORMAT (96H     ZONE          DENSITY  HSTAT SEPARATION        EFF
     1ICIENCY                   VALUE OF ADDING)
660   FORMAT (95H RADIAL AZIMUTHAL           RADIAL AZIMUTHAL   ANNUAL D
     1ESIGN PT. SYM. DES. PT.     1 MORE HSTAT)
670   FORMAT (4X,I2,6X,I2,5X,F5.3,4X,F5.3,3X,F5.3,6X,F5.3,3X,F5.3,7X,F5.
     13,11X,F6.4)
      END
      SUBROUTINE MAX
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE MAX PERFORMS FIELD LAYOUT BY SELECTING ZONES IN
C     THE ORDER OF DECREASING PERFORMANCE/COST. WITH EACH ZONE ADDED,
C     BALANCE OF PLANT LOSSES ARE CALCULATED. ON REACHING A REQUESTED
C     DESIGN POWER LEVEL, PLANT CAPITAL AND ENERGY COSTS ARE CALCULATED.
C     ONLY THAT COMBINATION OF FIELD LAYOUT, TOWER HEIGHT, AND RECEIVER
C     SIZE YIELDING THE LOWEST ENERGY COST IS SAVED AND RETURNED TO
C     SUBROUTINE OPTCAL.
C
C     THIS ROUTINE IS CALLED BY
C          OPTCAL
C
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /PERF/ HREFIN(16,13,7),REFTOT(13,12),YATMOS(13,12),
     1  YBLOCK(13,7),YCOSI(13,7),YSHAD(13,7),YSPILL(13,7),YTOTAL(13,7)
     2  ,HROP,RREFL,RWCAV(4),NHREF
C>>>>R LINE 2 ABOVE
C
      COMMON /FLUX1/ CFA(169),CFP(169),DIAMFN,FAZMAS,FAZMIS,FLX(169),
     1  FZMAXN,FZMINN,SFA(169),SFP(169),XFCN,XFLX(169),
     2  XNFLX(169),YFCN,YFLX(169),YNFLX(169),ZFCN,ZFLX(169),NFLXPT
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /REC1/ CAZMMN(4),CAZMMX(4),JREC(12,2),NREC(12),RCRA(4),
     1  RSRA(4)
C
      COMMON /EFF/ ACWOSY,AREF,CCMULC,CCMULY,CSTREF,CSTRMD,EFFSTR,
     1  EMPTY,ESTREF,ETAREF,FCR,FEFF,FLXLIM(4),FPLH,FSP,
     2  REFLP,REFPIP,REFPRL,REFRC,REFRC1,REFRC2,REFRC3,REFTHP,RHOML,
     X  RNHOML,
     3  SMULT,VMAX,VSTREF,XST,ITHEL,IRADFL
C>>>>R LINE 1 ABOVE
C
      COMMON /EFF1/ HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5
     *             ,PARL6,PARL7,PARL8,PARL9,PARL10,TPRE,TSTRT
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
C>>>>RCOMMON /OPT1/ CH,DATM(93),DSAVE(13,12),FLUXMB(13,7,4),
      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
     2  HVAL(93),HWDA,HWDR,HWR,THT2,
     3  AZMSER(13,12),DENSIR(13,12),DAREAF(13,7),ICOUNT,IHOPT,
     4  IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
     5  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH
C>>>>R LINE 4 ABOVE
C>>>>R LINE 3 ABOVE
C
      COMMON /OPT2/ AEVMAX,AEVREF,ALLATM(20),ALLBUA(20),ALLBUS(20),
     1  ALLCC(20),ALLCI(20),ALLCIC(20),ALLCIY(20),ALLCOS(20),
     2  ALLETA(20),ALLFLU(20,4),ALLHR(20),ALLKWH(20),ALLPCA(20),
     3  ALLPCB(20),ALLPCE(20),ALLPCF(20),ALLPCH(20),ALLPCL(20),
     4  ALLPCP(20),ALLPCR(20),ALLPCS(20),ALLPCT(20),ALLPCW(20),
     5  ALLRCP(20),ALLRCR(20),ALLSAB(20),ALLSPL(20),ALLSTR(20),
     6  ALLTC(20),ALLTOA(20),ALLTOB(20),ALLTP(20),ANGMAX(20,4),
     7  ANGMIN(20,4),APHMAX,APHREF,AREC,ARECRC,ARECRF,ARESAV(20),
     8  ARHMAX,ARHREF,ASHMAX,ASHREF,AVGPRL,BBHTW(20),BBW(20),CCPREF,
     9  CEGREF,CEVREF,CFIXED,CHEREF,CHPREF,CL,CPHREF,CREC,CREC1,
     A  CRHREF,CRPU,CSHREF,CSPU,CTOW,CWOSCH(20),CWOSCL(20),CWOSYH(20),
     B  CWOSYL(20),FLAST(20),HBOT(20,4),HSTSAV(20),
     C  HTOP(20,4),
     D  PEGREF,
     E  PEVREF,PHEREF,PMXH,POPT(21),PPHREF,PPIREF,PRHREF,PSHREF,PXHTW,
     F  SMPI,XEPGS,XHEA,XHEP,XPI,XREC,XRP,XSP
     G ,CKNREF,PKNREF,XKN,ALLXFLX(20,4,3),
     H  ICHE,IOPTHT(20),IOPTL,IOPTPO(20),IOPTRE(20),
     I  IOPTTH(20),IOPT1,IOPT2,IOPT20,IOPT21,IOPT3,IOPT30,IOPT40,
     J  IOPT6,IOPT7,IOPT8,IOPT9,IPLFL(20),IPOS,
     K  ISET(20,20),KLAST(20),LLAST(20),MHTW,MREC,MTHT,NUMOPT,ICKN
C>>>>R LINE D AND E ABOVE
C>>>>R LINE B ABOVE
C>>>3I
C
      COMMON /OPT3/ DRTHP,F,RHELNO
C
C
      DIMENSION DFLUX(4),HREFP(16),NRMIN(10),NRTOT(10),TFLUX(4),
     1             RHRFP(16),RCEFF(16),PIPEFF(16),HREFPN(16)
     2            ,MINR(4),MAXR(4),ANGMN(4),ANGMX(4),HT(4),HB(4),MZ(12)
C
C             DETERMINE MAXIMUM POSSIBLE DESIGN POINT POWER
C
      RTHP=0.
C>>>>DIFLSTP=0
      IOPT=IOPT20
      RTHPMX=0.
C>>>>DDO 110 I=1,NMAX
C>>>>DIVAL=(2+I)*IHOPT
C>>>4R   IF (IVAL.GT.NVALH-2) GO TO 120
      NMX = NMAX
      IF ( IHOPT .EQ. 1 ) NMX = NVALH - 2
      NMX1 = NMX - 2*IHOPT
      DO 110 I = 1,NMX1
      K=KMAX(I)
      L=LMAX(I)
C>>>>RDMIRA=DAREA(K,L)*WTAREA(L)*DENTR(K,L)*THT2
      DMIRA = DAREAF(K,L)*WTAREA(L)*DENTR(K,L)*THT2
C>>>>RIF (IHOPT.GT.0) DMIRA=DSAVE(K,L)
         IF ( IHOPT .EQ. 1 ) DMIRA = DSAVE(I)
      DRTHP=REFTOT(K,L)*REFSOL*DMIRA*1000.
      RTHPMX=RTHPMX+DRTHP/(YSPILL(K,L)+1.E-05)
      RTHP=RTHP+DRTHP
110   CONTINUE
C>>>>DCONTINUE
      RADCON=1.0-(1.0-REFRC)*(ARECRC/RTHP)/(AREF/REFTHP)
      AREC1=AREC
      IF(IRADFL.EQ.1) GO TO  120
      HSAVE=WN*THTI
      HMIX=15.00
      IF(HSAVE.LE.4.0) RE=1.751E5*HSAVE
      IF(HSAVE.LE.4.0) HMIXF=1.0/(HSAVE)*
     1(0.3+0.488*RE**0.5*(1.0+(RE/282000.0)**0.625)**0.8*0.04199)
      IF(HSAVE.LE.4.0) HMIX=(HMIXF**3.2+1.168E3)**(1.0/3.2)
      IF(HSAVE.GT.125.0)
     1HMIX=((33.75*(HSAVE)**(-0.19))**3.2+9.09**3.2)**(1/3.2)
      IF(IREC.EQ.0)RADCON=1.0-ARECRC/RTHP*(REFRC1+REFRC2*HMIX)
      IF(IREC.EQ.1.OR.IREC.EQ.2)RADCON=1.0-ARECRC/RTHP*
     1(REFRC1+REFRC2/(RXN(1)*2.0*THTI)**0.2+REFRC3*AREC/ARECRC)
 120  CONTINUE
      REFP=RADCON*RTHP
      ALPHP=(1.0-REFPIP)*SQRT(REFRC*REFTHP)/REFLP
      PIPE=1.0-ALPHP*FPLH*THTI/SQRT(ABS(REFP))
         IF (ITHEL.EQ.0) GO TO 140
      REFPTH=REFP*PIPE/SMULT
         IF (REFPTH.GE.5.75E+08) GO TO 130
      ETAREF=105.483*ATAN(2.2450E-05*REFPTH)-165.25783+2.77618E-11*REFPT
     1H
      IF (REFPTH.LE.2.5E+07) ETAREF=0.247
         GO TO 140
130   ETAREF=0.43425/(1.0-2.9644E-11*REFPTH)
140   ETATHE=FEFF*ETAREF
      REFP=ETAREF*(1.0-REFPRL)*PIPE*REFP/SMULT
      IF (REFP.GT.PMXH) PMXH=REFP
      SAVE=REFP*RTHPMX/RTHP
      IF (PXHTW.LT.SAVE) PXHTW=SAVE
      SAVE=REFP/1.0E+06
      SAVE2=PXHTW/1.0E+06
      SAVE3=POPT(IOPT)/1.E+06
      WRITE (6,720) SAVE,SAVE2,SAVE3
         IF (REFP.LT.POPT(IOPT)) GO TO 690
      IOPTMX=IOPT
      IF (NUMOPT.GT.0) IOPTMX=1+INT((REFP-POPT(1))/(POPT(2)-POPT(1)))
      IF (IOPTMX.GT.NUMOPT) IOPTMX=NUMOPT
C
C             THIS LOOP ORDERS THE FIELD ZONES FROM MAXIMUM TO
C             MINIMUM PERFORMANCE/COST
C
         IF (IHOPT+ISKIP.GT.0) GO TO 180
      ISKIP=IPOS/2
      DO 170 NM=2,NMAX
      K=KMAX(NM)
      L=LMAX(NM)
      NS=NM-1
150   CONTINUE
      KS=KMAX(NS)
      LS=LMAX(NS)
         IF (HCOST(K,L).LE.HCOST(KS,LS)) GO TO 160
      LMAX(NS+1)=LS
      KMAX(NS+1)=KS
      NS=NS-1
         IF (NS.NE.0) GO TO 150
160   KMAX(NS+1)=K
      LMAX(NS+1)=L
170   CONTINUE
C
C             OPTIMAL FIELD LAYOUT BASED ON
C             PERFORMANCE/COST RANKING
C
180   CONTINUE
      TMIRA=0.0
      TCL=0.0
      TCW=0.0
      TCH=0.0
      TPWRTH=0.0
      TAREA=0.0
      DO 190 I=1,4
      DFLUX(I)=0.
190   TFLUX(I)=0.
      RTHP=0.0
      DO 200 J=1,NHREF
      HREFP(J)=0.0
200   CONTINUE
      REFP=0.0
      DTT=0.
      DCS=0.
      DSL=0.
      DSD=0.
      DBK=0.
      DAM=0.
      RCOS=1.0
      RSPL=1.0
      RATM=1.0
      RSAB=1.0
      IOPT30=1
      REFPWR=POPT(IOPT)*SMULT
      DO 210 L=1,NAZM2P
      NRTOT(L)=0
      NRMIN(L)=16
210   CONTINUE
C>>>>R   IF (IPROPT.EQ.0) GO TO 220
         IF ( IPROPT .NE. 1 ) GO TO 220
      WRITE (6,730)
      WRITE (6,740)
220   CONTINUE
C
C             BUILD UP OPTIMAL FIELD
C
C>>>>RDO 660 I=1,NMAX
      DO 660 I = 1,NMX
      PLAST=REFP
      K=KMAX(I)
      L=LMAX(I)
      NRTOT(L)=NRTOT(L)+1
      IF (K.LE.NRMIN(L)) NRMIN(L)=K
C>>>>RDMIRA=DAREA(K,L)*WTAREA(L)*DENTR(K,L)*THT2
      DMIRA = DAREAF(K,L)*WTAREA(L)*DENTR(K,L)*THT2
C>>>>DIVAL = (2 + I)*IHOPT
C>>>>D   IF (IVAL.GT.NVALH+2) GO TO 670
C>>>>R   IF (IHOPT.EQ.1) DMIRA = DSAVE(K,L)
         IF ( IHOPT .EQ. 1 ) DMIRA = DSAVE(I)
         IF (DMIRA*YTOTAL(K,L).LT.1.E-10) GO TO 660
C>>>>R   IF (IFLSTP.EQ.1) GO TO 680
         IF ( IOPT30 .EQ. 0 ) GO TO 680
C
C             ITERATE FLUX
C
      SAVE = 1.
      DO 240 II=1,NFLXMX
      DFLUX(II)=FLUXMB(K,L,II)*DMIRA
      TFLUX(II)=TFLUX(II)+DFLUX(II)
         IF (TFLUX(II).LT.FLXLIM(II)) GO TO 240
      SAVE=SAVE*(TFLUX(II)-FLXLIM(II))/DFLUX(II)
C>>>>RIFLSTP=1
      IOPT30 = 0
      IIS=II
      DMIRA=DMIRA*SAVE
      TFLUX(II)=FLXLIM(II)
240   CONTINUE
C>>>>I
      DWTSR = DAREAF(K,L)*WTAREA(L)*THT2*SAVE/REALD(K,L)
      DCZ=YTOTAL(K,L)/HCOST(K,L)*HCOST1*DMIRA
C>>>>RDCL=CL*DAREA(K,L)*WTAREA(L)*THT2*SAVE/REALD(K,L)
      DCL = CL*DWTSR
      DCH=CH*DMIRA
      DCHOM=0.
      DCW=DCZ-DCL-DCH-DCHOM
      DPWRTH=YTOTAL(K,L)*YINSOL*DMIRA
      DRTHP=REFTOT(K,L)*REFSOL*DMIRA*1000.0
      DCS=DCS+DMIRA
C
C             ITERATE OPTICAL PERFORMANCE TERMS
C
      SAVEIN=YCOSI(K,L)*DMIRA
      DSD=DSD+SAVEIN
      SAVEIN=SAVEIN*YSHAD(K,L)
      DBK=DBK+SAVEIN
      SAVEIN=SAVEIN*YBLOCK(K,L)
      DAM=DAM+SAVEIN
      SAVEIN=SAVEIN*YATMOS(K,L)
      DSL=DSL+SAVEIN
      DTT=DTT+SAVEIN*YSPILL(K,L)
      TCL=TCL+DCL
      TCW=TCW+DCW
      TCH=TCH+DCH
C>>>>RTAREA=TAREA+DAREA(K,L)*WTAREA(L)*THT2*SAVE/REALD(K,L)
      TAREA = TAREA + DWTSR
      TMIRA=TMIRA+DMIRA
      THELNO=TMIRA/(DENSMS*ASTAT)
      TPWRTH=TPWRTH+DPWRTH
      YTOT=TPWRTH/(TMIRA*YINSOL)
      RTHP=RTHP+DRTHP
C
C             RADIATION + CONVECTION LOSSES
C
      RADCON=1.0-(1.0-REFRC)*(ARECRC/RTHP)/(AREF/REFTHP)
      IF(IRADFL.EQ.1) GO TO  245
      HSAVE=WN*THTI
      HMIX=15.00
      IF(HSAVE.LE.4.0) RE=1.751E5*HSAVE
      IF(HSAVE.LE.4.0) HMIXF=1.0/(HSAVE)*
     1(0.3+0.488*RE**0.5*(1.0+(RE/282000.0)**0.625)**0.8*0.04199)
      IF(HSAVE.LE.4.0) HMIX=(HMIXF**3.2+1.168E3)**(1.0/3.2)
      IF(HSAVE.GT.125.0)
     1HMIX=((33.75*(HSAVE)**(-0.19))**3.2+9.09**3.2)**(1/3.2)
      IF(IREC.EQ.0)RADCON=1.0-ARECRC/RTHP*(REFRC1+REFRC2*HMIX)
      IF(IREC.EQ.1.OR.IREC.EQ.2)RADCON=1.0-ARECRC/RTHP*
     1(REFRC1+REFRC2/(RXN(1)*2.0*THTI)**0.2+REFRC3*AREC/ARECRC)
 245  CONTINUE
      REFP=RADCON*RTHP
      PIPE=1.0-ALPHP*FPLH*THTI/SQRT(ABS(REFP))
C
C             THERMAL TO ELECTRIC CONVERSION
C
         IF (ITHEL.EQ.0) GO TO 260
      REFPTH=REFP*PIPE/SMULT
         IF (REFPTH.GE.5.75E+08) GO TO 250
      ETAREF=105.483*ATAN(2.24502E-05*REFPTH)-165.25783+2.77618E-11*REFP
     1TH
      IF (REFPTH.LE.2.5E+07) ETAREF=0.247
         GO TO 260
250   ETAREF=0.43425/(1.0-2.9644E-11*REFPTH)
260   ETATHE=FEFF*ETAREF
      REFP=ETAREF*(1.0-REFPRL)*PIPE*REFP
      DO 270 J=1,NHREF
      HREFP(J)=HREFP(J)+HREFIN(J,K,L)*DMIRA*1000.0
270   CONTINUE
C>>>>R   IF (IPROPT.EQ.0) GO TO 280
         IF ( IPROPT .NE. 1 ) GO TO 280
      RTHPM=RTHP/1.0E+06
      REFPM=REFP/1.0E+06/SMULT
      WRITE (6,750) I,K,L,YTOTAL(K,L),YTOT,RTHPM,REFPM,THELNO,TAREA,
     1TCL,TCW,TCH,TFLUX(1)
280   CONTINUE
C
C             TEST TO SEE IF THIS ZONE HAS TAKEN THE FIELD PAST
C             AN OUTPUT POWER LEVEL OF INTEREST
C
         IF (REFP.LT.REFPWR) GO TO 660
C
C             DESIGN POINT
C
290   CONTINUE
      IOPT40=1
C
C             F IS THE FRACTION OF THE LAST ZONE ADDED THAT MUST
C             BE ELIMINATED IN ORDER TO BE AT THE POWER LEVEL OF
C             INTEREST
C
      F=(REFP-REFPWR)/(REFP-PLAST)
      RCL=TCL-F*DCL
      RCW=TCW-F*DCW
      RCH=TCH-F*DCH
C>>>>RRAREA=TAREA-F*DAREA(K,L)*WTAREA(L)*THT2*SAVE/REALD(K,L)
      RAREA = TAREA - F*DWTSR
      RMIRA=TMIRA-F*DMIRA
      RHELNO=(TMIRA-F*DMIRA)/(DENSMS*ASTAT)
C
C             INTERPOLATION FOR HELIOSTAT DENSITY OPTIMIZATION
C
         IF (IHOPT.EQ.0) GO TO 320
      SAVE=HCSDP(K,L,NUMFF+1)*REFSOL*1000.
         IF (SAVE.LT.F*DRTHP) GO TO 310
C
C             TRIM LAST ZONE ADDED
C
      F3=F*DRTHP/SAVE
      RCW=TCW-F3*HCSDP(K,L,NUMFF+4)
      RCH=TCH-F3*HCSDP(K,L,NUMFF+2)*CH
      RHELNO=(TMIRA-F3*HCSDP(K,L,NUMFF+2))/(DENSMS*ASTAT)
         GO TO 300
310   CONTINUE
C
C             ELIMINATE LAST ZONE ADDED + DECREASE ALL ZONE DENSITIES
C
      F3=(F*DRTHP-SAVE)/(DRTHP-SAVE)
      RCW=TCW-F3*(DCW-HCSDP(K,L,NUMFF+4))-HCSDP(K,L,NUMFF+4)
      RCH=TCH-F3*(DCH-CH*HCSDP(K,L,NUMFF+2))-CH*HCSDP(K,L,NUMFF+2)
      RHELNO=(TMIRA-HCSDP(K,L,NUMFF+2)-F3*(DMIRA-HCSDP(K,L,NUMFF+2)))/
     1(DENSMS*ASTAT)
      F3=1.0
  300 CONTINUE
      RCL = TCL - F3*DCL
C>>>>RRAREA = TAREA - F3*DAREA(K,L)/REALD(K,L)*THT2*WTAREA(L)
      RAREA = TAREA - F3*DWTSR
320   CONTINUE
      RPWRTH=TPWRTH-F*DPWRTH
      RRTHP=RTHP-F*DRTHP
C
C  ADJUST ANNUAL FIELD EFFICIENCIES
C
      SAVEIN=F*DMIRA
      FDCS=DCS-SAVEIN
      SAVEIN=SAVEIN*YCOSI(K,L)
      FDSD=DSD-SAVEIN
      SAVEIN=SAVEIN*YSHAD(K,L)
      FDBK=DBK-SAVEIN
      SAVEIN=SAVEIN*YBLOCK(K,L)
      FDAM=DAM-SAVEIN
      SAVEIN=SAVEIN*YATMOS(K,L)
      FDSL=DSL-SAVEIN
      FDTT=DTT-SAVEIN*YSPILL(K,L)
      RCOS=FDSD/FDCS
      RSHD=FDBK/FDSD
      RBLK=FDAM/FDBK
      RATM=FDSL/FDAM
      RSPL=FDTT/FDSL
      RSAB=RSHD*RBLK
C
C             ANNUAL RADIATION + CONVECTION LOSSES
C
      RADCS=1.0-(1.-RADCON)*RTHP/(RTHP-F*DRTHP)
      PIPES=1.0-(1.-PIPE)*RTHP/(RTHP-F*DRTHP)
      ELRCTO=RRTHP*(1.0-RADCS)*HROP
      ELPTOT=RADCS*RRTHP*(1.0-PIPES)*HROP
      RCAVG=(RPWRTH*1000.-ELRCTO)/(RPWRTH*1000.)
      PIAVG=(RPWRTH*1000.-ELRCTO-ELPTOT)/(RPWRTH*1000.-ELRCTO)
      DO 330 J=1,NHREF
      RHRFP(J)=HREFP(J)-F*HREFIN(J,K,L)*DMIRA*1000.0
330   CONTINUE
C
C             CALCULATE POWER-LEVEL DEPENDENT COSTS
C
C             SIZE STORAGE
C
      PTH=POPT(IOPT)/ETAREF/(1.0-REFPRL)
      ESTOR=0.0
         IF (SMULT.EQ.1.) GO TO 360
      DO 350 J=1,NHREF
      RCEFF(J)=1.0-(1.0-REFRC)*(ARECRC/RHRFP(J))/(AREF/REFTHP)
      IF(IRADFL.EQ.1) GO TO  355
      HSAVE=WN*THTI
      HMIX=15.00
      IF(HSAVE.LE.4.0) RE=1.751E5*HSAVE
      IF(HSAVE.LE.4.0) HMIXF=1.0/(HSAVE)*
     1(0.3+0.488*RE**0.5*(1.0+(RE/282000.0)**0.625)**0.8*0.04199)
      IF(HSAVE.LE.4.0) HMIX=(HMIXF**3.2+1.168E3)**(1.0/3.2)
      IF(HSAVE.GT.125.0)
     1HMIX=((33.75*(HSAVE)**(-0.19))**3.2+9.09**3.2)**(1/3.2)
      IF(IREC.EQ.0)RCEFF(J)=1.0-ARECRC/RHRFP(J)*(REFRC1+REFRC2*HMIX)
      IF(IREC.EQ.1.OR.IREC.EQ.2)RCEFF(J)=1.0-ARECRC/RHRFP(J)*
     1(REFRC1+REFRC2/(RXN(1)*2.0*THTI)**0.2+REFRC3*AREC/ARECRC)
 355  CONTINUE
      PIPEFF(J)=1.0-ALPHP*FPLH*THTI/SQRT(ABS(RCEFF(J)*RHRFP(J)))
      HREFPN(J)=RHRFP(J)*RCEFF(J)*PIPEFF(J)
         IF (J.EQ.1) GO TO 350
         IF (HREFPN(J-1).LT.PTH) GO TO 360
      DT=HRDEL
      F1=HREFPN(J-1)-PTH
      F2=HREFPN(J)-PTH
         IF (HREFPN(J).GT.PTH) GO TO 340
      DT=(HREFPN(J-1)-PTH)/(HREFPN(J-1)-HREFPN(J))*DT
      F2=0.0
340   ESTOR=ESTOR+(F1+F2)*DT/2.0
350   CONTINUE
360   CONTINUE
      ESTOR=ESTOR*2.
      HRSTOR=ESTOR/PTH
      AVGPRL=(HROP/365.+FSP*HRSTOR)*REFPRL/(HROP/365.+HRSTOR)
      TNSTOR=1.0
      VSTOR=VSTREF*ESTOR/ESTREF
         IF (VSTOR.LE.VMAX) GO TO 370
      TNSTOR=FLOAT( INT(VSTOR/VMAX))+1.0
      VRATIO=VSTOR/VMAX+1.0
      IF ((VRATIO-TNSTOR).LE.0.001) TNSTOR=TNSTOR-1.0
      VSTOR=VSTOR/TNSTOR
370   CSTOR=TNSTOR*(CSTREF*(1.0+EMPTY/TNSTOR)*(VSTOR/VSTREF)**XST+CSTRMD
     1*(VSTOR/VSTREF))
      CPUMP=CRPU*(PTH*THTI)**XRP+CSPU*PTH**XSP
      CPIPE=THTI*(FPLH*CHPREF+FPLC*CCPREF)*(SMULT*PTH/(SMPI*PPIREF))**
     1(XPI/2.0)
         IF (ICHE.EQ.0) GO TO 420
      RHN=1.0
      SHN=1.0
      EVN=1.0
      PHN=1.0
      ARH=ARHREF*PTH/PRHREF
      ASH=ASHREF*PTH/PSHREF
      AEV=AEVREF*PTH/PEVREF
      APH=APHREF*PTH/PPHREF
         IF (ARH.LE.ARHMAX) GO TO 380
      RHN=FLOAT( INT(ARH/ARHMAX))+1.0
      RHRATI=ARH/ARHMAX+1.0
      IF ((RHRATI-RHN).LE.0.001) RHN=RHN-1.0
      ARH=ARH/RHN
380      IF (ASH.LE.ASHMAX) GO TO 390
      SHN=FLOAT( INT(ASH/ASHMAX))+1.0
      SHRATI=ASH/ASHMAX+1.0
      IF ((SHRATI-SHN).LE.0.001) SHN=SHN-1.0
      ASH=ASH/SHN
390      IF (AEV.LE.AEVMAX) GO TO 400
      EVN=FLOAT( INT(AEV/AEVMAX))+1.0
      EVRATI=AEV/AEVMAX+1.0
      IF ((EVRATI-EVN).LE.0.001) EVN=EVN-1.0
      AEV=AEV/EVN
400      IF (APH.LE.APHMAX) GO TO 410
      PHN=FLOAT( INT(APH/APHMAX))+1.0
      PHRATI=APH/APHMAX+1.0
      IF ((PHRATI-PHN).LE.0.001) PHN=PHN-1.0
      APH=APH/PHN
410   CHTXCH=RHN*CRHREF*(ARH/ARHREF)**XHEA+SHN*CSHREF*(ASH/ASHREF)**XHEA
     1+EVN*CEVREF*(AEV/AEVREF)**XHEA+PHN*CPHREF*(APH/APHREF)**XHEA
         GO TO 430
420   CHTXCH=CHEREF*(PTH/PHEREF)**XHEP
430   CONTINUE
      CKNAX=CKNREF*(PTH*SMULT/PKNREF)**XKN
      CHTXCH=CHTXCH+CKNAX
      CEPGS=CEGREF*(PTH*ETAREF/PEGREF)**XEPGS
C 2-27-85    HEL COST ALGORITHM CHANGED TO ADD BCS AND WEATHER EQUIPMENT
C     AND TO SUBTRACT COST OF WIRING FROM QUOTED HELIOSTAT PRICE
      RCH=RCH+1.0E6-RCW
      RCL=RCL*1.3+0.18E6*CL
      CCH=RCL+RCH+RCW
C
C             CALCULATE CAVITY RECEIVER COST FROM OPTIMIZED FIELD
C
         IF (IREC.LT.1) GO TO 500
         IF (IREC.GT.2) GO TO 500
      AREC=0.
      DO 440 MM=1,NUMCAV
      ANGMN(MM)=0.
      ANGMX(MM)=0.
      MINR(MM)=16
      MAXR(MM)=0
440   CONTINUE
C
C             FIND MINIMUM AND MAXIMUM RADIAL ZONES FOR EACH CAVITY
C
      DO 460 LAZM=1,NAZM
      N=NREC(LAZM)
         IF (N.EQ.0) GO TO 460
         IF (LAZM.GT.NAZM2P) GO TO 450
      LT=MZONE(LAZM)
      MZ(LT)=LAZM
      LC=LAZM
450   IF (LAZM.GT.NAZM2P) LC=MZ(LAZM)
      DO 477 MM=1,N
      IC=JREC(LAZM,MM)
      IF (MINR(IC).GT.NRMIN(LC)) MINR(IC)=NRMIN(LC)
          IF (NRMIN(LC).EQ.16) GO TO 477
      IF (MAXR(IC).LT.NRMIN(LC)+NRTOT(LC)-1) MAXR(IC)=NRMIN(LC)+NRTOT(LC
     1)-1
477   CONTINUE
460   CONTINUE
C
C             CALCULATE ACTUAL AREA OF HEAT ABSORBING SURFACES WITHIN
C             CAVITY
C
      DO 490 MM=1,NUMCAV
      ANGCAV=0.
      MN=MINR(MM)
      RMN=RADIUS(MN)
      MX=MAXR(MM)
         IF (MX.LT.MN) GO TO 490
      RMX=RADIUS(MX)
      W2D=RWCAV(MM)*W2
      ANG=180.-RELV(MM)
      HT(MM)=(1.+RYN(MM)*SIN(ANG*RR))*(RMN-W2+W2D)/(RMN-W2-RYN(MM)*COS
     1(ANG*RR))-1.
      HB(MM)=(1.-RYN(MM)*SIN(ANG*RR))*(RMX-W2+W2D)/(RMX-W2+RYN(MM)*COS
     1(ANG*RR))-1.
      HT1=HB(MM)+HN*RYN(MM)
      IF(HT1.GT.HT(MM)) WRITE (6,*) 'WARN**MAX--DEL HCAV USED'
      IF(HT1.LT.HT(MM)) HT(MM)=HT1
      HD=HT(MM)-HB(MM)
      CANG=RAZM(MM)-180.
      IF (CANG.LT.0.) CANG=CANG+360.
      IF (CANG.GE.360.) CANG=CANG-360.
      ANG=CAZMMN(MM)
      ICK=0
470   ANG=ABS(CANG-ANG)
      IF (ANG.GT.90.) ANG=360.-ANG
      BSIN=RMN*COS(ANG*RR)-W2
      BCOS=RMN*SIN(ANG*RR)+RXN(MM)
      B=(BCOS*BCOS+BSIN*BSIN)**0.5
      BP=W2D-RXN(MM)
      A=ASIN(BP*BSIN/(B*W2D))/RR
      ANGCAV=ANGCAV+90.-A
         IF (ICK.EQ.1) GO TO 480
      ANGMX(MM)=180.-A
      ANG=CAZMMX(MM)
      ICK=1
         GO TO 470
480   ANGMN(MM)=A
      AREC=AREC+RR*W2D*HD*THT2*ANGCAV
      IF(MM.EQ.1)AREC2=HD*THT2*RR*180.0*W2D
490   CONTINUE
      IF(ABS((AREC2-AREC1)/AREC2).GT.1.E-06.AND.IRADFL.EQ.0)
     1WRITE (6,*) 'ERROR*****ACTUAL REC AREA IS LESS THAN USED FORLOSS'
C
C             CALCULATE COST OF OPTIMIZED CAVITY RECEIVER
C
      CREC=CREC1*(AREC/ARECRF)**XREC
500   CONTINUE
      CTHERM=CCH+CTOW+CREC+CPIPE+CPUMP
      CCBAL=CTOW+CREC+CPIPE+CSTOR+CPUMP+CHTXCH+CEPGS
      CC=CCH+CCBAL
      CFIXED1=CFIXED+2.E6+0.14*CC+93.0*POPT(IOPT)/1.E3
      IF(IPH.NE.0)CFIXED1=CFIXED+2.E6+0.14*CC+93.0*0.32*POPT(IOPT)/1.E3
      CCBAL=CCBAL+CFIXED1
      CC=CC+CFIXED1
      CI=CCMUL*CC
      CINVY=CCMULY*CC
      CINVC=CCMULC*CC
C>>>>RCINVYH=CINVY*CCH/CC
      CINVYH = CCMULY*CCH
C>>>>RCINVYS=CINVY*CSTOR/CC
      CINVYS = CCMULY*CSTOR
      ACWOSY=(FCR+RNHOML)*(CINVY-CINVYS-CINVYH)
      ACHY=(FCR+RHOML)*CINVYH
      ACWOSC=ACWOSY*CCMULC/CCMULY
      ACHC=ACHY*CCMULC/CCMULY
      ACSY=(FCR+RNHOML)*CINVYS
      ACSC=ACSY*CCMULC/CCMULY
      TKWHR=PF*RCAVG*PIAVG*ETATHE/SMULT*((1.0-REFPRL)+(1.0-FSP*REFPRL)*(
     1SMULT-1.0)*EFFSTR)*RPWRTH
      HRNIGHT=(SMULT*(-HROP)+24.*365.-(HRSTRT+HRPRE+HRWEATH))
      PARSTRT=HRSTRT*(PARL1+PARL2*POPT(IOPT)/1.E6)*1000.0*PF
      PARPRE=HRPRE*(PARL3+PARL4*POPT(IOPT)/1.E6)*1000.0*PF
      PARNITE=HRNIGHT*(PARL5+PARL6*POPT(IOPT)/1.E6)*1000.0*PF
      PARWEAT=HRWEATH*(PARL7+PARL8*POPT(IOPT)/1.E6)*1000.0*PF
      PARPF=24.*365.*(1.-PF)*(PARL9+PARL10*POPT(IOPT)/1.E6)*1000.0
      TKWHR=TKWHR-(PARSTRT+PARPRE+PARNITE+PARWEAT+PARPF)
         IF (TKWHR.LE.0.) GO TO 630
      BBECY=(ACWOSY+ACHY+ACSY)/TKWHR*1000.0
      BBECC=(ACWOSC+ACHC+ACSC)/TKWHR*1000.0
      C=100.0/CC
C
C             SAVE MINIMUM ENERGY COSTS FOR SMART SEARCH
C
      IF (BBECY.LT.BBHTW(IOPT)) BBHTW(IOPT)=BBECY
      IF (BBECY.LT.BBW(IOPT)) BBW(IOPT)=BBECY
      IOPTL=IOPT
C
C             TEST TO SEE IF ENERGY COST IS A NEW MINIMUM
C
         IF (BBECY.GT.ALLBUA(IOPT)) GO TO 630
      MM=IOPTTH(IOPT)
      NN=IOPTRE(IOPT)
      IOPTTH(IOPT)=MTHT
      IOPTHT(IOPT)=MHTW
      IOPTPO(IOPT)=IPOS
      IOPTRE(IOPT)=MREC
      ISET(MM,NN)=ISET(MM,NN)-1
      ISET(MTHT,MREC)=ISET(MTHT,MREC)+1
         IF (IREC.NE.1.AND.IREC.NE.2) GO TO 520
      DO 510 MM=1,NUMCAV
      HTOP(IOPT,MM)=HT(MM)*THTI
      HBOT(IOPT,MM)=HB(MM)*THTI
      ANGMIN(IOPT,MM)=ANGMN(MM)
      ANGMAX(IOPT,MM)=ANGMX(MM)
510   CONTINUE
520   CONTINUE
C     2-28-85 AREA CHANGE TO INCLUDE TOWER LAND AND ROADS AROUND PERIM
C      AS PER H NORRIS MEMO
      RAREA=RAREA*1.3+0.18E6
      HSTSAV(IOPT)=RHELNO
      ARESAV(IOPT)=RAREA/1.0E+06
      ALLCOS(IOPT)=RCOS
      ALLSAB(IOPT)=RSAB
      ALLATM(IOPT)=RATM
      ALLSPL(IOPT)=RSPL
      ALLRCR(IOPT)=RCAVG
      ALLRCP(IOPT)=PIAVG
      ALLETA(IOPT)=ETATHE
      ALLTOB(IOPT)=ALLCOS(IOPT)*ALLSAB(IOPT)*ALLATM(IOPT)*ALLSPL(IOPT)*R
     1CAVG*PIAVG*RREFL
      ALLTOA(IOPT)=ALLTOB(IOPT)*ETATHE
      ALLHR(IOPT)=HRSTOR
      DO 530 II=1,4
530   ALLFLU(IOPT,II)=TFLUX(II)-F*DFLUX(II)
      DO 525 JJ=1,NFLXMX
      II=NMXFLX(JJ)
      ALLXFLX(IOPT,JJ,1)=XFLX(II)*THTI
      ALLXFLX(IOPT,JJ,2)=YFLX(II)*THTI
      ALLXFLX(IOPT,JJ,3)=ZFLX(II)*THTI
 525  CONTINUE
      ALLPCL(IOPT)=C*RCL
      ALLPCW(IOPT)=C*RCW
      ALLPCH(IOPT)=C*RCH
      ALLPCT(IOPT)=C*CTOW
      ALLPCR(IOPT)=C*CREC
      ALLPCP(IOPT)=C*CPIPE
      ALLPCS(IOPT)=C*CSTOR
      ALLPCB(IOPT)=C*CPUMP
      ALLPCE(IOPT)=C*CEPGS
      ALLPCA(IOPT)=C*CHTXCH
      ALLPCF(IOPT)=C*CFIXED1
      ALLCC(IOPT)=CC/1.0E+06
      ALLBUA(IOPT)=BBECY
      ALLBUS(IOPT)=BBECC
      ALLCI(IOPT)=CI/1.0E+06
      ALLCIY(IOPT)=CINVY/1.0E+06
      ALLCIC(IOPT)=CINVC/1.0E+06
      ALLKWH(IOPT)=TKWHR
      ALLTC(IOPT)=CTHERM
      ALLTP(IOPT)=(RTHP-F*DRTHP)*RADCS*PIPES
      CWOSYL(IOPT)=ACWOSY
      CWOSCL(IOPT)=ACWOSC
      CWOSYH(IOPT)=ACHY
      CWOSCH(IOPT)=ACHC
      ALLSTR(IOPT)=ESTOR
      SAVE2=ALLFLU(IOPT,1)/1.E+06
      SAVE=POPT(IOPT)/1.E+06
      SAVE3=ALLFLU(IOPT,2)/1.E+06
      SAVE4=ALLFLU(IOPT,3)/1.E+06
      SAVE5=ALLFLU(IOPT,4)/1.E+06
      WRITE (6,760) SAVE,ALLTOA(IOPT),SAVE2,SAVE3,SAVE4,SAVE5
     1,ALLKWH(IOPT),ALLCC(IOPT),ALLBUA(IOPT)
C
C             SAVE FIELD DATA AT THIS POWER IF USER ASKED FOR IT
C
         IF (IPLFL(IOPT).EQ.0) GO TO 610
      II=IPLFL(IOPT)
C
C             FLAST(KLAST,LLAST) IS FRACTION OF LAST ZONE IN FIELD
C
      KLAST(II)=K
      LLAST(II)=L
      FLAST(II)=1.-F
C>>>>D   IF (IHOPT.LT.1) GO TO 550
C>>>>DSAVE=HCSDP(K,L,NUMFF+1)*REFSOL*1000.
C>>>>D   IF (SAVE.LT.F*DRTHP) GO TO 540
C>>>>DFLAST(II)=1.-F*DRTHP/SAVE
C>>>>DF2=1.0
C>>>>D   GO TO 550
C>>>>DFLAST(II)=1.0
C>>>>DKLAST(II)=KMAX(I-1)
C>>>>DLLAST(II)=LMAX(I-1)
C>>>>DF2=(1.0-F)*DRTHP/(DRTHP-SAVE)
C>>>>DCONTINUE
C>>>>DI1 = II + NUMFF4
C>>>>DI2 = II + NUMFF3
      DO 600 M=1,NAZM2P
      ILLNR(II,M)=NRTOT(M)
      ILLRMI(II,M)=NRMIN(M)
      IF (ILLRMI(II,M).EQ.16) ILLRMI(II,M)=0
C>>>>D   IF (IHOPT.EQ.0.OR.ILLNR(II,M).EQ.0) GO TO 600
C>>>>DDO 560 JJ=1,NRAD
C>>>>DHCSDP(JJ,M,I2)=SCS(JJ,M,1+NUMFF3)
C>>>>DHCSDP(JJ,M,I1)=SCS(JJ,M,1+NUMFF)
C>>>>DIF (L.EQ.M.AND.F2.LT.1.0) ILLNR(II,L)=ILLNR(II,L)-1
C>>>>DIF (L.EQ.M.AND.F2.LT.1.0.AND.K.EQ.ILLRMI(II,L)) ILLRMI(II,L)=ILLRM
C>>>>DI(II,L)+1
C>>>>DIF (ILLNR(II,M).EQ.0) ILLRMI(II,M)=0
C>>>>DVAL=HVAL(I+2)+(1.0-F2)*(HVAL(I+1)-HVAL(I+2))
C>>>>DIII=ILLRMI(II,M)
C>>>>DJ=ILLNR(II,M)+III-1
C>>>>D   IF (J.LT.1.OR.J.GT.NRAD) GO TO 600
C>>>>DDO 590 JJ=III,J
C>>>>DDO 570 MM=2,NUMFF
C>>>>DMMM=NUMFF-MM+1
C>>>>D   IF (HCSDP(JJ,M,MMM+NUMFF2).GT.VAL) GO TO 580
C>>>>DCONTINUE
C>>>>DCONTINUE
C>>>>DMMM1=MMM+1
C>>>>DG=(VAL-HCSDP(JJ,M,MMM1+NUMFF2))/(HCSDP(JJ,M,MMM+NUMFF2)-HCSDP(JJ,
C>>>>DM,MMM+NUMFF2+1))
C>>>>DIF (G.GT.1.0) G=1.0
C>>>>DG=1.-G
C>>>>DHCSDP(JJ,M,I1)=(1.-G)*SCS(JJ,M,MMM+NUMFF)+G*SCS(JJ,M,MMM1+NUMFF)
C>>>>DHCSDP(JJ,M,I2)=(1.-G)*SCS(JJ,M,MMM+NUMFF3)+G*SCS(JJ,M,MMM1+NUMFF3)
C>>>>DCONTINUE
600   CONTINUE
         IF ( IHOPT .EQ. 1 ) CALL OPTDEN(I,II,K,L)
610   CONTINUE
      IOPT1=1
      IOPT2=1
      IOPT3=1
      IOPT8=1
      IOPT9=1
C>>>>DIOPT5=1
      IF (IOPT.LT.IOPT21) IOPT21=IOPT
         IF (IOPT.LT.NUMOPT) GO TO 640
      IOPT7=1
      IOPT6=1
         GO TO 700
630   CONTINUE
C>>>>R   IF (IPROP2.EQ.0) GO TO 640
         IF ( IPROPT .EQ. 0 ) GO TO 640
      SAVE=POPT(IOPT)/1.E+06
      SAVE2=(TFLUX(1)-F*DFLUX(1))/1.E+06
      SAVE3=RCOS*RSAB*RATM*RSPL*RCAVG*PIAVG*RREFL*ETATHE
      SAVE4=CC/1.E+06
      SAVE5=(TFLUX(2)-F*DFLUX(2))/1.E+06
      SAVE6=(TFLUX(3)-F*DFLUX(3))/1.E+06
      SAVE7=(TFLUX(4)-F*DFLUX(4))/1.E+06
      WRITE (6,770) SAVE,SAVE3,SAVE2,SAVE5,SAVE6,SAVE7,TKWHR,SAVE4,
     1BBECY
640   CONTINUE
C
C             ITERATE USERS OUTPUT DESIGN POWER
C
      IOPT=IOPT+1
         IF (IOPT.GT.IOPTMX) GO TO 700
      REFPWR=SMULT*POPT(IOPT)
C
C             SEE IF NEW POWER CAN BE MET WITHOUT ADDING A NEW ZONE
C
         IF (REFP.GE.REFPWR) GO TO 290
660   CONTINUE
C>>>>DCONTINUE
      SAVE=REFP/1.E+06/SMULT
      WRITE (6,780) SAVE
         GO TO 710
680   CONTINUE
      SAVE=REFP/1.0E+06/SMULT
      WRITE (6,790) IIS,SAVE,BBECY
C>>>>DIOPT30=0
         GO TO 710
690   CONTINUE
C     SAVE=REFP/1.E+06/SMULT
C     THE ABOVE LINE IS WRONG, SINCE AT THAT POINT IN CALCULATIONS
C     REFP IS STILL DIVIDED BY SMULT--KISTLER--2/8/85
      SAVE=REFP/1.E+06
      WRITE (6,800) SAVE,SAVE3
         GO TO 710
700   CONTINUE
      SAVE=REFP/1.E+06/SMULT
      WRITE (6,810) SAVE
710   CONTINUE
C>>19I
         IF ( IHOPT .EQ. 0 ) GO TO 990
      DO 980 M = 1,NVALH
      K = KMAX(M)
      L = LMAX(M)
      YCOSI(K,L) = DCOS(M)
      YSHAD(K,L) = DSHD(M)
      YBLOCK(K,L) = DBLK(M)
      YATMOS(K,L) = DATM(M)
      YSPILL(K,L) = DSPL(M)
      YTOTAL(K,L) = DTOT(M)
      HCOST(K,L) = HCST(M)
      DO 960 I = 1,NFLXMX
      FLUXMB(K,L,I) = FLXM(I,M)
  960 CONTINUE
      DO 970 I = 1,NHREF
      HREFIN(I,K,L) = HREFS(I,M)
  970 CONTINUE
  980 CONTINUE
  990 CONTINUE
      RETURN
720   FORMAT (5X,33H MAX. DESIGN PT. PWR - THIS REC.=,F8.2,20H ANY REC.,
     1 THIS THT=,F8.2,28H, MINIMUM POWER OF INTEREST=,F8.2)
730   FORMAT (' RANK    ZONE   ZONE  FIELD   REF.THERM.    DESIGN',
     1' NO.HEL',6X,'AREA',5X,'LAND COST   WIRE COST   HEL. COST',6X,
     2'TOT.PK.FLUX')
740   FORMAT (7X,'RAD AZM  EFF.   EFF.   POWER(MWTH) POWER(MWE)',16X,
     1'(SQ.M)',7X,3(3H($),9X),'(MWTH/SQ.M)')
750   FORMAT (2X,I3,2(2X,I2),2X,F5.3,1X,F5.3,3X,7(E10.4,2X),3X,E10.4)
760   FORMAT (17H OPTIMUM - POWER=,F6.1,9HMWE, EFF=,F5.3,6H FLUX=,4F5.2,
     113HMWT/M2, KWHR=,E11.3,10H, CAPITAL=,F9.2,11HM$, BUSBAR=,F9.2)
770   FORMAT (17H         - POWER=,F6.1,9HMWE, EFF=,F5.3,6H FLUX=,4F5.2,
     113HMWT/M2, KWHR=,E11.3,10H, CAPITAL=,F9.2,11HM$, BUSBAR=,F9.2)
780   FORMAT (/34H ALL ZONES ADDED TO FIELD - POWER=,F7.2)
790   FORMAT (/38H NEXT ZONE WILL EXCEED THE FLUX AT THE,I2,29H MAXIMUM
     1FLUX POINT, DES PWR=,F8.2,12H, LAST BBEC=,F9.2)
800   FORMAT (25H MAXIMUM POSSIBLE POWER =,F7.2,55H MWE THIS IS LESS THA
     1N THE  MINIMUM POWER OF INTEREST =,F7.2,21H MWE NO FIELD BUILDUP)
810   FORMAT (/64H ADDING MORE ZONES WILL NOT REACH NEXT POWER LEVEL - L
     1AST POWER=,F8.2)
      END
      SUBROUTINE OPTDEN(I,II,K,L)
      IMPLICIT REAL*8 (A-H,O-Z)
C>>>>I NEW ROUTINE
C
C          THIS ROUTINE IS USED IF YOU ARE OPTIMIZING DENSITIES.
C
C     THIS ROUTINE IS CALLED BY
C          MAX
C
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /OPT1/ CH,DATM(93),DSAVE(93),FLUXMB(13,7,4),
     1  FLXM(4,93),HCOST(13,7),HCOST1,HCST(93),HL,HREFS(16,93),
     2  HVAL(93),HWDA,HWDR,HWR,THT2,
     3  AZMSER(13,12),DENSIR(13,12),DAREAF(13,7),ICOUNT,IHOPT,
     4  IHOPTP,ILLNR(5,10),ILLRMI(5,10),ISKIP,
     5  KMAX(93),LMAX(93),NMAX,NUMFF2,NUMFF3,NUMFF4,NVALH
C
      COMMON /OPT2/ AEVMAX,AEVREF,ALLATM(20),ALLBUA(20),ALLBUS(20),
     1  ALLCC(20),ALLCI(20),ALLCIC(20),ALLCIY(20),ALLCOS(20),
     2  ALLETA(20),ALLFLU(20,4),ALLHR(20),ALLKWH(20),ALLPCA(20),
     3  ALLPCB(20),ALLPCE(20),ALLPCF(20),ALLPCH(20),ALLPCL(20),
     4  ALLPCP(20),ALLPCR(20),ALLPCS(20),ALLPCT(20),ALLPCW(20),
     5  ALLRCP(20),ALLRCR(20),ALLSAB(20),ALLSPL(20),ALLSTR(20),
     6  ALLTC(20),ALLTOA(20),ALLTOB(20),ALLTP(20),ANGMAX(20,4),
     7  ANGMIN(20,4),APHMAX,APHREF,AREC,ARECRC,ARECRF,ARESAV(20),
     8  ARHMAX,ARHREF,ASHMAX,ASHREF,AVGPRL,BBHTW(20),BBW(20),CCPREF,
     9  CEGREF,CEVREF,CFIXED,CHEREF,CHPREF,CL,CPHREF,CREC,CREC1,
     A  CRHREF,CRPU,CSHREF,CSPU,CTOW,CWOSCH(20),CWOSCL(20),CWOSYH(20),
     B  CWOSYL(20),FLAST(20),HBOT(20,4),HSTSAV(20),
     C  HTOP(20,4),
     D  PEGREF,
     E  PEVREF,PHEREF,PMXH,POPT(21),PPHREF,PPIREF,PRHREF,PSHREF,PXHTW,
     F  SMPI,XEPGS,XHEA,XHEP,XPI,XREC,XRP,XSP
     G ,CKNREF,PKNREF,XKN,ALLXFLX(20,4,3),
     H  ICHE,IOPTHT(20),IOPTL,IOPTPO(20),IOPTRE(20),
     I  IOPTTH(20),IOPT1,IOPT2,IOPT20,IOPT21,IOPT3,IOPT30,IOPT40,
     J  IOPT6,IOPT7,IOPT8,IOPT9,IPLFL(20),IPOS,
     K  ISET(20,20),KLAST(20),LLAST(20),MHTW,MREC,MTHT,NUMOPT,ICKN
C
      COMMON /OPT3/ DRTHP,F,RHELNO
C
C
C
C     9-4-85 DEFAULT F2LAST ADDED TO STOP INFINITE LOOP
      F2LAST=1.0
C
      SAVE = HCSDP(K,L,NUMFF+1)*REFSOL*1000.
         IF ( SAVE .LT. F*DRTHP ) GO TO 540
      FLAST(II) = 1. - F*DRTHP/SAVE
      F2 = 1.
         GO TO 550
  540 CONTINUE
      FLAST(II) = 1.
      KLAST(II) = KMAX(I-1)
      LLAST(II) = LMAX(I-1)
      F2 = (1. -F)*DRTHP/(DRTHP - SAVE)
  550 CONTINUE
      DO 552 LL = 1,NAZM2P
         IF ( ILLNR(II,LL) .EQ. 0 ) GO TO 552
      IF ( L .EQ. LL .AND. F2 .LT. 1. ) ILLNR(II,L) = ILLNR(II,L) - 1
      IF ( L .EQ. LL .AND. F2 .LT. 1. .AND. K .EQ. ILLRMI(II,L) )
     C     ILLRMI(II,L) = ILLRMI(II,L) + 1
         IF ( ILLNR(II,LL) .EQ. 0 ) ILLRMI(II,LL) = 0
  552 CONTINUE
      KLST = KLAST(II)
      LLST = LLAST(II)
      FLST = FLAST(II)
      FLAND(KLST,LLST) = FLAND(KLST,LLST)*FLST
      M = MZONE(LLST)
      IF ( M .GT. MZONE(NAZM2P) ) FLAND(KLST,M) = FLAND(KLST,M)*FLST
      I1 = II + NUMFF4
      I2 = II + NUMFF3
  555 CONTINUE
      AMIRRN = 0.
      VAL = HVAL(I+2) + (1. - F2)*(HVAL(I+1) - HVAL(I+2))
      DO 600 M = 1,NAZM2P
         IF ( ILLNR(II,M) .EQ. 0 ) GO TO 600
      MZ = MZONE(M)
      DO 560 J = 1,NRAD
      HCSDP(J,M,I2) = SCS(J,M,1+NUMFF3)
      HCSDP(J,M,I1) = SCS(J,M,1+NUMFF)
  560 CONTINUE
      III = ILLRMI(II,M)
      J = ILLNR(II,M) + III - 1
         IF ( J .LT. 1 .OR. J .GT. NRAD ) GO TO 600
      DO 590 JJ = III,J
      DO 570 MM = 2,NUMFF
      MMM = NUMFF - MM + 1
         IF ( HCSDP(JJ,M,MMM+NUMFF2) .GT. VAL ) GO TO 580
  570 CONTINUE
      GO TO 585
  580 CONTINUE
      MMM1 = MMM + 1
      G = (VAL - HCSDP(JJ,M,MMM1+NUMFF2))/(HCSDP(JJ,M,MMM+NUMFF2)
     C         - HCSDP(JJ,M,MMM1+NUMFF2))
      IF ( G .GT. 1. ) G = 1.
      G = 1. - G
      HCSDP(JJ,M,I1) = (1. - G)*SCS(JJ,M,MMM+NUMFF)
     C                      + G*SCS(JJ,M,MMM1+NUMFF)
      HCSDP(JJ,M,I2) = (1. - G)*SCS(JJ,M,MMM+NUMFF3)
     C                      + G*SCS(JJ,M,MMM1+NUMFF3)
  585 CONTINUE
      RAD = RADIUS(JJ)*THTI
      DF = HCSDP(JJ,M,I1)
      T = HCSDP(JJ,M,I2)
      RSEP = SQRT(T + SQRT(T**2 + 1./DF**2))
      SAVE2 = (FLAND(JJ,M) + FLAND(JJ,MZ))/2.
      SAVE3 = RAD*(FSLIP - 1.)/(RSEP*HM*(1. + FSLIP))
      SAVE3 = (2.*SAVE3 + 1./FSLIP)/(2.*SAVE3 + 1.)
      IF ( ILAY .GT. 1 ) SAVE3 = 1.
      AMIRRN = AMIRRN + DAREA(JJ,M)*DF*DENSMS*WTAREA(M)*SAVE2*SAVE3
     C         /REALD(JJ,M)
  590 CONTINUE
  600 CONTINUE
      HELN = AMIRRN*THT2/(ASTAT*DENSMS)
      IHELN = HELN
      IRHELNO = RHELNO
      IF ( IHELN .EQ. IRHELNO ) RETURN
         IF ( RHELNO .GT. HELN ) GO TO 780
      F2LAST = F2
      F2 = F2/2.
      GO TO 555
  780 CONTINUE
      F2 = (F2 + F2LAST)/2.
      GO TO 555
 
      END
      SUBROUTINE PERCAL
      IMPLICIT REAL*8 (A-H,O-Z)
C
C
C          THIS ROUTINE IS ACCESSED FOR PERFORMANCE RUNS.  IT PERFORMS
C     AN INITIAL, DETAILED FIELD PERFORMANCE CALCULATION FOR A USER
C     DEFINED OR DEFAULT FIELD, OR FOR A CIRCULAR GRID OF ZONES.
C
C
C     THIS ROUTINE IS CALLED BY
C          DELSOL
C
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /DEL4/ H2O,INSOL,IPRINT(9),ITAPE,NUAZ,NUEL,NYEAR,
     1  PRES,SOLCON,UAZ(20),UDAY,UEL(9),UTIME
C
      COMMON /GEN1/ ATM1S,ATM2S,ATM3S,ATM4S,CTAANG(13),CTPANG(13),
     1  DCANT,DIAMF,FAZMAX,FAZMIN,FZMAX,FZMIN,HCANT,
     2  HCS(13,7,50),HEAD1,HEAD11,HEAD12,HEAD2,HEAD3,HEAD4,HPANL,
     3  HXCANT(25),HYCANT(25),
     4  RADMAX,RADMIN,RANGE(13),
     5  RCANT(13),RMIRL,ROUND,SIGAZ,SIGEL,SIGSX,SIGSY,SIGTX,
     6  SIGTY,SIPAK(16),STAANG(13),TOWD,TOWL,WM,WPANL,XFC,XFOCUS,
     7  YFC,YFOCUS,ZFC,DENSMR,
     8  ICANT,ICPANL,IFLX,IFOCUS,IFXOUT(9,20),IDENS,
     9  IROTFL,NCANTX,NCANTY,NUMK(16)
C>>>>R LINE 7 ABOVE
C
C>>>4I
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>>RCOMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),DENSMS,
      COMMON /GEN5/ ASTAT,DAREA(13,12),DBLK(93),DCOS(93),
     1  DSHD(93),DSPL(93),DTOT(93),LZONE(12),REALD(13,12),
     2  REFFLU(13,12),WTAREA(7)
C>>>>R LINE 2 ABOVE
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /FIELDC/ ALP(5),AZM(13),CLE(5),CLN(5),FLAND(13,12),
     1  RR,SLEW(5),SLNS(5),
     2  INORTH,NAZM,NAZM2P,NLAND,NRAD
C
      COMMON /PERF/ HREFIN(16,13,7),REFTOT(13,12),YATMOS(13,12),
     1  YBLOCK(13,7),YCOSI(13,7),YSHAD(13,7),YSPILL(13,7),YTOTAL(13,7)
     2  ,HROP,RREFL,RWCAV(4),NHREF
C>>>>R LINE 2 ABOVE
C
      COMMON /AIM/ IAUTOP,INDC,NAX(13,12),NAY(13,12),NUMPT(13,12),
     1  XAIM(13,12,2),YAIM(13,12,2)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
      COMMON /FLUXXX/ AZMF,POLF,IFLAUT,NXFLX,NYFLX
C
      COMMON /FLUX1/ CFA(169),CFP(169),DIAMFN,FAZMAS,FAZMIS,FLX(169),
     1  FZMAXN,FZMINN,SFA(169),SFP(169),XFCN,XFLX(169),
     2  XNFLX(169),YFCN,YFLX(169),YNFLX(169),ZFCN,ZFLX(169),NFLXPT
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
      COMMON /REC1/ CAZMMN(4),CAZMMX(4),JREC(12,2),NREC(12),RCRA(4),
     1  RSRA(4)
C
C>>>4I
      COMMON /EFF/ ACWOSY,AREF,CCMULC,CCMULY,CSTREF,CSTRMD,EFFSTR,
     1  EMPTY,ESTREF,ETAREF,FCR,FEFF,FLXLIM(4),FPLH,FSP,REFLP,
     2  REFPIP,REFPRL,REFRC,REFRC1,REFRC2,REFRC3,REFTHP,RHOML,RNHOML,
     3  SMULT,VMAX,VSTREF,XST,ITHEL,IRADFL
C
      COMMON /EFF1/ HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5
     *             ,PARL6,PARL7,PARL8,PARL9,PARL10,TPRE,TSTRT
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
      COMMON /PER1/ A(3,3),B(2,2),BIN(7,7),CNA,CNP,DAY(9),DAYDEL,
     1  DR(20),DT(20),FLAY,FY(9),GCANTA,GCANTB,GCANTX,GCANTY,
     2  GRSAVE(15,15),GSAVE(7,7,4),HCOS,HCTHT(16,4),HFLTOT(9,20),
     3  HINSOL(9,20),HXCANN(25),HYCANN(25),
     4  PSAVE(7,7),RRMULT(5,5),RRR(20),
     5  RUS(7,7),SAVESB(5,5),SIGAZ2,SIGEL2,SIGSX2,SIGSY2,SIGTX2,
     6  SIGTY2,SIH1,SIH2,SNA,SNP,SRANGE(7),TIME(9,20),TSUNSE(9),
     7  ZZMULT(5,5),ZZZ(20),
     8  IDP,INEAR,IPRT,JDP,MSTP(7),NH(9),NNEARS
C>>>>R LINE 4 ABOVE
C
C
      DIMENSION AZMAX(12),AZMIN(12),CSAANG(9,20),CSPANG(9,20),
     1  DEC(9),DFLATM(9),DFLBLO(9),DFLCOS(9),DFLSHA(9),DFLSPI(9),
     2  DFLTOT(9),DINSOL(9),FLFLX(169),FLWT(13,12),FTIME(9,20),
     3  HEADIH(4),HFLATM(9,20),HFLBLO(9,20),HFLCOS(9,20),HFLSHA(9,20),
     4  HFLSPI(9,20),NNEARI(5),
     5  NNIHS(5),NUMHIR(50),RSPA(7,7),SAANG(9,20),SSAANG(9,20),
     6  SSPANG(9,20),FACT2(15),LORDER(12)
C
C
      DATA ((RSPA(I,J),I=1,7),J=1,7) / 2.,0.,1.,0.,.75,0.,.625,
     C                                 0.,0.,0.,0.,0.,0.,0.,
     C                                 1.,0.,.25,0.,.125,0.,0.,
     C                                 0.,0.,0.,0.,0.,0.,0.,
     C                                 .75,0.,.125,0.,0.,0.,0.,
     C                                 0.,0.,0.,0.,0.,0.,0.,
     C                                 .625,0.,0.,0.,0.,0.,0. /
C
      DATA SQ2/.7072/
C
C
      NAMELIST /FMTP/   THT,WM,HM,IROUND,NCANTX,NCANTY,ROUND,NRAD,NAZM,
     1                  RMIRL,DENSMR,DENSIT,XFOCUS,YFOCUS,SIGAZ,SIGEL,
     2                  SIGSX,SIGSY,SIGTX,SIGTY,HEAD,IFOCUS,XFOCAL,
     3                  YFOCAL,HRDEL,NYEAR,HCANT,DCANT,RADMIN,RADMAX,
     4                  RCANT,WEATH,DWEATH,IWEATH,ILAY,IATM,ATM1,ATM2,
     5                  ATM3,ATM4,INDC,IROTFL,INSOL,
     6                  NSUN,SUNI,SUNR,INORTH,AMAXN,RAD,
     7                  AZM,AZMSEP
C
C
C
C               FACT2 = 1*3*5...
C
      NOMAX2 = 2*NOMAX
      FACT2(2) = 1.
      DO 130 I = 4,NOMAX2,2
      FACT2(I) = FACT2(I-2)*FLOAT(I-1)
  130 CONTINUE
C
C               BINOMIAL COEFFICIENTS
C
      DO 140 I = 1,NOMAX
      DO 140 J = 1,I
      BIN(I,J) = FACT(I)/FACT(J)/FACT(I-J+1)
  140 CONTINUE
      DO 150 I = 1,NYEAR
      FY(I) = 4./3.
         IF ( I .EQ. 2*(I/2) ) FY(I) = 8./3.
  150 CONTINUE
      FY(NYEAR) = 2./3.
      FY(1) = FY(NYEAR)
      ISABL = 1
         IF ( ILAY .LT. 2 ) ISABL = 0
C>>>>RIUTIME = 0
C>>>>D   IF ( IPROB. EQ. 1 ) IUTIME = 1
C>>>>D   IF ( IPROB .EQ. 2 ) IUTIME = 2
      ITYPE = 0
         IF ( IPROB .EQ. 4 ) ITYPE = 1
C
C               LORDER IS THE ORDER IN WHICH THE AZIMUTHAL ZONES ARE
C               USED
C
      NAZM2M = NAZM/2 - 1
      LORDER(NAZM) = NAZM2P
      LORDER(1) = 1
      L = 1
      LL2 = NAZM + 1
      DO 160 LL = 1,NAZM2M
      L = L + 1
      LORDER(L) = LL + 1
      L = L + 1
      LORDER(L) = LL2 - LL
  160 CONTINUE
C
C               GENERATE NEIGHBORS FOR STAGGER LAYOUT
C
      DT(1) = 2.
      DT(2) = - 2.
      DT(3) = 1.
      DT(4) = 1.
      DT(5) = - 1.
      DT(6) = - 1.
      DT(7) = 0.
      DT(8) = 0.
      DT(9) = 3.
      DT(10) = 3.
      DT(11) = - 3.
      DT(12) = - 3.
      DR(1) = 0.
      DR(2) = 0.
      DR(3) = - .5
      DR(4) = .5
      DR(5) = - .5
      DR(6) = .5
      DR(7) = - 1.
      DR(8) = 1.
      DR(9) = - .5
      DR(10) = .5
      DR(11) = - .5
      DR(12) = .5
C
C               GENERATE NEAREST NEIGHBORS FOR CORNFIELD LAYOUT
C
                  IF ( ILAY .EQ. 2*(ILAY/2) ) GO TO 170
      DR(3) = - 1.
      DR(4) = 1.
      DR(5) = - 1.
      DR(6) = 1.
      DR(9) = - 1.
      DR(10) = 1.
      DR(11) = - 1.
      DR(12) = 1.
      DT(1) = 1.
      DT(2) = - 1.
      DT(9) = 2.
      DT(10) = 2.
      DT(11) = - 2.
      DT(12) = - 2.
  170 CONTINUE
      DO 180 I = 2,NORDER
      MSTP(I) = 1
  180 CONTINUE
      MSTP(1) = ISTEP
      SIGSY2 = SIGSY**2
      SIGSX2 = SIGSX**2
      SIGTX2 = SIGTX**2
      SIGTY2 = SIGTY**2
      SIGEL2 = SIGEL**2
      SIGAZ2 = SIGAZ**2
      CLAT = COS(RR*PLAT)
      SLAT = SIN(RR*PLAT)
      CSTART = COS(RR*ASTART)
      DAYDEL = 0.
C>>>>RIF ( IUTIME .EQ. 0 ) DAYDEL = 365./FLOAT(NYEAR-1)/2.
      IF ( IPROB .NE. 1 .AND. IPROB .NE. 2 )
     C     DAYDEL = 365./FLOAT(NYEAR-1)/2.
      HGAP = 0.
      NORDEA = 2*NORDER
      HEAD6 = 7HFIELDWT
      HEAD7 = 7HNRADMIN
      HEAD8 = 7HNRADMAX
      HEAD23 = 7HHSTATS     
      HEAD84 = 7HFR LAND
C
C     NORMALIZE ATMOSPHERIC ATTENUATION TO UNITS OF TOWER HEIGHT
C
      THTI = THT
      THT2 = THT**2
      ATM1N = ATM1S
      ATM2N = ATM2S*THT
      ATM3N = ATM3S*THT2
      ATM4N = ATM4S*THT**3
      HN = H/THT
      WN = W/THT
      HMN = HM/THT
      WMN = WM/THT
      ASTATN = ASTAT/THT2
      HGAPN = HGAP/THT
      W2 = WN/2.
      DO 190 I = 1,13
      DO 190 J = 1,12
         FLWT(I,J) = 0.
  190 CONTINUE
C
C              MOMENTS OF SUNSHAPE
C
      IF (NSUN .GT. 2) GO TO 820
C
C             NSUN = 1  SUN = 1.-0.5138(R/SUN RADIUS)**4
C
      SSAVE=.5138
      DO 800 I=1,NORDER,2
      JMAX=NORDER-I+1
      SAVE=1.
      IF (I.GT.1) SAVE=FACT2(I-1)
      DO 800 J=1,JMAX,2
      SAVE2=1.
      IF (J.GT.1) SAVE2=FACT2(J-1)
      IJ = I + J
      RUSS=(1./FLOAT(IJ)-SSAVE/FLOAT(IJ+4))/(.5-SSAVE/6.)*SAVE*
     1SAVE2/FACT((IJ)/2)/2**((IJ-2)/2)
C
C             NSUN = 0 IS A POINT SUN OF UNIT INTENSITY
C
      IF (NSUN.EQ.0) RUSS=0.
      RUS(I,J)=RUSS*(4.65E-03)**(IJ-2)
800   CONTINUE
         IF (NSUN.NE.2) GO TO 870
      SAVE = 1.
      DO 810 I=1,NORDER,2
      IF (I.GT.1) SAVE=FACT2(I-1)
      SAVE2 = 1.
      DO 810 J=1,NORDER,2
      IF (J.GT.1) SAVE2=FACT2(J-1)
      IJ = I + J
      RUSS=2.*SAVE*SAVE2/FACT((IJ)/2)/2.**((IJ-2)/2)/FLOAT(IJ)
      RUS(I,J)=RUSS*(4.645E-03)**(IJ-2)
810   CONTINUE
         GO TO 870
820   CONTINUE
C
C             NSUN = 3  USER DEFINED SUNSHAPE
C
      DO 830 N=1,7,2
830   AZMIN(N)=0.
      NN=NSUNPT-1
      DO 840 N=1,NN
      SAVE3=SUNR(N)/(SUNR(N+1)-SUNR(N))
      SAVE4=1./(SUNR(N+1)-SUNR(N))
      DO 840 M=1,7,2
      FM=FLOAT(M+1)
      L=M+1
      SAVE=(SUNR(N+1)**L-SUNR(N)**L)/FM
      SAVE2=(SUNR(N+1)**(M+2)-SUNR(N)**(M+2))/(FM+1.)
840   AZMIN(M)=AZMIN(M)+SUNI(N)*(SAVE*(1.+SAVE3)-SAVE2*SAVE4)+SUNI(N+1)*
     1(-SAVE*SAVE3+SAVE2*SAVE4)
      SAVE=1.
      DO 850 I=1,NORDER,2
      JMAX=NORDER-I+1
      DO 850 J=1,JMAX,2
      IJ = I + J
      RUS(I,J)=AZMIN(IJ-1)*RSPA(I,J)/SAVE*PI
      SAVE=RUS(1,1)
850   CONTINUE
870   CONTINUE
      RUS(1,1)=1.0
C
C             MOMENTS OF ERROR DISTRIBUTION
C
      DO 880 I=1,NORDER
      DO 880 J=1,NORDER
880   PSAVE(I,J)=0.
      SAVE = 1.
      DO 900 I=1,NORDER,2
      IF (I.GT.1) SAVE=FACT2(I-1)
      SAVE2 = 1.
      DO 900 J=1,NORDER,2
      IF (J.GT.1) SAVE2=FACT2(J-1)
      PSAVE(I,J)=SAVE*SAVE2
900   CONTINUE
      WM2=WMN/2.
      HM2=HMN/2.
      DO 920 I=1,NORDER
      JMAX=NORDER-I+1
      JMIN=2-I+2*(I/2)
      DO 920 J=JMIN,JMAX,2
      II=(I-1)/2+1
      DO 920 K=1,II
      SAVE=1.
C>>>>RIF (I+J-2*K.GT.0) SAVE=FACT2(I+J-2*K)
      IF ( I + J .GT. 2*K ) SAVE = FACT2(I+J-2*K)
      GSAVE(I,J,K)=SAVE*FACT(I)/(FACT(I-2*K+2)*FACT(K))
920   CONTINUE
      DO 930 K=1,NORDEA
      DO 930 I=1,NORDEA
      GRSAVE(I,K)=0.
930   CONTINUE
C
C             MOMENTS OF MIRROR SHAPE
C
C             COORDINATES OF CENTER OF CANT PANEL
C
         IF (ICPANL.GT.0) GO TO 950
      DX=(WMN-HGAPN)/FLOAT(NCANTX)
      DY=HMN/FLOAT(NCANTY)
      XMS=-WM2-DX/2.
      K=0
      DO 940 I=1,NCANTX
      XMS=XMS+DX
      IF (I.EQ.NCANTX/2+1) XMS=XMS+HGAPN
      YMS=-HM2-DY/2.
      DO 940 J=1,NCANTY
      YMS=YMS+DY
      K=K+1
      HYCANN(K) = YMS
      HYCANT(K) = HYCANN(K)*THT
      HXCANN(K) = XMS
      HXCANT(K) = HXCANN(K)*THT
940   CONTINUE
      WM2S = (WM2 - HGAPN/2.)/FLOAT(NCANTX)
      HM2S = HM2/FLOAT(NCANTY)
         GO TO 970
950   CONTINUE
      DO 960 I=1,NCANT
      HXCANN(I) = HXCANT(I)/THT
960   HYCANN(I) = HYCANT(I)/THT
      WM2S = WPANL/THT/2.0
      HM2S = HPANL/THT/2.0
970   CONTINUE
      DO 980 K=1,NORDER,2
      DO 980 L=1,NORDER,2
      KL = K*L
      GRSAVE(K,L)=4./FLOAT(KL)*WM2S**K*HM2S**L
980   CONTINUE
C
C             CIRCULAR HELIOSTATS
C
         IF (IROUND.EQ.0) GO TO 1000
      DO 990 K=1,NORDER,2
      DO 990 L=1,NORDER,2
      SAVE=1.
C>>>>RIF (K.GT.1) SAVE=SAVE*FACT2(K-1)
      IF ( K .GT. 1 ) SAVE = FACT2(K-1)
      IF (L.GT.1) SAVE=SAVE*FACT2(L-1)
      KL = K + L
      GRSAVE(K,L)=WM2S**(KL)*PI/FLOAT(KL)*SAVE/FACT((KL)/2)*2.**(1-
     1(KL-2)/2)
990   CONTINUE
1000  CONTINUE
C
C             TIME INDEPENDENT FIELD QUANTITIES
C
      DO 1030 J=1,NAZM
      AZM(J)=FLOAT(J-1)*AZMDEL
      AZMIN(J)=AZM(J)-AZMDEL/2.
      AZMAX(J)=AZM(J)+AZMDEL/2.
         IF (INORTH*J.NE.NAZM2P) GO TO 1010
      AZM(J)=180.
      AZMIN(J)=AMAXN
      AZMAX(J)=360.-AMAXN
1010  CONTINUE
      IF (INORTH*J.GT.NAZM2P) AZM(J)=360.-FLOAT(NAZM+1-J)*AZMDEL
      CTAANG(J)=COS(RR*AZM(J))
      STAANG(J)=SIN(AZM(J)*RR)
      DO 1030 I=1,NRAD
      RADIUS(I)=RADMIN+FLOAT(I-1)*RADDEL
      RANGE(I)=SQRT(1.+(RADIUS(I)-W2)**2)
      RDMIN(I)=RADIUS(I)-RADDEL/2.
      RDMAX(I)=RADIUS(I)+RADDEL/2.
      YATMOS(I,J)=1.0-ATM1N-ATM2N*RANGE(I)-ATM3N*RANGE(I)**2-ATM4N*
     1  RANGE(I)**3
      TPANG=ATAN(RADIUS(I)-W2)
      CTPANG(I)=COS(TPANG)
      STPANG(I)=SIN(TPANG)
      REFFLU(I,J)=0.
      REFTOT(I,J)=0.
1030  CONTINUE
      RDMIN(1)=RADMIN
C>>>>RRDMAX(NAZM)=RADMAX
      RDMAX(NRAD) = RADMAX
C
C             ZERO OUT PERFORMANCE QUANTITIES
C
      AZM(NAZM+1)=AZM(1)
      DO 1070 I=1,NRAD
      DO 1040 J=1,NAZM2P
      YSHAD(I,J) = 0.
      YBLOCK(I,J) = 0.
      YTOTAL(I,J) = 0.
      YSPILL(I,J) = 0.
      YCOSI(I,J) = 0.
 1040 CONTINUE
      DO 1050 J=1,NAZM
      DO 1050 NN=1,25
1050  HCSDP(I,J,NN)=0.
      DO 1070 J=1,7
      DO 1060 NN=1,25
1060  SCS(I,J,NN)=0.
      DO 1070 NN=1,50
      HCS(I,J,NN)=0.0
1070  CONTINUE
      DO 1080 L=1,NAZM
1080  NREC(L)=1
C
C             CAVITY RECEIVERS
C
      WRITE (6,3010)
      WRITE (6,3020) THT
      WRITE (6,3030) TOWL,TOWD
      TOWDN = TOWD/THT
      TOWLN = TOWL/THT
      TOWD2=TOWDN/2.
         IF (IREC.EQ.0) GO TO 1100
      CALL SETCAV
         GO TO 1101
1100  CONTINUE
      WRITE (6,3040) H,W
 1101 CONTINUE
      DO 1105 I=1,NUMCAV
      RXN(I) = RX(I)/THT/2.
      RYN(I) = RY(I)/THT/2.
      RA2(I) = RXN(I)**2
      RB2(I) = RYN(I)**2
 1105 CONTINUE
      IF (IAUTOP.GE.0) GO TO 1115
      WRITE (6,3045) -IAUTOP
      GO TO 1180
 3045 FORMAT(//' ZONES CLOSE TO RECEIVER AIMED IN A 2D ARRAY AS CLOSE',
     1' TO THE BOTTOM OF THE RECEIVER AS POSSIBLE.  STARTING WITH ZONE'
     2,I5,',',/' (RADIALLY OUTWARD), CODE CALCULATES A 2D SMART AIMING'
     3,'FOR EACH OUTER ZONE AT EACH TIME STEP')
 1115 CONTINUE
         IF (IAUTOP.GT.0) GO TO 1120
      WRITE (6,3050)
         GO TO 1180
1120     IF (IAUTOP.GT.2) GO TO 1130
      WRITE (6,3060) IAUTOP
         GO TO 1180
1130     IF (IAUTOP.GT.4) GO TO 1140
      I=IAUTOP-2
      WRITE (6,3070)
      WRITE (6,3080) I
         GO TO 1180
1140  WRITE (6,3090)
      DO 1170 K=1,NRAD
      DO 1170 L=1,NAZM
         IF (IAUTOP.GT.5) GO TO 1150
      WRITE (6,3100) K,L,NAX(K,L),XAIM(K,L,1),XAIM(K,L,2),NAY(K,L),
     1YAIM(K,L,1),YAIM(K,L,2)
         GO TO 1170
1150  NN=NUMPT(K,L)
      DO 1160 N=1,NN
      WRITE (6,3110) K,L,N,XAIM(K,L,N),YAIM(K,L,N)
1160  CONTINUE
1170  CONTINUE
1180  CONTINUE
C
C             AUTOMATIC GENERATION OF THE FLUX POINTS -
C             EXTERNAL RECEIVER
C
C>>>>DDAZM=AZMDEL
      IF (IPROB.EQ.4) IFLX=0
         IF (IFLX.EQ.0) GO TO 1250
C
C             FLUX POINTS SET TO HEAT ABSORBING SURFACE
C
         IF (IFLAUT.NE.4) GO TO 1220
         IF (IREC.GT.0) GO TO 1190
      DIAMF = W
      IFLAUT=1
      XFC=0.
      YFC=0.
      ZFC=0.
         GO TO 1220
1190     IF (IREC.GT.2) GO TO 1200
      DIAMF=W*RWCAV(1)
      IFLAUT=2
         GO TO 1210
1200  IFLAUT=3
      POLF=RELV(1)
      AZMF=RAZM(1)
C>>>>RIF (FAZMIN.GT.100) FAZMIN=0.
      IF ( FAZMIN .GT. 100. ) FAZMIN = 0.
      IF (FAZMAX.GT.100.) FAZMAX=0.
1210  ZFC=0.
      XFC=-W/2.*SIN(RR*RAZM(1))
      YFC=-W/2.*COS(RR*RAZM(1))
1220  CONTINUE
C
C     SET UP FLUX POINTS
C     NORMALIZE FLUX POINTS TO TOWER HEIGHT
C
      FZMINN = FZMIN/THT
      FZMAXN = FZMAX/THT
      XFCN = XFC/THT
      YFCN = YFC/THT
      ZFCN = ZFC/THT
      FAZMIS = FAZMIN
      FAZMAS = FAZMAX
      DIAMFN = DIAMF/THT
      NFLXPT = NXFLX*NYFLX
      NFLSAV = NXFLX
         IF ( NXFLX .GT. 12 ) NFLSAV = 12
      MFLSAV = 1 + NXFLX/13
      CALL SETFLX
      WRITE (6,3120)
      RADF=0.5*DIAMF
         IF (IFLAUT.EQ.1) WRITE (6,3130) RADF,XFC,YFC,ZFC
         IF (IFLAUT.EQ.2) WRITE (6,3140) RADF,XFC,YFC,ZFC
         IF (IFLAUT.EQ.3) WRITE (6,3150) XFC,YFC,ZFC,POLF,AZMF
         IF (IUSERF .NE. 3) WRITE (6,3160)
         IF (IUSERF .EQ. 3) WRITE (6,3170)
         IF (IFLAUT.LT.3) WRITE (6,3750)
         IF (IFLAUT.GT.2) WRITE (6,3180)
      WRITE (6,3760) (XNFLX(N),N=1,NXFLX)
      WRITE (6,3820)
      J1=1-NXFLX
      DO 1230 N=1,NYFLX
      J1=J1+NXFLX
      J2=J1+NXFLX-1
      K1=J1-NFLSAV
      DO 1230 M=1,MFLSAV
      K1=K1+NFLSAV
      K2=K1+NFLSAV-1
      KK=(K2-1)/J2
      K2=K2*(1-KK)+J2*KK
      WRITE (6,3190) YNFLX(N),(KK,KK=K1,K2)
1230  CONTINUE
      WRITE (6,3200)
      WRITE (6,3210)
      WRITE (6,3220)
      DO 1240 N=1,NFLXPT
      SAVE=XFLX(N)*THT
      SAVE2=YFLX(N)*THT
      SAVE3=ZFLX(N)*THT
      ASAVE=ATAN2(SFA(N),CFA(N))/RR
      IF (ASAVE.LT.0.) ASAVE=ASAVE+360.
      SSAVE=ATAN2(SFP(N),CFP(N))/RR
      WRITE (6,3230) N,SAVE,SAVE2,SAVE3,SSAVE,ASAVE
1240  CONTINUE
1250  CONTINUE
      WRITE (6,3280)
C
C             LAND CONSTRAINT
C
         IF (NLAND.EQ.0) GO TO 1290
      WRITE (6,3240)
      WRITE (6,3250) NLAND
      WRITE (6,3260) XTOWER,YTOWER
      XTOWES = XTOWER
      YTOWES = YTOWER
      DO 1260 JJ=1,NLAND
      WRITE (6,3270) JJ,CLE(JJ),CLN(JJ),SLEW(JJ),SLNS(JJ),ALP(JJ)
 1260 CONTINUE
      CALL LAND
      DO 1270 K=1,NRAD
      DO 1270 L=1,NAZM
      FLAND(K,L)=FLAND(K,L)+1.E-10
      DAREA(K,L)=DAREA(K,L)*FLAND(K,L)
1270  CONTINUE
1290  CONTINUE
C
C             CALCULATE WEIGHT OF FIELD POINTS FOR INTEGRATION
C
         IF (IUSERF.EQ.3) GO TO 1380
         IF (IUSERF.EQ.0) GO TO 1330
      NRADMN(NAZM+1)=NRADMN(1)
      NRADMX(NAZM+1)=NRADMX(1)
      FLAREA=0.
      AMIRRN=0.
      DO 1310 J=1,NAZM
      NR1=NRADMN(J)
      NR2=NRADMX(J)
         IF (NR1.EQ.0) GO TO 1310
      DO 1300 I=NR1,NR2
      FL=DAREA(I,J)
      FLWT(I,J)=FL*DENSIT(I,J)
      AMIRRN = AMIRRN + FLWT(I,J)
      FLAREA=FLAREA+FL/REALD(I,J)
1300  CONTINUE
1310  CONTINUE
      AVGDEN=AMIRRN/FLAREA
      AMIRRO = AMIRRN*THT2
      FLAREA=FLAREA*THT2
      FLAREA=1.3*FLAREA+0.18E6
C     CHANGE IN AREA TO ADD CENTER AND ROADS, ETC, BASED ON H. NORRIS
      DO 1320 I=1,NRAD
      DO 1320 J=1,NAZM
      FLWT(I,J)=FLWT(I,J)/AMIRRN
1320  CONTINUE
      HELN=AMIRRN/(ASTATN*DENSMS)
C
C             PRINT OUT FIELD DESCRIPTION
C
      WRITE (6,3290)
      WRITE (6,3300) FLAREA,AMIRRO,HELN,AVGDEN
      WRITE (6,3310) HEAD7,(NRADMN(L),L=1,NAZM)
      WRITE (6,3310) HEAD8,(NRADMX(L),L=1,NAZM)
         GO TO 1350
1330  CONTINUE
      AMIRRO=0.
      FLAREA=0.
      DO 1340 K=1,NRAD
      DO 1340 L=1,NAZM
1340  FLWT(K,L)=1.0
      WRITE (6,3320)
1350  CONTINUE
         IF (IROTFL.GT.0) WRITE (6,3330)
         IF (IATM.EQ.2) WRITE (6,3340)
         IF (ILAY.EQ.0) WRITE (6,3350)
         IF (ILAY.EQ.1) WRITE (6,3360)
         IF (ILAY.EQ.2) WRITE (6,3370)
         IF (ILAY.EQ.3) WRITE (6,3380)
         IF (INORTH.EQ.1) WRITE (6,3390)
      WRITE (6,3450) RADMAX,RADMIN,RADDEL
      WRITE (6,3820)
      WRITE (6,3400) HM
      WRITE (6,3410) WM
      WRITE (6,3820)
      WRITE (6,3420) THT
      WRITE (6,3430)
      WRITE (6,3440)
      WRITE (6,3820)
      WRITE (6,3780)
      WRITE (6,3810) (AZM(L),L=1,NAZM)
      WRITE (6,3820)
      DO 1370 K=1,NRAD
      WRITE (6,3800) RADIUS(K),HEAD5,(DENSIT(K,L),L=1,NAZM)
      WRITE (6,3800) RADIUS(K),HEAD67,(REALD(K,L),L=1,NAZM)
      DO 1360 L=1,NAZM
      REFFLU(K,L)=DAREA(K,L)*DENSIT(K,L)/ASTATN/DENSMS
1360  REFTOT(K,L)=1./(AZMSEP(K,L)*DENSIT(K,L))*DENSMS
      WRITE (6,3800) RADIUS(K),HEAD19,(REFTOT(K,L),L=1,NAZM)
      WRITE (6,3800) RADIUS(K),HEAD20,(AZMSEP(K,L),L=1,NAZM)
      WRITE (6,3800) RADIUS(K),HEAD84,(FLAND(K,L),L=1,NAZM)
      IF (IUSERF.NE.0) WRITE (6,3800) RADIUS(K),HEAD6,(FLWT(K,L),L=1,
     1NAZM)
      WRITE (6,3850) RADIUS(K),HEAD23,(REFFLU(K,L),L=1,NAZM)
      WRITE (6,3820)
1370  CONTINUE
      NNEARS=13
      NRADS=NRAD
      NAZMS=NAZM
      NUMHIR(1)=1
      SIH1=1.
      SIH2=0.
         GO TO 1430
C
C             SET UP FOR A USER DEFINED SET OF INDIVIDUAL
C             HELIOSTAT COORDINATES
C
1380  CONTINUE
      NNEARS=16
      NUMTT=1
      NUMFF=1
      NNIHS(1)=3
      NNIHS(2)=5
      NNIHS(3)=5
      NNIHS(4)=5
      NNIHS(5)=3
      SIH1=0.
      SIH2=1.
      ILAY=4
      IAUTOP=6
      ZZMULT(1,1)=1.0
      RRMULT(1,1)=1.0
      SAVESB(1,1)=1.0
      ISABL=1
      FLAY=0.
      READ (5,2610) (HEADIH(I),I=1,4)
      READ (5,3460) NUMIH,NUMIR
      WRITE (6,3280)
      WRITE (6,3470)
      WRITE (6,3480) (HEADIH(I),I=1,4)
      WRITE (6,3490) NUMIH,NUMIR
      WTIH=1./FLOAT(NUMIH)
      DAREA(1,1)=WMN*HMN*DENSMS
      DENSIT(1,1)=1.0
      FLWT(1,1)=WTIH
      AMIRRO = WM*HM*DENSMS/WTIH
      DO 1390 I=1,5
      I1=2*I-1
      I2=2*I
      NUMHIR(I)=1
      DO 1390 L=1,50
      DO 1390 J=1,6
      HCS(I1,J,L)=1.E+10
      HCS(I2,J,L)=1.E+10
1390  CONTINUE
      NSAVE=5
      IF (NUMIR+2.LT.NSAVE) NSAVE=NUMIR+2
C
C             READ IN FIRST THREE ROWS OF HELIOSTATS
C
      DO 1420 NIR=3,NSAVE
      IJK=2*NIR-1
      READ (5,3460) NR,NUMHIR(NIR)
      NR=NUMHIR(NIR)+2
      DO 1400 NIRH=3,NR
      KJI=NIRH-50*(NIRH/51)
      IF (NIRH.EQ.51) IJK=IJK+1
      READ (5,3500) (HCS(IJK,JJ,KJI),JJ=1,6)
1400  CONTINUE
         IF (NR.LT.8) GO TO 1420
C
C             ADD FIRST(LAST) ROW HELIOSTATS TO BACK(FRONT) END
C             THESE EXTRA HELIOSTATS ARE ONLY USED TO FIND
C             THE NEAREST NEIGHBORS FOR SHADOWING AND BLOCKING
C
      DO 1410 JJ=1,6
      NR1N = (NR+1)/51
      NR2N = (NR+2)/51
      NIR2N = 2*NIR - 1
      NIR1 = NIR2N + NR1N
      NR1 = NR + 1 - 50*NR1N
      NIR2 = NIR2N + (NR - 1)/51
      NR2 = NR - 1 - 50*((NR - 1)/50)
      NIR3 = NIR2N + NR2N
      NR3 = NR + 2 - 50*NR2N
      HCS(2*NIR-1,JJ,2)=HCS(IJK,JJ,KJI)
      HCS(NIR1,JJ,NR1)=HCS(2*NIR-1,JJ,3)
      HCS(2*NIR-1,JJ,1)=HCS(NIR2,JJ,NR2)
      HCS(NIR3,JJ,NR3)=HCS(2*NIR-1,JJ,4)
1410  CONTINUE
1420  CONTINUE
      NRADS=NUMIR
      NAZMS=NUMHIR(3)
1430  CONTINUE
      IF (IPROB.EQ.3) NYEAR=NUEL
      IPRT=1
      SAVE3=0.
      IF (IPROB.EQ.2.OR.IPROB.EQ.3) IPRT=0
         IF ( IWEATH .GT. 0 ) GO TO 1445
      DO 1440 I=1,NYEAR
      DWEATH(I)=WEATH
      DH2O(I)=H2O
      DPRES(I)=PRES
1440  CONTINUE
 1445 CONTINUE
C
C             SUN POSITION AND INSOLATION
C
C             YEARLY INTEGRATION DAYS BEGINS WITH WINTER SOLSTICE
C             (DAY 355)
C
C>>>>RIF (IUTIME.GT.0) NYEAR=1
      IF ( IPROB .EQ. 1 .OR. IPROB .EQ. 2 ) NYEAR = 1
      YINSOL=0.
      HROP=0.0
      HRSTRT=0.0
      HRPRE=0.0
      HRWEATH=0.0
      D=354.75-DAYDEL
      WRITE (6,3510)
C>>>>R   IF (IUTIME.EQ.2) WRITE (6,3570) UDAY,UTIME
         IF ( IPROB .EQ. 2 ) WRITE (6,3570) UDAY,UTIME
C>>>>R   IF (IUTIME.EQ.1) WRITE (6,3520) UDAY
         IF ( IPROB .EQ. 1 ) WRITE (6,3520) UDAY
         IF (IPROB.NE.3) GO TO 1450
      SOLCON=1.00
      INSOL=2
      WRITE (6,3530)
      WRITE (6,3540)
      WRITE (6,3550)
1450  CONTINUE
      HEADI=8HMEINEL    
      IF (INSOL.EQ.1) HEADI=8HHOTTEL  
      IF (INSOL.EQ.2) HEADI=8HCONSTANT 
      IF (INSOL.EQ.3) HEADI=8HMOON         
      IF (INSOL.EQ.4) HEADI=8HALLEN      
      WRITE (6,3560) HEADI
      DO 1570 I=1,NYEAR
      DFLCOS(I)=0.
      DFLSPI(I)=0.
      DFLSHA(I)=0.
      DFLBLO(I)=0.
      DFLATM(I)=0.
      DFLTOT(I)=0.
         IF (IPROB.EQ.3) GO TO 1470
      D=D+DAYDEL
      IF (D.GT.365.) D=D-365.
      DAY(I)=D
C>>>>RIF (IUTIME.GT.0) DAY(I)=UDAY
      IF ( IPROB .EQ. 1 .OR. IPROB .EQ. 2 ) DAY(I) = UDAY
      SAVE=2.*PI/365.24*(DAY(I)+284.)
      SAVE2=2.*SAVE
      SAVE=SAVE+0.007133*SIN(SAVE)+0.032680*COS(SAVE)-0.000318*SIN(SAVE2
     1)+0.000145*COS(SAVE2)
      SDEC=SIN(23.442274*RR)*SIN(SAVE)
      CDEC=SQRT(1.-SDEC**2)
      DEC(I)=ATAN2(SDEC,CDEC)/RR
      DINSOL(I)=0.
      TSUNSE(I)=ACOS((CSTART-SDEC*SLAT)/CLAT/CDEC)/15./RR
      NH(I)=INT(TSUNSE(I)/HRDEL)+2
         IF (IPROB.LT.4.OR.NH(I).LT.17) GO TO 1460
      WRITE (6,3580) I
      W = - 98
        GO TO 2550
1460  CONTINUE
C>>>>RIF (IUTIME.GT.1) NH(I)=1
      IF ( IPROB .EQ. 2 ) NH(I) = 1
      HROP=HROP+2.0*TSUNSE(I)*FY(I)*DAYDEL*DWEATH(I)
      HRSTRT=HRSTRT+TSTRT*2.0*FY(I)*DAYDEL*DWEATH(I)
      HRPRE=HRPRE+TPRE*2.0*FY(I)*DAYDEL*DWEATH(I)
      HRWEATH=HRWEATH+24.*(1.-DWEATH(I))*FY(I)*DAYDEL
      F=1.
         GO TO 1480
1470  TSUNSE(I)=4.
      SDEC=0.
      CDEC=1.
      NH(I)=NUAZ
      DAY(I)=UEL(I)
1480  CONTINUE
      NN=NH(I)
      IF (I.EQ.NYEAR) NHREF=NH(I)
      DO 1540 J=1,NN
C>>>>RT=15.*FLOAT(J-1)*HRDEL
      T = FLOAT(J-1)*HRDEL
C>>>>RIF (J.EQ.NN) T=TSUNSE(I)*15.
      IF ( J .EQ. NN ) T = TSUNSE(I)
C>>>>RIF (IUTIME.GT.1) T=UTIME*15.
      IF ( IPROB .EQ. 2 ) T = UTIME
C>>>2RTIME(I,J)=T/15.
      TIME(I,J) = T
      T = T*15.
C>>>>R   IF (IUTIME.GT.1) GO TO 1490
         IF ( IPROB .EQ. 2 ) GO TO 1490
C>>>>R   IF (IUTIME.EQ.1.AND.(TIME(I,J)-REFTIM)**2.LT.0.001) GO TO 1490
         IF ( IPROB .EQ. 1 .AND. (TIME(I,J) - REFTIM)**2 .LT. .001 )
     C        GO TO 1490
         IF ((TIME(I,J)-REFTIM)**2+(DAY(I)-REFDAY)**2.GT.0.001) GO TO 15
     100
1490  CONTINUE
      IDP=I
      JDP=J
      REFDAY=DAY(I)
      REFTIM=TIME(I,J)
1500  CONTINUE
C>>>>R   IF (IUTIME.GT.1) GO TO 1510
         IF ( IPROB .EQ. 2 ) GO TO 1510
      IF (J.GT.1.AND.J.EQ.(NN-1)) F=1.+(TSUNSE(I)-TIME(I,J))/HRDEL
      IF (J.EQ.(NN-1).AND.J.EQ.1) F=(TSUNSE(I)-TIME(I,J))/HRDEL
      IF (J.EQ.NN) F=(TIME(I,J)-TIME(I,J-1))/HRDEL
1510  CONTINUE
      FTIME(I,J)=F
      ST=SIN(T*RR)
      CT=COS(T*RR)
      CSPANG(I,J)=SLAT*SDEC+CLAT*CDEC*CT
      SSPANG(I,J)=SQRT(1.-CSPANG(I,J)**2)
      SSAANG(I,J)=0.
      CSAANG(I,J)=1.0
         IF (IROTFL.EQ.1) GO TO 1520
      IF (SSPANG(I,J).GT.1.E-05) SSAANG(I,J)=ST*CDEC/SSPANG(I,J)
      CSAANG(I,J)=CDEC*(SLAT*CT-CLAT*SDEC/CDEC)/(SSPANG(I,J)+1.E-08)
1520  CONTINUE
         IF (IPROB.NE.3) GO TO 1530
      TIME(I,J)=UAZ(J)/100.
      CSPANG(I,J)=COS(RR*UEL(I))
      SSPANG(I,J)=SIN(RR*UEL(I))
      CSAANG(I,J)=COS(RR*UAZ(J))
      SSAANG(I,J)=SIN(RR*UAZ(J))
1530  CONTINUE
C
C             INSOL = 0  INSOLATION VS. ZENITH FROM MEINEL
C             INSOL = 1  H C HOTTEL SOLAR ENERGY V18 P129 1976
C             INSOL = 2  CONSTANT .95 KW/M2 INSOLATION
C             INSOL = 3  MOON MODEL
C             INSOL = 4  ALLEN MODEL
C
      HINSOL(I,J)=(1.-.14*ALT)*EXP(-.357/CSPANG(I,J)**.678)+.14*ALT
      IF (INSOL.EQ.1) HINSOL(I,J)=0.4237-0.00821*(6.-ALT)**2+(0.5055+0.0
     10595*(6.5-ALT)**2)*EXP(-(0.2711+0.01858*(2.5-ALT)**2)/(CSPANG(I,J)
     2+.00001))
      SAVE2= 90. -ATAN2(SSPANG(I,J),CSPANG(I,J))/RR
      SAVE=1.0/CSPANG(I,J)
      IF (SAVE2.LE.30.0) SAVE=SAVE-41.972213*SAVE2**(-2.0936381-0.041173
     141*SAVE2+0.000849854*SAVE2**2)
      IF (INSOL.EQ.3) HINSOL(I,J)=1.0-0.263*((DH2O(I)+2.72)/(DH2O(I)+5.0
     1))*(SAVE*DPRES(I))**(0.367*((DH2O(I)+11.53)/(DH2O(I)+7.88)))
      IF (INSOL.EQ.4) HINSOL(I,J)=0.183*EXP(-SAVE*DPRES(I)/0.48)+0.715*E
     1XP(-SAVE*DPRES(I)/4.15)+.102
      HINSOL(I,J)=HINSOL(I,J)*1.353*(1.+.0335*COS(2.*PI*(DAY(I)+10.)/365
     1.))
      IF (INSOL.EQ.2) HINSOL(I,J)=SOLCON
      DINSOL(I)=DINSOL(I)+F*HINSOL(I,J)
      F=2.
1540  CONTINUE
      F=2.
      DINSOL(I)=DINSOL(I)*HRDEL
      SAVE3=SAVE3+DINSOL(I)*DAYDEL*FY(I)
      FY(I)=FY(I)*DWEATH(I)
      YINSOL=YINSOL+DINSOL(I)*DAYDEL*FY(I)
         IF (IPRT.GT.0) WRITE (6,3610) DAY(I),DEC(I),DINSOL(I)
         IF (IPRT.GT.0) WRITE (6,3600) DWEATH(I)
         IF (IPRT.GT.0) WRITE (6,3590) DH2O(I),DPRES(I)
      DO 1570 J=1,NN
      SPANG=ACOS(CSPANG(I,J))/RR
      SAANG(I,J)=ATAN2(SSAANG(I,J),CSAANG(I,J))/RR
      T=TIME(I,J)
      SAVE=SPANG
         IF (IPROB.EQ.3.OR.SAVE.LT.1.E-08) GO TO 1550
      SAVE=PI/2.-SPANG*RR
C
C             CORRECT POLAR ANGLE FOR ATMOSPHERIC REFRACTION
C
      SAVE2=.004013327*(SAVE/.06476916)**(-.66956539+.019276169*SAVE)-.0
     100512969
      SAVE2 = SAVE + SAVE2*DPRES(I)
      SAVE2=PI/2.-SAVE2
      CSPANG(I,J)=COS(SAVE2)
      SSPANG(I,J)=SIN(SAVE2)
      SPANG=SAVE2/RR
      SAVE=(PI/2.-SAVE)/RR
1550  CONTINUE
      WRITE (6,3620) T,HINSOL(I,J),SAVE,SPANG,SAANG(I,J)
1570  CONTINUE
      IF (SAVE3.GT.1.E-10) WEATH=YINSOL/SAVE3
         IF (IPRT.GT.0) WRITE (6,3630) SAVE3,WEATH,YINSOL
          IF (IPRT.GT.0) WRITE (6,3640) HROP,HRSTRT,HRPRE,HRWEATH
C
C             SUN ANGLES AT CANTING TIME
C
         IF (ICANT.NE.3) GO TO 1580
      SAVE=2.*PI/365.24*(DCANT+284.)
      SAVE2=2.*SAVE
      SAVE=SAVE+0.007133*SIN(SAVE)+0.032680*COS(SAVE)-0.000318*SIN(SAVE2
     1)+0.000145*COS(SAVE2)
      SDEC=SIN(23.442274*RR)*SIN(SAVE)
      CDEC=SQRT(1.-SDEC**2)
      CT=COS(HCANT*RR*15.)
      ST=SIN(HCANT*RR*15.)
      CSPCAN=SLAT*SDEC+CLAT*CDEC*CT
      SSPCAN=SQRT(1.-CSPCAN**2)
      SSACAN=0.
      IF (SSPCAN.GT.1.E-05) SSACAN=ST*CDEC/SSPCAN
      CSACAN=SQRT(1.-SSACAN**2)
      IF (SSACAN**2.GT.0.0001) CSACAN=SSACAN*(SLAT*CT-CLAT*SDEC/CDEC)/(S
     1T+1.E-08)
1580  CONTINUE
      IPRSUM=0
C
C             ZERO OUT QUANTITIES TO BE CALCULATED
C
      DO 1590 I=1,NYEAR
      IPRSUM=IPRSUM+IPRINT(I)
      NN=NH(I)
      DO 1590 J=1,NN
      HFLCOS(I,J)=0.
      HFLSHA(I,J)=0.
      HFLBLO(I,J)=0.
      HFLATM(I,J)=0.
      HFLSPI(I,J)=0.
      HFLTOT(I,J)=0.
1590  CONTINUE
      DO 1600 K=1,NRAD
      DO 1600 L=1,NAZM2P
      NN=NH(NYEAR)
      DO 1600 J=1,NN
1600  HREFIN(J,K,L)=0.
      NFMID=NUMFF/2+1
      NTMID=NUMTT/2+1
      DO 1610 N=1,NFLXPT
1610  FLFLX(N)=0.
      YFLCOS=0.
      YFLATM=0.
      YFLSPI=0.
      YFLTOT=0.
      YFLSHA=0.
      YFLBLO=0.
      IHNUM=0
         IF (IPRSUM.LT.1) GO TO 1620
      WRITE (6,3650)
         IF (IPROB .EQ. 4) WRITE (6,3660)
1620  CONTINUE
C
C             SUM OVER ZONES
C
      DO 2100 KKI=1,NRADS
      K=KKI
C>>>>I
      KK = K
      DO 1630 NNN=1,5
1630  NNEARI(NNN)=3
      DO 2020 LLI=1,NAZMS
C
C             READ IN COORDINATES OF NEXT ROW OF HELIOSTATS
C
C             READ IN ANOTHER ROW OF INDIVIDUAL HELIOSTATS
C
C>>>>DL=LORDER(LLI)
C
C             TEST IF INDIVIDUAL HELIOSTAT PERFORMANCE RUN
C
         IF (IUSERF.NE.3) GO TO 1700
      K=1
      L=1
C>>>>I
      KK = K
      LL = L
      M = 1
         IF (LLI*IHPR.EQ.1) WRITE (6,3670) KKI,NUMHIR(3)
      NR=3
      IHNUM=IHNUM+1
      NIRH=LLI+2-50*(LLI/49)
      NNN=2*NR-1+(LLI/49)
      XHEL=HCS(NNN,1,NIRH)/THT
      YHEL=HCS(NNN,2,NIRH)/THT
      ZHEL=HCS(NNN,3,NIRH)/THT
         IF (IAUTOP.LT.6) GO TO 1640
      NUMPT(K,L)=1
      XAIM(K,L,1)=HCS(NNN,4,NIRH)
      YAIM(K,L,1)=HCS(NNN,5,NIRH)
1640  CONTINUE
      XFOCAL(K)=HCS(NNN,6,NIRH)/THT+1.E-08
      YFOCAL(K)=XFOCAL(K)
      RAD=SQRT(XHEL**2+YHEL**2)
      RADIUS(K)=RAD
      RDMIN(K)=RAD-HM2
C
C             FIND THE NEAREST NEIGHBORS IN INIDIVIDUAL
C             HELIOSTAT FIELD
C
      ZZZ(1)=1.E+08
      RRR(1)=1.E+08
      NN3=0
      DO 1670 NN=1,5
      N=NNEARI(NN)
      NIRH1=N-50*(N/51)
      NNN1=2*NN-1+N/51
      XH1=HCS(NNN1,1,NIRH1)/THT
      YH1=HCS(NNN1,2,NIRH1)/THT
      D1=(XH1-XHEL)**2+(YH1-YHEL)**2
1650  NIRH2=(N+1)-50*((N+1)/51)
      NNN2=2*NN-1+(N+1)/51
      XH2=HCS(NNN2,1,NIRH2)/THT
      YH2=HCS(NNN2,2,NIRH2)/THT
      D2=(XH2-XHEL)**2+(YH2-YHEL)**2
         IF (D2+1.e-08.GE.D1) GO TO 1660
      N=N+1
      IF (N.GT.NUMHIR(NN)+2) N=3
      D1=D2
      XH1=XH2
      YH1=YH2
         GO TO 1650
1660  NNEARI(NN)=N
      NN1=NNIHS(NN)
      DO 1670 NN2=1,NN1
C
C             CANNOT SHADE ITSELF
C
         IF (NN*NN2.EQ.9) GO TO 1670
      NN3=NN3+1
      NN4=N-NN1/2-1+NN2
      NN5=NN4-(NN4/51)*50
      NN6=2*NN+(NN4/51)-1
      RRR(NN3)=HCS(NN6,1,NN5)/THT-XHEL
      ZZZ(NN3)=HCS(NN6,2,NN5)/THT-YHEL
C
C             IF HELIOSTAT IS FAR AWAY DO NOT CALCULATE
C             SHADOWING AND BLOCKING
C
C>>>2RIF (RRR(NN3)**2+ZZZ(NN3).GT.100*(HMN+WMN)) NN3=NN3-1
      IF ( RRR(NN3)**2 + ZZZ(NN3) .GT. 100.*(HMN + WMN) )
     1     NN3 = NN3 - 1
1670  CONTINUE
      IF (NN3.LT.1) NN3=1
      NNEARS=NN3+1
C
C             PRINT 9468,(III,RRR(III),ZZZ(III),III=1,NNEARS)
C
         IF (IREC.GT.0) GO TO 1680
      STA=XHEL/RAD
      CTA=YHEL/RAD
      RANGE(K)=SQRT((XHEL-STA*W2)**2+(YHEL-CTA*W2)**2+(1.-ZHEL)**2)
      CTP=(1.-ZHEL)/RANGE(K)
      STP=SQRT(1.-CTP**2)
         GO TO 1690
1680  CONTINUE
      I=1
      XRECVR=-W2*RSRA(I)
      YRECVR=-W2*RCRA(I)
      RSAVE=SQRT((XHEL-XRECVR)**2+(YHEL-YRECVR)**2+(ZHEL-1.)**2)
      RAD=SQRT((XHEL-XRECVR)**2+(YHEL-YRECVR)**2)
      RSAVE=SQRT(RAD**2+(1.-ZHEL)**2)
      CTP=(1.-ZHEL)/RSAVE
      STP=SQRT(1.-CTP**2)
      CTA=(YHEL-YRECVR)/(RSAVE*STP)
      STA=-(XRECVR-XHEL)/(RSAVE*STP)
      RANGE(K)=RSAVE
1690  CONTINUE
      AZMIH=ATAN2(STA,CTA)/RR
      ELVIH=ATAN2(STP,CTP)/RR
C>>>>DKK=1
      YATMOS(K,L)=1.0-ATM1N-ATM2N*RANGE(K)-ATM3N*RANGE(K)**2-ATM4N*
     1  RANGE(K)**3
C>>>>DLL=1
C>>>>DM=1
C>>>>DK=1
C>>>>DL=1
      FLWTS=WTIH
      SAVE=XHEL*THT
      SAVE2=YHEL*THT
      IF (IHPR.GT.0) WRITE (6,3680) IHNUM,KKI,SAVE,SAVE2,RAD,RANGE(K),
     1AZMIH,ELVIH
         GO TO 1710
1700  CONTINUE
C
C             DO NOT CALCULATE PERFORMANCE IF ZONE NOT IN FIELD
C
C>>>>I
      L = LORDER(LLI)
C change 3/27/87 commented next line and added the following to fix
C a problem with fland initialization. This is Kistler's fix.
C        IF (IUSERF.GT.0.AND.FLWT(K,L).LT.1.E-06) GO TO 2020
         IF  (IUSERF.GT.0.AND.FLWT(K,L).LT.1.E-06) GO TO 1760
C>>>>DKS=K
      CTP=CTPANG(K)
      STP=STPANG(K)
C>>>>DKK=K
      CTA=CTAANG(L)
      STA=STAANG(L)
      FLWTS=FLWT(K,L)
C>>>>RM=MZONE(L)
      LL = L
      M = LZONE(L)
      INEAR=1
C>>>>DLS=L
C>>>>DLL=L
      CALL HELPOS
      RSAVE=RANGE(K)
         IF (IREC.EQ.0) GO TO 1710
C
C             FOR FLAT PLATE AND CAVITY RECEIVERS
C             THE HELIOSTATS ARE AIMED AT THE CENTER OF THE
C             APERTURE OR PLATE
C             FOR EXTERNAL CYLINDERS THE HELIOSTATS ARE AIMED
C             RADIALLY INWARD
C
      I=JREC(L,1)
         IF (I.LT.1) GO TO 1710
      XRECVR=-W2*RSRA(I)
      YRECVR=-W2*RCRA(I)
      ZRECVR=0.
      XHEL=STAANG(L)*RADIUS(K)
      YHEL=CTAANG(L)*RADIUS(K)
      RSAVE=SQRT((XHEL-XRECVR)**2+(YHEL-YRECVR)**2+1.)
      CTP=(1.+ZRECVR)/RSAVE
      STP=SQRT(1.-CTP**2)
      CTA=-(YRECVR-YHEL)/(RSAVE*STP)
      STA=-(XRECVR-XHEL)/(RSAVE*STP)
1710  CONTINUE
      SRANGE(1)=1.
      DO 1720 II=2,NORDER
1720  SRANGE(II)=SRANGE(II-1)*RANGE(K)
C
C             DETERMINE CANTING OF MIRROR PANELS
C
         IF (ICANT.EQ.0) GO TO 1760
         IF (ICANT.GT.2) GO TO 1740
C
C             USER DEFINED ON-AXIS CANTING
C
      GCANTA=0.
      GCANTY=0.
      GCANTX=-.5/RCANT(K)*SRANGE(2)
      GCANTB=-.5/RCANT(K)*SRANGE(2)
         IF (ICANT.GT.0) GO TO 1760
C
C             ON-AXIS CANTING = SLANT RANGE
C
      GCANTX=-.5/RANGE(K)*SRANGE(2)
      GCANTB=-.5/RANGE(K)*SRANGE(2)
         GO TO 1760
C
C             OFF-AXIS CANTING AT USER DEFINED TIME
C
1740  CONTINUE
      CSP=CSPCAN
      SSP=SSPCAN
      SSA=SSACAN
      CSA=CSACAN
      HCOS=SQ2*SQRT(1.+CSP*CTP+SSP*STP*(CTA*CSA+STA*SSA))
      CNP=(CSP+CTP)/HCOS/2.
      SNP=SQRT(1.-CNP**2)
      CNA=CTA
      SNA=STA
         IF (CNP.GT.0.999) GO TO 1750
      CNA=(SSP*CSA+STP*CTA)/HCOS/SNP/2.
      SNA=(SSP*SSA+STP*STA)/HCOS/SNP/2.
1750  CONTINUE
      SBNT=SNA*CTA-CNA*STA
      CBNT=CTA*CNA+SNA*STA
      CBNS=CSA*CNA+SNA*SSA
      SBNS=SNA*CSA-CNA*SSA
      A(1,1)=CBNT
      A(1,2)=CNP*SBNT
      A(2,2)=SNP*STP+CNP*CTP*CBNT
      A(2,1)=-CTP*SBNT
      B(1,1)=2.*A(2,2)
      B(1,2)=-A(2,1)*2.
      B(2,1)=-A(1,2)*2.
      B(2,2)=2.*A(1,1)
      SAVE=4.*HCOS*RANGE(K)
      D11=A(2,1)*B(1,2)-A(1,1)*B(2,2)
      D22=A(1,2)*B(2,1)-A(2,2)*B(1,1)
      D21=A(1,1)*B(2,1)-A(2,1)*B(1,1)
      D12=A(2,2)*B(1,2)-A(1,2)*B(2,2)
      GCANTX=D11/SAVE*SRANGE(2)
      GCANTA=D12/SAVE*SRANGE(2)
      GCANTY=D21/SAVE*SRANGE(2)
      GCANTB=D22/SAVE*SRANGE(2)
1760  CONTINUE
C
C             FOCUSING
C
         IF (L.GT.1) GO TO 1780
         IF (IFOCUS.EQ.1) GO TO 1770
C
C             FOCAL LENGTH SET EQUAL TO RANGE
C
      XFOCAL(K)=RANGE(K)
      YFOCAL(K)=RANGE(K)
1770  CONTINUE
      XFOCAL(K)=-1./XFOCAL(K)*XFOCUS
      YFOCAL(K)=-1./YFOCAL(K)*YFOCUS
1780  CONTINUE
C change 3/27/87 added next line to fix fland initialization problem.
C This is Kistler's fix.
      IF (IUSERF.GT.0.AND.FLWT(K,L).LT.1.E-06) GO TO 2020
         IF (IPRSUM.LT.1) GO TO 1790
         IF (IUSERF.LT.3) WRITE (6,3690) RADIUS(K),AZM(L)
      WRITE (6,3710)
1790  CONTINUE
C
C             INTEGRATE OVER THE HALF YEAR
C
      DO 2010 I=1,NYEAR
      DSPILL=0.
      DBLOCK=0.
      DCOSI=0.
      DSHAD=0.
      DTOTAL=0.
      NN=NH(I)
C
C             INTEGRATE OVER THE DAY
C
      DO 1980 J=1,NN
      F=FTIME(I,J)
      CSP=CSPANG(I,J)
         IF (IAUTOP.GT.4) GO TO 1810
      NAY(K,L)=0
      NAX(K,L)=0
      NUMPT(K,L)=0
C>>>>RDO 1800 LL=1,2
      DO 1800 LL2 = 1,2
C>>>>RXAIM(K,L,LL)=0.
      XAIM(K,L,LL2) = 0.
C>>>>RYAIM(K,L,LL)=0.
      YAIM(K,L,LL2) = 0.
1800  CONTINUE
1810  CONTINUE
      FDP=0.
C
C             TEST TO SEE IF THIS TIME IS THE DESIGN POINT
C
      IF (I.EQ.IDP.AND.J.EQ.JDP) FDP=1.00
      SSP=SSPANG(I,J)
      CSA=CSAANG(I,J)
      SSA=SSAANG(I,J)
      SOL=HINSOL(I,J)
C
C             COSINE
C
      HCOS=SQ2*SQRT(1.+CSP*CTP+SSP*STP*(CTA*CSA+STA*SSA))
      DCOSI=DCOSI+F*SOL*HCOS
      HFLCOS(I,J)=HCOS*FLWTS+HFLCOS(I,J)
      DFLCOS(I)=DFLCOS(I)+F*SOL*HCOS*FLWTS*HRDEL
      CNP=(CSP+CTP)/HCOS/2.
      SNP=SQRT(1.-CNP**2)
      CNA=2.*CTA
C
C             HELIOSTAT ANGLES OF ORIENTATION
C
      SNA=2.*STA
         IF (CNP.GT.0.999) GO TO 1820
      CNA=(SSP*CSA+STP*CTA)/HCOS/SNP
      SNA=(SSP*SSA+STP*STA)/HCOS/SNP
1820  CONTINUE
      CNA=CNA/2.
      SNA=SNA/2.
      SBNT=SNA*CTA-CNA*STA
      CBNT=CTA*CNA+SNA*STA
      CBNS=CSA*CNA+SNA*SSA
      SBNS=SNA*CSA-CNA*SSA
C
C             HELIOSTAT AIMING ANGLES
C
      AZMANG=ATAN2(SBNT,CBNT)/RR
      ELVANG=ATAN2(SNP,CNP)/RR
      SAVEIN=HCOS
C
C             CALCULATE SHADOWING AND BLOCKING
C
C             TOWER SHADOW
C
      TSHAD=1.0
      TLS=TOWLN*SSP/CSP
         IF (RDMIN(K).GT.TLS) GO TO 1840
         IF (IUSERF.GT.2) GO TO 1830
         IF (SAANG(I,J).LT.AZMIN(L)) GO TO 1840
         IF (SAANG(I,J).GT.AZMAX(L)) GO TO 1840
      SAVE=RDMAX(K)
      IF (SAVE.GT.TLS) SAVE=TLS
      TSHAD=1.0-TOWDN*(SAVE-RDMIN(K))/DAREA(K,L)/REALD(K,L)
      IF (TSHAD.LT.1.E-06) TSHAD=1.E-06
         GO TO 1840
1830  CONTINUE
      YS=CSA*YHEL+SSA*XHEL
         IF (YS.LT.0.0) GO TO 1840
      XS=CSA*XHEL-SSA*YHEL
         IF (XS-WM2.GT.TOWD2) GO TO 1840
         IF (XS+WM2.LT.-TOWD2) GO TO 1840
      SAVE=TOWD2
      IF (XS+WM2.LT.SAVE) SAVE=XS+WM2
      SAVE2=-TOWD2
      IF (XS-WM2.GT.SAVE2) SAVE2=XS-WM2
      SAVE3=YS+HM2*HCOS
      IF (SAVE3.GT.TLS) SAVE3=TLS
      TSHAD=1.0-(SAVE-SAVE2)*(SAVE3-YS+HM2*HCOS)/ASTATN
1840  CONTINUE
      DO 1850 MM=1,NUMTT
      DO 1850 MN=1,NUMFF
1850  SAB(MM,MN)=TSHAD+.000001
C
C             HELIOSTAT SHADOWING
C
C>>>>DKS=K
C>>>>DLS=L
      INEAR=1
      CALL SHADOW (SSP,CSP,SSA,CSA,SBNS,CBNS)
      HSHAD=SAB(NFMID,NTMID)
      HFLSHA(I,J)=HFLSHA(I,J)+HSHAD*FLWTS*SAVEIN
      DFLSHA(I)=DFLSHA(I)+HSHAD*SAVEIN*FLWTS*F*SOL
      DSHAD=DSHAD+HSHAD*F*SOL*SAVEIN
      SAVEIN=SAVEIN*HSHAD
      INEAR=2-ISABL
C
C             HELIOSTAT BLOCKING
C
      CALL SHADOW (STP,CTP,STA,CTA,SBNT,CBNT)
      HBLOCK=SAB(NFMID,NTMID)/HSHAD
         IF (ISB.NE.1) GO TO 1860
C
C             SHADOWING AND BLOCKING ASSUMED TO
C             COMPLETELY OVERLAP
C
      SAVE=1.0
      IF (HBLOCK.LT.HSHAD) SAVE=HBLOCK/HSHAD
      HBLOCK=SAVE
1860  CONTINUE
C>>>>DM=LZONE(L)
         IF (IPROB .NE. 4) GO TO 1880
      SPAK=SOL*F*DAYDEL*FY(I)*HRDEL/WTAREA(M)
      SPAK=SPAK*SAVEIN/HSHAD
      IPAK=0
C
C             SAVE SHADOWING AND BLOCKING RESULTS
C             FOR HELIOSTAT OPTIMIZATION
C
      DO 1870 II=1,NUMFF
      DO 1870 JJ=1,NUMTT
      IPAK=IPAK+1
      HCSDP(K,L,IPAK)=HCSDP(K,L,IPAK)+FDP*SAB(II,JJ)
1870  SCS(K,M,IPAK)=SCS(K,M,IPAK)+SAB(II,JJ)*SPAK
1880  CONTINUE
      HFLBLO(I,J)=HFLBLO(I,J)+HBLOCK*FLWTS*SAVEIN
      DFLBLO(I)=DFLBLO(I)+SAVEIN*HBLOCK*FLWTS*F*SOL
      DBLOCK=DBLOCK+HBLOCK*F*SOL*SAVEIN
      SAVEIN=SAVEIN*HBLOCK
      HFLATM(I,J)=HFLATM(I,J)+YATMOS(K,L)*FLWTS*SAVEIN
      DFLATM(I)=DFLATM(I)+YATMOS(K,L)*SAVEIN*FLWTS*F*SOL
      SAVEIN=SAVEIN*YATMOS(K,L)
C
C             CALCULATE SPILLAGE
C
C             MULTIPLE AIM POINTS
C
C             SPILLAGE AND FLUX DENSITY
C
C             EXPANSION COEFFICIENTS
C
      A(1,1)=CBNT
      A(1,2)=CNP*SBNT
      A(2,2)=SNP*STP+CNP*CTP*CBNT
      A(2,3)=CNP*STP-SNP*CTP*CBNT
      A(3,1)=STP*SBNT
      A(3,2)=SNP*CTP-CNP*STP*CBNT
      A(3,3)=HCOS
      A(1,3)=-SNP*SBNT
      A(2,1)=-CTP*SBNT
      B(1,1)=2.*A(2,2)
      B(1,2)=-A(2,1)*2.
      B(2,1)=-A(1,2)*2.
      B(2,2)=2.*A(1,1)
C>>>>DKK=K
C>>>>DLL=L
C
C             CALL SUBROUTINES TO CALCULATE HERMITE COEFS9+
C             SPILLAGE
C
 1903 CONTINUE
      CALL MOMENT
      IF (SPL.GT.1.00) SPL=1.00
      FL = SPL+1.E-08
         IF (IUSERF.EQ.3) GO TO 1910
      SPAK=SOL*F*DAYDEL*FY(I)*HRDEL*SAVEIN/WTAREA(M)
      SAVE2=0.
      IF (L.GT.NAZM2P) SAVE2=1.0
      IPAK=0
C
C             SAVE DEPENCE OF HERMITE COEFFICIENTS OF FLUX ON
C             TOWER HEIGHT FOR USE IN DETERMINING
C             SPILLAGE AND FLUX IN OPTIMIZATION
C
      DO 1900 II=1,NPAK
      MM=NUMK(II)
      SAVE=(SAVE2*SIPAK(II)+1.0-SAVE2)*SPAK
C>>>>RDO 1900 KK=1,MM
      DO 1900 KK2 = 1,MM
      IPAK=IPAK+1
C>>>>RHCS(K,M,IPAK)=HCS(K,M,IPAK)+HCTHT(II,KK)*SAVE
      HCS(K,M,IPAK) = HCS(K,M,IPAK) + HCTHT(II,KK2)*SAVE
1900  CONTINUE
1910  CONTINUE
C
C             CALCULATE THE FLUX DENSITY
C
         IF (IFXOUT(I,J).EQ.0) GO TO 1940
      CON1=FLWT(K,L)*SOL*SAVEIN*AMIRRO*RMIRL/THT2
      CALL FLXMAP (K,L)
      CON=SOL*SAVEIN*RMIRL*DAREA(K,L)*DENSIT(K,L)
      DO 1920 N=1,NFLXPT
      FLFLX(N)=FLFLX(N)+FLX(N)*CON1
      FLX(N)=FLX(N)*CON
1920  CONTINUE
         IF (IFXOUT(I,J).GT.0) GO TO 1940
C
C             PRINT OUT ZONE BY ZONE FLUX MAP
C
      WRITE (6,4300) DAY(I),TIME(I,J),REFSOL
      WRITE (6,3730) RADIUS(K),AZM(L)
      WRITE (6,3740)
         IF (IFLAUT.LT.3) WRITE (6,3750)
         IF (IFLAUT.GT.2) WRITE (6,3180)
      WRITE (6,3820)
      WRITE (6,3760) (XNFLX(N),N=1,NXFLX)
      WRITE (6,3820)
      J1=1-NXFLX
      REWIND (32)
      WRITE (32) (FLFLX(NNNN),NNNN=1,NFLXPT)
      DO 1930 N=1,NYFLX
      J1=J1+NXFLX
      J2=J1+NXFLX-1
      K1=J1-NFLSAV
C>>>>RDO 1930 M=1,MFLSAV
      DO 1930 MM = 1,MFLSAV
      K1=K1+NFLSAV
      K2=K1+NFLSAV-1
C>>>>RKK=(K2-1)/J2
      KK2 = (K2 - 1)/J2
C>>>>RK2=K2*(1-KK)+J2*KK
      K2 = K2*(1 - KK2) + J2*KK2
C>>>>RWRITE (6,3770) YNFLX(N),(FLX(KK),KK=K1,K2)
      WRITE (6,3770) YNFLX(N),(FLX(KK2),KK2=K1,K2)
      WRITE (32) (FLX(KK2),KK2=K1,K2)
1930  CONTINUE
      DPSPILL=HFLSPI(I,J)+FLWTS*FL*SAVEIN
      IF(IAUTOP.EQ.5.AND.IFXOUT(I,J).EQ.-2)
     *CALL PLOTMAP(XAIM(K,L,1),XAIM(K,L,2),YAIM(K,L,1),
     1YAIM(K,L,2),DPSPILL,FL,CON1,NXFLX,NYFLX,FLFLX,IREFLX,
     2IFXOUT(I,J),XNFLX,YNFLX,RADIUS(K),AZM(L))
      IF(IAUTOP.EQ.5.AND.IREFLX.EQ.1) GO TO 1903
1940  CONTINUE
      HSPILL=FL
      DSPILL=DSPILL+F*SOL*FL*SAVEIN
      HFLSPI(I,J)=HFLSPI(I,J)+FL*FLWTS*SAVEIN
      HTOTAL=HCOS*FL*RREFL*YATMOS(K,L)*HBLOCK*HSHAD
      HFLTOT(I,J)=HFLTOT(I,J)+FLWTS*HTOTAL
      DFLTOT(I)=DFLTOT(I)+HTOTAL*FLWTS*F*SOL*HRDEL
      DFLSPI(I)=DFLSPI(I)+FL*SAVEIN*FLWTS*F*SOL
      DTOTAL=DTOTAL+F*SOL*HTOTAL
         IF (IPROB.EQ.3) GO TO 1960
C
C             REFERENCE TIME OF YEAR USED TO DETERMINE FIELD SIZE
C
         IF (I.NE.IDP.OR.J.NE.JDP) GO TO 1950
      REFFLU(K,L)=HTOTAL/HSPILL/YATMOS(K,L)
      REFTOT(K,LL)=HTOTAL
1950  CONTINUE
C
C             JUNE 21 HOURLY PERFORMANCE SAVED FOR CALCULATION
C             OF STORAGE CAPACITY
C
         IF (I.NE.NYEAR) GO TO 1960
      SOL1=HINSOL(NYEAR,JDP)
C>>>>DM=LZONE(L)
      HREFIN(J,K,M)=REFSOL*SOL/SOL1/YATMOS(K,L)*HTOTAL/HSPILL/WTAREA(M)+
     1HREFIN(J,K,M)
1960  CONTINUE
         IF (IPRINT(I).LT.2) GO TO 1980
C
C             PRINT OUT HOURLY DATA
C
      WRITE (6,3700) DAY(I),TIME(I,J),AZMANG,ELVANG,HCOS,HSHAD,HBLOCK,YA
     1TMOS(K,L),HSPILL,HTOTAL,NAX(K,L),NAY(K,L),XAIM(K,L,1),YAIM(K,L,1)
1980  CONTINUE
         IF (IPROB.EQ.2.OR.IPROB.EQ.3) GO TO 2010
C>>>>DM=LZONE(L)
      SAVE=FY(I)*DAYDEL*HRDEL/WTAREA(M)
      YTOTAL(K,M)=YTOTAL(K,M)+DTOTAL*SAVE
      YSPILL(K,M)=YSPILL(K,M)+DSPILL*SAVE
      YSHAD(K,M)=YSHAD(K,M)+DSHAD*SAVE
      YBLOCK(K,M)=YBLOCK(K,M)+DBLOCK*SAVE
      YCOSI(K,M)=YCOSI(K,M)+DCOSI*SAVE
      DSPILL=DSPILL/DBLOCK/YATMOS(K,L)
      DBLOCK=DBLOCK/DSHAD
      DSHAD=DSHAD/DCOSI
      DCOSI=DCOSI*HRDEL/DINSOL(I)
      DTOTAL=DTOTAL*HRDEL/DINSOL(I)
         IF (WTAREA(M).LT.1.5) GO TO 2000
         IF (L.LT.NAZM2P) GO TO 1990
      DCOSI=(DCOSI+DCOS(I))/2.
      DSHAD=(DSHAD+DSHD(I))/2.
      DBLOCK=(DBLOCK+DBLK(I))/2.
      DSPILL=(DSPILL+DSPL(I))/2.
      DTOTAL=(DTOTAL+DTOT(I))/2.
         GO TO 2000
1990  CONTINUE
      DCOS(I)=DCOSI
      DBLK(I)=DBLOCK
      DSHD(I)=DSHAD
      DTOT(I)=DTOTAL
      DSPL(I)=DSPILL
         IF (IPRINT(I)*IPRT.GT.0) WRITE (6,3830) DAY(I)
         GO TO 2010
2000  CONTINUE
         IF (IPRINT(I)*IPRT.LT.1) GO TO 2010
C
C             PRINT OUT DAILY DATA
C
      WRITE (6,3720) DAY(I),DCOSI,DSHAD,DBLOCK,YATMOS(K,L),DSPILL,
     1DTOTAL
2010  CONTINUE
2020  CONTINUE
         IF (IUSERF.LT.3.OR.KKI.EQ.NRADS) GO TO 2100
      NAZMS=NUMHIR(4)
      DO 2030 NNN=1,4
      NN2=NUMHIR(NNN+1)+4
      NUMHIR(NNN)=NUMHIR(NNN+1)
      DO 2030 NN=1,NN2
      NNN2=2*NNN-1+NN/51
      IJK=NN-50*(NN/51)
      DO 2030 J=1,6
      HCS(NNN2,J,IJK)=HCS(NNN2+2,J,IJK)
2030  CONTINUE
      DO 2040 JJ=1,6
      DO 2040 NNN2=1,50
      HCS(9,JJ,NNN2)=1.E+10
2040  HCS(10,JJ,NNN2)=1.E+10
         IF (KKI+3.GT.NRADS) GO TO 2080
      READ (5,3460) NR,NUMHIR(NNN)
      NNN2=NUMHIR(5)+2
      DO 2050 N=3,NNN2
      KJI=N-50*(N/51)
      NNN1=9+N/51
      READ (5,3500) (HCS(NNN1,J,KJI),J=1,6)
2050  CONTINUE
         IF (NNN2.LT.8) GO TO 2100
      DO 2060 JJ=1,6
      NNN21 = NNN2+1-50*((NNN2+1)/51)
      NNN22 = NNN2-1-50*((NNN2-1)/51)
      NNN23 = NNN2+2-50*((NNN2+2)/51)
      HCS(9,JJ,2)=HCS(NNN1,JJ,KJI)
      HCS(9+(NNN2+1)/51,JJ,NNN21)=HCS(9,JJ,3)
      HCS(9,JJ,1)=HCS(9+(NNN2-1)/51,JJ,NNN22)
      HCS(9+(NNN2+2)/51,JJ,NNN23)=HCS(9,JJ,4)
2060  CONTINUE
         GO TO 2100
2080  CONTINUE
      NUMHIR(5)=1
      HCS(9,4,NN)=1.E+08
2100  CONTINUE
C
C             PRINT OUT YEARLY DATA
C
         IF (IPRT.LT.1) GO TO 2160
      WRITE (6,3840)
      WRITE (6,3820)
      WRITE (6,2610) HEAD
      WRITE (6,3820)
      WRITE (6,3790)
      WRITE (6,3820)
         IF (IPROB .EQ. 4) WRITE (6,3660)
      WRITE (6,3820)
      WRITE (6,3780)
      WRITE (6,3810) (AZM(L),L=1,NAZM2P)
      DO 2150 K=1,NRAD
         IF (IUSERF.EQ.3) GO TO 2130
      DO 2120 L=1,NAZM2P
      SPAK=YCOSI(K,L)+1.E-10
      DO 2110 N=1,25
2110  SCS(K,L,N)=SCS(K,L,N)/SPAK
      SPAK=YATMOS(K,L)*YBLOCK(K,L)+1.E-10
      DO 2120 N=1,50
      HCS(K,L,N)=HCS(K,L,N)/SPAK
2120  CONTINUE
2130  CONTINUE
      DO 2140 L=1,NAZM2P
         IF (DAREA(K,L).LT.1.E-06) GO TO 2140
         IF (IFLAUT.GT.1.AND.FLWT(K,L).LT.1.E-06) GO TO 2140
      YSPILL(K,L)=YSPILL(K,L)/YBLOCK(K,L)/YATMOS(K,L)
      YBLOCK(K,L)=YBLOCK(K,L)/YSHAD(K,L)
      YSHAD(K,L)=YSHAD(K,L)/YCOSI(K,L)
      YCOSI(K,L)=YCOSI(K,L)/YINSOL
         IF (IUSERF.EQ.3) YCOSI(K,L)=YCOSI(K,L)*FLWTS
      YTOTAL(K,L)=YTOTAL(K,L)/YINSOL
         IF (IUSERF.EQ.3) YTOTAL(K,L)=YTOTAL(K,L)*FLWTS
2140  CONTINUE
      WRITE (6,3800) RADIUS(K),HEAD1,(YCOSI(K,L),L=1,NAZM2P)
      WRITE (6,3800) RADIUS(K),HEAD2,(YSPILL(K,L),L=1,NAZM2P)
      WRITE (6,3800) RADIUS(K),HEAD3,(YATMOS(K,L),L=1,NAZM2P)
      WRITE (6,3800) RADIUS(K),HEAD11,(YSHAD(K,L),L=1,NAZM2P)
      WRITE (6,3800) RADIUS(K),HEAD12,(YBLOCK(K,L),L=1,NAZM2P)
      WRITE (6,3800) RADIUS(K),HEAD4,(YTOTAL(K,L),L=1,NAZM2P)
      WRITE (6,3820)
2150  CONTINUE
2160  CONTINUE
         IF (IUSERF.EQ.0) GO TO 2510
C
C             PRINT OUT RESULTS FROM USER DEFINED FIELD
C
      WRITE (6,3860)
         IF (IPROB.NE.3) GO TO 2170
      DO 2161 J=1,7
      JJ=8-J
      WRITE (31,2162) HFLTOT(6,JJ),HFLTOT(5,JJ),HFLTOT(4,JJ),
     1 HFLTOT(3,JJ),HFLTOT(2,JJ),HFLTOT(1,JJ)
2161  CONTINUE
2162  FORMAT (5X,6(F6.3,1H,))
      WRITE (6,3530)
      WRITE (6,3540)
      WRITE (6,3550)
2170  CONTINUE
      WRITE (6,3870)
      WRITE (6,3880) RMIRL,RRECL
      WRITE (6,3890)
      DO 2190 I=1,NYEAR
      NN=NH(I)
      DO 2180 J=1,NN
      HFLSPI(I,J)=HFLSPI(I,J)/HFLATM(I,J)
      HFLATM(I,J)=HFLATM(I,J)/HFLBLO(I,J)
      HFLBLO(I,J)=HFLBLO(I,J)/HFLSHA(I,J)
      HFLSHA(I,J)=HFLSHA(I,J)/HFLCOS(I,J)
2180  WRITE (6,3900) DAY(I),TIME(I,J),HFLCOS(I,J),HFLSHA(I,J),
     1HFLBLO(I,J),HFLATM(I,J),HFLSPI(I,J),HFLTOT(I,J)
         IF (IPRT.LT.1) GO TO 2190
      SAVE=FY(I)*DAYDEL
      YFLTOT=YFLTOT+DFLTOT(I)*SAVE
      YFLATM=YFLATM+DFLATM(I)*SAVE
      YFLCOS=YFLCOS+DFLCOS(I)*SAVE
      YFLSPI=YFLSPI+DFLSPI(I)*SAVE
      YFLBLO=YFLBLO+DFLBLO(I)*SAVE
      YFLSHA=YFLSHA+DFLSHA(I)*SAVE
      DFLSPI(I)=DFLSPI(I)/DFLATM(I)
      DFLATM(I)=DFLATM(I)/DFLBLO(I)
      DFLBLO(I)=DFLBLO(I)/DFLSHA(I)
      DFLSHA(I)=DFLSHA(I)/DFLCOS(I)*HRDEL
      DFLCOS(I)=DFLCOS(I)/DINSOL(I)
      DFLTOT(I)=DFLTOT(I)/DINSOL(I)
      WRITE (6,3910) DAY(I),DFLCOS(I),DFLSHA(I),DFLBLO(I),DFLATM(I),DFLS
     1PI(I),DFLTOT(I)
2190  CONTINUE
         IF (IPRT.LT.1) GO TO 2200
      YFLSPI=YFLSPI/YFLATM
      YFLATM=YFLATM/YFLBLO
      YFLBLO=YFLBLO/YFLSHA
      YFLSHA=YFLSHA/YFLCOS*HRDEL
      YFLCOS=YFLCOS/YINSOL
      YFLTOT=YFLTOT/YINSOL
      WRITE (6,3920) YFLCOS,YFLSHA,YFLBLO,YFLATM,YFLSPI,YFLTOT
2200  CONTINUE
         IF (IPROB.EQ.3) GO TO 2510
      IF(IRADFL.EQ.1.OR.(IREC.NE.2.AND.IREC.NE.1)) GO TO 2195
C
C             CALCULATE FAKE CAV HEAT ABSORPTION AREA
C
      AREC=H*RR*180.0*W2*THT*RWCAV(1)
      HI=H
      H=AREC
2195  CONTINUE
      CALL POWER
      IF(IRADFL.EQ.0.AND.(IREC.EQ.1.OR.IREC.EQ.2))
     1H=HI
         IF (IFLX.EQ.0) GO TO 2510
      WRITE (6,3120)
      WRITE (6,3740)
      DO 2460 I=1,NYEAR
      N=NH(I)
      DO 2460 J=1,N
         IF (IFXOUT(I,J).NE.0) GO TO 2470
2460  CONTINUE
      WRITE (6,4290)
         GO TO 2510
2470  CONTINUE
      SAVE=HINSOL(I,J)
      IF (IFUDGE.EQ.1) SAVE = REFSOL
      WRITE (6,4300) DAY(I),TIME(I,J),SAVE
         IF ( IFUDGE .EQ. 0 ) GO TO 2495
      SAVE2 = REFSOL/HINSOL(I,J)
      DO 2490 N=1,NFLXPT
      FLFLX(N)=FLFLX(N)*SAVE2
2490  CONTINUE
 2495 CONTINUE
         IF (IFLAUT.LT.3) WRITE (6,3750)
         IF (IFLAUT.GT.2) WRITE (6,3180)
      WRITE (6,3820)
      WRITE (6,3760) (XNFLX(N),N=1,NXFLX)
      WRITE (6,3820)
      WRITE (11,3761) HEAD,NXFLX
 3761 FORMAT(5A8,I5)
      WRITE (11,3760) (XNFLX(N),N=1,NXFLX)
      WRITE (11,3820)
      J1=1-NXFLX
      DO 2500 N=1,NYFLX
      J1=J1+NXFLX
      J2=J1+NXFLX-1
      K1=J1-NFLSAV
      DO 2500 M=1,MFLSAV
      K1=K1+NFLSAV
      K2=K1+NFLSAV-1
      KK=(K2-1)/J2
      K2=K2*(1-KK)+J2*KK
      WRITE (6,3770) YNFLX(N),(FLFLX(KK),KK=K1,K2)
      WRITE (11,3770) YNFLX(N),(FLFLX(KK),KK=K1,K2)
2500  CONTINUE
2510  CONTINUE
      DO 2520 K=1,NRAD
      IF (ABS(YFOCAL(K)).LT.1.E-06) YFOCAL(K)=-1./7.15
      IF (ABS(XFOCAL(K)).LT.1.E-06) XFOCAL(K)=-1./7.15
      XFOCAL(K)=-1./XFOCAL(K)
      YFOCAL(K)=-1./YFOCAL(K)
2520  CONTINUE
C>>>7I
C
C             CONVERT FROM UNITS OF THT TO REGULAR UNITS
C
      DO 2530 K = 1,NRAD
      DO 2530 L = 1,NAZM
         DAREA(K,L) = DAREA(K,L)/REALD(K,L)/FLAND(K,L)
2530  CONTINUE
C
C          WRITE TAPE10 - STORE THE DATA FROM THIS PERFORMANCE
C                         RUN FOR SUBSEQUENT DESIGN RUNS
C
      IF (ITAPE .NE. 1) GO TO 2550
      WRITE (10) THT,WM,HM,IROUND,ICANT,NCANTX,NCANTY,ROUND,ASTAT,AMIRRO
     1,AZM,RADIUS,RADDEL,AZMDEL,NRAD,NAZM,RR,RMIRL,DAREA,DENSMR,DENSIT,S
     1TAANG,CTPANG,STPANG,CTAANG,XFOCUS,YFOCUS,SIGAZ,SIGEL,SIGSX,SIGSY,S
     1IGTX,SIGTY,WTAREA,HEAD,NHREF,IFOCUS,REFFLU,HREFIN,AZMSEP,XFOCAL,YF
     1OCAL,REFTOT,YINSOL,REFSOL,HRDEL,YCOSI,YSHAD,YBLOCK,HCS,HCANT,DCANT
     1,RADMIN,RADMAX,RCANT,RANGE,HROP,WEATH,DPRES,DH2O,IWEATH,NYEAR,
     1DWEATH,INSOL,SOLCON,NSUN,NSUNPT,SUNI,SCS,HCSDP,NUMFF,NUMTT,IFILL,
     1ILAY,REALD,LZONE,MZONE,HCANT,DCANT,RCANT,INDC,IROTFL,ATM1,ATM2,ATM
     13,ATM4,IATM,INORTH,AMAXN,PLAT,ALT,REFDAY,REFTIM,NSUNPT,SUNR,SUNI,N
     1SUM,IPROB,ICPANL,WPANL,HPANL,HXCANT,HYCANT,RDMIN,RDMAX,
     1H,IAUTOP,IREC,NAX,NAY,NUMCAV,NUMPT,RAZM,RELV,RRECL,RWCAV,RX,RY,
     1TOWD,TOWL,W,XAIM,YAIM,AREF,EFFSTR,ETAREF,FEFF,FEP,FPLC,FPLH,FSP,
     1IPH,ITHEL,PF,REFLP,REFPIP,REFPRL,REFRC,REFTHP,SMULT,AZMF,DIAMF,
     1FAZMAX,FAZMIN,FLXLIM,FZMAX,FZMIN,ICAVF,IFLAUT,IFLX,IFXOUT,
     1NFLXMX,NMXFLX,NXFLX,NYFLX,POLF,XFC,YFC,ZFC,ISB,ALP,ASTART,CLE,
     1CLN,FLAND,FSLIP,IDENS,IHPR,IUSERF,NLAND,NRADMN,NRADMX,SLEW,SLNS,
     1XTOWER,YTOWER,REFRC1,REFRC2,REFRC3,IRADFL
     *,HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5,PARL6,PARL7
     *,PARL8,PARL9,PARL10,TPRE,TSTRT
      ENDFILE 10
C>>>7I ABOVE LINES
C>>>>R LINE 5 ABOVE
      WRITE (6,9876) '$FMTP'
 9876 FORMAT(1H1,A8)
C mjw     PRINT FMTP
2550  CONTINUE
C>>>>D
C>>>>D           CONVERT FROM UNITS OF THT TO REGULAR UNITS
C>>>>D
C>>>>DDO 2750 K = 1,NRAD
C>>>>DDO 2570 L = 1,NAZM
C>>>>DDAREA(K,L) = DAREA(K,L)/REALD(K,L)/FLAND(K,L)
C>>>>D2750 CONTINUE
      RETURN
2610  FORMAT(1X,4A10)
3010  FORMAT(1H1,20X,8HRECEIVER////)
3020  FORMAT(14H TOWER HEIGHT=,F6.2,34H M ABOVE PLANE OF HELIOSTAT PIVOT
     1S//)
3030  FORMAT (/59H THE TOWER SHADOW IS CALCULATED USING A CYLINDER OF HE
     1IGHT=,F6.1,17H M AND DIAMETER =,F5.1,2H M/)
3040  FORMAT (34H EXTERNAL RECEIVER - CYLINDER HT.=,F6.2,8H  DIAM.=,F6.2
     1,2H M)
3050  FORMAT (//59H SINGLE AIM POINT AT THE CENTER OF THE RECEIVER OR AP
     1ERTURE)
3060  FORMAT (//40H THE CODE WILL AUTOMATICALLY CALCULATE A,I2,57H DIMEN
     1SIONAL SMART AIMING FOR EACH ZONE AT EACH TIME STEP)
3070  FORMAT (//88H AIMING OF HELIOSTATS DESIGNED TO MINIMIZE RAD+CONV L
     1OSSES OR MECHANICAL FATIGUE FAILURE)
3080  FORMAT (10X,31HIMAGES FROM EACH ZONE ARE IN A ,I2,78HD ARRAY AS CL
     1OSE TO THE BOTTOM OF RECEIVER AS POSSIBLE W/O INCREASING SPILLAGE)
3090  FORMAT (//69H USER DEFINED AIMING FOR EACH ZONE - AIM POINTS DO NO
     1T VARY WITH TIME//)
3100  FORMAT (8H ZONE K=,I2,3H L=,I2,23H NO. OF PTS. IN HORIZ.=,I2,5H MI
     1N=,F6.2,6H  MAX=,F6.2,10H NO. VERT=,I2,5H MIN=,F6.2,5H MAX=,F6.2)
3110  FORMAT (8H ZONE K=,I2,3H L=,I2,4H THE,I2,26H AIM PT. HAS COORDS HD
     1RIZ=,F7.2,6H VERT=,F7.2)
3120  FORMAT (1H1,20X,16HFLUX CALCULATION//)
3130  FORMAT (///20X,45H FLUX POINTS ON OUTSIDE OF CYLINDER OF RADIUS,F5
     1.2,17H M CENTERED AT X=,F5.2,3H Y=,F5.2,3H Z=,F5.2/)
3140  FORMAT (///20X,45H FLUX POINTS ON  INSIDE OF CYLINDER OF RADIUS,F5
     1.2,17H M CENTERED AT X=,F5.2,3H Y=,F5.2,3H Z=,F5.2/)
3150  FORMAT (///10X,38H FLUX POINTS ON A PLANE CENTERED AT X=,F5.2,3H Y
     1=,F5.2,5H Z = ,F5.2,23H ANGLES OF NORMAL, POL=,F5.1,5H AZM=,F5.1)
3160  FORMAT (//90H SEVERAL HELIOSTATS AT DIFFERENT POSITIONS WITHIN EAC
     1H ZONE ARE USED IN GENERATING FLUXES//)
3170  FORMAT (//99H ONLY ONE HELIOSTAT PER ZONE IS USED TO GENERATE THE
     1FLUX MAP - THIS MUST BE A PERFORMANCE ONLY RUN//)
3180  FORMAT (54H   HT(M)      HORIZONTAL DISPLACEMENT IN FLUX PLANE(M)/
     1)
3190  FORMAT (F10.3,12(2X,I3,5X))
3200  FORMAT (///20X,57H COORDINATES OF FLUX POINTS AND ANGLES OF SURFAC
     1E NORMALS)
3210  FORMAT (  /60H NUMBER      COORDINATES IN METERS        POLAR    A
     1ZIMUTHAL)
3220  FORMAT (57H            EAST     NORTH       UP       ANGLE     ANG
     1LE)
3230  FORMAT (I5,4X,5(F7.2,3X))
3240  FORMAT (/30H LAND CONSTRAINED FIELD LAYOUT)
3250  FORMAT (10X,39H THE FIELD IS CONSTRAINED TO BE WITHIN ,I2,11H RECT
     1ANGLES)
3260  FORMAT (10X,36H THE CENTER OF THE TOWER IS LOCATED ,F7.1,11H M EAS
     1T AND ,F7.1,8H M NORTH)
3270  FORMAT (10X,3HTHE,I2,22H RECTANGLE IS CENTERED,F7.1,7HM EAST,,F7.1
     1,17HM NORTH, EW SIDE=,F7.1,12HM, N-S SIDE=,F7.1,18HM, ROTATION ANG
     1LE=,F6.2)
3280  FORMAT (1H1,20X,7H ZONING//)
3290  FORMAT (///20X,20H USER DEFINED FIELDS/)
3300  FORMAT (//37H FIELD DESCRIPTION - LAND AREA(M2  )=,E11.3,13H MIRR0
     1R AREA=,E11.3,17H  NO. OF HSTATS =,F7.0,19H AVERAGE DENSITY = ,F5.
     13)
3310  FORMAT (8X,A7,20I6)
3320  FORMAT (/22H NO USER DEFINED FIELD/)
3330  FORMAT (/62H HELIOSTAT FIELD ROTATES  - SUN ALWAYS APPEARS TO BE D
     1UE SOUTH/)
3340  FORMAT (//37H USER DEFINED ATMOSPHERIC ATTENUATION/)
3350  FORMAT (//52H HELIOSTATS ARE LAID OUT IN A RADIAL STAGGER PATTERN/
     1/)
3360  FORMAT (//54H HELIOSTATS ARE LAID OUT IN A RADIAL CONRFIELD PATTER
     1N//)
3370  FORMAT (//49H HELIOSTATS ARE LAID OUT IN A N/S STAGGER PATTERN//)
3380  FORMAT (//51H HELIOSTATS ARE LAID OUT IN A N/S CORNFIELD PATTERN//
     1 )
3390  FORMAT (//79H NORTH ONLY HELIOSTAT - RECEIVER MUST BE A SINGLE N F
     1ACING CAVITY OR FLAT PLATE//)
3400  FORMAT (5X,83H NOTE - THE UNITS OF THE RADIAL SEPARATION ARE THE N
     1UMBER OF HELIOSTAT HEIGHTS, HM=,F6.2,7H METERS)
3410  FORMAT (5X,85H      - THE UNITS OF THE AZIMUTHAL SEPARATION ARE TH
     1E NUMBER OF HELIOSTAT WIDTHS, WM=,F6.2,7H METERS/13X,71H AZM SEP I
     1S ONE HALF THE AZIMUTHAL SEPARATION BETWEEN HELIOSTAT CENTERS)
3420  FORMAT (5X,74H NOTE - THE RADIUS OF THE HELIOSTAT IS GIVEN IN UNIT
     1S OF THE TOWER HEIGHT=,F6.2,7H METERS)
3430  FORMAT (5X,94H NOTE - MISS. H IS THE FRACTION OF THE ZONE AVAILABL
     1E IN RADIAL LAYOUTS BECAUSE OF SLIP PLANES)
3440  FORMAT (5X,' NOTE - FR LAND IS THE FRACTION OF THE LAND WITHIN ',
     1'A ZONE THAT IS AVAILABLE DUE TO LAND CONSTRAINTS OR FIELD TRIM')
3450  FORMAT (    20X,17HFIELD RADIUS MAX=,F5.2,5H MIN=,F5.2,6H STEP=,F5
     1.2,14H TOWER HEIGHTS)
3460  FORMAT (3I10)
3470  FORMAT (58H INDIVIDUAL HELIOSTAT COORDINATES ARE SUPPLIED BY THE U
     1SER// )
3480  FORMAT (26H THE LABEL OF THE FIELD = ,4A10)
3490  FORMAT (5H THE ,I5,29H HELIOSTATS ARE GROUPED INTO ,I3,5H ROWS/)
3500  FORMAT (8F10.4)
3510  FORMAT (1H1,20X,28H INSOLATION AND SUN POSITION//)
3520  FORMAT (1X,40H USER DEFINED SINGLE CALCULATIONAL DAY =,F7.2)
3530  FORMAT (/98H USER DEFINED MATRIX OF SUN POSITIONS TO BE USED IN PE
     1RFORMANCE CALCULATIONS INSTEAD OF TIME STEPS)
3540  FORMAT (20X,'THE VALUE OF THE DAY IS SET EQUAL TO THE ELEVATION',
     1' ANGLE')
3550  FORMAT (20X,'THE VALUE OF THE HOUR IS SET TO 1/100 TIMES THE AZI'
     1,'MUTHAL ANGLE'/)
3560  FORMAT (29H INSOLATION CALCULATED USING ,A8,6H MODEL/)
3570  FORMAT (1X,' USER DEFINED SINGLE CALCULATION - DAY=',F7.2,' HOUR='
     1,F5.2)
3580  FORMAT (//18H PROGRAM STOP- THE,I3,68H DAY HAS MORE THAN 16 TIME S
     1TEPS - INCREASE HOURDEL (NAMELIST BASIC))
3590  FORMAT (20X,13H MM OF WATER=,F7.2,32H ATMOSPHERIC PRESSURE/SEA LEV
     1EL=,F7.3/)
3600  FORMAT (20X,73HTHE AVERAGE CORRECTION TO CLEAR SKY INSOLATION TO A
     1CCOUNT FOR CLOUDINESS=,F6.3)
3610  FORMAT (//18H DAY OF THE YEAR =,F7.3,27H  DECLINATION OF SUN (DEG)
     1=,F7.2,37H DAILY CLEAR SKY INSOLATION (KWH/M2)=,F7.2)
3620  FORMAT (8X,6HTIME =,F5.1,20H  INSOLATION(KW/M2)=,F7.3,20H TRUE ZEN
     1ITH ANGLE =,F7.2,18H  OBSERVED ZENITH=,F7.2,14H SUN AZIMUTH =,F7.2
     2)
3630  FORMAT (/50H YEARLY INSOLATION (KWATT-HOUR/M2)   - CLEAR SKY =,F7.
     11,25H  TIMES WEATHER FACTOR OF,F5.2,13H GIVES NET OF,F7.1//)
3640  FORMAT (31H HRS DIRECT OPERATION PER YEAR=,E10.4/
     1        31H HRS STARTUP/SHUTDOWN PER YEAR=,E10.4/
     2        34H HRS PRESTARTUP/SHUTDOWN PER YEAR=, E10.4/
     3        31H HRS OF WEATH OUTAGE PER YEAR =,E10.4/
     4        49H HRS OF NIGHT TIME AND STORAGE-ONLY OPER NOT INCL/
     5        30H ALL HOURS ABOVE ASSUME PF=1.0//)
3650  FORMAT (1H1,20X,31H----- OPTICAL PERFORMANCE -----//)
3660  FORMAT (105H THIS PERFORMANCE CALCULATION IS ONLY USED AS INPUT FO
     1R DESIGN OPTIMIZATION - ALL SPILLAGES ARE SET = 1.0//)
3670  FORMAT (//4H THE,I4,13H ROW CONTAINS,I4,11H HELIOSTATS/)
3680  FORMAT (16H IND. HSTAT NO.=,I5,6H ROW =,I4,7H EAST =,F7.2,8H NORTH
     1 =,F7.2,9H  RADIUS=,F6.2,12HTHT, RANGE =,F6.2,17H THT,  REC. AZM =
     2,F6.2,5H ELV=,F6.2)
3690  FORMAT (//20X,8HRADIUS =,F7.3,11H AZIMUTH = ,F7.1)
3700  FORMAT (2X,F6.2,5X,F5.2,3X,F6.1,4X,F5.1,6(3X,F6.3),5X,I2,8X,I2,
     1   7X,F5.2,5X,F5.2)
3710  FORMAT (3X,3HDAY,7X,4HHOUR,4X,6HAZ ANG,4X,6HEL ANG,4X,6HCOSINE,3X,
     16HSHADOW,2X,5HBLOCK,2X,18HATM TRANS SPILLAGE,3X,5HTOTAL,1X,
     2'NO.X AIM NO. Y AIM 1ST X AIM 1ST Y AIM')
3720  FORMAT (2X,F6.2,3X,7HAVERAGE,18X,         6(3X,F6.3),5X,I2,4X,I2)
3730  FORMAT (/,20X,22H FLUX MAP FROM RADIUS=,F6.2,10H AZIMUTH =,F6.1/)
3740  FORMAT (/37H UNITS OF FLUX ARE KILOWATT/SQ. METER/)
3750  FORMAT (51H   HT(M)      AZIMUTH OF NORMAL(180 = NORTH FACING))
3760  FORMAT (10X,12F10.3)
3770  FORMAT (F10.3,12E10.3)
3780  FORMAT (7H RADIUS,14X,52HAZIMUTHAL ANGLE OF FIELD POINT  (NORTH=0.
     1, EAST=90.))
3790  FORMAT (/1X,82H TOTAL PERFORMANCE INCLUDES REFLECTIVITIES BUT NOT
     1RADIATION AND CONVECTION LOSSES/)
3800  FORMAT (1X,F5.2,3X,A7,3X,12F8.3)
3810  FORMAT (19X,12F8.1)
3820  FORMAT (/)
3830  FORMAT (2X,F6.2,3X,7HAVERAGE,5X,74HTHE DAILY AVERAGE FOR THIS ZONE
     1 EQUALS THE DAILY AVERAGE FOR THE NEXT ZONE)
3840  FORMAT (1H1,20X,27H YEARLY AVERAGE PERFORMANCE)
3850  FORMAT (1X,F5.2,3X,A7,3X,12(F6.1,2X) )
3860  FORMAT (1H1,20X,44H PERFORMANCE SUMMARY FROM USER DEFINED FIELD//)
3870  FORMAT (10X,18HOPTICAL EFFICIENCY//)
3880  FORMAT (1X,54H TOTAL EFFICIENCY INCLUDES REFLECTIVITIES, HELIOSTAT
     1 =,F6.3,12H  RECEIVER =,F6.3//)
3890  FORMAT (3X,3HDAY,7X,4HHOUR,5X,6HCOSINE,4X,6HSHADOW,5X,5HBLOCK,2X,1
     19HATM.TRANS. SPILLAGE,4X,5HTOTAL)
3900  FORMAT (2X,F6.2,4X,F5.2,5X,F6.3,5(4X,F6.3))
3910  FORMAT (/2X,F6.2,3X,7HAVERAGE,4X,F6.3,5(4X,F6.3),/)
3920  FORMAT (//4X,14HYEARLY AVERAGE,4X,F6.3,5(4X,F6.3)/)
4290  FORMAT  (//51H FLUX NOT CALCULATED AT ANY TIME - CHECK IFXOUT ON ,
     115HNAMELIST NLFLUX)
4300  FORMAT (/32H THE FLUX IS CALCULATED FOR DAY=,F7.2,6H HOUR=,F7.2,12
     1H INSOLATION=,F7.3,6H KW/M2/ )
      END
      SUBROUTINE HELPOS
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE GENERATES THE COORDINATES OF THE
C     NEAREST NEIGHBORS FOR SHADOWING AND BLOCKING CALCULATIONS
C
C     THIS ROUTINE IS CALLED BY
C          PERCAL
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /PER1/ A(3,3),B(2,2),BIN(7,7),CNA,CNP,DAY(9),DAYDEL,
     1  DR(20),DT(20),FLAY,FY(9),GCANTA,GCANTB,GCANTX,GCANTY,
     2  GRSAVE(15,15),GSAVE(7,7,4),HCOS,HCTHT(16,4),HFLTOT(9,20),
     3  HINSOL(9,20),HXCANN(25),HYCANN(25),
     4  PSAVE(7,7),RRMULT(5,5),RRR(20),
     5  RUS(7,7),SAVESB(5,5),SIGAZ2,SIGEL2,SIGSX2,SIGSY2,SIGTX2,
     6  SIGTY2,SIH1,SIH2,SNA,SNP,SRANGE(7),TIME(9,20),TSUNSE(9),
     7  ZZMULT(5,5),ZZZ(20),
     8  IDP,INEAR,IPRT,JDP,MSTP(7),NH(9),NNEARS
C>>>>R LINE 4 ABOVE
 
C>>>>I
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
 
C>>>>I
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
C
C>>>>RRAD=RADIUS(KS)
      RAD = RADIUS(KK)
C>>>>RDENS=DENSIT(KS,LS)*AZMSEP(KS,LS)/DENSMR
      DENS = DENSIT(KK,LL)*AZMSEP(KK,LL)/DENSMS
      NNEAR=13-INEAR
      SAVE=1./DENS*HMN
C>>>>RSAVE1=WMN*AZMSEP(KS,LS)
      SAVE1 = WMN*AZMSEP(KK,LL)
         IF (ILAY.GT.1) GO TO 120
C
C             RADIAL LAYOUT PATTERN
C
      FLAY=1.0
      DO 110 I=1,NNEAR
      ZZZ(I)=1.0
      RRR(I)=1.0
110   CONTINUE
         GO TO 140
C
C             N-S LAYOUT PATTERN
C
120      IF (ILAY.GT.3) GO TO 160
      DO 130 II=1,NNEAR,INEAR
      RRR(II)=DR(II)*SAVE
130   ZZZ(II)=DT(II)*SAVE1
      FLAY=0.
140   CONTINUE
      FFFST=WMN*HMN/SAVE/SAVE1
C>>>>RTTTST=.5*((SAVE/HMN)**2-AZMSEP(KS,LS)**2)
      TTTST = .5*((SAVE/HMN)**2 - AZMSEP(KK,LL)**2)
      RRST=SQRT(TTTST+SQRT(TTTST**2+1./FFFST**2))
      ZZST=1./(RRST*FFFST)
      DO 150 II=1,NUMFF
      FF=FFFST*(1.+FLOAT(II-NUMFF/2-1)*DFFF)
      DO 150 JJ=1,NUMTT
      TT=TTTST*(1.+FLOAT(JJ-NUMTT/2-1)*DTTT)
      RRMULT(II,JJ)=SQRT(TT+SQRT(TT**2+1./FF**2))/RRST
      ZZMULT(II,JJ)=1./(FF*RRMULT(II,JJ)*ZZST*RRST)
      SAVESB(II,JJ)=1.0
         IF (ILAY.GT.1) GO TO 150
      SAVE2=RRMULT(II,JJ)*SAVE*HMN
      SAVE3=RAD*(FSLIP-1.)/(SAVE2*(1.+FSLIP))
      SAVESB(II,JJ)=(2.*SAVE3+1./FSLIP)/(2.*SAVE3+1.)
150   CONTINUE
      RETURN
C
C             USER DEFINED HELIOSTAT LOCATIONS
C
160   CONTINUE
      FLAY=0.
      RETURN
      END
      SUBROUTINE MOMENT
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         THIS SUBROUTINE CALCULATES THE MOMENTS OF THE
C     SUNSHAPE, HELIOSTAT ERROR DISTRIBUTION, AND
C     HELIOSTAT SHAPE ON THE IMAGE PLANE AND THEN CONVOLVES
C     THESE TO FIND THE MOMENTS OF THE HELIOSTAT IMAGE
C
C     THIS ROUTINE IS CALLED BY
C          PERCAL
C
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /AIM/ IAUTOP,INDC,NAX(13,12),NAY(13,12),NUMPT(13,12),
     1  XAIM(13,12,2),YAIM(13,12,2)
C
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
C
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
      COMMON /HELIO/ XCENT(25),YCENT(25)
C
      COMMON /PER1/ A(3,3),B(2,2),BIN(7,7),CNA,CNP,DAY(9),DAYDEL,
     1  DR(20),DT(20),FLAY,FY(9),GCANTA,GCANTB,GCANTX,GCANTY,
     2  GRSAVE(15,15),GSAVE(7,7,4),HCOS,HCTHT(16,4),HFLTOT(9,20),
     3  HINSOL(9,20),HXCANN(25),HYCANN(25),
     4  PSAVE(7,7),RRMULT(5,5),RRR(20),
     5  RUS(7,7),SAVESB(5,5),SIGAZ2,SIGEL2,SIGSX2,SIGSY2,SIGTX2,
     6  SIGTY2,SIH1,SIH2,SNA,SNP,SRANGE(7),TIME(9,20),TSUNSE(9),
     7  ZZMULT(5,5),ZZZ(20),
     8  IDP,INEAR,IPRT,JDP,MSTP(7),NH(9),NNEARS
C>>>>R LINE 4 ABOVE
C
C
      DIMENSION START(2),TSAVE(7),UG(7,7),UGSAVE(7,7),UM(7,7),US(7,7),
     1  XC(25,7),YC(25,7)
C
C             CALCULATE THE MOMENTS OF SUN, S
C
      DO 110 I=1,NORDER,2
      JMAX=NORDER-I+1
      JMIN=2-I+2*(I/2)
      DO 110 J=JMIN,JMAX,2
      US(I,J)=SRANGE(I+J-1)*RUS(I,J)
110   CONTINUE
C
C             MOMENTS OF THE ERROR FUNCTION, G
C
      SAVE=SIGAZ2*SNP**2+SIGSX2
      SAVE2=SIGEL2+SIGSY2
      DELTA1=SIGTX2+SAVE*B(1,1)**2+SAVE2*B(1,2)**2
      DELTA2=SIGTY2+SAVE*B(2,1)**2+SAVE2*B(2,2)**2
      DELTAA=SAVE*B(1,1)*B(2,1)+SAVE2*B(2,2)*B(1,2)
      SIGX=SRANGE(2)*SQRT(DELTA1)
      SIGY=SRANGE(2)*SQRT(DELTA2)
         IF (DELTAA**2.LT.1.E-20) GO TO 140
      RHO=DELTAA/SQRT(DELTA1*DELTA2)+1.E-10
      RHO2=RHO**2
      RHOSAV=(1.-RHO2)/2.
      DO 130 I=1,NORDER
      JMAX=JMX(I)
      JMIN=IMN(I)
      DO 130 J=JMIN,JMAX,2
      RS2=RHO2
      RS=1./RHOSAV
      SAVE=0.
      KKK=(I-1)/2+1
      DO 120 K=1,KKK
      RS=RS*RHOSAV
      RS2=RS2/RHO2
120   SAVE=SAVE+GSAVE(I,J,K)*RS*RS2
      UG(I,J)=(SIGX*RHO)**(I-1)*SIGY**(J-1)*SAVE
130   CONTINUE
         GO TO 160
140   CONTINUE
      DO 150 I=1,NORDER
      SAVE2=FLOAT(I-2*(I/2))
      SAVE=SIGX**(I-1)
      JMAX=JMX(I)
      JMIN=IMN(I)
      DO 150 J=JMIN,JMAX,2
      UG(I,J)=PSAVE(I,J)*SAVE*SIGY**(J-1)*SAVE2
150   CONTINUE
160   CONTINUE
C
C             MOMENTS OF THE MIRROR PROJECTION, M
C
      E11=A(1,1)+B(1,1)*XFOCAL(KK)*SRANGE(2)/2.+1.E-10
      E12=A(1,2)+B(1,2)*YFOCAL(KK)*SRANGE(2)/2.+1.E-10
      E21=A(2,1)+B(2,1)*XFOCAL(KK)*SRANGE(2)/2.+1.E-10
      E22=A(2,2)+B(2,2)*YFOCAL(KK)*SRANGE(2)/2.+1.E-10
      STEP=(E21/E22)**ISTEP
      DO 180 I=1,NORDER
      JMAX=JMX(I)
      JMIN=IMN(I)
      DO 180 J=JMIN,JMAX,ISTEP
      SAVE=0.
      IJ=I+J+1
      MSTEP=MSTP(J)
      S11=E12**(I-1)
      SAVE2=(E11/E12)**MSTEP
C     S11=S11/SAVE2
      START(1)=E22**(J-1)
      START(2)=START(1)*E21/E22
      DO 170 K=1,I,MSTEP
      IJK=IJ-K
      S11=S11*SAVE2
      BNS=BIN(I,K)*S11/SAVE2
      LMIN=IMN(K)
      S22=START(LMIN)
      DO 170 L=LMIN,J,ISTEP
      S22=S22*STEP
C
C             PRINT 2566,I,J,K,L,S11,S22,BNS,BIN(J,L),
C             GRSAVE(K+L-1,IJK-L)
C
      SAVE=SAVE+BNS*S22*BIN(J,L)*GRSAVE(K+L-1,IJK-L)/STEP
170   CONTINUE
      UGSAVE(I,J)=SAVE*HCOS
      UM(I,J)=SAVE*HCOS
180   CONTINUE
         IF (NCANT.EQ.1) GO TO 220
C
C             CANTED HELIOSTATS
C
      DO 190 I=1,NCANT
      XCENT(I)=(A(1,1)+B(1,1)*GCANTX+B(1,2)*GCANTY)*HXCANN(I)+(A(1,2)+B(
     11,1)*GCANTA+B(1,2)*GCANTB)*HYCANN(I)
      YCENT(I)=(A(2,1)+B(2,1)*GCANTX+B(2,2)*GCANTY)*HXCANN(I)+(A(2,2)+B(
     12,1)*GCANTA+B(2,2)*GCANTB)*HYCANN(I)
      XC(I,1)=1.
      YC(I,1)=1.
      DO 190 K=2,NORDER
      XC(I,K)=XC(I,K-1)*XCENT(I)
      YC(I,K)=YC(I,K-1)*YCENT(I)
190   CONTINUE
         IF (INDC.EQ.1) GO TO 220
      DO 210 I=1,NORDER
      JMAX=JMX(I)
      JMIN=JMN(I)
      DO 210 J=JMIN,JMAX,ISTEP
      SAVE=0.
      MSTEP=MSTP(J)
      DO 200 N=1,NCANT
      DO 200 II=1,I,MSTEP
      S11=BIN(I,II)*XC(N,I-II+1)
      JJMIN=JMN(II)
      DO 200 JJ=JJMIN,J,ISTEP
      SAVE=SAVE+S11*BIN(J,JJ)*UGSAVE(II,JJ)*YC(N,J-JJ+1)
200   CONTINUE
      UM(I,J)=SAVE
210   CONTINUE
C
C             COMBINE MOMENTS OF S, G + M TO GET MOMENTS OF F
C
220   CONTINUE
      IPAK=0
      SAVE2=US(1,1)*UG(1,1)*UM(1,1)
      DO 240 M=1,NORDER
      NMIN=JMN(M)
      NMAX=JMX(M)
      DO 240 N=NMIN,NMAX,ISTEP
      IPAK=IPAK+1
      SAVE=0.
      TSAVE(1)=0.
      TSAVE(3)=0.
      TSAVE(5)=0.
      TSAVE(7)=0.
      MSTEP=MSTP(N)
      DO 230 K=1,M,MSTEP
      LMIN=JMN(K)
      BN1=BIN(M,K)
      MK=M-K+1
      DO 230 L=LMIN,N,ISTEP
      UGS=UG(MK,N-L+1)*BN1*BIN(N,L)
      DO 230 I=1,K,2
      KI=K-I+1
      BN3=BIN(K,I)
      DO 230 J=1,L,2
C
C             PRINT 8342,M,N,K,L,I,J,BN3,US(I,J),UM(KI,L-J+1),UGS
C
      TSAVE(KI+L-J)=TSAVE(KI+L-J)+BIN(L,J)*BN3*US(I,J)*UM(KI,L-J+1)*UGS/
     1SAVE2
      SAVE=SAVE+BIN(L,J)*BN3*US(I,J)*UM(KI,L-J+1)*UGS
230   CONTINUE
C
C             NORMALIZE FOR UNIT FLUX
C
      UF(M,N)=SAVE/SAVE2
      HCTHT(IPAK,1)=TSAVE(1)
      HCTHT(IPAK,2)=TSAVE(3)
      HCTHT(IPAK,3)=TSAVE(5)
      HCTHT(IPAK,4)=TSAVE(7)
240   CONTINUE
      CALL HERMIT
      RETURN
      END
      SUBROUTINE POWER
      IMPLICIT REAL*8 (A-H,O-Z)
C
C
C               THIS ROUTINE CALCULATES THE POWER PRODUCTION FROM A USER
C          DEFINED FIELD
C
C          THIS ROUTINE IS CALLED BY
C               PERCAL
C
C
c      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
c     1  DWEATH(9),HRDEL,IATM,IFUDGE,IWEATH,NSUN,NSUNPT,PLAT,REFDAY,
c     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),IPROB
      COMMON /DEL2/ ALT,ASTART,ATM1,ATM2,ATM3,ATM4,DH2O(9),DPRES(9),
     1  DWEATH(9),HRDEL,PLAT,REFDAY,
     2  REFSOL,REFTIM,SUNI(50),SUNR(50),WEATH,W,HEAD(5),
     3  IATM,IFUDGE,IWEATH,NSUN,NSUNPT,
     4  IPROB
C
      COMMON /DEL4/ H2O,INSOL,IPRINT(9),ITAPE,NUAZ,NUEL,NYEAR,
     1  PRES,SOLCON,UAZ(20),UDAY,UEL(9),UTIME
C
C
C     ADDED GEN2 8-15-85 TO GET PF FOR BETTER ANNUAL ENERGY EST
C
      COMMON /GEN2/ ACHC,ACWOSC,AZMTR(13,12),CCMUL,DENTR(13,12),FPLC,
     1  HC(16),HXN(7,7),PF,
     2  ICAVF(4),IPH,ISTR,NFLXMX,NMXFLX(4),NTMAX(13),
     3  NTMIN(13)
C
C>>>>RCOMMON /GEN3/ ACHY,AZMDEL,DAZM,ESTOR,IHPR,IMN(7),ISB,IUSERF,
      COMMON /GEN3/ ACHY,AZMDEL,ESTOR,IHPR,IMN(7),ISB,IUSERF,
     1  NCANT,NOMAX,NPAK,NRADMN(13),NRADMX(13),NSUM,RADDEL,RDMAX(13),
     2  RDMIN(13),STPANG(13),XTOWER,YTOWER
C>>>>R LINE 2 ABOVE
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /GEN8/ AMAXN,RX(4),RY(4),XFOCAL(13),YFOCAL(13)
C
      COMMON /EFF/ ACWOSY,AREF,CCMULC,CCMULY,CSTREF,CSTRMD,EFFSTR,
     1  EMPTY,ESTREF,ETAREF,FCR,FEFF,FLXLIM(4),FPLH,FSP,REFLP,
     2  REFPIP,REFPRL,REFRC,REFRC1,REFRC2,REFRC3,REFTHP,RHOML,RNHOML,
     3  SMULT,VMAX,VSTREF,XST,ITHEL,IRADFL
C>>>>R LINES 1 AND 2 ABOVE
C
      COMMON /EFF1/ HRPRE,HRSTRT,HRWEATH,PARL1,PARL2,PARL3,PARL4,PARL5
     *             ,PARL6,PARL7,PARL8,PARL9,PARL10,TPRE,TSTRT
C
      COMMON /PER1/ A(3,3),B(2,2),BIN(7,7),CNA,CNP,DAY(9),DAYDEL,
     1  DR(20),DT(20),FLAY,FY(9),GCANTA,GCANTB,GCANTX,GCANTY,
     2  GRSAVE(15,15),GSAVE(7,7,4),HCOS,HCTHT(16,4),HFLTOT(9,20),
     3  HINSOL(9,20),HXCANN(25),HYCANN(25),
     4  PSAVE(7,7),RRMULT(5,5),RRR(20),
     5  RUS(7,7),SAVESB(5,5),SIGAZ2,SIGEL2,SIGSX2,SIGSY2,SIGTX2,
     6  SIGTY2,SIH1,SIH2,SNA,SNP,SRANGE(7),TIME(9,20),TSUNSE(9),
     7  ZZMULT(5,5),ZZZ(20),
     8  IDP,INEAR,IPRT,JDP,MSTP(7),NH(9),NNEARS
C>>>>R LINE 4 ABOVE
C
      COMMON /RECC/ IREC,NUMCAV,RAZM(4),RELV(4)
C
C
C
C             DESIGN POINT POWER PRODUCTION FROM A USER
C             DEFINED FIELD
C
         IF (IREC.EQ.0) GO TO 1100
      ARECRC=0.
      AREC=H
      DO 1090 I=1,NUMCAV
      ARECRC=ARECRC+RX(I)*RY(I)
1090  CONTINUE
         IF ( IREC .NE. 2*(IREC/2) ) ARECRC = ARECRC*PI/4.
         GO TO 1101
1100  CONTINUE
      ARECRC=PI*W*H
1101  CONTINUE
      WRITE (6,3930)
      WRITE (6,3940) REFDAY,REFTIM,REFSOL
      RTHP=REFSOL*AMIRRO*HFLTOT(IDP,JDP)/RRECL*1000.
      SAVE=RTHP/1.0E+06
      HEADFL=7H MW-TH  
      WRITE (6,3950) SAVE,HEADFL
      SAVE2=SAVE*(1.-RRECL)
      SAVE=SAVE-SAVE2
      WRITE (6,3960) SAVE2,HEADFL
      RCPOW=ARECRC*REFTHP*(1.0-REFRC)/AREF
      IF(IRADFL.EQ.1) GO TO  1200
      HMIX=15.00
      IF(W.LE.4.0) RE=1.751E5*W
      IF(W.LE.4.0) HMIXF=1.0/(W)*
     1(0.3+0.488*RE**0.5*(1.0+(RE/282000.0)**0.625)**0.8*0.04199)
      IF(W.LE.4.0) HMIX=(HMIXF**3.2+1.168E3)**(1.0/3.2)
      IF(W.GT.125.0)
     1HMIX=((33.75*(W)**(-0.19))**3.2+9.09**3.2)**(1/3.2)
      IF(IREC.EQ.0)RCPOW=ARECRC*(REFRC1+REFRC2*HMIX)
      IF(IREC.EQ.1.OR.IREC.EQ.2)RCPOW=ARECRC*(REFRC1+
     1REFRC2/(RX(1))**0.2+REFRC3*AREC/ARECRC)
 1200 CONTINUE
      SAVE2=RCPOW/1.E+06
      WRITE (6,3970) SAVE2,HEADFL
      SAVE=SAVE-SAVE2
      RADCON=1.0-(1.0-REFRC)*(ARECRC/RTHP)/(AREF/REFTHP)
      IF(IRADFL.EQ.1) GO TO  1210
      IF(IREC.EQ.0)RADCON=1.0-ARECRC/RTHP*(REFRC1+REFRC2*HMIX)
      IF(IREC.EQ.1.OR.IREC.EQ.2)RADCON=1.0-ARECRC/RTHP*
     1(REFRC1+REFRC2/(RX(1))**0.2+REFRC3*AREC/ARECRC)
1210  CONTINUE
      REFP=RADCON*RTHP
      ALPHP=(1.0-REFPIP)*SQRT(REFRC*REFTHP)/REFLP
      PIPE=1.0-ALPHP*FPLH*THT/SQRT(ABS(REFP))
      PIPPOW=SAVE*(1.0-PIPE)
      RLOSS=RCPOW/1.E+06+PIPPOW
      WRITE (6,3980) PIPPOW,HEADFL
      SAVE=SAVE-PIPPOW
      WRITE (6,3990) SAVE,HEADFL
      SAVE2=SAVE*(SMULT-1.0)/SMULT
      PTURB = SAVE - SAVE2
      WRITE (6,4000) SAVE2,HEADFL
      WRITE (6,4010) PTURB,HEADFL
      REFP=PTURB*1.E+06
      REFPTH=REFP
         IF (ITHEL.EQ.0) GO TO 2220
         IF (REFPTH.GE.5.75E+08) GO TO 2210
      ETAREF=105.483*ATAN(2.2450E-05*REFPTH)-165.25783+2.77618E-11*REFPT
     1H
      IF (REFPTH.LE.2.5E+07) ETAREF=0.247
         GO TO 2220
2210  ETAREF=0.43425/(1.0-2.9644E-11*REFPTH)
2220  ETATHE=FEFF*ETAREF
      WRITE (6,4020) ETAREF
      SAVE = PTURB*ETAREF
      SAVEP=SAVE
      HEADFL= 7H MW-EL  
      WRITE (6,4030) SAVE,HEADFL
      SAVE2=SAVE*REFPRL
      SAVE=SAVE-SAVE2
      WRITE (6,4040) SAVE2,HEADFL
      WRITE (6,4050) SAVE,HEADFL
      RCPOW=RCPOW/1.E+06
         IF (IPRT.LT.1) GO TO 2450
         IF (IFUDGE.GT.0.OR.ITAPE.EQ.3) GO TO 2230
      WRITE (6,4060)
      WRITE (6,4070)
      WRITE (6,4080)
         GO TO 2450
2230  CONTINUE
C
C               ANNUAL POWER PRODUCTION FROM A USER DEFINED FIELD
C
      WRITE (6,4090)
      WRITE (6,4100)
      WRITE (6,4120)
      WRITE (6,4130)
      WRITE (6,4110)
      IPSTR=0
      FSTR=1.0
      ECSAVY=1.0E+11
      ECOLD=1.0E+10
      EFF1=ETAREF
      EFF2=FEFF*EFF1
      DO 2430 IS=1,NSTR
      IST=IS
 2240 CONTINUE
         IF ( IPSTR .EQ. 0 ) GO TO 2250
      WRITE (6,4170) FSAVE,CCSAVY,CCSAVC,ECSAVY,ECSAVC
      WRITE (6,4150)
 2250 CONTINUE
C  12-18-85 BLK
C  THESE ALLOW FOR OPTIMIZATION OF STORAGE ABOVE
C  THE MAXIMUM SIZE.  THIS IS IMPORTANT FOR NORTH
C  FIELDS, WHERE DAY 172 MAY NOT BE THE CRITICAL DAY
C  FOR STORAGE.  IT IS SUGGESTED THAT THIS ALWAYS
C  BE USED FOR NORTH FIELDS.  ISTR IS FACTOR OF
C  HOW HIGH ABOVE MAXIMUM STORAGE SIZE IS TO BE
C  CONSIDERED.  NSTR IS AS DEFINED IN CODE.
C  ISTR WAS NOT USED BEFORE THIS CHANGE.
C
      IF (NSTR.NE.1.AND.ISTR.NE.0) FSTR=1.-(FLOAT(IST-1)/FLOAT(NSTR-1))
      IF (NSTR.NE.1.AND.ISTR.NE.0) FSTR=FSTR*FLOAT(ISTR)
C
      ESTCAP=FSTR*ESTOR/1.0E+06
      TNSTOR=1.0
      VSTOR=(VSTREF*ESTCAP/ESTREF)*1.E+06
         IF (VSTOR.LE.VMAX) GO TO 2260
      TNSTOR=FLOAT( INT(VSTOR/VMAX))+1.0
      VRATIO=VSTOR/VMAX+1.0
      IF ((VRATIO-TNSTOR).LE.0.001) TNSTOR=TNSTOR-1.0
      VSTOR=VSTOR/TNSTOR
2260  CSTRN=TNSTOR*(CSTREF*(1.0+EMPTY/TNSTOR)*(VSTOR/VSTREF)**XST+CSTRMD
     1*(VSTOR/VSTREF))
      YSOL=0.0
      SUMTHR=0.0
      SUMRCP=0.0
      SUMSTR=0.0
      SUMSTL=0.0
      SUMPAR=0.0
      SUMELP=0.0
      HRNIGHT=0.0
      DO 2410 I=1,NYEAR
      M=NH(I)
      IT=INT(TSUNSE(I)/.25)
      T=-.25*FLOAT(IT+1)
      TL=-TSUNSE(I)
      PTHRL=HFLTOT(I,M)*AMIRRO*HINSOL(I,M)/1000.
      SOLL=HINSOL(I,M)
      PTHL=PTHRL-RLOSS
      ETTSTR=0.
      ELOST=0.0
2270  CONTINUE
C
C             RESET TIME, TEST FOR MORNING, AFTERNOON,
C             OR SHUTDOWN
C
      T=T+.25
      IF (T.GT.TSUNSE(I)) T=TSUNSE(I)
      TA=ABS(T)
2280     IF (TA.LE.TIME(I,M)*1.0001) GO TO 2290
      M = M + 1
         GO TO 2280
2290     IF (TA.GE.TIME(I,M-1)) GO TO 2300
      M = M - 1
         GO TO 2290
2300  CONTINUE
      SAVE=(TA-TIME(I,M-1))/(TIME(I,M)-TIME(I,M-1))
      EFF=SAVE*HFLTOT(I,M)+(1.0-SAVE)*HFLTOT(I,M-1)
      SOL=SAVE*HINSOL(I,M)+(1.0-SAVE)*HINSOL(I,M-1)
      PTHR=SOL*EFF*AMIRRO/1000.
      PTH=PTHR-RLOSS
      IF(PTH.LT.PTHL.AND.PTH.LT.0.0.AND.T.GT.0.0) T=T-0.25
      IF(PTH.LT.PTHL.AND.PTH.LT.0.0.AND.T.GT.0.0) GO TO 2390
      DTIME=(T-TL)
      F=DTIME*FY(I)*DAYDEL
      YSOL=YSOL+F/2.*(SOL+SOLL)
      SUMTHR=SUMTHR+F*(PTHR+PTHRL)/2.
      SUMRCP=SUMRCP+F*RCPOW
      IF(PTH.LT.PTHL.AND.T.GT.0.0) GO TO 2330
C
C             OPERATION BEFORE NOON
C
         IF (PTH.GT.PTURB) GO TO 2310
C
C             OPERATION BEFORE FULL TURBINE POWER ACHIEVED
C
      ETOSTR=0.0
      ETURB=EFF2*DTIME*(PTH+PTHL)/2.0
      PSTR=0.0
      PTOTB=PTH
      ELGRS=PTH*EFF2
      IF(PTH.LT.0.0) PTOTB=0.0
      IF(PTH.LT.0.0) ELGRS=0.0
         GO TO 2315
2310     IF (PTHL.GT.PTURB) GO TO 2320
C
C             OPERATION DURING FIRST TIME STEP WHEN FULL
C             TURBINE POWER ACHIEVED
C
      PSTR = PTH - PTURB
      ELGRS = EFF1*PTURB
      FDT=DTIME*PSTR/(PTH-PTHL)
      ETOSTR=FDT*PSTR/2.0
      ETURB=(DTIME-FDT)*EFF2*(PTURB+PTHL)/2.0+FDT*ELGRS
      ETTSTR=ETOSTR
      PTOTB=PTURB
      IF(ETTSTR.LE.ESTCAP) GO TO 2316
      PSTR=0.
      ETOSTR=0.
      ELOST=ETTSTR
      ETTSTR=ESTCAP
 2316 CONTINUE
 2315 CONTINUE
      ESTR=ETOSTR
         GO TO 2345
C
C             OPERATION WHEN TURBINE AT FULL CAPACITY. EXCESS
C             ENERGY SENT TO STORAGE UNTIL STORAGE FULLY CHARGED,
C             REMAINDER THROWN AWAY.
C
2320  ETOSTR=DTIME*(PTH+PTHL-2.0*PTURB)/2.0
      ELGRS = EFF1*PTURB
      ETURB=DTIME*ELGRS
      ETTSTR=ETTSTR+ETOSTR
      PSTR=PTH-PTURB
      PTOTB=PTURB
         IF (ETTSTR.LE.ESTCAP) GO TO 2315
      ELOST=ETTSTR-ESTCAP
      ESTR=ETOSTR-ELOST
      ETTSTR=ESTCAP
      PSTR=0.0
         GO TO 2345
C
C             OPERATION AFTER NOON
C
2330     IF (PTH.GT.PTURB) GO TO 2320
         IF (PTHL.LE.PTURB) GO TO 2360
C
C             OPERATION DURING FIRST TIME STEP IN AFTERNOON WHEN
C             DELIVERED FIELD POWER FALLS BELOW FULL TURBINE POWER
C
      FDT=DTIME*(PTURB-PTH)/(PTHL-PTH)
      ESTR1=(DTIME-FDT)*(PTHL-PTURB)/2.0
      ESTR=ESTR1
      ETTSTR=ETTSTR+ESTR1
         IF (ETTSTR.LE.ESTCAP) GO TO 2340
      ELOST=ETTSTR-ESTCAP
      ETTSTR=ESTCAP
 2340 CONTINUE
      PSTR = PTH - PTURB
      ELGRS = EFF1*PTURB
      ESTR2=FDT*PSTR/2.0/EFFSTR
      ETTSTR=ETTSTR+ESTR2
      FDTST=FDT
      IF (ETTSTR.LE.0.0) FDTST=FDT*(ETTSTR-ESTR2)/ABS(ESTR2)
      ETURB=(DTIME-FDT+FDTST)*ELGRS+(FDT-FDTST)*EFF2*(PTURB+PTH)/2.
      EPAR=REFPRL*ETURB*(1.0)
      PTOTB=PTURB
      PAR=REFPRL*ELGRS*(1.0)
         IF (ETTSTR.GT.0.0) GO TO 2380
      PSTR=0.0
      PTOTB=PTH
      ELGRS=EFF2*PTH
      ESTR=0.0
      ETTSTR=0.
 2345 CONTINUE
      PAR=REFPRL*ELGRS
      EPAR=REFPRL*ETURB
         GO TO 2380
C
C             OPERATION IN AFTERNOON WHEN FIELD NO LONGER DELIVERS FULL
C             TURBINE POWER. ENERGY WITHDRAWN FROM STORAGE, IF AVAILABLE
C
2360  FDT=0.0
      PSTR=0.0
      ESTR=0.0
      ELOST=0.
         IF (ETTSTR.LE.0.0) GO TO 2375
      ETOSTR=DTIME*(PTH+PTHL-2.0*PTURB)/2.0/EFFSTR
      ETTSTR=ETTSTR+ETOSTR
      FDT=DTIME
      PSTR=PTH-PTURB
      PTOTB=PTURB
      ELGRS=EFF1*PTURB
      PAR=REFPRL*ELGRS*(1.0)
         IF (ETTSTR.GT.0.0) GO TO 2370
      FDT=(ETTSTR-ETOSTR)/ABS(ETOSTR)*DTIME
      PSTR=0.0
 2375 PTOTB=PTH
      ELGRS=EFF2*PTH
      PAR=REFPRL*ELGRS
2370  ETURB=FDT*EFF1*PTURB+(DTIME-FDT)*EFF2*(PTH+PTHL)/2.0
      EPAR=REFPRL*ETURB
      IF (ETTSTR.GT.0.0) EPAR=EPAR*(1.0)
2380  CONTINUE
      ELN = ELGRS - PAR
      SUMPAR=SUMPAR+F*EPAR/DTIME
      SUMELP=SUMELP+F*(ETURB-EPAR)/DTIME
      HSTR=ETTSTR/PTURB*EFFSTR
      SUMSTR=SUMSTR+F*ESTR/DTIME
      SUMSTL=SUMSTL+F*ELOST/DTIME
      IF (IPSTR.NE.0) WRITE (6,4140) DAY(I),T,SOL,PTHR,PTH,PSTR,HSTR,
     1PTOTB,ELGRS,PAR,ELN
         IF (T.GE.TSUNSE(I)) GO TO 2390
      PTHRL=PTHR
      SOLL=SOL
      PTHL=PTH
      TL=T
         GO TO 2270
2390  CONTINUE
      TNIGHT=(24.0-(TSUNSE(I)+T)-2.0*(TPRE+TSTRT))
      IF(ETTSTR.LE.0.0) HRNIGHT=HRNIGHT+F*TNIGHT/DTIME
      IF(ETTSTR.LE.0.0) GO TO 2400
      SAVE2=(1.0-REFPRL)   /(1.0-REFPRL)
      T=T+HSTR*SAVE2
      TNIGHT=TNIGHT-HSTR*SAVE2
      HRNIGHT=HRNIGHT+F*TNIGHT/DTIME
      SAVE=PTURB*ETAREF*SAVE2
      PAR=SAVE*REFPRL*FSP
      SUMPAR=SUMPAR+F*HSTR*PAR/DTIME
      ELN=SAVE-PAR
      SUMELP=SUMELP+F*HSTR*ELN/DTIME
      HSTR=0.
      PTHR=0.
      PTH=0.
      SOL=0.
      PSTR=PTURB
      PTOTB=PTURB
      ELGRS=ELN+PAR
      IF (IPSTR.NE.0) WRITE (6,4140) DAY(I),T,SOL,PTHR,PTH,PSTR,HSTR,
     1PTOTB,ELGRS,PAR,ELN
2400  CONTINUE
         IF (IPSTR.NE.0) WRITE (6,4160)
2410  CONTINUE
      PARSTRT=HRSTRT*(PARL1+PARL2*SAVEP)*PF
      PARPRE=HRPRE*(PARL3+PARL4*SAVEP)*PF
      PARNITE=HRNIGHT*(PARL5+PARL6*SAVEP)*PF
      PARWEAT=HRWEATH*(PARL7+PARL8*SAVEP)*PF
      PARPF=24.0*365.0*(1.0-PF)*(PARL9+PARL10*SAVEP)
      SUMELP=SUMELP*PF
      SUMELP=SUMELP-(PARSTRT+PARPRE+PARNITE+PARWEAT+PARPF)
      ACSY=CCMULY*CSTRN*(FCR+RNHOML)
      BBECNY=(ACWOSY+ACHY+ACSY)/SUMELP
      IF (IPSTR.NE.0) GO TO 2440
         IF (BBECNY.GT.ECSAVY.AND.ECOLD.GT.ECSAVY) GO TO 2420
      ECOLD=ECSAVY
      ECSAVY=BBECNY
      ICSAVE=IST
      FSAVE=FSTR
      CCSAVY=((ACWOSY+ACSY)/(FCR+RNHOML)+ACHY/(FCR+RHOML))/1.E+06
      CCSAVC=CCSAVY*CCMULC/CCMULY
      ECSAVC=BBECNY*CCMULC/CCMULY
         IF (IS.NE.NSTR) GO TO 2430
2420  IPSTR=1
      IST=ICSAVE
         GO TO 2240
2430  CONTINUE
2440  CONTINUE
      WRITE (6,4180)
      WRITE (6,4190) YSOL
      SAVE=SUMTHR/RRECL*PF
      HEADFL= 8H MW-TH H
      WRITE (6,4195) PF
 4195 FORMAT(1X,30HBASED ON A PLANT FACTOR OF    ,F10.3,1H,)
      WRITE (6,4200) SAVE,HEADFL
      SAVE2=SAVE*(1.-RRECL)
      SAVE=SAVE-SAVE2
      WRITE (6,3960) SAVE2,HEADFL
      WRITE (6,3970) SUMRCP*PF,HEADFL
      SAVE=SAVE-SUMRCP*PF
      SAVE2=SUMRCP*PF*PIPPOW/RCPOW
      WRITE (6,3980) SAVE2,HEADFL
      SAVE=SAVE-SAVE2
      WRITE (6,4210) SAVE,HEADFL
      SAVE3=EFFSTR*(SUMSTR)*PF
      WRITE (6,4220) SUMSTR*PF,HEADFL
      WRITE (6,4230) EFFSTR
      WRITE (6,4240) SUMSTL*PF,HEADFL
      SAVE=SAVE-SUMSTR*PF-SUMSTL*PF
      WRITE (6,4250) SAVE,HEADFL
      WRITE (6,4260) SAVE3,HEADFL
      ETATHE=(SUMELP+SUMPAR*PF+PARNITE+PARSTRT+PARPRE+PARWEAT+PARPF)
      ETATHE=ETATHE/(SAVE+SAVE3)
      SAVE=(SUMELP+SUMPAR*PF+PARNITE+PARSTRT+PARPRE+PARWEAT+PARPF)
C     SAVE=ETATHE*(SAVE+SAVE3)--OLD WAY ASSUMES ETATHE=FEFF*ETAREF
      WRITE (6,4020) ETATHE
      HEADFL= 8H MW-EL H
      WRITE (6,4270) SAVE,HEADFL
      WRITE (6,4040) SUMPAR*PF,HEADFL
      WRITE (6,4282) PARSTRT,PARPRE,PARNITE,PARWEAT,PARPF
 4282 FORMAT (26H NONOPERATING PARASITICS: /
     119H   STARTUP/SHUTDOWN,15X,F12.3/
     322H   PRESTARTUP/SHUTDOWN,12X,F12.3/
     3 8H   NIGHT,26X,F12.3/
     410H   WEATHER,24X,F12.3/
     516H   PLANT OUTAGES,18X,F12.3)
      WRITE (6,4280) SUMELP,HEADFL
2450  CONTINUE
      RETURN
3930  FORMAT (1H1,20X,24H----- DESIGN POINT -----//)
3940  FORMAT (1X,5H DAY=,F8.2,6H TIME=,F8.2,13H INSOLATION =,F7.3,6H KW/
     1M2//)
3950  FORMAT (28H GROSS POWER ONTO RECEIVER  ,9X,F12.3 ,A8/)
3960  FORMAT (25H REFLECTIVITY LOSS       ,12X,F12.3,A8)
3970  FORMAT (33H RADIATION/CONVECTION LOSS       ,4X,F12.3,A8)
3980  FORMAT (21H PIPING LOSSES       ,16X,F12.3,A8/)
3990  FORMAT (31H THERMAL POWER TOWER BASE      ,6X,F12.3,A8/)
4000  FORMAT (23H POWER TO STORAGE      ,14X,F12.3,A8)
4010  FORMAT (/17H POWER TO TURBINE,20X,F12.3,A8)
4020  FORMAT (23H   THERM. TO ELEC. EFF.,14X,F12.3)
4030  FORMAT (27H GROSS ELECTRIC POWER      ,10X,F12.3,A8)
4040  FORMAT (36H ELECTRICAL PARASITICS (OPERATIONAL),1X,F12.3,A8/)
4050  FORMAT (25H NET ELECTRIC POWER      ,12X,F12.3,A8//)
4060  FORMAT (///109H  -----DETAILED ANNUAL PERFORMANCE CAN ONLY BE DONE
     1 WITH DELSOL OPTIMIZED SYSTEM READ FROM TAPE OR RERUN-----)
4070  FORMAT (/86H  -----WE ARE VERY DISAPPOINTED IN YOU FOR NOT USING D
     1ELSOL TO DESIGN THIS SYSTEM-----)
4080  FORMAT (/97H  -----WE CANNOT BE RESPONSIBLE FOR THE CODE"S ACCURAC
     1Y NOW THAT YOU HAVE HURT ITS FEELINGS -----)
4090  FORMAT (1H1,20X,48H ANNUAL POWER PRODUCTION FROM USER DEFINED FIEL
     1D//)
4100  FORMAT (100H THE DISPATCH STRATEGY FOR STORAGE IS TO RUN THE TURBI
     1NE AT ITS DESIGN POINT POWER WHENEVER POSSIBLE/)
4110  FORMAT (/89H THE STORAGE POWER HAS A POSITIVE SIGN WHEN CHARGING A
     1ND A NEGATIVE SIGN WHEN DISCHARGING/)
4120  FORMAT (56H ALL POWERS ARE IN MEGAWATT-THERMAL OR MEGAWATT ELECTRI
     1C)
4130  FORMAT (40H INSOLATION UNITS ARE KILOWATTS/SQ.METER)
4140  FORMAT (2X,F6.2,4X,F6.3,3X,F5.3,2X,F9.3,2X,F9.3,2X,F9.3,2X,F9.3,
     1  2X,F9.3,2X,F9.3,2X,F9.3,2X,F9.3)
4150  FORMAT (3X,3HDAY,7X,4HHOUR,5X,6HINSOL.,2X,9HREC. PWR.,2X,9HTWR. PW
     1R.,2X,9HSTR. PWR.,2X,8HHR. STR.,3X,10HTURB. PWR.,2X,9HGROSS EL.,2X
     2,8HEL. PAR.,2X,7HNET EL.)
4160  FORMAT (/)
4170  FORMAT (/46H THE OPTIMUM STORAGE CAPACITY IS APPROXIMATELY,F5.2,23
     1H TIMES THE MAXIMUM SIZE/23H   TOTAL CAPITAL COST =,F7.2,51H(M$,1S
     2T YR OPERATION)          TOTAL CAPITAL COST =,F7.2,14H(M$,CURRENT
     3$)/26H   LEVELIZED ENERGY COST =,F7.2,41H MILLS/KWHR       LEVELIZ
     4ED ENERGY COST =,F7.2,22H MILLS/KWHR(CURRENT $)/)
4180  FORMAT (1H1,20X,27H YEARLY AVERAGE PERFORMANCE///)
4190  FORMAT (18H YEARLY INSOLATION,19X,F12.3,9H KW-HR/M2/)
4200  FORMAT (27H GROSS ENERGY ONTO RECEIVER,10X,F12.3,A8/)
4210  FORMAT (29H THERMAL ENERGY AT TOWER BASE,8X,F12.3,A8/)
4220  FORMAT (18H ENERGY TO STORAGE,19X,F12.3,A8)
4230  FORMAT (18H   ROUND TRIP EFF.,18X,F12.3)
4240  FORMAT (17H ENERGY DISCARDED,20X,F12.3,A8/)
4250  FORMAT (32H ENERGY TO TURBINE FROM RECEIVER,5X,F12.3,A8)
4260  FORMAT (31H ENERGY TO TURBINE FROM STORAGE,6X,F12.3,A8)
4270  FORMAT (22H GROSS ELECTRIC ENERGY,15X,F12.3,A8/)
4280  FORMAT (26H NET ELECTRICAL PRODUCTION,11X,F12.3,A8//)
      END
      SUBROUTINE SHADOW (SSP,CSP,SSA,CSA,SBNS,CBNS)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C         SUBROUTINE SHADOW IS CALLED IN THE INITIAL PERFORMANCE CALCU-
C     LATION IN DELSOL  FOR FIRST THE SHADOWING, THEN THE BLOCKING
C     CALCULATION. THE 12 NEAREST HELIOSTATS ARE CONSIDERED.
C
C     THIS ROUTINE IS CALLED BY
C          PERCAL
C
      COMMON /GEN4/ HCSDP(13,12,25),HEAD19,HEAD20,HEAD5,HEAD67,NUMFF,
     1  NUMTT,SAB(7,7),SCS(13,7,25),YINSOL
C
C>>>2RCOMMON /GEN6/ AZMSEP(13,12),DENSMR,FSLIP,ILAY,RADIUS(13)
      COMMON /GEN6/ AZMSEP(13,12),DENSMS,FSLIP,ILAY,MZONE(7),
     1  RADIUS(13)
C
      COMMON /GEN7/ AMIRRO,DENSIT(13,12),DFFF,DTTT,FEP,H,HM,HMN,
     1  IROUND,NSTR,PI,RRECL,THT,WMN
C
      COMMON /ALL1/ CTA,CTP,FLTOT,HN,RA2(4),RB2(4),RXN(4),
     1  RYN(4),STA,STP,THTI,WN,W2,XTOWES,YTOWES,IPROPT
C
      COMMON /PER1/ A(3,3),B(2,2),BIN(7,7),CNA,CNP,DAY(9),DAYDEL,
     1  DR(20),DT(20),FLAY,FY(9),GCANTA,GCANTB,GCANTX,GCANTY,
     2  GRSAVE(15,15),GSAVE(7,7,4),HCOS,HCTHT(16,4),HFLTOT(9,20),
     3  HINSOL(9,20),HXCANN(25),HYCANN(25),
     4  PSAVE(7,7),RRMULT(5,5),RRR(20),
     5  RUS(7,7),SAVESB(5,5),SIGAZ2,SIGEL2,SIGSX2,SIGSY2,SIGTX2,
     6  SIGTY2,SIH1,SIH2,SNA,SNP,SRANGE(7),TIME(9,20),TSUNSE(9),
     7  ZZMULT(5,5),ZZZ(20),
     8  IDP,INEAR,IPRT,JDP,MSTP(7),NH(9),NNEARS
C>>>>R LINE 4 ABOVE
 
C>>>>I
      COMMON /HERM2/ FACT(7),ISTEP,JMN(7),JMX(7),LL,NORDER
 
C>>>>I
      COMMON /HERM3/ ITYPE,KK,SPL,UF(7,7)
C
C
C>>>>RK=KS
      K = KK
C>>>>RL=LS
      L = LL
      NNEAR=NNEARS-INEAR
      ZCOS=HCOS/2.
      RAD=RADIUS(K)
      DXZ=SSP*CSA
      DYZ=-SSP*SSA
      DXX=SNA-SNP/2./ZCOS*SSP*CNA*SBNS
      DYX=CNA+SNP/2./ZCOS*SSP*SNA*SBNS
      C=-(SSP*CNP*CBNS-CSP*SNP)*SNP/2./ZCOS+CNP
      DXY=CNA*C
      DYY=-SNA*C
C
C             CALCULATE SHADOWING
C
      DO 150 II=1,NNEAR,INEAR
      DO 140 MM=1,NUMTT
      DO 140 NN=1,NUMFF
         IF (ILAY.LT.2) GO TO 110
      T=ZZZ(II)*ZZMULT(NN,MM)
      R=RRR(II)*RRMULT(NN,MM)
         GO TO 120
110   CONTINUE
C>>>>RDENS=DENSIT(K,L)*AZMSEP(K,L)/DENSMR
      DENS = DENSIT(K,L)*AZMSEP(K,L)/DENSMS
      S1=SIN(WMN*AZMSEP(K,L)*DT(II)*ZZMULT(NN,MM)/RAD)
      C1=COS(WMN*AZMSEP(K,L)*DT(II)*ZZMULT(NN,MM)/RAD)
      R=RAD*(C1-1.)+C1*DR(II)*RRMULT(NN,MM)*HMN/DENS
      T=(RAD+DR(II)*RRMULT(NN,MM)*HMN/DENS)*S1
120   CONTINUE
      DX=FLAY*(-CTA*R+STA*T)-(1.-FLAY)*R
      DY=FLAY*(STA*R+CTA*T)+(1.-FLAY)*T
C
C             DX IS SOUTH, DY IS EAST, X,Y,Z ARE IN SUN COORDINATES
C
      DX=DX*SIH1-ZZZ(II)*SIH2
      DY=DY*SIH1+RRR(II)*SIH2
      Z=DX*DXZ+DY*DYZ
C
C             TEST TO SEE IF HELIOSTAT IS BEHIND REFERENCE
C
         IF (Z.LT.0) GO TO 150
      X=ABS(DX*DXX+DY*DYX)
C
C             TEST FOR OVERLAPPING SIDES OF HELIOSTAT PROJECTIONS
C
         IF (X.GE.WMN) GO TO 140
      Y=ABS(DX*DXY+DY*DYY)
C                 TEST FOR OVERLAP OF TOP+BOT OF HSTAT PROJECTIONS
         IF (Y.GT.HMN) GO TO 140
         IF (IROUND.GT.0) GO TO 130
C
C             SHADOWING AND BLOCKING IS REDUCED BY MISSING HELIOSTATS
C
      SAB(NN,MM)=SAB(NN,MM)*(1.-(WMN-X)*(HMN-Y)/(WMN*HMN*SAB(NN,MM))*
     1SAVESB(NN,MM))
C
C             SET SHADOWING TO ZERO IF NEGATIVE
C
      SAB(NN,MM)=(SAB(NN,MM)+ABS(SAB(NN,MM)))/2.+1.E-08
         GO TO 140
C
C             ROUND HELIOSTATS
C
130   SAVE=X**2+Y**2
         IF (SAVE.GT.WMN**2) GO TO 140
      SAVE=SQRT(SAVE)
      SAVE=ACOS(SAVE/WMN)
      SAB(NN,MM)=SAB(NN,MM)*(1.-(SAVE*2.-SIN(2.*SAVE))/(PI*SAB(NN,MM))*S
     1AVESB(NN,MM))
       SAB(NN,MM)=(SAB(NN,MM)+ABS(SAB(NN,MM)))/2.+1.E-08
140   CONTINUE
150   CONTINUE
      RETURN
      END
      SUBROUTINE PLOTMAP(XAIM1,XAIM2,YAIM1,YAIM2,DPSPILL,HSPILL,
     1CON1,NXFLX,NYFLX,FLFLX,IREFLX,IFXOUT,XNFLX,YNFLX,
     2RADIUS,AZM)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C     THIS ROUTINE USES DISPLAY TO GIVE INSTANT AFFECTS OF CHANGING
C     USER AIMPOINTS
C     STARTED 8-20-85 BY B. L. KISTLER
C     CALLED BY PERCAL ONLY WHEN IAUTOP=1 AND IFXOUT(I,J)=-2
C     CALLED FOR EACH ZONE
C
      COMMON WORK(5000)
      DIMENSION FLX(200),FLFLX(NXFLX*NYFLX),FLXFLX(20,20)
      DIMENSION XXNFLX(200),YYNFLX(200)
      DIMENSION XNFLX(NXFLX),YNFLX(NYFLX)
C
      WRITE (1,980) RADIUS,AZM,XAIM1,XAIM2,YAIM1,YAIM2,HSPILL,DPSPILL
 980  FORMAT(/1X,'ZONE RADIUS=',E10.3,' ZONE ANGLE=',E10.3/
     11X,'MIN-MAX X AIMPOINTS=',2E10.3,' YAIMPOINTS=',2E10.3,/
     21X,'ZONE INTERCEPT=',E10.3,' CUMULATIVE INTERCEPT SO FAR=',E10.3/)
      DO 5 I=1,NYFLX
      DO 6 J=1,NXFLX
      K=J+(I-1)*NXFLX
      XXNFLX(K)=XNFLX(J)
      YYNFLX(K)=YNFLX(I)
 6    CONTINUE
 5    CONTINUE
      IREFLX=0
      REWIND(32)
      READ (32) (FLFLX(I),I=1,NXFLX*NYFLX)
      IFLAG=0
      FLMAX=0.
      FLXMAX=0.
      DO 10 I=1,NYFLX
      K1=1+NXFLX*(I-1)
      K2=NXFLX*(I)
      READ (32) (FLX(J),J=K1,K2)
      DO 15 II=1,NXFLX
      KII=(I-1)*NXFLX+II
      IF(FLX(KII).NE.0.0) IFLAG=1
      IF(FLFLX(KII).GT.FLMAX) FLMAX=FLFLX(KII)
      IF(FLX(KII).GT.FLXMAX) FLXMAX=FLX(KII)
 15   CONTINUE
 10   CONTINUE
      IF(IFLAG.EQ.0) WRITE (1,970)
 970  FORMAT(/1X,'**ERROR** NO FLUX FROM THIS ZONE--TRY OTHER AIM'/)
      IF(IFLAG.EQ.0) GO TO 100
      REWIND(11)
C
C      ALWAYS COME TO THIS MENU
C
 100  CONTINUE
      WRITE (1,900)
 900  FORMAT(//1X,'1.  PLOT THIS ZONE FLUX MAP'/
     11X,'2.  PLOT TOTAL FLUX MAP SO FAR'/
     21X,'3.  CHANGE XAIM,YAIM AND RECALCULATE FLUX MAP'/
     31X,'4.  ACCEPT THIS ZONE AIMING AND GO TO NEXT ZONE'/
     31X,'5.  DONT DO INTERACTIVITY NO MORE!'//
     41X,'INPUT NUMBER OF SELECTION DESIRED')
      READ (1,910) MENUNUM
 910  FORMAT(I1)
      IF(MENUNUM.LT.1.OR.MENUNUM.GT.5) GO TO 100
      IF(MENUNUM.EQ.1) GO TO 200
      IF(MENUNUM.EQ.2) GO TO 300
      IF(MENUNUM.EQ.3) GO TO 400
      IF(MENUNUM.EQ.4) GO TO 500
      IF(MENUNUM.EQ.5) GO TO 600
      GO TO 100
 500  CONTINUE
      IREFLX=0
      RETURN
 400  CONTINUE
      WRITE (1,920) XAIM1
 920  FORMAT(/1X,'OLD VALUE=',E10.3,'INPUT NEW MINIMUM XAIM')
      READ (1,930) XAIM1
      WRITE (1,940) XAIM2
 930  FORMAT(F20.0)
 940  FORMAT(/1X,'OLD VALUE=',E10.3,'INPUT NEW MAXIMUM XAIM')
      READ (1,930) XAIM2
      WRITE (1,950) YAIM1
 950  FORMAT(/1X,'OLD VALUE=',E10.3,'INPUT NEW MINIMUM YAIM')
      READ (1,930) YAIM1
      WRITE (1,960) YAIM2
 960  FORMAT(/1X,'OLD VALUE=',E10.3,'INPUT NEW MAXIMUM YAIM')
      READ (1,930) YAIM2
      IREFLX=1
      DO 30 K=1,NXFLX*NYFLX
C     ACTUALLY BELOW LINE SHOULD HAVE CON1/CON*FLX(K)
      FLFLX(K)=FLFLX(K)-FLX(K)
 30   CONTINUE
      RETURN
 200  CONTINUE
C     PLOT ZONE FLUX MAP HERE
C     CALL STRTT1
C     CALL SETDEV(6,6)
C     CALL BGNPL(0)
C     CALL TITLE('CONTOUR OF FLUX$',100,'CIRCUMFERENCE$',100,
C    1'HEIGHT$',100,7.,7.)
C     CALL GRAF(XNFLX(1),(XNFLX(NXFLX)-XNFLX(1))/10.,XNFLX(NXFLX),
C    1YNFLX(NYFLX),(YNFLX(NYFLX)-YNFLX(1))/10.,YNFLX(1))
C     CALL FRAME
C     CALL BGNMAT(20,20)
C     CALL GETMAT(XXNFLX,YYNFLX,FLX,NXFLX*NYFLX,0)
C     CALL ENDMAT(FLXFLX,0)
C     CALL BCOMON(5000)
C     CALL CONMAK(FLXFLX,20,20,FLXMAX/10.)
C     CALL CONANG(30.)
C     CALL CONLIN(0,'SOLID','LABELS',1,5)
C     CALL CONLIN(1,'DASH','NOLABELS',1,1)
C     CALL CONLIN(2,'DASH','NOLABELS',1,1)
C     CALL CONLIN(3,'DASH','NOLABELS',1,1)
C     CALL CONLIN(4,'DASH','NOLABELS',1,1)
C     CALL PARA3
C     CALL CONDIG(2)
C     CALL CONTUR(1,'LABELS','DRAW')
C     CALL MESSAG('ZONE PEAK FLUX=$',100,.6,.6)
C     CALL REALNO(FLXMAX,3,'ABUT','ABUT')
C     CALL MESSAG('ZONE INTERCEPT=$',100,.6,.8)
C     CALL REALNO(HSPILL,3,'ABUT','ABUT')
C     CALL ENDPL(0)
C     CALL DONEPL
      GO TO 100
 300  CONTINUE
C     PLOT TOTAL FLUX MAP HERE
C     CALL STRTT1
C     CALL SETDEV(6,6)
C     CALL BGNPL(0)
C     CALL TITLE('CONTOUR OF FLUX$',100,'CIRCUMFERENCE$',100,
C    1'HEIGHT$',100,7.,7.)
C     CALL GRAF(XNFLX(1),(XNFLX(NXFLX)-XNFLX(1))/10.,XNFLX(NXFLX),
C    1YNFLX(NYFLX),(YNFLX(NYFLX)-YNFLX(1))/10.,YNFLX(1))
C     CALL FRAME
C     CALL BGNMAT(20,20)
C     CALL GETMAT(XXNFLX,YYNFLX,FLFLX,NXFLX*NYFLX,0)
C     CALL ENDMAT(FLXFLX,0)
C     CALL BCOMON(5000)
C     CALL CONMAK(FLXFLX,20,20,FLMAX/10.)
C     CALL CONANG(30.)
C     CALL CONLIN(0,'SOLID','LABELS',1,5)
C     CALL CONLIN(1,'DASH','NOLABELS',1,1)
C     CALL CONLIN(2,'DASH','NOLABELS',1,1)
C     CALL CONLIN(3,'DASH','NOLABELS',1,1)
C     CALL CONLIN(4,'DASH','NOLABELS',1,1)
C     CALL PARA3
C     CALL CONDIG(2)
C     CALL CONTUR(1,'LABELS','DRAW')
C     CALL MESSAG('TOTAL PEAK FLUX=$',100,.6,.6)
C     CALL REALNO(FLMAX,3,'ABUT','ABUT')
C     CALL MESSAG('TOTAL INTERCEPT=$',100,.6,.8)
C     CALL REALNO(DPSPILL,3,'ABUT','ABUT')
C     CALL ENDPL(0)
C     CALL DONEPL
      GO TO 100
 600  CONTINUE
      IFXOUT=-1
      IREFLX=0
      RETURN
      END
